/*****************************************************************************
 *  rsfpl_adem.c
 *
 *  Rohde & Schwarz FPL spectrum analyzer Instrument Driver
 *  Original Release: May 2018
 *  By: Jiri Kominek
 *
 *  Should you have any technical questions please contact the hotline of
 *  Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
 *
 *  e-mail: CustomerSupport@rohde-schwarz.com
 *
 *  Modification History:
 *    see ChangeLog
 *
 *  Copyright (c) 2018, Rohde & Schwarz GmbH & Co. KG
 *  All rights reserved.
 *****************************************************************************/

#include "rsfpl.h"

#if (defined(WIN32) || defined(_WIN32) || defined(__WIN32__) || defined(__NT__))
#define ANSI
#endif
#ifndef ANSI                /* UNIX compatible */
#include <sys/timeb.h>
#include <sys/types.h>
#include <unistd.h>
#define Sleep sleep
#else                       /* ANSI compatible */
#include <windows.h>
#endif
#include <string.h>
#if defined (_CVI_)
#include <utility.h>
#define _strnicmp strnicmp
#endif

 /// HIFN  This function selects the Analog Demodulation mode.
 /// HIRET Returns the status code of this operation.
 /// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
 /// HIPAR instrSession/rsfpl_InitWithOptions function.  The handle identifies a particular
 /// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_SetADEMMode(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_WriteWithOpc(instrSession, "INST:SEL ADEM", 0));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Loads a measurement configuration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fileName/Loads a measurement configuration.
ViStatus _VI_FUNC rsfpl_ADEMLoadConfiguration(ViSession instrSession,
                                              ViString  fileName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_ADEM_LOAD_CONFIGURATION, fileName),
            2, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Saves the current Analog Demodulation measurement configuration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fileName/Saves the current Analog Demodulation measurement configuration.
ViStatus _VI_FUNC rsfpl_ADEMSaveConfiguration(ViSession instrSession,
                                              ViString  fileName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_ADEM_SAVE_CONFIGURATION, fileName),
            2, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Restores the default configurations of predefined Analog Demodulation
/// HIFN  standards.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_ADEMRestoreConfigurations(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_ADEM_RESTORE_CONFIGURATIONS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the level when AM-modulated signals are used as trigger source.
/// HIFN     For triggering to be successful, the measurement time must cover at
/// HIFN  least 5 periods of the audio signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR amLevel/Sets the level when AM-modulated signals are used as trigger source.
ViStatus _VI_FUNC rsfpl_ConfigureADEMTriggerAMLevel(ViSession instrSession,
                                                    ViReal64  amLevel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_ADEM_TRIGGER_AM_LEVEL, amLevel),
            2, "AM Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the level when RF power signals are used as trigger source.
/// HIFN     For triggering to be successful, the measurement time must cover at
/// HIFN  least 5 periods of the audio signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR rfLevel/Sets the level when RF power signals are used as trigger source.
ViStatus _VI_FUNC rsfpl_ConfigureADEMTriggerRFLevel(ViSession instrSession,
                                                    ViReal64  rfLevel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_ADEM_TRIGGER_RF_LEVEL, rfLevel),
            2, "RF Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the level when FM-modulated signals are used as trigger source.
/// HIFN     For triggering to be successful, the measurement time must cover at
/// HIFN  least 5 periods of the audio signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fmLevel/Sets the level when FM-modulated signals are used as trigger source.
ViStatus _VI_FUNC rsfpl_ConfigureADEMTriggerFMLevel(ViSession instrSession,
                                                    ViReal64  fmLevel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_ADEM_TRIGGER_FM_LEVEL, fmLevel),
            2, "FM Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the level when PM-modulated signals are used as trigger source.
/// HIFN     For triggering to be successful, the measurement time must cover at
/// HIFN  least 5 periods of the audio signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR pmLevel/Sets the level when PM-modulated signals are used as trigger source.
ViStatus _VI_FUNC rsfpl_ConfigureADEMTriggerPMLevel(ViSession instrSession,
                                                    ViReal64  pmLevel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_ADEM_TRIGGER_PM_LEVEL, pmLevel),
            2, "PM Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the bandwidth for analog demodulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR demodulationBandwidth/Sets the bandwidth for analog demodulation. Depending on the selected
/// HIPAR demodulationBandwidth/demodulation bandwidth, the instrument selects the required sample
/// HIPAR demodulationBandwidth/rate.
/// HIPAR demodulationFilter/Defines the type of demodulation filter to be used.
/// HIPAR measurementTime/Defines the measurement time for analog demodulation.
ViStatus _VI_FUNC rsfpl_ConfigureADEMBandwidth(ViSession instrSession,
                                               ViReal64  demodulationBandwidth,
                                               ViInt32   demodulationFilter,
                                               ViReal64  measurementTime)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_ADEM_BANDWIDTH, demodulationBandwidth),
            2, "Demodulation Bandwidth");

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_ADEM_FILTER_TYPE, demodulationFilter),
            3, "Demodulation Filter");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_ADEM_MEASUREMENT_TIME, measurementTime),
            4, "Measurement Time");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the sample rate set up for the current analog demodulation
/// HIFN  measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR sampleRate/Returns the sample rate set up for the current analog demodulation
/// HIPAR sampleRate/measurement.
ViStatus _VI_FUNC rsfpl_QueryADEMSampleRate(ViSession instrSession,
                                            ViReal64* sampleRate)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_GetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_ADEM_SAMPLE_RATE, sampleRate),
            2, "Sample Rate");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the record length set up for the current analog demodulation
/// HIFN  measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR recordLength/Returns the record length set up for the current analog demodulation
/// HIPAR recordLength/measurement.
ViStatus _VI_FUNC rsfpl_QueryADEMRecordLength(ViSession instrSession,
                                              ViInt32*  recordLength)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_GetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_ADEM_RECORD_LENGTH, recordLength),
            2, "Record Length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the analog demodulator of the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR sampleRate/The frequency at which measurement values are taken from the
/// HIPAR sampleRate/A/D-converter and stored in I/Q memory.
/// HIPAR recordLength/Number of samples to be stored in I/Q memory.
/// HIPAR triggerSource/Trigger source.
/// HIPAR triggerSlope/Used slope of the trigger signal.
/// HIPAR offsetSamples/Number of samples to be used as an offset to the trigger signal.
/// HIPAR numberOfMeasurements/Number of repetitions of the measurement to be executed. The value
/// HIPAR numberOfMeasurements/indicated here is especially necessary for the average/maxhold/minhold
/// HIPAR numberOfMeasurements/function.
ViStatus _VI_FUNC rsfpl_ConfigureADEMDemodulator(ViSession instrSession,
                                                 ViReal64  sampleRate,
                                                 ViInt32   recordLength,
                                                 ViInt32   triggerSource,
                                                 ViInt32   triggerSlope,
                                                 ViInt32   offsetSamples,
                                                 ViInt32   numberOfMeasurements)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2mode;
    ViChar*  p2slope;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsFpl_rngTriggerSource, triggerSource, &p2mode),
        3, "Trigger Mode");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsFpl_rngPosNeg, triggerSlope, &p2slope),
        4, "Trigger Slope");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "ADEM:SET %.12G,%ld,%s,%s,%ld,%ld",
            sampleRate, recordLength, p2mode, p2slope, offsetSamples, numberOfMeasurements);
    checkErr(rsfpl_WriteCommandWithOPCSync(instrSession, cmd, 0));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the unit for angles.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR unit/Selects the unit for angles.
ViStatus _VI_FUNC rsfpl_ConfigureADEMUnitAngle(ViSession instrSession,
                                               ViInt32   unit)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_ADEM_UNIT_ANGLE, unit),
            2, "Unit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the unit for THD measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR unit/Selects the unit for THD measurements.
ViStatus _VI_FUNC rsfpl_ConfigureADEMUnitTHD(ViSession instrSession,
                                             ViInt32   unit)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_ADEM_UNIT_THD, unit),
            2, "Unit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the squelch function, i.e. if the signal falls below a
/// HIFN  defined threshold, the demodulated data is automatically set to 0.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR squelchEnabled/Activates the squelch function, i.e. if the signal falls below a
/// HIPAR squelchEnabled/defined threshold, the demodulated data is automatically set to 0.
/// HIPAR squelchLevel/Defines the level threshold below which the demodulated data is set to
/// HIPAR squelchLevel/0 if squelching is enabled. This control has effect only if Squelch
/// HIPAR squelchLevel/Enabled == On.
ViStatus _VI_FUNC rsfpl_ConfigureADEMSquelch(ViSession instrSession,
                                             ViBoolean squelchEnabled,
                                             ViReal64  squelchLevel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_ADEM_SQUELCH_ENABLED, squelchEnabled),
            2, "Squelch Enabled");
    if (squelchEnabled)
    {
        viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_ADEM_SQUELCH_LEVEL, squelchLevel),
                3, "Squelch Level");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Determines the position where the phase of the PM-demodulated signal
/// HIFN  is set to 0 rad. The maximum possible value depends on the measurement
/// HIFN  time selected in the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR zeroPhaseReferencePosition/Determines the position where the phase of the PM-demodulated signal
/// HIPAR zeroPhaseReferencePosition/is set to 0 rad. The maximum possible value depends on the measurement
/// HIPAR zeroPhaseReferencePosition/time selected in the instrument.
ViStatus _VI_FUNC rsfpl_ConfigureADEMZeroPhaseReferencePosition(
                            ViSession instrSession,
                            ViReal64  zeroPhaseReferencePosition)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_ADEM_ZERO_PHASE_REFERENCE_POSITION, zeroPhaseReferencePosition),
            2, "Zero Phase Reference Position");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines how the reference position in time for 0 rad is determined.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mode/Defines how the reference position in time for 0 rad is determined.
ViStatus _VI_FUNC rsfpl_ConfigureADEMZeroPhaseReferencePositionMode(
                            ViSession instrSession,
                            ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, "", RSFPL_ATTR_ADEM_ZERO_PHASE_REFERENCE_POSITION_MODE, mode),
            2, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the coupling of the AF path of the analyzer in the specified
/// HIFN  window.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR afCoupling/Selects the coupling of the AF path of the analyzer in the specified
/// HIPAR afCoupling/window.
ViStatus _VI_FUNC rsfpl_ConfigureADEMAFCoupling(ViSession instrSession,
                                                ViInt32   window,
                                                ViInt32   afCoupling)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_ADEM_AF_COUPLING,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_ADEM_AF_COUPLING, afCoupling),
            3, "AF Coupling");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the time domain zoom function for the analog-demodulated
/// HIFN  measurement data in the specified window.
/// HIFN     If the zoom function is enabled, the defined number of sweep points
/// HIFN  are displayed from the start time specified with
/// HIFN  [SENSe:]ADEMod<n>:ZOOM:STARt.
/// HIFN     If the zoom function is disabled, data reduction is used to adapt the
/// HIFN  measurement points to the number of points available on the display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR zoomEnabled/The command enables or disables the time domain zoom function for the
/// HIPAR zoomEnabled/analog-demodulated measurement data in the specified window.
/// HIPAR window/This control selects the measurement window.
/// HIPAR start/Selects the start time for the zoomed display of analog-demodulated
/// HIPAR start/measurements in the specified window. The maximum possible value
/// HIPAR start/depends on the measurement time.
/// HIPAR lengthMode/Defines whether the length of the zoom area for the analog-demodulated
/// HIPAR lengthMode/measurement data is defined automatically or manually in the specified
/// HIPAR lengthMode/window.
/// HIPAR length/Allows you to define the length of the time domain zoom area for the
/// HIPAR length/analog-demodulated measurement data in the specified window manually.
/// HIPAR length/If the length is defined manually using this control, the zoom mode is
/// HIPAR length/also set to manual.
ViStatus _VI_FUNC rsfpl_ConfigureADEMTimeDomainZoom(ViSession instrSession,
                                                    ViInt32   window,
                                                    ViBoolean zoomEnabled,
                                                    ViReal64  start,
                                                    ViInt32   lengthMode,
                                                    ViReal64  length)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_ADEM_TIME_DOMAIN_ZOOM_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_ADEM_TIME_DOMAIN_ZOOM_ENABLED, zoomEnabled),
            3, "Zoom Enabled");
    if (zoomEnabled)
    {
        viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_ADEM_TIME_DOMAIN_ZOOM_START, start),
                4, "Start");

        viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_ADEM_TIME_DOMAIN_ZOOM_LENGTH_MODE, lengthMode),
                5, "Length Mode");
        if (lengthMode == RSFPL_VAL_MANUAL)
        {
            viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_ADEM_TIME_DOMAIN_ZOOM_LENGTH, length),
                    6, "Length");
        }
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the center frequency for AF spectrum result display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR afCenter/Sets the center frequency for AF spectrum result display.
ViStatus _VI_FUNC rsfpl_ConfigureADEMAFCenter(ViSession instrSession,
                                              ViReal64  afCenter)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_ADEM_AF_CENTER, afCenter),
            2, "AF Center");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the span (around the center frequency) for AF spectrum result
/// HIFN  display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR afSpan/Sets the span (around the center frequency) for AF spectrum result
/// HIPAR afSpan/display.
ViStatus _VI_FUNC rsfpl_ConfigureADEMAFSpan(ViSession instrSession,
                                            ViReal64  afSpan)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_ADEM_AF_SPAN, afSpan),
            2, "AF Span");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the maximum span for AF spectrum result display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_SetADEMAFSpanFull(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_ADEM_AF_SPAN_FULL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the start frequency for AF spectrum result display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR afStart/Sets the start frequency for AF spectrum result display.
ViStatus _VI_FUNC rsfpl_ConfigureADEMAFStart(ViSession instrSession,
                                             ViReal64  afStart)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_ADEM_AF_START, afStart),
            2, "AF Start");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the stop frequency for AF spectrum result display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR afStop/Sets the stop frequency for AF spectrum result display.
ViStatus _VI_FUNC rsfpl_ConfigureADEMAFStop(ViSession instrSession,
                                            ViReal64  afStop)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_ADEM_AF_STOP, afStop),
            2, "AF Stop");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates automatic scaling of the y-axis in all diagrams according to
/// HIFN  the current measurement results. Currently auto-scaling is only
/// HIFN  available for AF measurements. RF power and RF spectrum measurements
/// HIFN  are not affected by the auto-scaling.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR afAutoScaleEnabled/Activates automatic scaling of the y-axis in all diagrams according to
/// HIPAR afAutoScaleEnabled/the current measurement results. Currently auto-scaling is only
/// HIPAR afAutoScaleEnabled/available for AF measurements. RF power and RF spectrum measurements
/// HIPAR afAutoScaleEnabled/are not affected by the auto-scaling.
ViStatus _VI_FUNC rsfpl_ConfigureADEMAFAutoScaleEnabled(ViSession instrSession,
                                                        ViBoolean afAutoScaleEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_ADEM_AF_AUTO_SCALE_ENABLED, afAutoScaleEnabled),
            2, "Auto Scale Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the DBW to the specified value and the span (around the center
/// HIFN  frequency) of the RF data to be evaluated to its new maximum (the
/// HIFN  demodulation bandwidth).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR rfSpan/Sets the DBW to the specified value and the span (around the center
/// HIPAR rfSpan/frequency) of the RF data to be evaluated to its new maximum (the
/// HIPAR rfSpan/demodulation bandwidth).
ViStatus _VI_FUNC rsfpl_ConfigureADEMRFSpan(ViSession instrSession,
                                            ViReal64  rfSpan)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_ADEM_RF_SPAN, rfSpan),
            2, "RF Span");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the span (around the center frequency) for RF spectrum result
/// HIFN  display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR rfSpanZoom/Sets the span (around the center frequency) for RF spectrum result
/// HIPAR rfSpanZoom/display.
ViStatus _VI_FUNC rsfpl_ConfigureADEMRFSpanZoom(ViSession instrSession,
                                                ViReal64  rfSpanZoom)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_ADEM_RF_SPAN_ZOOM, rfSpanZoom),
            2, "RF Span Zoom");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Switches all AF filters for the selected evaluation off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_ADEMAFFilterAllOff(ViSession instrSession,
                                           ViInt32   window)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_ADEM_AF_HIGH_PASS_FILTER_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    checkErr(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_ADEM_AF_FILTER_ALL_OFF, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates/deactivates the selected high pass filter for the specified
/// HIFN  evaluation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR filterEnabled/Activates/deactivates the selected high pass filter for the specified
/// HIPAR filterEnabled/evaluation.
/// HIPAR filterType/Selects the high pass filter type for the specified evaluation.
ViStatus _VI_FUNC rsfpl_ConfigureADEMAFHighPassFilter(ViSession instrSession,
                                                      ViInt32   window,
                                                      ViBoolean filterEnabled,
                                                      ViInt32   filterType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_ADEM_AF_HIGH_PASS_FILTER_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");
    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_ADEM_AF_HIGH_PASS_FILTER_ENABLED, filterEnabled),
            3, "Filter Enabled");
    if (filterEnabled)
    {
        viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_ADEM_AF_HIGH_PASS_FILTER_TYPE, filterType),
                4, "Filter Type");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the cutoff frequency of the high pass filter for the specified
/// HIFN  evaluation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR cutoffFrequency/Selects the cutoff frequency of the high pass filter for the specified
/// HIPAR cutoffFrequency/evaluation.
ViStatus _VI_FUNC rsfpl_ConfigureADEMAFHighPassFilterCutoffFrequency(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64  cutoffFrequency)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_ADEM_AF_HIGH_PASS_FILTER_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_ADEM_AF_HIGH_PASS_FILTER_CUTOFF_FREQUENCY, cutoffFrequency),
            3, "Cutoff Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates/deactivates the selected low pass filter for the specified
/// HIFN  evaluation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR filterEnabled/Activates/deactivates the selected low pass filter for the specified
/// HIPAR filterEnabled/evaluation.
ViStatus _VI_FUNC rsfpl_ConfigureADEMAFLowPassFilterEnabled(ViSession instrSession,
                                                            ViInt32   window,
                                                            ViBoolean filterEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_ADEM_AF_LOW_PASS_FILTER_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");
    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_ADEM_AF_LOW_PASS_FILTER_ENABLED, filterEnabled),
            3, "Filter Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the cutoff frequency of the low pass filter for the specified
/// HIFN  evaluation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR cutoffFrequency/Selects the cutoff frequency of the low pass filter for the specified
/// HIPAR cutoffFrequency/evaluation.
ViStatus _VI_FUNC rsfpl_ConfigureADEMAFLowPassFilterCutoffFrequency(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64  cutoffFrequency)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_ADEM_AF_LOW_PASS_FILTER_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_ADEM_AF_LOW_PASS_FILTER_CUTOFF_FREQUENCY, cutoffFrequency),
            3, "Cutoff Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the relative low pass filter type for the specified
/// HIFN  evaluation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR filterTypeRelative/Selects the relative low pass filter type for the specified
/// HIPAR filterTypeRelative/evaluation.
ViStatus _VI_FUNC rsfpl_ConfigureADEMAFLowPassFilterTypeRelative(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   filterTypeRelative)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_ADEM_AF_LOW_PASS_FILTER_TYPE_RELATIVE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_ADEM_AF_LOW_PASS_FILTER_TYPE_RELATIVE, filterTypeRelative),
            3, "Filter Type Relative");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the absolute low pass filter type for the specified
/// HIFN  evaluation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR filterTypeAbsolute/Selects the absolute low pass filter type for the specified
/// HIPAR filterTypeAbsolute/evaluation.
ViStatus _VI_FUNC rsfpl_ConfigureADEMAFLowPassFilterTypeAbsolute(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   filterTypeAbsolute)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_ADEM_AF_LOW_PASS_FILTER_TYPE_ABSOLUTE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_ADEM_AF_LOW_PASS_FILTER_TYPE_ABSOLUTE, filterTypeAbsolute),
            3, "Filter Type Absolute");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates/deactivates the CCITT (CCITT P.53) weighting filter for the
/// HIFN  specified evaluation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR ccittEnabled/Activates/deactivates the CCITT (CCITT P.53) weighting filter for the
/// HIPAR ccittEnabled/specified evaluation.
/// HIPAR window/This control selects the measurement window.
ViStatus _VI_FUNC rsfpl_ConfigureADEMAFWeightedFilterCCITTEnabled(
                            ViSession instrSession,
                            ViInt32   window,
                            ViBoolean ccittEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_ADEM_AF_WEIGHTED_FILTER_CCITT_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_ADEM_AF_WEIGHTED_FILTER_CCITT_ENABLED, ccittEnabled),
            3, "CCITT Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates/deactivates the unweighted CCIR filter in the specified
/// HIFN  window.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR unweightedCCIREnabled/Activates/deactivates the unweighted CCIR filter in the specified
/// HIPAR unweightedCCIREnabled/window.
/// HIPAR window/This control selects the measurement window.
ViStatus _VI_FUNC rsfpl_ConfigureADEMAFWeightedFilterUnweightedCCIREnabled(
                            ViSession instrSession,
                            ViInt32   window,
                            ViBoolean unweightedCCIREnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_ADEM_AF_WEIGHTED_FILTER_UNWEIGHTED_CCIR_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_ADEM_AF_WEIGHTED_FILTER_UNWEIGHTED_CCIR_ENABLED, unweightedCCIREnabled),
            3, "Unweighted CCIR Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates/deactivates the weighted CCIR filter for the specified
/// HIFN  evaluation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR weightedCCIREnabled/Activates/deactivates the weighted CCIR filter for the specified
/// HIPAR weightedCCIREnabled/evaluation.
/// HIPAR window/This control selects the measurement window.
ViStatus _VI_FUNC rsfpl_ConfigureADEMAFWeightedFilterWeightedCCIREnabled(
                            ViSession instrSession,
                            ViInt32   window,
                            ViBoolean weightedCCIREnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_ADEM_AF_WEIGHTED_FILTER_WEIGHTED_CCIR_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_ADEM_AF_WEIGHTED_FILTER_WEIGHTED_CCIR_ENABLED, weightedCCIREnabled),
            3, "Weighted CCIR Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates/deactivates the 'A' weighting filter for the specified
/// HIFN  evaluation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR aWeightedEnabled/Activates/deactivates the 'A' weighting filter for the specified
/// HIPAR aWeightedEnabled/evaluation.
/// HIPAR window/This control selects the measurement window.
ViStatus _VI_FUNC rsfpl_ConfigureADEMAFWeightedAEnabled(ViSession instrSession,
                                                        ViInt32   window,
                                                        ViBoolean aWeightedEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_ADEM_AF_WEIGHTED_A_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_ADEM_AF_WEIGHTED_A_ENABLED, aWeightedEnabled),
            3, "A Weighted Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates/deactivates the selected deemphasis for the specified
/// HIFN  evaluation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR filterEnabled/Activates/deactivates the selected deemphasis for the specified
/// HIPAR filterEnabled/evaluation.
/// HIPAR window/This control selects the measurement window.
/// HIPAR filterType/Selects the deemphasis for the specified evaluation.
ViStatus _VI_FUNC rsfpl_ConfigureADEMAFDeemphasisFilter(ViSession instrSession,
                                                        ViInt32   window,
                                                        ViBoolean filterEnabled,
                                                        ViInt32   filterType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_ADEM_AF_DEEMPHASIS_FILTER_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");
    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_ADEM_AF_DEEMPHASIS_FILTER_ENABLED, filterEnabled),
            3, "Filter Enabled");
    if (filterEnabled)
    {
        viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_ADEM_AF_DEEMPHASIS_FILTER_TYPE, filterType),
                4, "Filter Type");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables or disables the YIG filter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR filterEnabled/Enables or disables the YIG filter.
ViStatus _VI_FUNC rsfpl_ConfigureADEMRFInputYIGFilter(ViSession instrSession,
                                                      ViBoolean filterEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_ANALOG_DEMODULATION_RF_INPUT_YIG_FILTER, filterEnabled),
            2, "Filter Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the unit for relative demodulation results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR unit/Selects the unit for relative demodulation results.
ViStatus _VI_FUNC rsfpl_ConfigureADEMRelativeDemodulationResultsUnit(
                            ViSession instrSession,
                            ViInt32   unit)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_ADEM_RELATIVE_DEMODULATION_RESULTS_UNIT, unit),
            2, "Unit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the reference value to be used for relative demodulation results
/// HIFN  to the currently measured value for all relative detectors.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR demodulationType/This parameter selects the demodulation.
ViStatus _VI_FUNC rsfpl_SetADEMDemodulationReference(ViSession instrSession,
                                                     ViInt32   demodulationType)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attributes[3] = { RSFPL_ATTR_ADEM_AM_DEMODULATION_SET_REFERENCE,RSFPL_ATTR_ADEM_FM_DEMODULATION_SET_REFERENCE,RSFPL_ATTR_ADEM_PM_DEMODULATION_SET_REFERENCE };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, demodulationType, 0, 2),
            2, "Demodulation Type");

    checkErr(rsfpl_SetAttributeViString(instrSession, NULL, attributes[demodulationType], NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures relative demodulation for the selected detector.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR demodulationType/This parameter selects the demodulation.
/// HIPAR detectorType/Selects measurement type.
/// HIPAR relativeDemodulationEnabled/Activates relative demodulation for the selected detector. If
/// HIPAR relativeDemodulationEnabled/activated, the demodulated result is set in relation to the reference
/// HIPAR relativeDemodulationEnabled/value.
/// HIPAR resultMode/Defines the mode with which the demodulation result is determined.
/// HIPAR referenceValue/Defines the reference value to be used for relative demodulation
/// HIPAR referenceValue/results and recalculates the results. If necessary, the detector is
/// HIPAR referenceValue/activated.
ViStatus _VI_FUNC rsfpl_ConfigureADEMRelativeDemodulation(ViSession instrSession,
                                                          ViInt32   demodulationType,
                                                          ViInt32   detectorType,
                                                          ViBoolean relativeDemodulationEnabled,
                                                          ViInt32   resultMode,
                                                          ViReal64  referenceValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr_enabled[3] = {
             RSFPL_ATTR_ADEM_AM_RELATIVE_DEMODULATION_ENABLED,
             RSFPL_ATTR_ADEM_FM_RELATIVE_DEMODULATION_ENABLED,
             RSFPL_ATTR_ADEM_PM_RELATIVE_DEMODULATION_ENABLED
             };
    ViAttr   attr_mode[3] = {
             RSFPL_ATTR_ADEM_AM_RELATIVE_DEMODULATION_RESULT_MODE,
             RSFPL_ATTR_ADEM_FM_RELATIVE_DEMODULATION_RESULT_MODE,
             RSFPL_ATTR_ADEM_PM_RELATIVE_DEMODULATION_RESULT_MODE
             };
    ViAttr   attr_ref_value[3] = {
             RSFPL_ATTR_ADEM_AM_RELATIVE_DEMODULATION_REFERENCE_VALUE,
             RSFPL_ATTR_ADEM_FM_RELATIVE_DEMODULATION_REFERENCE_VALUE,
             RSFPL_ATTR_ADEM_PM_RELATIVE_DEMODULATION_REFERENCE_VALUE
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, demodulationType, 0, 2),
            2, "Demodulation Type");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_ADEM_AM_RELATIVE_DEMODULATION_ENABLED,
            "ADEMDetector", detectorType , RS_REPCAP_BUF_SIZE, repCap),
            3, "Detector Type");
    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, attr_enabled[demodulationType], relativeDemodulationEnabled),
            4, "Relative Demodulation Enabled");

    if (relativeDemodulationEnabled)
    {
        viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, attr_mode[demodulationType], resultMode),
                5, "Result Mode");

        viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, attr_ref_value[demodulationType], referenceValue),
                6, "Reference Value");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Adds a window to the display in the active channel setup.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR windowName/String containing the name of the existing window the new window is
/// HIPAR windowName/inserted next to.
/// HIPAR direction/Direction the new window is added relative to the existing window.
/// HIPAR windowType/Type of result display (evaluation method) you want to add.
/// HIPAR arraySize/Defines the memory size allocated for Returned Window Name.
/// HIPAR returnedWindowName/String containing the name of the existing window the new window is
/// HIPAR returnedWindowName/inserted next to.
ViStatus _VI_FUNC rsfpl_ADEMLayoutAddWindow(ViSession instrSession,
                                            ViString  windowName,
                                            ViInt32   direction,
                                            ViInt32   windowType,
                                            ViUInt32  arraySize,
                                            ViChar    returnedWindowName[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   response[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2direction;
    ViChar*  p2type;

    checkErr(RsCore_LockSession(instrSession));

    if (direction == RSFPL_VAL_DIRECTION_REPLACE)
    {
        checkErr(rsfpl_ADEMLayoutReplaceWindow(instrSession, windowName, windowType));
    }
    else
    {
        viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsFpl_rngLayoutDirection, direction, &p2direction),
            3, "Direction");
        viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsFpl_rngADEMLayoutWindowType, windowType, &p2type),
            4, "Window Type");

        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:ADD? '%s',%s,%s", windowName, p2direction, p2type);
        checkErr(RsCore_QueryViString(instrSession, cmd, response));
        checkErr(rsfpl_CheckStatus(instrSession));

        if (returnedWindowName != NULL)
        {
            RsCore_TrimString(response, RS_VAL_TRIM_WHITESPACES_AND_ALL_QUOTES);
            strncpy(returnedWindowName, response, arraySize);
        }
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Replaces the window type (for example from 'Spectrum' to 'Result
/// HIFN  Summary') of an already existing window in the active channel setup
/// HIFN  while keeping its position, index and window name.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR windowName/String containing the name of the existing window the new window is
/// HIPAR windowName/inserted next to.
/// HIPAR windowType/Type of result display (evaluation method) you want to add.
ViStatus _VI_FUNC rsfpl_ADEMLayoutReplaceWindow(ViSession instrSession,
                                                ViString  windowName,
                                                ViInt32   windowType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2type;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsFpl_rngADEMLayoutWindowType, windowType, &p2type),
        3, "Window Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:REPL '%s',%s", windowName, p2type);
    checkErr(rsfpl_WriteCommandWithOPCSync(instrSession, cmd, 0));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the trace modes of the evaluated signal to be measured
/// HIFN  simultaneously. For each of the six available traces a mode can be
/// HIFN  defined.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR evaluationMethod/Selects evaluation method.
/// HIPAR traceMode/Array of 6 trace modes, one for each available trace.
ViStatus _VI_FUNC rsfpl_ConfigureADEMTrace(ViSession instrSession,
                                           ViInt32   evaluationMethod,
                                           ViInt32   traceMode[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2buf = cmd;
    ViChar*  p2type;
    ViChar*  p2trace;
    ViInt32  i = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsFpl_rngADEMEvaluationMethod, evaluationMethod, &p2type),
        2, "Evaluation Method");

    p2buf += snprintf(p2buf, RS_MAX_MESSAGE_BUF_SIZE, "ADEM:%s ", p2type);
    for (i = 0; i < 6; i++)
    {
        viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsFpl_rngADEMTraceMode, traceMode[i], &p2trace),
            3, "Trace Mode");

        p2buf += sprintf(p2buf, "%s,", p2trace);
    }

    *--p2buf = '\0';
    checkErr(rsfpl_WriteCommandWithOPCSync(instrSession, cmd, 0));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Reads the result data of the evaluated signal in the specified trace
/// HIFN  mode.
/// HIFN     The trace results are configured for a specific evaluation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR evaluationMethod/Selects evaluation method.
/// HIPAR tracemode/The specified trace mode must be one of those configured by
/// HIPAR tracemode/SENS:ADEM:<Evaluation>:TYPE. Otherwise a query error is generated.
/// HIPAR arrayLength/Number of array points requested.
/// HIPAR actualPoints/Number of points actually returned in the array.
/// HIPAR result/For any graphical result display, the function returns one result for
/// HIPAR result/each measurement point.
ViStatus _VI_FUNC rsfpl_ReadADEMTrace(ViSession instrSession,
                                      ViInt32   evaluationMethod,
                                      ViInt32   traceMode,
                                      ViUInt32  arrayLength,
                                      ViUInt32* actualPoints,
                                      ViReal64  result[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2type;
    ViChar*  p2trace;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsFpl_rngADEMEvaluationMethod, evaluationMethod, &p2type),
        2, "Evaluation Method");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsFpl_rngADEMTraceMode, traceMode, &p2trace),
        3, "Trace Mode");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":FORM REAL,32;ADEM:%s:RES? %s", p2type, p2trace);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arrayLength, result, (ViInt32*)actualPoints));
    checkErr(rsfpl_CheckStatus(instrSession));

Error:
    (void)RsCore_Write(instrSession, "FORM ASC");
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates/deactivates the phase wrap for the specified PM time domain
/// HIFN  display with DC coupling.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR resultType/Activates/deactivates the phase wrap for the specified PM time domain
/// HIPAR resultType/display with DC coupling.
ViStatus _VI_FUNC rsfpl_ConfigureADEMRangeResultType(ViSession instrSession,
                                                     ViInt32   window,
                                                     ViInt32   resultType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_ADEM_SELECT_TRACE_FOR_RESULT_SUMMARY,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_ANALOG_DEMODULATION_RESULT_RANGE_RESULT_TYPE, resultType),
            3, "Result Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the trace specified by the index <t> in the window specified
/// HIFN  by the index <n>. Only traces that are active in the specified result
/// HIFN  display can be selected. The selected trace is used to determine the
/// HIFN  'Result Summary' for the corresponding result display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR trace/Selects the trace.
ViStatus _VI_FUNC rsfpl_ADEMSelectTraceForResultSummary(ViSession instrSession,
                                                        ViInt32   window,
                                                        ViInt32   trace)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_trace[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_ADEM_SELECT_TRACE_FOR_RESULT_SUMMARY,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_ADEM_SELECT_TRACE_FOR_RESULT_SUMMARY,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCap_trace),
            3, "Trace");
    (void)strncat(repCap, ",", strlen(repCap) - RS_MAX_MESSAGE_BUF_SIZE - 1);
    (void)strncat(repCap, repCap_trace, strlen(repCap) - RS_MAX_MESSAGE_BUF_SIZE - 1);

    checkErr(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_ADEM_SELECT_TRACE_FOR_RESULT_SUMMARY, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the current value of the demodulated signal for the specified
/// HIFN  trace (as displayed in the Result Summary in manual operation).
/// HIFN
/// HIFN     Note that all windows with the same evaluation method have the same
/// HIFN  traces.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trace/Selects the trace.
/// HIPAR modulation/Selects the modulation type.
/// HIPAR measType/Selects measurement type.
/// HIPAR modulationResult/Returns selected result.
ViStatus _VI_FUNC rsfpl_QueryADEMModulationResult(ViSession instrSession,
                                                  ViInt32   trace,
                                                  ViInt32   modulation,
                                                  ViInt32   measType,
                                                  ViReal64* modulationResult)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2modulation;
    ViChar*  p2type;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trace, 1, 6),
            2, "Trace");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsFpl_rngADEMType, modulation, &p2modulation),
        3, "Modulation");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsFpl_rngADEMSummaryMeasType, measType, &p2type),
        4, "Meas Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:MARK:FUNC:ADEM:%s:RES%ld? %s", p2modulation, trace, p2type);
    checkErr(rsfpl_QueryViReal64(instrSession, cmd, modulationResult));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the current relative value of the demodulated signal for the
/// HIFN  specified trace (as displayed in the Result Summary in manual
/// HIFN  operation).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trace/Selects the trace.
/// HIPAR modulation/Selects the modulation type.
/// HIPAR measType/Selects measurement type.
/// HIPAR modulationResult/Returns selected result.
ViStatus _VI_FUNC rsfpl_QueryADEMModulationResultRelative(ViSession instrSession,
                                                          ViInt32   trace,
                                                          ViInt32   modulation,
                                                          ViInt32   measType,
                                                          ViReal64* modulationResult)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2modulation;
    ViChar*  p2type;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trace, 1, 6),
            2, "Trace");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsFpl_rngADEMType, modulation, &p2modulation),
        3, "Modulation");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsFpl_rngADEMSummaryMeasType, measType, &p2type),
        4, "Meas Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:MARK:FUNC:ADEM:%s:RES%ld:REL? %s", p2modulation, trace, p2type);
    checkErr(rsfpl_QueryViReal64(instrSession, cmd, modulationResult));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the modulation (audio) frequency for the demodulation method
/// HIFN  in the specified window.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR modulationFrequency/Queries the modulation (audio) frequency for the demodulation method
/// HIPAR modulationFrequency/in the specified window.
ViStatus _VI_FUNC rsfpl_QueryADEMResultModulationFrequency(ViSession instrSession,
                                                           ViInt32   window,
                                                           ViReal64* modulationFrequency)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_ADEM_RESULT_MODULATION_FREQUENCY,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsfpl_GetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_ADEM_RESULT_MODULATION_FREQUENCY, modulationFrequency),
            3, "Modulation Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the carrier power, which is determined from the Clr/Write
/// HIFN  data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR carrierPower/Queries the carrier power, which is determined from the Clr/Write
/// HIPAR carrierPower/data.
ViStatus _VI_FUNC rsfpl_QueryADEMResultCarrierPower(ViSession instrSession,
                                                    ViInt32   window,
                                                    ViReal64* carrierPower)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_ADEM_SELECT_TRACE_FOR_RESULT_SUMMARY,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsfpl_GetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_ADEM_RESULT_CARRIER_POWER, carrierPower),
            3, "Carrier Power");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the result of the total harmonic distortion (THD) measurement
/// HIFN  in the specified window.
/// HIFN     Note that this value is only calculated if an AF Spectrum window is
/// HIFN  displayed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR trace/Selects the trace.
/// HIPAR thd/Queries the result of the total harmonic distortion (THD) measurement
/// HIPAR thd/in the specified window.
ViStatus _VI_FUNC rsfpl_QueryADEMResultTHD(ViSession instrSession,
                                           ViInt32   window,
                                           ViInt32   trace,
                                           ViReal64* thd)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_trace[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_ADEM_SELECT_TRACE_FOR_RESULT_SUMMARY,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_ADEM_SELECT_TRACE_FOR_RESULT_SUMMARY,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCap_trace),
            3, "Trace");
    strncat(repCap, ",", strlen(repCap) - RS_MAX_MESSAGE_BUF_SIZE - 1);
    strncat(repCap, repCap_trace, strlen(repCap) - RS_MAX_MESSAGE_BUF_SIZE - 1);

    viCheckParm(rsfpl_GetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_ADEM_RESULT_THD, thd),
            4, "THD");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the result of the signal-to-noise-and-distortion (SINAD)
/// HIFN  measurement in the specified window for the specified trace.
/// HIFN     Note that this value is only calculated if an AF Spectrum window is
/// HIFN  displayed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR trace/Selects the trace.
/// HIPAR sinad/Queries the result of the signal-to-noise-and-distortion (SINAD)
/// HIPAR sinad/measurement in the specified window for the specified trace.
ViStatus _VI_FUNC rsfpl_QueryADEMResultSINAD(ViSession instrSession,
                                             ViInt32   window,
                                             ViInt32   trace,
                                             ViReal64* sinad)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_trace[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_ADEM_SELECT_TRACE_FOR_RESULT_SUMMARY,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_ADEM_SELECT_TRACE_FOR_RESULT_SUMMARY,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCap_trace),
            3, "Trace");
    (void)strncat(repCap, ",", strlen(repCap) - RS_MAX_MESSAGE_BUF_SIZE - 1);
    (void)strncat(repCap, repCap_trace, strlen(repCap) - RS_MAX_MESSAGE_BUF_SIZE - 1);

    viCheckParm(rsfpl_GetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_ADEM_RESULT_SINAD, sinad),
            4, "SINAD");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the carrier offset (= frequency error) for FM and PM
/// HIFN  demodulation. The carrier offset is determined from the current
/// HIFN  measurement data (CLR/WRITE). The modulation is removed using low pass
/// HIFN  filtering.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR trace/Selects the trace.
/// HIPAR carrierOffset/Queries the carrier offset (= frequency error) for FM and PM
/// HIPAR carrierOffset/demodulation. The carrier offset is determined from the current
/// HIPAR carrierOffset/measurement data (CLR/WRITE). The modulation is removed using low pass
/// HIPAR carrierOffset/filtering.
ViStatus _VI_FUNC rsfpl_QueryADEMResultCarrierOffset(ViSession instrSession,
                                                     ViInt32   window,
                                                     ViInt32   trace,
                                                     ViReal64* carrierOffset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_trace[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_ADEM_SELECT_TRACE_FOR_RESULT_SUMMARY,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_ADEM_SELECT_TRACE_FOR_RESULT_SUMMARY,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCap_trace),
            3, "Trace");
    (void)strncat(repCap, ",", strlen(repCap) - RS_MAX_MESSAGE_BUF_SIZE - 1);
    (void)strncat(repCap, repCap_trace, strlen(repCap) - RS_MAX_MESSAGE_BUF_SIZE - 1);

    viCheckParm(rsfpl_GetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_ADEM_RESULT_CARRIER_OFFSET, carrierOffset),
            4, "Carrier Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the modulation (audio) frequency for the demodulation method
/// HIFN  in the specified window.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR trace/Selects the trace.
/// HIPAR modulationFrequency/Queries the modulation (audio) frequency for the demodulation method
/// HIPAR modulationFrequency/in the specified window.
ViStatus _VI_FUNC rsfpl_QueryADEMTraceResultModulationFrequency(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   trace,
                            ViReal64* modulationFrequency)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_trace[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_ADEM_TRACE_RESULT_MODULATION_FREQUENCY,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_ADEM_TRACE_RESULT_MODULATION_FREQUENCY,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCap_trace),
            3, "Trace");
    (void)snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s",
        repCap_window, repCap_trace);

    checkErr(rsfpl_GetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_ADEM_TRACE_RESULT_MODULATION_FREQUENCY, modulationFrequency));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the carrier power, which is determined from the Clr/Write
/// HIFN  data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR trace/Selects the trace.
/// HIPAR carrierPower/Queries the carrier power, which is determined from the Clr/Write
/// HIPAR carrierPower/data.
ViStatus _VI_FUNC rsfpl_QueryADEMTraceResultCarrierPower(ViSession instrSession,
                                                         ViInt32   window,
                                                         ViInt32   trace,
                                                         ViReal64* carrierPower)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_trace[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_ADEM_TRACE_RESULT_CARRIER_POWER,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_ADEM_TRACE_RESULT_CARRIER_POWER,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCap_trace),
            3, "Trace");
    (void)snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s",
        repCap_window, repCap_trace);

    checkErr(rsfpl_GetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_ADEM_TRACE_RESULT_CARRIER_POWER, carrierPower));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the result of the modulation distortion measurement in the
/// HIFN  specified window for the specified trace.
/// HIFN
/// HIFN     Note that this value is only calculated if an AF Spectrum window is
/// HIFN  displayed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR trace/Selects the trace.
/// HIPAR distortion/Queries the result of the modulation distortion measurement in the
/// HIPAR distortion/specified window for the specified trace.
ViStatus _VI_FUNC rsfpl_QueryADEMTraceResultDistortion(ViSession instrSession,
                                                       ViInt32   window,
                                                       ViInt32   trace,
                                                       ViReal64* distortion)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_trace[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_ADEM_TRACE_RESULT_DISTORTION,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_ADEM_TRACE_RESULT_DISTORTION,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCap_trace),
            3, "Trace");
    (void)snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s",
        repCap_window, repCap_trace);

    checkErr(rsfpl_GetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_ADEM_TRACE_RESULT_DISTORTION, distortion));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}
