/*****************************************************************************
 *  Rohde&Schwarz FPL Spectrum Analyzer instrument driver
 *  Callbacks
 *
 *  Original Release: June 2018
 *  By: Jiri Kominek
 *
 *  Should you have any technical questions please contact the hotline of
 *  Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
 *
 *  e-mail: CustomerSupport@rohde-schwarz.com
 *

 *****************************************************************************/

#include "rsfpl.h"
#if (defined(WIN32) || defined(_WIN32) || defined(__WIN32__) || defined(__NT__))
#define ANSI
#endif
#ifndef ANSI                /* UNIX compatible */
#include <sys/timeb.h>
#include <sys/types.h>
#include <unistd.h>
#define Sleep sleep
#else                       /* ANSI compatible */
#include <windows.h>
#endif
#if defined (_CVI_)
#include <utility.h>
#endif

#pragma warning (disable:4100) // unreferenced formal parameter
#pragma warning (disable:4055)


 /*****************************************************************************
  *- Read Callbacks ----------------------------------------------------------*
  *****************************************************************************/

  /*****************************************************************************
   * Function: rsfpl_bin_ReadCallback
   * Purpose:  This function overrides standard calback funtion.
   *****************************************************************************/
ViStatus rsfpl_bin_ReadCallback(ViSession          instrSession,
	ViConstString      repCapName,
	RsCoreAttributePtr attr)
{
	ViStatus error = VI_SUCCESS;
	ViInt32  int32Value;
	ViChar* cmd = NULL;
	ViChar   response[RS_MAX_MESSAGE_BUF_SIZE];

	// If no command available, exit the function
	if (attr->command == NULL)
		return VI_SUCCESS;

	checkErr(RsCore_BuildAttrCommandString(instrSession, attr, repCapName, &cmd));
	checkErr(RsCore_QueryAttributeCommand(instrSession, attr, cmd, response));

	switch (attr->dataType)
	{
	case RS_VAL_INT32:
		int32Value = (ViInt32)RsCore_Convert_BinaryString_To_ViInt64(response + 2); // skip #B at the beginning of string
		checkErr(RsCore_SetAttributeP2Value(instrSession, attr, &int32Value));
		break;
	default:
		viCheckErrElab(RS_ERROR_INVALID_TYPE,
			"rsfpl_bin_ReadCallback only accepts RS_VAL_INT32 data type");
	}

Error:
	if (cmd)
		free(cmd);

	return error;
}

/*****************************************************************************
 * Function: rsfpl_bin_WriteCallback
 * Purpose:  This function overrides standard calback funtion.
 *****************************************************************************/
ViStatus rsfpl_bin_WriteCallback(ViSession          instrSession,
	ViConstString      repCapName,
	RsCoreAttributePtr attr,
	void* value)
{
	ViStatus error = VI_SUCCESS;
	ViChar* cmd = NULL;
	ViChar   bin_num[33];
	ViInt32  c = 0, d = 0, count = 0;
	ViInt32  n = *(ViInt32*)value;

	// If no command available, exit the function
	if (attr->command == NULL)
		return VI_SUCCESS;

	checkErr(RsCore_BuildAttrCommandString(instrSession, attr, repCapName, &cmd));

	switch (attr->dataType)
	{
	case RS_VAL_INT32:
		for (c = 31; c >= 0; c--)
		{
			d = n >> c;

			if (d & 1)
				*(bin_num + count) = 1 + '0';
			else
				*(bin_num + count) = 0 + '0';

			count++;
		}
		*(bin_num + count) = '\0';
		RsCore_StrCatDynamicString(instrSession, &cmd, " #B", bin_num, NULL);
		break;
	default:
		RsCore_GenerateInvalidDataTypeError(instrSession, attr->dataType, NULL);
	}
	checkErr(RsCore_WriteAttributeCommand(instrSession, attr, cmd));

Error:
	if (cmd)
		free(cmd);

	return error;
}

/*****************************************************************************
* Function: rsfpl_SpurList_Real64_ReadCallback
* Purpose:  This function overrides standard calback funtion. Reads int
*           returned with apostrophes
*****************************************************************************/
ViStatus rsfpl_SpurList_Real64_ReadCallback(ViSession          instrSession,
	ViConstString      repCapName,
	RsCoreAttributePtr attr)
{
	ViStatus error = VI_SUCCESS;

	if (attr->dataType == RS_VAL_REAL64)
	{
		checkErr(RsCore_ReadCallback(instrSession, repCapName, attr));

		// The spur list returns unexpected strings when measurements are not valid
		// An attr value of 0.0 means that atoi in RsCoreInt_ReadCallbackViReal64 failed
		// Since this indicates a non-numeric string, set attribute value to NAN
		if (*((ViPReal64)attr->p2value) == 0.0)
		{
			*((ViPReal64)attr->p2value) = RS_VAL_NAN_VI_REAL64;
		}
	}
	else
	{
		checkErr(RsCore_GenerateInvalidDataTypeError(instrSession, attr->dataType,
			"rsfpl_SpurList_Real64_ReadCallback: Allowed data types: RS_VAL_REAL64"));
	}

Error:
	return error;
}

/*****************************************************************************
* Function: rsfpl_quotedInt_ReadCallback
* Purpose:  This function overrides standard calback funtion. Reads int
*           returned with apostrophes
*****************************************************************************/
ViStatus rsfpl_quotedInt_ReadCallback(ViSession          instrSession,
	ViConstString      repCapName,
	RsCoreAttributePtr attr)
{
	return RsCore_SpecialCallback_AnyQuotedParamRead(instrSession, repCapName, attr);
}

/*****************************************************************************
 * Function: rsfpl_quotedInt_WriteCallback
 * Purpose:  This function overrides standard calback funtion. Writes int
 *           with apostrophes
 *****************************************************************************/
ViStatus rsfpl_quotedInt_WriteCallback(ViSession          instrSession,
	ViConstString      repCapName,
	RsCoreAttributePtr attr,
	void* value)
{
	return RsCore_SpecialCallback_SurroundParamWrite(instrSession, repCapName, attr, " '", value, "'");
}
