/*****************************************************************************
 *  rsfpl_phase_noise.c
 *
 *  Rohde & Schwarz FPL spectrum analyzer Instrument Driver
 *  Original Release: March 2021
 *  By: John Reid
 *
 *  Should you have any technical questions please contact the hotline of
 *  Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
 *
 *  e-mail: CustomerSupport@rohde-schwarz.com
 *
 *  Modification History:
 *    see ChangeLog
 *
 *  Copyright (c) 2021, Rohde & Schwarz GmbH & Co. KG
 *  All rights reserved.
 *****************************************************************************/

#include "rsfpl.h"

#if (defined(WIN32) || defined(_WIN32) || defined(__WIN32__) || defined(__NT__))
#define ANSI
#endif
#ifndef ANSI                /* UNIX compatible */
#include <sys/timeb.h>
#include <sys/types.h>
#include <unistd.h>
#define Sleep sleep
#else                       /* ANSI compatible */
#include <windows.h>
#endif
#include <string.h>
#if defined (_CVI_)
#include <utility.h>
#define _strnicmp strnicmp
#endif

/// HIFN  Selects the Phase Noise application (channel type) for the current
/// HIFN  channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_PhaseNoiseMode(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_PHASE_NOISE_MODE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

 /// HIFN  Initiates a reference measurement that determines the inherent phase
 /// HIFN  noise of the R&S FPL1000.
 /// HIRET Returns the status code of this operation.
 /// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
 /// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
 /// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_PhaseNoiseInitiateReferenceMeasurement(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_PHASE_NOISE_INITIATE_REFERENCE_MEASUREMENT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Stops the measurement in the current half decade and continues
/// HIFN  measuring in the subsequent half decade.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_PhaseNoiseFinishHalfDecade(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_PHASE_NOISE_FINISH_HALF_DECADE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the front end of the RF measurement hardware. Use
/// HIFN  rsfpl_ConfigureAttenuation to configure mechanical attenuation at the
/// HIFN  front end and rsfpl_ConfigurePhaseNoiseFrontEndCoupling to configure
/// HIFN  coupling.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR nominalFrequency/Defines the nominal frequency of the measurement.
/// HIPAR nominalLevel/Defines the nominal level.
ViStatus _VI_FUNC rsfpl_ConfigurePhaseNoiseFrontEnd(ViSession instrSession,
                                                    ViReal64  nominalFrequency,
                                                    ViReal64  nominalLevel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_FREQUENCY_CENTER, nominalFrequency),
            2, "Nominal Frequency");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_PHASE_NOISE_NOMINAL_LEVEL, nominalLevel),
            3, "Nominal Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the coupling method at the RF input. AC coupling blocks any
/// HIFN  DC voltage from the input signal. DC coupling lets DC voltage through.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR coupling/Selects the coupling method at the RF input. AC coupling blocks any DC
/// HIPAR coupling/voltage from the input signal. DC coupling lets DC voltage through.
ViStatus _VI_FUNC rsfpl_ConfigurePhaseNoiseFrontEndCoupling(ViSession instrSession,
                                                            ViInt32   coupling)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_VSA_RF_INPUT_COUPLING, coupling),
            2, "Coupling");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures frequency verification.
/// HIFN     If frequency verification is on, the R&S FPL1000 initiates the phase
/// HIFN  noise measurement only if the frequency of the DUT is within a certain
/// HIFN  frequency tolerance range. The tolerance range is either a percentage
/// HIFN  range of the nominal frequency or a absolute deviation from the nominal
/// HIFN  frequency. If you define both an absolute and relative tolerance, the
/// HIFN  application uses the higher tolerance level.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR verifyFrequencyEnabled/Turns frequency verification on and off.
/// HIPAR relativeTolerance/Defines a relative frequency tolerance in % for frequency
/// HIPAR relativeTolerance/verification. If you define both an absolute and relative tolerance,
/// HIPAR relativeTolerance/the application uses the higher tolerance level.
/// HIPAR absoluteTolerance/Defines an absolute frequency tolerance in Hz for frequency
/// HIPAR absoluteTolerance/verification. If you define both an absolute and relative tolerance,
/// HIPAR absoluteTolerance/the application uses the higher tolerance level.
ViStatus _VI_FUNC rsfpl_ConfigurePhaseNoiseVerifyFrequency(ViSession instrSession,
                                                           ViBoolean verifyFrequencyEnabled,
                                                           ViReal64  relativeTolerance,
                                                           ViReal64  absoluteTolerance)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_PHASE_NOISE_VERIFY_FREQUENCY_ENABLED, verifyFrequencyEnabled),
            2, "Verify Frequency Enabled");

    if (verifyFrequencyEnabled)
    {
        viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_PHASE_NOISE_FREQUENCY_RELATIVE_TOLERANCE, relativeTolerance),
                3, "Relative Tolerance");

        viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_PHASE_NOISE_FREQUENCY_ABSOLUTE_TOLERANCE, absoluteTolerance),
                4, "Absolute Tolerance");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures level verification.
/// HIFN     If level verification is on, the R&S FPL1000 initiates the phase noise
/// HIFN  measurement only if the level of the DUT is within a certain level
/// HIFN  tolerance range. The tolerance range is a level range relative to the
/// HIFN  nominal level.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR verifyLevelEnabled/Turns level verification on and off.
/// HIPAR tolerance/Defines a relative level tolerance for level verification.
ViStatus _VI_FUNC rsfpl_ConfigurePhaseNoiseVerifyLevel(ViSession instrSession,
                                                       ViBoolean verifyLevelEnabled,
                                                       ViReal64  tolerance)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_PHASE_NOISE_VERIFY_LEVEL_ENABLED, verifyLevelEnabled),
            2, "Verify Level Enabled");

    if (verifyLevelEnabled)
    {
        viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_PHASE_NOISE_LEVEL_TOLERANCE, tolerance),
                3, "Tolerance");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns repeated tries to start the measurement if signal verification
/// HIFN  fails on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR onVerifyFailed/Turns repeated tries to start the measurement if signal verification
/// HIPAR onVerifyFailed/fails on and off.
ViStatus _VI_FUNC rsfpl_ConfigurePhaseNoiseOnVerifyFailed(ViSession instrSession,
                                                          ViInt32   onVerifyFailed)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_PHASE_NOISE_ON_VERIFY_FAILED, onVerifyFailed),
            2, "On Verify Failed");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns tracking options on and off. The analyzer can track the
/// HIFN  frequency and level of the phase noise measurement and adjust the
/// HIFN  configuration accordingly.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frequencyTrackingEnabled/Turns frequency tracking on and off.
/// HIPAR levelTrackingEnabled/Turns level  tracking on and off.
ViStatus _VI_FUNC rsfpl_ConfigurePhaseNoiseTracking(ViSession instrSession,
                                                    ViBoolean frequencyTrackingEnabled,
                                                    ViBoolean levelTrackingEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_PHASE_NOISE_FREQUENCY_TRACKING_ENABLED, frequencyTrackingEnabled),
            2, "Frequency Tracking Enabled");

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_PHASE_NOISE_LEVEL_TRACKING_ENABLED, levelTrackingEnabled),
            3, "Level Tracking Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures I/Q processing.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR amRejectEnabled/Turns the suppression of AM noise on and off.
/// HIPAR maxFreqDrift/Defines the maximum tracking bandwidth (sample rate) for all half
/// HIPAR maxFreqDrift/decades.
/// HIPAR digitalPLLEnabled/Turns the digital PLL on and off.
/// HIPAR decimationEnabled/Turns decimation of results on and off.
ViStatus _VI_FUNC rsfpl_ConfigurePhaseNoiseIQProcessing(ViSession instrSession,
                                                        ViBoolean amRejectEnabled,
                                                        ViReal64  maxFreqDrift,
                                                        ViBoolean digitalPLLEnabled,
                                                        ViBoolean decimationEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_PHASE_NOISE_REJECT_AM_ENABLED, amRejectEnabled),
            2, "amRejectEnabled");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_PHASE_NOISE_MAX_FREQ_DRIFT, maxFreqDrift),
            3, "Max Freq Drift");

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_PHASE_NOISE_DIGITAL_PLL_ENABLED, digitalPLLEnabled),
            4, "Digital PLL Enabled");

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_PHASE_NOISE_DECIMATION_ENABLED, decimationEnabled),
            5, "Decimation Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Specifies the sweep direction. When switched on the sweep direction is
/// HIFN  from the start frequency to the stop frequency. When switched off the
/// HIFN  sweep direction is reversed.
/// HIFN     Use rsfpl_ConfigureFrequencyStartStop to set the start and stop
/// HIFN  frequency of the measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR sweepForward/Specifies the sweep direction. When switched on the sweep direction is
/// HIPAR sweepForward/from the start frequency to the stop frequency. When switched off the
/// HIPAR sweepForward/sweep direction is reversed.
ViStatus _VI_FUNC rsfpl_ConfigurePhaseNoiseSweepForwardEnabled(
                            ViSession instrSession,
                            ViBoolean sweepForward)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_PHASE_NOISE_SWEEP_FORWARD_ENABLED, sweepForward),
            2, "Sweep Forward");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the type of measurement configuration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR preset/Selects the type of measurement configuration.
ViStatus _VI_FUNC rsfpl_ConfigurePhaseNoiseSelectPreset(ViSession instrSession,
                                                        ViInt32   preset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_PHASE_NOISE_SELECT_PRESET, preset),
            2, "Preset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures global settings for phase noise measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR globalRBW/Defines the resolution bandwidth over all half decades.
/// HIPAR averageCount/Defines the number of measurements to be included in the averaging for
/// HIPAR averageCount/each and all half decades.
/// HIPAR multiplierEnabled/Turns a multiplier that is applied to the average count in each half
/// HIPAR multiplierEnabled/decade on and off.
/// HIPAR multiplier/Defines a multiplier that is applied to the average count in each half
/// HIPAR multiplier/decade. Before you can use the command you have to turn on the
/// HIPAR multiplier/multiplier with 'Multiplier Enabled'.
/// HIPAR globalSweepMode/Selects the sweep mode for all half decades.
ViStatus _VI_FUNC rsfpl_ConfigurePhaseNoiseGlobals(ViSession instrSession,
                                                   ViReal64  globalRBW,
                                                   ViInt32   averageCount,
                                                   ViBoolean multiplierEnabled,
                                                   ViInt32   multiplier,
                                                   ViInt32   globalSweepMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_PHASE_NOISE_GLOBAL_RBW, globalRBW),
            2, "Global RBW");

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_PHASE_NOISE_GLOBAL_AVERAGE_COUNT, averageCount),
            3, "Average Count");

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_PHASE_NOISE_MULTIPLIER_ENABLED, multiplierEnabled),
            4, "Multiplier Enabled");
    if (multiplierEnabled)
    {
        viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_PHASE_NOISE_MULTIPLIER, multiplier),
                5, "Multiplier");
    }
    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_PHASE_NOISE_GLOBAL_SWEEP_MODE, globalSweepMode),
            6, "Global Sweep Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures measurement settings for a specified half decade.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR halfDecade/Selects a half decade.
/// HIPAR rbw/Defines the resolution bandwidth for a particular half decade.
/// HIPAR sweepMode/Selects the sweep mode for a particular half decade.
/// HIPAR averageCount/Defines the number of measurements included in the averaging for a
/// HIPAR averageCount/half decade.
/// HIPAR windowType/Selects the window function for a particular half decade. Window
/// HIPAR windowType/functions are available for I/Q sweep mode.
ViStatus _VI_FUNC rsfpl_ConfigurePhaseNoiseHalfDecade(ViSession instrSession,
                                                      ViInt32   halfDecade,
                                                      ViReal64  rbw,
                                                      ViInt32   sweepMode,
                                                      ViInt32   averageCount,
                                                      ViInt32   windowType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_PHASE_NOISE_HALF_DECADE_RBW,
            "PNRange", halfDecade - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Half Decade");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_PHASE_NOISE_HALF_DECADE_RBW, rbw),
            3, "RBW");

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_PHASE_NOISE_HALF_DECADE_SWEEP_MODE, sweepMode),
            4, "Sweep Mode");

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_PHASE_NOISE_HALF_DECADE_AVERAGE_COUNT, averageCount),
            5, "Average Count");

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_PHASE_NOISE_HALF_DECADE_WINDOW_TYPE, windowType),
            6, "Window Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures horizontal axis scaling for the application.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR trace/Sets the trace.
/// HIPAR scope/Selects the way the application scales the horizontal axis.
/// HIPAR start/If Scope is set to Manual, selects the start frequency of the display
/// HIPAR start/range.
/// HIPAR stop/If Scope is set to Manual, selects the stop frequency of the display
/// HIPAR stop/range.
/// HIPAR halfDecade/If Scope is set to Half Decade, selects the half decade to be
/// HIPAR halfDecade/displayed.
ViStatus _VI_FUNC rsfpl_ConfigurePhaseNoiseTraceXAxisScope(ViSession instrSession,
                                                           ViInt32   window,
                                                           ViInt32   trace,
                                                           ViInt32   scope,
                                                           ViReal64  start,
                                                           ViReal64  stop,
                                                           ViInt32   halfDecade)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_trace[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_PHASE_NOISE_TRACE_X_AXIS_SCOPE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_PHASE_NOISE_TRACE_X_AXIS_SCOPE,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCap_trace),
            3, "Trace");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap_window, repCap_trace);

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_PHASE_NOISE_TRACE_X_AXIS_SCOPE, scope),
            4, "Scope");

    switch (scope)
    {
    case RSFPL_VAL_PHASE_NOISE_X_AXIS_SCOPE_MANUAL:
        viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_PHASE_NOISE_TRACE_X_AXIS_START, start),
                5, "Start");

        viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_PHASE_NOISE_TRACE_X_AXIS_STOP, stop),
                6, "Stop");
        break;

    case RSFPL_VAL_PHASE_NOISE_X_AXIS_SCOPE_HALF_DECADE:
        viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_PHASE_NOISE_TRACE_X_AXIS_HALF_DECADE, halfDecade),
                7, "Half Decade");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures scaling of the vertical axis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR trace/Sets the trace.
/// HIPAR yAxisScaling/Selects the type of manual scaling of the vertical axis.
/// HIPAR top/Defines the reference value or upper border of the diagram area, if
/// HIPAR top/you have selected an appropriate manual y-axis scaling method.
/// HIPAR range/Defines the display range of the y-axis, if you have selected an
/// HIPAR range/appropriate manual y-axis scaling method. The unit is dB.
/// HIPAR bottom/Defines the reference value or upper border of the diagram area, if
/// HIPAR bottom/you have selected an appropriate manual y-axis scaling method.
ViStatus _VI_FUNC rsfpl_ConfigurePhaseNoiseTraceYAxisScaling(ViSession instrSession,
                                                             ViInt32   window,
                                                             ViInt32   subwindow,
                                                             ViInt32   trace,
                                                             ViInt32   yAxisScaling,
                                                             ViReal64  top,
                                                             ViReal64  range,
                                                             ViReal64  bottom)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_withSubwindow[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_subwindow[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_trace[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_PHASE_NOISE_TRACE_Y_AXIS_SCALING,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_PHASE_NOISE_TRACE_Y_AXIS_TOP,
            "Subwindow", subwindow , RS_REPCAP_BUF_SIZE, repCap_subwindow),
            3, "Subwindow");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_PHASE_NOISE_TRACE_Y_AXIS_SCALING,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCap_trace),
            4, "Trace");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap_window, repCap_trace);
    snprintf(repCap_withSubwindow, RS_REPCAP_BUF_SIZE, "%s,%s,%s", repCap_window, repCap_subwindow, repCap_trace);

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_PHASE_NOISE_TRACE_Y_AXIS_SCALING, yAxisScaling),
            5, "Y Axis Scaling");

    if ((yAxisScaling == RSFPL_VAL_PHASE_NOISE_SCALING_TOP_BOTTOM)
        || (yAxisScaling == RSFPL_VAL_PHASE_NOISE_SCALING_TOP_RANGE))
    {
        viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap_withSubwindow, RSFPL_ATTR_PHASE_NOISE_TRACE_Y_AXIS_TOP, top),
                5, "Top");
    }

    if ((yAxisScaling == RSFPL_VAL_PHASE_NOISE_SCALING_TOP_RANGE)
        || (yAxisScaling == RSFPL_VAL_PHASE_NOISE_SCALING_BOTTOM_RANGE))
    {
        viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_PHASE_NOISE_TRACE_Y_AXIS_RANGE, range),
                6, "Range");
    }

    if ((yAxisScaling == RSFPL_VAL_PHASE_NOISE_SCALING_TOP_BOTTOM)
        || (yAxisScaling == RSFPL_VAL_PHASE_NOISE_SCALING_BOTTOM_RANGE))
    {
        viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_PHASE_NOISE_TRACE_Y_AXIS_BOTTOM, bottom),
                7, "Bottom");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns automatic scaling of the vertical axis on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR trace/Sets the trace.
/// HIPAR enabled/Turns automatic scaling of the vertical axis on and off.
ViStatus _VI_FUNC rsfpl_ConfigurePhaseNoiseTraceYAxisAutoScaleEnabled(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   trace,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_trace[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_PHASE_NOISE_TRACE_Y_AXIS_AUTO_SCALE_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_PHASE_NOISE_TRACE_Y_AXIS_AUTO_SCALE_ENABLED,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCap_trace),
            3, "Trace");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap_window, repCap_trace);

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_PHASE_NOISE_TRACE_Y_AXIS_AUTO_SCALE_ENABLED, enabled),
            4, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Performs auto scaling of the vertical axis once, then turns auto
/// HIFN  scaling off again.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR trace/Sets the trace.
ViStatus _VI_FUNC rsfpl_PhaseNoiseTraceYAxisAutoScaleOnce(ViSession instrSession,
                                                          ViInt32   window,
                                                          ViInt32   subwindow,
                                                          ViInt32   trace)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_subwindow[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_trace[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_PHASE_NOISE_TRACE_Y_AXIS_AUTO_SCALE_ONCE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_PHASE_NOISE_TRACE_Y_AXIS_AUTO_SCALE_ONCE,
            "Subwindow", subwindow , RS_REPCAP_BUF_SIZE, repCap_subwindow),
            3, "Subwindow");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_PHASE_NOISE_TRACE_Y_AXIS_AUTO_SCALE_ONCE,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCap_trace),
            4, "Trace");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s,%s", repCap_window, repCap_subwindow, repCap_trace);

    checkErr(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_PHASE_NOISE_TRACE_Y_AXIS_AUTO_SCALE_ONCE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures options for traces. Use
/// HIFN  rsfpl_ConfigureSubwindowTraceSmoothing to enable and adjust smoothing
/// HIFN  aperture.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR trace/Sets the trace.
/// HIPAR offset/Defines a reference level offset (for all traces in all windows).
/// HIPAR spurRemovalEnabled/Turns spur removal for a specific trace on and off.
/// HIPAR spurThreshold/Defines the level threshold for spur removal.
/// HIPAR smoothingMethod/Selects the trace smoothing method.
ViStatus _VI_FUNC rsfpl_ConfigurePhaseNoiseTraceOptions(ViSession instrSession,
                                                        ViInt32   window,
                                                        ViInt32   trace,
                                                        ViReal64  offset,
                                                        ViBoolean spurRemovalEnabled,
                                                        ViReal64  spurThreshold,
                                                        ViInt32   smoothingMethod)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_trace[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_PHASE_NOISE_TRACE_SPUR_REMOVAL_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_PHASE_NOISE_TRACE_SPUR_REMOVAL_ENABLED,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCap_trace),
            3, "Trace");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap_window, repCap_trace);

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_REFERENCE_LEVEL_OFFSET, offset),
            4, "Offset");

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_PHASE_NOISE_TRACE_SPUR_REMOVAL_ENABLED, spurRemovalEnabled),
            5, "Spur Removal Enabled");

    if (spurRemovalEnabled)
    {
        viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_PHASE_NOISE_SPUR_THRESHOLD, spurThreshold),
                6, "Spur Threshold");
    }

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_PHASE_NOISE_TRACE_SMOOTHING_METHOD, smoothingMethod),
            7, "Smoothing Method");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures and sets up limit lines.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR type/Selects the shape of a phase noise limit line.
/// HIPAR trace/Selects the trace to assign a phase noise limit line to. To assign
/// HIPAR trace/multiple traces to the limit line, use
/// HIPAR trace/rsfpl_ConfigurePhaseNoiseLimitLineSelectTraces.
/// HIPAR noiseFloor/Defines the noise floor level of the DUT. The noise floor level is
/// HIPAR noiseFloor/necessary for the calculation of a phase noise limit line.
ViStatus _VI_FUNC rsfpl_ConfigurePhaseNoiseLimitLine(ViSession instrSession,
                                                     ViInt32   window,
                                                     ViInt32   type,
                                                     ViInt32   trace,
                                                     ViReal64  noiseFloor)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_PHASE_NOISE_LIMIT_LINE_TYPE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_PHASE_NOISE_LIMIT_LINE_TYPE, type),
            3, "Type");

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_PHASE_NOISE_LIMIT_LINE_SELECTED_TRACE, trace),
            4, "Trace");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_PHASE_NOISE_LIMIT_LINE_NOISE_FLOOR, noiseFloor),
            5, "NoiseFloor");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the traces to assign a phase noise limit line to.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR bufferSize/Size of traces array.
/// HIPAR traces/The numbers of the traces to assign a phase noise limit line to.
ViStatus _VI_FUNC rsfpl_ConfigurePhaseNoiseLimitLineSelectTraces(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   bufferSize,
                            ViInt32   traces[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViString param = NULL;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, bufferSize, 1),
        3, "Buffer Size");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_PHASE_NOISE_LIMIT_LINE_SELECTED_TRACES,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    checkErr(RsCore_Convert_ViInt32array_To_String(instrSession, traces, bufferSize, NULL, &param));

    viCheckParm(RsCore_SetAttributeViRawString(instrSession, repCap,
        RSFPL_ATTR_PHASE_NOISE_LIMIT_LINE_SELECTED_TRACES,
        RS_VAL_DIRECT_USER_CALL,
        param),
        4, "Traces");

Error:
    if (param)
        free(param);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the corner frequency and slope of a phase noise limit line.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR limitSegment/Selects repcap PNLimSeg.
/// HIPAR cornerFrequency/Sets the corner frequency of a phase noise limit line.
/// HIPAR leftSlope/Defines the slope for a phase noise limit line segment.
ViStatus _VI_FUNC rsfpl_ConfigurePhaseNoiseLimitLineSegment(ViSession instrSession,
                                                            ViInt32   window,
                                                            ViInt32   limitSegment,
                                                            ViReal64  cornerFrequency,
                                                            ViReal64  leftSlope)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_seg[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_PHASE_NOISE_LIMIT_LINE_SEGMENT_CORNER_FREQUENCY,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_PHASE_NOISE_LIMIT_LINE_SEGMENT_CORNER_FREQUENCY,
            "PNLimSeg", limitSegment - 1, RS_REPCAP_BUF_SIZE, repCap_seg),
            3, "Limit Segment");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap_window, repCap_seg);

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_PHASE_NOISE_LIMIT_LINE_SEGMENT_CORNER_FREQUENCY, cornerFrequency),
            4, "Corner Frequency");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_PHASE_NOISE_LIMIT_LINE_SEGMENT_LEFT_SLOPE, leftSlope),
            5, "Left Slope");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the limit check results for phase noise limit lines.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR limitCheckResult/Queries the limit check results for phase noise limit lines.
ViStatus _VI_FUNC rsfpl_QueryPhaseNoiseLimitLineCheckResult(ViSession  instrSession,
                                                            ViInt32    window,
                                                            ViBoolean* limitCheckResult)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_PHASE_NOISE_LIMIT_LINE_CHECK_RESULT,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsfpl_GetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_PHASE_NOISE_LIMIT_LINE_CHECK_RESULT, limitCheckResult),
            3, "Limit Check Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Creates a new user limit line from the data of a phase noise limit
/// HIFN  line.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR limitLine/Selects the user limit line to which the phase noise limit line is
/// HIPAR limitLine/copied.
ViStatus _VI_FUNC rsfpl_PhaseNoiseLimitLineCopytoUserLimitLine(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   limitLine)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_lim[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_PHASE_NOISE_LIMIT_LINE_COPY_TO_USER_LIMIT_LINE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_PHASE_NOISE_LIMIT_LINE_COPY_TO_USER_LIMIT_LINE,
            "LimitLine", limitLine - 1, RS_REPCAP_BUF_SIZE, repCap_lim),
            3, "Limit Line");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap_window, repCap_lim);

    checkErr(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_PHASE_NOISE_LIMIT_LINE_COPY_TO_USER_LIMIT_LINE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the residual noise integration range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR rangeEnabled/Turns integration of the measurement range for residual noise
/// HIPAR rangeEnabled/calculation on and off.
/// HIPAR rangeStart/Defines the start point of the residual noise integration range, if
/// HIPAR rangeStart/range integration is enabled.
/// HIPAR rangeEnd/Defines the end point of the residual noise integration range, if
/// HIPAR rangeEnd/range integration is enabled.
ViStatus _VI_FUNC rsfpl_ConfigurePhaseNoiseResidualNoiseRange(
                            ViSession instrSession,
                            ViInt32   window,
                            ViBoolean rangeEnabled,
                            ViReal64  rangeStart,
                            ViReal64  rangeEnd)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_PHASE_NOISE_RESIDUAL_NOISE_USER_RANGE_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_PHASE_NOISE_RESIDUAL_NOISE_USER_RANGE_ENABLED, rangeEnabled),
            3, "Range Enabled");
    if (rangeEnabled)
    {
        viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_PHASE_NOISE_RESIDUAL_NOISE_RANGE_START, rangeStart),
                4, "Range Start");

        viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_PHASE_NOISE_RESIDUAL_NOISE_RANGE_END, rangeEnd),
                5, "Range End");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures a custom residual noise calculation range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR userRange/Selects repcap RNRange.
/// HIPAR userRangeTrace/Selects the trace for a custom residual noise calculation range.
/// HIPAR userRangeStart/Defines the start point of a custom residual noise calculation range.
/// HIPAR userRangeEnd/Defines the end point of a custom residual noise calculation range.
/// HIPAR userRangeEnd/Before you can use the command, you have to assign the user range to a
/// HIPAR userRangeEnd/trace with CALCulate<n>:EVALuation:USER<range>:TRACe.
ViStatus _VI_FUNC rsfpl_ConfigurePhaseNoiseResidualNoiseUserRange(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   userRange,
                            ViInt32   userRangeTrace,
                            ViReal64  userRangeStart,
                            ViReal64  userRangeEnd)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_range[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_PHASE_NOISE_RESIDUAL_NOISE_USER_RANGE_TRACE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_PHASE_NOISE_RESIDUAL_NOISE_USER_RANGE_TRACE,
            "RNRange", userRange - 1, RS_REPCAP_BUF_SIZE, repCap_range),
            3, "User Range");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap_window, repCap_range);

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_PHASE_NOISE_RESIDUAL_NOISE_USER_RANGE_TRACE, userRangeTrace),
            4, "User Range Trace");
    if (userRangeTrace != RSFPL_VAL_TRACE_NONE)
    {
        viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_PHASE_NOISE_RESIDUAL_NOISE_USER_RANGE_START, userRangeStart),
                5, "User Range Start");

        viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_PHASE_NOISE_RESIDUAL_NOISE_USER_RANGE_END, userRangeEnd),
                6, "User Range End");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns a user range off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR userRange/Selects repcap RNRange.
ViStatus _VI_FUNC rsfpl_PhaseNoiseResidualNoiseDisableUserRange(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   userRange)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_range[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_PHASE_NOISE_RESIDUAL_NOISE_USER_RANGE_TRACE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_PHASE_NOISE_RESIDUAL_NOISE_USER_RANGE_TRACE,
            "RNRange", userRange - 1, RS_REPCAP_BUF_SIZE, repCap_range),
            3, "User Range");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap_window, repCap_range);

    checkErr(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_PHASE_NOISE_RESIDUAL_NOISE_USER_RANGE_DISABLE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the Integrated Phase Noise for the first trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trace/Sets the trace.
/// HIPAR integratedPhaseNoise/Queries the Integrated Phase Noise for the first trace.
ViStatus _VI_FUNC rsfpl_QueryPhaseNoiseResidualNoiseIntegratedPhaseNoise(
                            ViSession instrSession,
                            ViReal64* integratedPhaseNoise)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_GetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_PHASE_NOISE_RESIDUAL_NOISE_INTEGRATED_PHASE_NOISE, integratedPhaseNoise),
            2, "Integrated Phase Noise");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the residual FM for the first trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trace/Sets the trace.
/// HIPAR residualFM/Queries the residual FM for the first trace.
ViStatus _VI_FUNC rsfpl_QueryPhaseNoiseResidualNoiseResidualFM(
                            ViSession instrSession,
                            ViReal64* residualFM)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_GetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_PHASE_NOISE_RESIDUAL_NOISE_RESIDUAL_FM, residualFM),
            2, "Residual FM");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the residual RMS jitter for the first trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trace/Sets the trace.
/// HIPAR residualRMSJitter/Queries the residual RMS jitter for the first trace.
ViStatus _VI_FUNC rsfpl_QueryPhaseNoiseResidualNoiseResidualRMSJitter(
                            ViSession instrSession,
                            ViReal64* residualRMSJitter)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_GetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_PHASE_NOISE_RESIDUAL_NOISE_RESIDUAL_RMS_JITTER, residualRMSJitter),
            2, "Residual RMS Jitter");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the residual PM for the first trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trace/Sets the trace.
/// HIPAR residualPM/Queries the residual PM for the first trace.
ViStatus _VI_FUNC rsfpl_QueryPhaseNoiseResidualNoiseResidualPM(
                            ViSession instrSession,
                            ViReal64* residualPM)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_GetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_PHASE_NOISE_RESIDUAL_NOISE_RESIDUAL_PM, residualPM),
            2, "Residual PM");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the Integrated Phase Noise for a particular user range. The
/// HIFN  trace that is queried depends on
/// HIFN  RSFPL_ATTR_PHASE_NOISE_RESIDUAL_NOISE_USER_RANGE_TRACE.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trace/Sets the trace.
/// HIPAR userRange/Selects repcap RNRange.
/// HIPAR integratedPhaseNoise/Queries the Integrated Phase Noise for a particular user range. The
/// HIPAR integratedPhaseNoise/trace that is queried depends on
/// HIPAR integratedPhaseNoise/RSFPL_ATTR_PHASE_NOISE_RESIDUAL_NOISE_USER_RANGE_TRACE.
ViStatus _VI_FUNC rsfpl_QueryPhaseNoiseResidualNoiseUserRangeIntegratedPhaseNoise(
                            ViSession instrSession,
                            ViInt32   userRange,
                            ViReal64* integratedPhaseNoise)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_PHASE_NOISE_RESIDUAL_NOISE_USER_RANGE_INTEGRATED_PHASE_NOISE,
            "RNRange", userRange - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "User Range");

    viCheckParm(rsfpl_GetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_PHASE_NOISE_RESIDUAL_NOISE_USER_RANGE_INTEGRATED_PHASE_NOISE, integratedPhaseNoise),
            3, "Integrated Phase Noise");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the residual FM for a particular user range. The trace that is
/// HIFN  queried depends on
/// HIFN  RSFPL_ATTR_PHASE_NOISE_RESIDUAL_NOISE_USER_RANGE_TRACE.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR userRange/Selects repcap RNRange.
/// HIPAR residualFM/Queries the residual FM for a particular user range. The trace that is
/// HIPAR residualFM/queried depends on
/// HIPAR residualFM/RSFPL_ATTR_PHASE_NOISE_RESIDUAL_NOISE_USER_RANGE_TRACE.
ViStatus _VI_FUNC rsfpl_QueryPhaseNoiseResidualNoiseUserRangeResidualFM(
                            ViSession instrSession,
                            ViInt32   userRange,
                            ViReal64* residualFM)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_PHASE_NOISE_RESIDUAL_NOISE_USER_RANGE_RESIDUAL_FM,
            "RNRange", userRange - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "User Range");

    viCheckParm(rsfpl_GetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_PHASE_NOISE_RESIDUAL_NOISE_USER_RANGE_RESIDUAL_FM, residualFM),
            3, "Residual FM");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the residual RMS jitter for a particular user range. The trace
/// HIFN  that is queried depends on
/// HIFN  RSFPL_ATTR_PHASE_NOISE_RESIDUAL_NOISE_USER_RANGE_TRACE.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR userRange/Selects repcap RNRange.
/// HIPAR residualRMSJitter/Queries the residual RMS jitter for a particular user range. The trace
/// HIPAR residualRMSJitter/that is queried depends on
/// HIPAR residualRMSJitter/RSFPL_ATTR_PHASE_NOISE_RESIDUAL_NOISE_USER_RANGE_TRACE.
ViStatus _VI_FUNC rsfpl_QueryPhaseNoiseResidualNoiseUserRangeResidualRMSJitter(
                            ViSession instrSession,
                            ViInt32   userRange,
                            ViReal64* residualRMSJitter)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_PHASE_NOISE_RESIDUAL_NOISE_USER_RANGE_RESIDUAL_RMS_JITTER,
            "RNRange", userRange - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "User Range");

    viCheckParm(rsfpl_GetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_PHASE_NOISE_RESIDUAL_NOISE_USER_RANGE_RESIDUAL_RMS_JITTER, residualRMSJitter),
            3, "Residual RMS Jitter");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the residual PM for a particular user range. The trace that is
/// HIFN  queried depends on
/// HIFN  RSFPL_ATTR_PHASE_NOISE_RESIDUAL_NOISE_USER_RANGE_TRACE.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR userRange/Selects repcap RNRange.
/// HIPAR residualPM/Queries the residual PM for a particular user range. The trace that is
/// HIPAR residualPM/queried depends on
/// HIPAR residualPM/RSFPL_ATTR_PHASE_NOISE_RESIDUAL_NOISE_USER_RANGE_TRACE.
ViStatus _VI_FUNC rsfpl_QueryPhaseNoiseResidualNoiseUserRangeResidualPM(
                            ViSession instrSession,
                            ViInt32   userRange,
                            ViReal64* residualPM)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_PHASE_NOISE_RESIDUAL_NOISE_USER_RANGE_RESIDUAL_PM,
            "RNRange", userRange - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "User Range");

    viCheckParm(rsfpl_GetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_PHASE_NOISE_RESIDUAL_NOISE_USER_RANGE_RESIDUAL_PM, residualPM),
            3, "Residual PM");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns the spot noise calculation on every 10^x offset frequency on and
/// HIFN  off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR onDecadeEdges/Turns the spot noise calculation on every 10^x offset frequency on and
/// HIPAR onDecadeEdges/off.
ViStatus _VI_FUNC rsfpl_ConfigurePhaseNoiseSpotNoiseOnAllDecadeEdgesEnabled(
                            ViSession instrSession,
                            ViInt32   window,
                            ViBoolean onDecadeEdges)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_PHASE_NOISE_SPOT_NOISE_ON_ALL_DECADE_EDGES_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_PHASE_NOISE_SPOT_NOISE_ON_ALL_DECADE_EDGES_ENABLED, onDecadeEdges),
            3, "On Decade Edges");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns all spot noise markers off (custom and 10^x markers).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
ViStatus _VI_FUNC rsfpl_PhaseNoiseSpotNoiseDisableAllMarkers(ViSession instrSession,
                                                             ViInt32   window)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_PHASE_NOISE_SPOT_NOISE_DISABLE_ALL_MARKERS,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    checkErr(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_PHASE_NOISE_SPOT_NOISE_DISABLE_ALL_MARKERS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns the spot noise calculation on every 10^x offset frequency on and
/// HIFN  off.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSFPL_ATTR_PHASE_NOISE_SPOT_NOISE_DECADE_EDGE_ENABLED
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     CALCulate<n>:SNOise<m>:DECades[:STATe]
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR snMarker/Selects repcap SNMarker.
/// HIPAR edgeMarkerEnabled/Turns the spot noise calculation on every 10^x offset frequency on and
/// HIPAR edgeMarkerEnabled/off.
ViStatus _VI_FUNC rsfpl_ConfigurePhaseNoiseSpotNoiseDecadeEdgeMarkerEnabled(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   snMarker,
                            ViBoolean edgeMarkerEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_PHASE_NOISE_SPOT_NOISE_DECADE_EDGE_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_PHASE_NOISE_SPOT_NOISE_DECADE_EDGE_ENABLED,
            "SNMarker", snMarker - 1, RS_REPCAP_BUF_SIZE, repCap),
            3, "SN Marker");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap_window, repCap_marker);

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_PHASE_NOISE_SPOT_NOISE_DECADE_EDGE_ENABLED, edgeMarkerEnabled),
            4, "Edge Marker Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables and configures a custom spot noise marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR snMarker/Selects repcap SNMarker.
/// HIPAR markerEnabled/Turns a custom spot noise marker on and off.
/// HIPAR frequency/Defines the horizontal position of a custom spot noise marker.
ViStatus _VI_FUNC rsfpl_ConfigurePhaseNoiseSpotNoiseCustomMarker(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   snMarker,
                            ViBoolean markerEnabled,
                            ViReal64  frequency)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_PHASE_NOISE_SPOT_NOISE_MARKER_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_PHASE_NOISE_SPOT_NOISE_MARKER_ENABLED,
            "SNMarker", snMarker - 1, RS_REPCAP_BUF_SIZE, repCap),
            3, "SN Marker");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap_window, repCap_marker);

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_PHASE_NOISE_SPOT_NOISE_MARKER_ENABLED, markerEnabled),
            4, "Marker Enabled");
    if (markerEnabled)
    {
        viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_PHASE_NOISE_SPOT_NOISE_CUSTOM_MARKER_FREQUENCY, frequency),
                5, "Frequency");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This method queries the horizontal positions of the 10^x offset
/// HIFN  frequency spot noise markers.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR buffersSize/Size of output arrays.
/// HIPAR frequencies/Returns the frequencies of the enabled decade edge markers.
/// HIPAR values/Returns the values of the enabled decade edge markers.
ViStatus _VI_FUNC rsfpl_QueryPhaseNoiseSpotNoiseDecadeEdgeResults(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   buffersSize,
                            ViReal64  frequencies[],
                            ViReal64  values[])
{
    ViStatus  error = VI_SUCCESS;
    ViChar    repCap[RS_REPCAP_BUF_SIZE] = "";
    ViPChar   response1 = NULL;
    ViPChar   response2 = NULL;
    ViPReal64 results1 = NULL;
    ViPReal64 results2 = NULL;
    ViInt32   outArraySize = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_PHASE_NOISE_SPOT_NOISE_DECADE_FREQUENCIES,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, buffersSize, 1),
        3, "Buffers Size");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, frequencies),
        4, "Frequencies");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, values),
        5, "Values");

    checkErr(RsCore_GetAttributeViStringUnknownLen(instrSession, repCap,
        RSFPL_ATTR_PHASE_NOISE_SPOT_NOISE_DECADE_FREQUENCIES,
        RS_VAL_DIRECT_USER_CALL, &response1));
    checkErr(RsCore_Convert_String_To_ViReal64Array(instrSession, response1, &results1, &outArraySize));
    checkErr(RsCore_CopyToUserBufferViReal64Array(frequencies, buffersSize, results1, outArraySize));

    outArraySize = 0;
    checkErr(RsCore_GetAttributeViStringUnknownLen(instrSession, repCap,
        RSFPL_ATTR_PHASE_NOISE_SPOT_NOISE_DECADE_VALUES,
        RS_VAL_DIRECT_USER_CALL, &response2));
    checkErr(RsCore_Convert_String_To_ViReal64Array(instrSession, response2, &results2, &outArraySize));
    checkErr(RsCore_CopyToUserBufferViReal64Array(frequencies, buffersSize, results2, outArraySize));

Error:
    if (response1) free(response1);
    if (response2) free(response2);
    if (results1) free(results1);
    if (results2) free(results2);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the vertical position of a custom spot noise marker in dBc/Hz.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR snMarker/Selects repcap SNMarker.
/// HIPAR markerValue/Queries the vertical position of a custom spot noise marker in dBc/Hz.
ViStatus _VI_FUNC rsfpl_QueryPhaseNoiseSpotNoiseMarkerValue(ViSession instrSession,
                                                            ViInt32   window,
                                                            ViInt32   snMarker,
                                                            ViReal64* markerValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_PHASE_NOISE_SPOT_NOISE_DECADE_EDGE_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_PHASE_NOISE_SPOT_NOISE_DECADE_EDGE_ENABLED,
            "SNMarker", snMarker - 1, RS_REPCAP_BUF_SIZE, repCap),
            3, "SN Marker");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap_window, repCap_marker);

    viCheckParm(rsfpl_GetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_PHASE_NOISE_SPOT_NOISE_MARKER_VALUE, markerValue),
            4, "Marker Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns spur removal for all traces and windows on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Turns spur removal for all traces and windows on and off.
ViStatus _VI_FUNC rsfpl_ConfigurePhaseNoiseSpurRemovalEnabled(
                            ViSession instrSession,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_PHASE_NOISE_SPUR_REMOVAL_ENABLED, enabled),
            2, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the level threshold for spur removal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR threshold/Defines the level threshold for spur removal.
ViStatus _VI_FUNC rsfpl_ConfigurePhaseNoiseSpurThreshold(ViSession instrSession,
                                                         ViReal64  threshold)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_PHASE_NOISE_SPUR_THRESHOLD, threshold),
            2, "Threshold");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns two values (frequency and level) for each each spur that has
/// HIFN  been detected.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trace/Sets the trace.
/// HIPAR buffersSize/Size of output arrays.
/// HIPAR frequencies/Location of each spur that has been detected.
/// HIPAR levels/Level of each spur that has been detected.
/// HIPAR returnedValues/Returns the number of values read from the instrument. If this number
/// HIPAR returnedValues/is larger than the allocated size of the buffers, not all values read
/// HIPAR returnedValues/from the instrument have been returned to the user. Consider
/// HIPAR returnedValues/reallocating and calling again.
ViStatus _VI_FUNC rsfpl_QueryPhaseNoiseSpurListResults(ViSession instrSession,
                                                       ViInt32   trace,
                                                       ViInt32   buffersSize,
                                                       ViReal64  frequencies[],
                                                       ViReal64  levels[],
                                                       ViInt32*  returnedValues)
{
    ViStatus  error = VI_SUCCESS;
    ViChar    repCap[RS_REPCAP_BUF_SIZE] = "";
    ViPChar   response = NULL;
    ViPReal64 results = NULL;
    ViInt32   outArraySize = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_PHASE_NOISE_SPUR_LIST_RESULTS,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trace");
    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, buffersSize, 1),
        3, "Buffers Size");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, frequencies),
        4, "Frequencies");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, levels),
        5, "Levels");

    checkErr(RsCore_GetAttributeViStringUnknownLen(instrSession, repCap,
        RSFPL_ATTR_PHASE_NOISE_SPUR_LIST_RESULTS,
        RS_VAL_DIRECT_USER_CALL,
        &response));
    checkErr(RsCore_Convert_String_To_ViReal64Array(instrSession, response, &results, &outArraySize));
    checkErr(RsCore_DecimateViReal64ArrayToUserBuffers(results, outArraySize, buffersSize, returnedValues,
        frequencies, levels, NULL, NULL, NULL, NULL));

Error:
    if (response) free(response);
    if (results) free(results);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the Discrete Jitter result.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trace/Sets the trace.
/// HIPAR discreteJitter/Queries the Discrete Jitter result.
ViStatus _VI_FUNC rsfpl_QueryPhaseNoiseSpurListDiscreteJitter(
                            ViSession instrSession,
                            ViInt32   trace,
                            ViReal64* discreteJitter)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_PHASE_NOISE_SPUR_LIST_DISCRETE_JITTER,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trace");

    viCheckParm(rsfpl_GetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_PHASE_NOISE_SPUR_LIST_DISCRETE_JITTER, discreteJitter),
            3, "Discrete Jitter");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the Random Jitter result.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trace/Sets the trace.
/// HIPAR randomJitter/Queries the Random Jitter result.
ViStatus _VI_FUNC rsfpl_QueryPhaseNoiseSpurListRandomJitter(ViSession instrSession,
                                                            ViInt32   trace,
                                                            ViReal64* randomJitter)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_PHASE_NOISE_SPUR_LIST_RANDOM_JITTER,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trace");

    viCheckParm(rsfpl_GetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_PHASE_NOISE_SPUR_LIST_RANDOM_JITTER, randomJitter),
            3, "Random Jitter");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the carrier frequency and level that has been actually
/// HIFN  measured. These values are shown in the channel bar.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trace/Sets the trace.
/// HIPAR frequency/Returns the carrier frequency that has been actually measured. The
/// HIPAR frequency/measured frequency is shown in the channel bar.
/// HIPAR level/Returns the level of the DUT that has been actually measured. The
/// HIPAR level/measured level is shown in the channel bar.
ViStatus _VI_FUNC rsfpl_QueryPhaseNoiseTraceResults(ViSession instrSession,
                                                    ViInt32   trace,
                                                    ViReal64* frequency,
                                                    ViReal64* level)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_PHASE_NOISE_TRACE_MEASURED_FREQUENCY,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trace");

    viCheckParm(rsfpl_GetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_PHASE_NOISE_TRACE_MEASURED_FREQUENCY, frequency),
            3, "Frequency");

    viCheckParm(rsfpl_GetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_PHASE_NOISE_TRACE_MEASURED_LEVEL, level),
            4, "Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the values for each half decade in the phase noise sweep
/// HIFN  results table.
/// HIFN     Null parameters will be ignored.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR buffersSize/Sizes of output arrays.
/// HIPAR start/Returns the start frequency offset of each half decade.
/// HIPAR stop/Returns the stop frequency offset of each half decade.
/// HIPAR samplingRate/Queries the sampling rate used in each half decade.
/// HIPAR averages/Queries the number of measurements that have been performed in each
/// HIPAR averages/half decade.
/// HIPAR frequencyDrift/Queries the frequency drift in each half decade.
/// HIPAR levelDrift/Queries the level drift in each half decade.
/// HIPAR maxDrift/Queries the maximum frequency drift in each half decade.
ViStatus _VI_FUNC rsfpl_QueryPhaseNoiseSweepResultList(ViSession instrSession,
                                                       ViInt32   buffersSize,
                                                       ViReal64  start[],
                                                       ViReal64  stop[],
                                                       ViReal64  samplingRate[],
                                                       ViReal64  averages[],
                                                       ViReal64  frequencyDrift[],
                                                       ViReal64  levelDrift[],
                                                       ViReal64  maxDrift[])
{
    ViStatus  error = VI_SUCCESS;
    ViPChar   response = NULL;
    ViPReal64 results = NULL;
    ViInt32   outArraySize = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, buffersSize, 1),
        2, "Buffers Size");

    if (start)
    {
        viCheckParm(RsCore_GetAttributeViStringUnknownLen(instrSession, NULL,
            RSFPL_ATTR_PHASE_NOISE_SWEEP_RESULT_LIST_START,
            RS_VAL_DIRECT_USER_CALL, &response),
            3, "Start");
        if (response)
        {
            checkErr(RsCore_Convert_String_To_ViReal64Array(instrSession, response, &results, &outArraySize));
            checkErr(RsCore_CopyToUserBufferViReal64Array(start, buffersSize, results, outArraySize));
            free(response);
            if (results) free(results);
            response = NULL;
            results = NULL;
            outArraySize = 0;
        }
    }

    if (stop)
    {
        viCheckParm(RsCore_GetAttributeViStringUnknownLen(instrSession, NULL,
            RSFPL_ATTR_PHASE_NOISE_SWEEP_RESULT_LIST_STOP,
            RS_VAL_DIRECT_USER_CALL, &response),
            4, "Stop");
        if (response)
        {
            checkErr(RsCore_Convert_String_To_ViReal64Array(instrSession, response, &results, &outArraySize));
            checkErr(RsCore_CopyToUserBufferViReal64Array(stop, buffersSize, results, outArraySize));
            free(response);
            if (results) free(results);
            response = NULL;
            results = NULL;
            outArraySize = 0;
        }
    }

    if (samplingRate)
    {
        viCheckParm(RsCore_GetAttributeViStringUnknownLen(instrSession, NULL,
            RSFPL_ATTR_PHASE_NOISE_SWEEP_RESULT_LIST_SAMPLING_RATE,
            RS_VAL_DIRECT_USER_CALL, &response),
            5, "Sampling Rate");
        if (response)
        {
            checkErr(RsCore_Convert_String_To_ViReal64Array(instrSession, response, &results, &outArraySize));
            checkErr(RsCore_CopyToUserBufferViReal64Array(samplingRate, buffersSize, results, outArraySize));
            free(response);
            if (results) free(results);
            response = NULL;
            results = NULL;
            outArraySize = 0;
        }
    }

    if (averages)
    {
        viCheckParm(RsCore_GetAttributeViStringUnknownLen(instrSession, NULL,
            RSFPL_ATTR_PHASE_NOISE_SWEEP_RESULT_LIST_AVERAGES,
            RS_VAL_DIRECT_USER_CALL, &response),
            6, "Averages");
        if (response)
        {
            checkErr(RsCore_Convert_String_To_ViReal64Array(instrSession, response, &results, &outArraySize));
            checkErr(RsCore_CopyToUserBufferViReal64Array(averages, buffersSize, results, outArraySize));
            free(response);
            if (results) free(results);
            response = NULL;
            results = NULL;
            outArraySize = 0;
        }
    }

    if (frequencyDrift)
    {
        viCheckParm(RsCore_GetAttributeViStringUnknownLen(instrSession, NULL,
            RSFPL_ATTR_PHASE_NOISE_SWEEP_RESULT_LIST_FREQUENCY_DRIFT,
            RS_VAL_DIRECT_USER_CALL, &response),
            7, "Frequency Drift");
        if (response)
        {
            checkErr(RsCore_Convert_String_To_ViReal64Array(instrSession, response, &results, &outArraySize));
            checkErr(RsCore_CopyToUserBufferViReal64Array(frequencyDrift, buffersSize, results, outArraySize));
            free(response);
            if (results) free(results);
            response = NULL;
            results = NULL;
            outArraySize = 0;
        }
    }

    if (levelDrift)
    {
        viCheckParm(RsCore_GetAttributeViStringUnknownLen(instrSession, NULL,
            RSFPL_ATTR_PHASE_NOISE_SWEEP_RESULT_LIST_LEVEL_DRIFT,
            RS_VAL_DIRECT_USER_CALL, &response),
            8, "Level Drift");
        if (response)
        {
            checkErr(RsCore_Convert_String_To_ViReal64Array(instrSession, response, &results, &outArraySize));
            checkErr(RsCore_CopyToUserBufferViReal64Array(levelDrift, buffersSize, results, outArraySize));
            free(response);
            if (results) free(results);
            response = NULL;
            results = NULL;
            outArraySize = 0;
        }
    }

    if (maxDrift)
    {
        viCheckParm(RsCore_GetAttributeViStringUnknownLen(instrSession, NULL,
            RSFPL_ATTR_PHASE_NOISE_SWEEP_RESULT_LIST_MAX_DRIFT,
            RS_VAL_DIRECT_USER_CALL, &response),
            9, "Max Drift");
        if (response)
        {
            checkErr(RsCore_Convert_String_To_ViReal64Array(instrSession, response, &results, &outArraySize));
            checkErr(RsCore_CopyToUserBufferViReal64Array(maxDrift, buffersSize, results, outArraySize));
            free(response);
            if (results) free(results);
            response = NULL;
            results = NULL;
            outArraySize = 0;
        }
    }

Error:
    if (response) free(response);
    if (results) free(results);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}
