/*****************************************************************************
 *  rsfpl.c
 *
 *  Rohde & Schwarz FPL spectrum analyzer Instrument Driver
 *  Original Release: May 2018
 *  By: Jiri Kominek
 *
 *  Should you have any technical questions please contact the hotline of
 *  Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
 *
 *  e-mail: CustomerSupport@rohde-schwarz.com
 *
 *  Modification History:
 *    see ChangeLog
 *
 *  Copyright (c) 2019, Rohde & Schwarz GmbH & Co. KG
 *  All rights reserved.
 *****************************************************************************/
#include "rsfpl.h"
#include "rsfpl_internal.h"

#include "time.h"
#if (defined(WIN32) || defined(_WIN32) || defined(__WIN32__) || defined(__NT__))
#define ANSI
#endif
//#ifndef ANSI                /* UNIX compatible */
//#include <sys/timeb.h>
//#include <sys/types.h>
//#include <unistd.h>
//#define Sleep sleep
//#else                       /* ANSI compatible */
//#include <windows.h>
//#endif
#include <string.h>
#include <stdlib.h>
#if defined (_CVI_)
#include <utility.h>
#define _strnicmp strnicmp
#endif

 /*****************************************************************************
  *- Value Definition and Range Tables ---------------------------------------*
  *****************************************************************************/
ViChar gsCommand[RS_MAX_MESSAGE_BUF_SIZE] = "";
extern ViString gsc_read_y_trace_arr[] = { "TRACE1", "TRACE2", "TRACE3", "TRACE4", "TRACE5", "TRACE6", "LIST", "SPUR", "SPEC", NULL };
static const ViString gsc_PowerMeasurementArr[] = { "ACP", "CPOW", "MCAC", "OBAN", "CN", "CN0", "AOB", "PPOW", "GACL", "MACM", "COB", NULL };
static const ViString gsc_ResultsString = "PASSED,FAILED,NONE";
static const ViString arrChannelType[] = { "SANALYZER", "ADEM", "IQ", "NOISE", "DDEM" };

/*****************************************************************************
 *- Callback Declarations ---------------------------------------------------*
 *****************************************************************************/

 /*****************************************************************************
  *- User-Callable Functions (Exportable Functions) --------------------------*
  *****************************************************************************/

  /// HIFN  This function performs instrument driver initialization actions
  /// HIRET Returns the status code of this operation.
  /// HIPAR resourceName/Pass the resource name of the device to initialize.
  /// HIPAR IDQuery/Specify whether you want the instrument driver to perform an ID Query.
  /// HIPAR resetDevice/Specify whether you want the to reset the instrument during the
  /// HIPAR resetDevice/initialization procedure.
  /// HIPAR newInstrSession/Returns a ViSession handle that you use to identify the instrument in
  /// HIPAR newInstrSession/all subsequent instrument driver function calls.
ViStatus _VI_FUNC rsfpl_init(ViRsrc resourceName, ViBoolean IDQuery,
    ViBoolean resetDevice, ViSession* newInstrSession)
{
    ViStatus error = VI_SUCCESS;

    if (newInstrSession == NULL)
    {
        (void)RsCore_SetErrorInfo(0, VI_FALSE, RS_ERROR_INVALID_PARAMETER,
            VI_ERROR_PARAMETER4, "Null address for Instrument Handle");

        checkErr(RS_ERROR_INVALID_PARAMETER);
    }

    checkErr(rsfpl_InitWithOptions(resourceName, IDQuery, resetDevice, "", newInstrSession));

Error:
    return error;
}

/// HIFN  This function performs instrument driver initialization actions
/// HIRET Returns the status code of this operation.
/// HIPAR resourceName/Pass the resource name of the device to initialize.
/// HIPAR IDQuery/Specify whether you want the instrument driver to perform an ID Query.
/// HIPAR resetDevice/Specify whether you want the to reset the instrument during the
/// HIPAR resetDevice/initialization procedure.
/// HIPAR optionString/You can use this control to set the initial value of certain
/// HIPAR optionString/attributes for the session.  The following table lists the attributes
/// HIPAR optionString/and the name you use in this parameter to identify the attribute.
/// HIPAR newInstrSession/Returns a ViSession handle that you use to identify the instrument in
/// HIPAR newInstrSession/all subsequent instrument driver function calls.
ViStatus _VI_FUNC rsfpl_InitWithOptions(
    ViRsrc resourceName,
    ViBoolean IDQuery,
    ViBoolean resetDevice,
    ViString optionString,
    ViSession* newInstrSession
)
{
    ViStatus error = VI_SUCCESS;
    RsCoreAttributePtr* attrList = NULL;
    ViSession instrSession = 0;
    RsCoreSessionPtr sessionProperties = NULL;

    if (newInstrSession == NULL)
    {
        (void)RsCore_SetErrorInfo(0, VI_FALSE, RS_ERROR_INVALID_PARAMETER, VI_ERROR_PARAMETER5,
            "Null address for Instrument Handle");
        checkErr(RS_ERROR_INVALID_PARAMETER);
    }

    attrList = (RsCoreAttributePtr*)rsfpl_RsCoreAttributeList;

    *newInstrSession = 0;

    checkErr(RsCore_NewSpecificDriver(
        resourceName, // VISA Resource Name
        "rsfpl", // driver prefix
        optionString, // Init Options string, is applied to the session settings
        rsfpl_RsCoreAttributeList, // List of all attributes from the rsxxx_AttrPropertiesList
        rsfpl_RsCoreAttributeListLength, // Length of the attributes list
        VI_FALSE, //idnModelFullName Model has the full *IDN? query name e.g. true: "CMW100A", VI_FALSE: "CMW" Used in CheckInstrument()
        0, // WriteDelay
        0, // ReadDelay
        1000000, // IO Segment Size
        RS_VAL_OPCWAIT_STBPOLLING, // OPC Wait Mode
        RSFPL_OPC_TIMEOUT, // OPC timeout
        15000, // VISA Timeout
        600000, // Self-test timeout
        RS_VAL_BIN_FLOAT_FORMAT_SINGLE_4BYTES, // BinaryFloatNumbersFormat
        RS_VAL_BIN_INTEGER_FORMAT_INT32_4BYTES, // binaryIntegerNumbersFormat
        &instrSession));

    // Minimal allowed rscore version is 4.3.0
    checkErr(RsCore_CheckMinimalEngineVersion(instrSession, 4, 3, 0));

    checkErr(RsCore_GetRsSession(instrSession, &sessionProperties));

    // No SCPI command has been sent yet.
    // Call viClear before sending any SCPI command
    checkErr(RsCore_ViClear(instrSession));

    /* --- Here perform settings that are default for this driver,
    but can be overwritten by the optionsString settings */

    sessionProperties->autoSystErrQuery = VI_FALSE;

    // Parse option string and optionally sets the initial state of the following session attributes
    checkErr(RsCore_ApplyOptionString(instrSession, optionString));
    checkErr(RsCore_BuildRepCapTable(instrSession, rsfpl_RsCoreRepCapTable, rsfpl_RsCoreRepCapTableCount));

    // Query *IDN? string, parse it and set the following attributes:
    // -RS_ATTR_INSTRUMENT_MANUFACTURER
    // - RS_ATTR_INSTRUMENT_MODEL - based on the idnModelFullName true: "CMW100A", false : "CMW"
    // - RS_ATTR_INSTRUMENT_FIRMWARE_REVISION
    checkErr(RsCore_QueryAndParseIDNstring(instrSession, RSFPL_SIMULATION_ID_QUERY, NULL));

    // Query *OPT? string, parse the options, remove the duplicates, sort them and store the result string to RS_ATTR_OPTIONS_LIST
    checkErr(RsCore_QueryAndParseOPTstring(instrSession, RSFPL_SIMULATION_OPT_QUERY, RS_VAL_OPTIONS_PARSING_AUTO));

    // Default Instrument Setup + optional *RST
    if (resetDevice == VI_TRUE)
    {
        checkErr(rsfpl_reset(instrSession));
    }
    else
    {
        checkErr(rsfpl_DefaultInstrSetup(instrSession));
    }

    if (IDQuery == VI_TRUE)
    {
        checkErr(RsCore_FitsIDNpattern(instrSession, RSFPL_VALID_ID_RESPONSE_STRING, NULL));
    }

    checkErr(rsfpl_CheckStatus(instrSession));

Error:
    if (error < VI_SUCCESS)
    {
        if (sessionProperties != NULL && error == RS_ERROR_INSTRUMENT_STATUS && sessionProperties->autoSystErrQuery == VI_FALSE)
        {
            sessionProperties->autoSystErrQuery = VI_TRUE;
            (void)rsfpl_CheckStatus(instrSession);
            sessionProperties->autoSystErrQuery = VI_FALSE;
        }

        if (instrSession != 0)
            (void)RsCore_ViClose(instrSession);
    }
    else
    {
        *newInstrSession = instrSession;
    }

    return error;
}

/// HIFN  This function selects the spectrum analyzer mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_SetSANMode(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsfpl_WriteCommandWithOPCSync(instrSession, "INST:SEL SAN", 0));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the start/stop frequency of the spectrum
/// HIFN  analyzer
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR startFrequency/The start frequency of the frequency sweep.
/// HIPAR stopFrequency/The stop frequency of the frequency sweep.
ViStatus _VI_FUNC rsfpl_ConfigureFrequencyStartStop(ViSession instrSession,
                                                    ViReal64  startFrequency,
                                                    ViReal64  stopFrequency)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_FREQUENCY_START, startFrequency),
            2, "Start Frequency");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_FREQUENCY_STOP, stopFrequency),
            3, "Stop Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the center/span frequency the spectrum
/// HIFN  analyzer
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR centerFrequency/Defines the center frequency.
/// HIPAR span/Defines the frequency span.
ViStatus _VI_FUNC rsfpl_ConfigureFrequencyCenterSpan(ViSession instrSession,
                                                     ViReal64  centerFrequency,
                                                     ViReal64  span)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_FREQUENCY_CENTER, centerFrequency),
            2, "Center Frequency");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_FREQUENCY_SPAN, span),
            3, "Span");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the center frequency the spectrum analyzer
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR centerFrequency/Defines the center frequency.
ViStatus _VI_FUNC rsfpl_ConfigureFrequencyCenter(ViSession instrSession,
                                                 ViReal64  centerFrequency)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_FREQUENCY_CENTER, centerFrequency),
            2, "Center Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines a frequency offset.
/// HIFN     If this value is not 0Hz, the application assumes that the input
/// HIFN  signal was frequency shifted outside the application. All results of
/// HIFN  type 'frequency' will be corrected for this shift numerically by the
/// HIFN  application.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frequencyOffset/Defines a frequency offset.
ViStatus _VI_FUNC rsfpl_ConfigureFrequencyOffset(ViSession instrSession,
                                                 ViReal64  frequencyOffset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_FREQUENCY_OFFSET, frequencyOffset),
            2, "Frequency Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Switches between the MultiView tab and the most recently displayed
/// HIFN  channel setup. If only one channel setup is active, this attribute has
/// HIFN  no effect.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR modeMultiviewEnabled/Switches between the MultiView tab and the most recently displayed
/// HIPAR modeMultiviewEnabled/channel setup. If only one channel setup is active, this control has no
/// HIPAR modeMultiviewEnabled/effect.
ViStatus _VI_FUNC rsfpl_ConfigureOperatingModeMultiviewEnabled(
                            ViSession instrSession,
                            ViBoolean modeMultiviewEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_OPERATING_MODE_MULTIVIEW_ENABLED, modeMultiviewEnabled),
            2, "Multiview Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function adds an additional measurement channel. The number of
/// HIFN  measurement channels you can configure at the same time depends on
/// HIFN  available memory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channelType/This control selects the channel type of the new channel.
/// HIPAR channelName/String containing the name of the channel.
ViStatus _VI_FUNC rsfpl_AddMeasurementChannel(ViSession instrSession,
                                              ViString  channelType,
                                              ViString  channelName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "INST:CRE %s,'%s'", channelType, channelName);
    checkErr(rsfpl_WriteCommandWithOPCSync(instrSession, cmd, 0));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects an existing measurement channel with the
/// HIFN  specified name.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channelName/String containing the name of the channel.
ViStatus _VI_FUNC rsfpl_SelectMeasurementChannel(ViSession instrSession,
                                                 ViString  channelName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "INST %s", channelName);
    checkErr(rsfpl_WriteCommandWithOPCSync(instrSession, cmd, 0));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects an existing measurement channel type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channelType/Channel type of the new channel setup. For a list of available channel
/// HIPAR channelType/setup types see RSFPL_ATTR_OPERATING_MODE_LIST_OF_CHANNEL_SETUPS
/// HIPAR channelType/(ListOfChannelSetups).
ViStatus _VI_FUNC rsfpl_SelectMeasurementChannelByType(ViSession instrSession,
                                                       ViInt32   channelType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channelType, RSFPL_VAL_CHAN_TYPE_SANALYZER, RSFPL_VAL_CHAN_TYPE_VSA),
            2, "Channel Type");


    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "INST %s", arrChannelType[channelType]);
    checkErr(RsCore_WriteWithOpc(instrSession, cmd, 0));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function duplicates the currently selected measurement channel,
/// HIFN  i.e starts a new measurement channel of the same type and with the
/// HIFN  identical measurement settings. The name of the new channel is the same
/// HIFN  as the copied channel, extended by a consecutive number (e.g.
/// HIFN  "Spectrum" -> "Spectrum 2").
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_DuplicateMeasurementChannel(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsfpl_WriteCommandWithOPCSync(instrSession, "INST:CRE:DUPL", 0));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function replaces a measurement channel with another one.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR oldChannelName/String containing the name of the channel you want to replace.
/// HIPAR newChannelType/This control selects the channel type of the new channel.
/// HIPAR newChannelName/String containing the name of the new channel.
ViStatus _VI_FUNC rsfpl_ReplaceMeasurementChannel(ViSession instrSession,
                                                  ViString  oldChannelName,
                                                  ViString  newChannelType,
                                                  ViString  newChannelName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "INST:CRE:REPL '%s',%s,'%s'", oldChannelName, newChannelType, newChannelName);
    checkErr(rsfpl_WriteCommandWithOPCSync(instrSession, cmd, 0));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function renames a measurement channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR oldChannelName/String containing the name of the channel you want to rename.
/// HIPAR newChannelName/String containing the new channel name.
ViStatus _VI_FUNC rsfpl_RenameMeasurementChannel(ViSession instrSession,
                                                 ViString  oldChannelName,
                                                 ViString  newChannelName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "INST:REN '%s','%s'", oldChannelName, newChannelName);
    checkErr(rsfpl_WriteCommandWithOPCSync(instrSession, cmd, 0));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function deletes a measurement channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channelName/String containing the name of the channel you want to delete.
ViStatus _VI_FUNC rsfpl_DeleteMeasurementChannel(ViSession instrSession,
                                                 ViString  channelName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_CHANNEL_SETUP_DELETE, channelName),
            2, "Channel Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries all active measurement channels.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR arraySize/This parameter determines the size of array Channel Type and Channel
/// HIPAR arraySize/Name. The Channel Type and Channel Name arrays must contain at least
/// HIPAR arraySize/this number of items.
/// HIPAR channelType/This control returns an array of the channel types of all channels.
/// HIPAR channelName/This control returns a comma separated list of names of channels.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsfpl_QueryAllMeasurementChannels(ViSession instrSession,
                                                    ViUInt32  arraySize,
                                                    ViChar    channelCatalog[],
                                                    ViUInt32* returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsfpl_QueryViString(instrSession, "INST:LIST?", arraySize, channelCatalog, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns the Sequencer on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR sequencerEnabled/Turns the Sequencer on and off.
ViStatus _VI_FUNC rsfpl_ConfigureSequencerEnabled(ViSession instrSession,
                                                  ViBoolean sequencerEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_SEQUENCER_ENABLED, sequencerEnabled),
            2, "Sequencer Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the way the FPL application performs measurements
/// HIFN  sequentially.
/// HIFN     Before this command can be executed, the Sequencer must be activated
/// HIFN  (see RSFPL_ATTR_SEQUENCER_ENABLED).
/// HIFN     Note: In order to synchronize to the end of a sequential measurement
/// HIFN  using *OPC, *OPC? or *WAI you must use Sinle Sequence mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR sequencerMode/Selects the way the FPL application performs measurements
/// HIPAR sequencerMode/sequentially.
ViStatus _VI_FUNC rsfpl_ConfigureSequencerMode(ViSession instrSession,
                                               ViInt32   sequencerMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_SEQUENCER_MODE, sequencerMode),
            2, "Sequencer Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Starts a new sequence of measurements by the Sequencer. Before it can
/// HIFN  be executed, the Sequencer must be activated (see
/// HIFN  RSFPL_ATTR_SEQUENCER_ENABLED).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_SequencerInit(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_SEQUENCER_INIT, NULL),
            2, "Sequencer Init");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Stops the currently active sequence of measurements. The Sequencer
/// HIFN  itself is not deactivated, so you can start a new sequence immediately
/// HIFN  using RSFPL_ATTR_SEQUENCE_INIT.
/// HIFN     To deactivate the Sequencer use
/// HIFN  RSFPL_ATTR_SEQUENCER_ENABLED.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_SequencerAbort(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_SEQUENCER_ABORT, NULL),
            2, "Sequencer Abort");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Determines which IF path the R&S FPL1000 hardware uses.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR rfInputSAWFilter/Determines which IF path the R&S FPL1000 hardware uses.
ViStatus _VI_FUNC rsfpl_ConfigureRFInputSAWFilter(ViSession instrSession,
                                                  ViInt32   RFInputSAWFilter)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_RF_INPUT_SAW_FILTER, RFInputSAWFilter),
            2, "RF Input SAW Filter");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the nominal input impedance of the RF input. In some
/// HIFN  applications, only 50 Ohm are supported.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR rfInputImpedance/Selects the nominal input impedance of the RF input. In some
/// HIPAR rfInputImpedance/applications, only 50 Ohm are supported.
ViStatus _VI_FUNC rsfpl_ConfigureRFInputImpedance(ViSession instrSession,
                                                  ViInt32   rfInputImpedance)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_RF_INPUT_IMPEDANCE, rfInputImpedance),
            2, "RF Input Impedance");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Resets the attenuator and reconnects the RF input with the input mixer
/// HIFN  after an overload condition occurred and the protection mechanism
/// HIFN  intervened. The error status bit (bit 3 in the STAT:QUES:POW status
/// HIFN  register) and the INPUT OVLD message in the status bar are cleared.
/// HIFN     The attribute/method works only if the overload condition has been
/// HIFN  eliminated first.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_RFInputProtectionReset(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_RF_INPUT_PROTECTION_RESET, NULL),
            2, "RF Input Protection Reset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns a power sensor on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR sensor/Selects the sensor.
/// HIPAR powerSensorEnabled/Turns a power sensor on and off.
ViStatus _VI_FUNC rsfpl_ConfigurePowerSensorEnabled(ViSession instrSession,
                                                    ViInt32   sensor,
                                                    ViBoolean powerSensorEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_POWER_SENSOR_TO_REFERENCE,
            "PowerSensor", sensor - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sensor");
    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_POWER_SENSOR_ENABLED, powerSensorEnabled),
            3, "Power Sensor Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns continuous update of power sensor measurements on and off.
/// HIFN     If on, the results are update even if a single sweep is complete.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR sensor/Selects the sensor.
/// HIPAR continuousUpdateEnabled/Turns continuous update of power sensor measurements on and off.
ViStatus _VI_FUNC rsfpl_ConfigurePowerSensorContinuousUpdateEnabled(
                            ViSession instrSession,
                            ViInt32   sensor,
                            ViBoolean continuousUpdateEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_POWER_SENSOR_TO_REFERENCE,
            "PowerSensor", sensor - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sensor");

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_POWER_SENSOR_CONTINUOUS_UPDATE_ENABLED, continuousUpdateEnabled),
            3, "Continuous Update Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the number of power sensors currently connected to the
/// HIFN  instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR powerSensorCount/Queries the number of power sensors currently connected to the
/// HIPAR powerSensorCount/instrument.
ViStatus _VI_FUNC rsfpl_QueryPowerSensorCount(ViSession instrSession,
                                              ViInt32*  powerSensorCount)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_GetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_POWER_SENSOR_COUNT, powerSensorCount),
            2, "Power Sensor Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function assign the connected power sensors to the configuration
/// HIFN  sets.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR sensor/Selects the sensor.
/// HIPAR autoAssignEnabled/Defines whether the selected power sensor index is automatically
/// HIPAR autoAssignEnabled/assigned to a subsequently connected power sensor.
/// HIPAR placeholder/This control is reserved for future use.
/// HIPAR type/The power sensor type.
/// HIPAR interface/This control is reserved for future use.
/// HIPAR serialNumber/The power sensor serial number.
ViStatus _VI_FUNC rsfpl_ConfigurePowerSensorAssignment(ViSession instrSession,
                                                       ViInt32   sensor,
                                                       ViBoolean autoAssignEnabled,
                                                       ViString  placeholder,
                                                       ViString  type,
                                                       ViString  powerSensorInterface,
                                                       ViString  serialNumber)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
             powerSensorInterface;
             placeholder;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_POWER_SENSOR_TO_REFERENCE,
            "PowerSensor", sensor - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sensor");
    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_POWER_SENSOR_AUTO_ASSIGN_ENABLED, autoAssignEnabled),
            3, "Auto Assign Enabled");

    if (!autoAssignEnabled)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SYST:COMM:RDEV:PMET%ld:DEF '','%s','','%s'", sensor,
            type,
            serialNumber);
        checkErr(rsfpl_WriteCommandWithOPCSync(instrSession, cmd, 0));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the current measurement result as the reference level for
/// HIFN  relative measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR sensor/Selects the sensor.
ViStatus _VI_FUNC rsfpl_PowerSensorMeasurementToReference(ViSession instrSession,
                                                          ViInt32   sensor)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_POWER_SENSOR_TO_REFERENCE,
            "PowerSensor", sensor - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sensor");
    viCheckParm(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_POWER_SENSOR_TO_REFERENCE, NULL),
            3, "Power Sensor To Reference");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the reference value for relative measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR sensor/Selects the sensor.
/// HIPAR referenceValue/Defines the reference value for relative measurements.
ViStatus _VI_FUNC rsfpl_ConfigurePowerSensorReferenceValue(ViSession instrSession,
                                                           ViInt32   sensor,
                                                           ViReal64  referenceValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_POWER_SENSOR_REFERENCE_VALUE,
            "PowerSensor", sensor - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sensor");
    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_POWER_SENSOR_REFERENCE_VALUE, referenceValue),
            3, "Reference Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns relative power sensor measurements on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR sensor/Selects the sensor.
/// HIPAR relativePowerEnabled/Turns relative power sensor measurements on and off.
ViStatus _VI_FUNC rsfpl_ConfigurePowerSensorRelativePowerEnabled(
                            ViSession instrSession,
                            ViInt32   sensor,
                            ViBoolean relativePowerEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_POWER_SENSOR_RELATIVE_POWER_ENABLED,
            "PowerSensor", sensor - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sensor");
    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_POWER_SENSOR_RELATIVE_POWER_ENABLED, relativePowerEnabled),
            3, "Relative Power Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the external power trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR sensor/Selects the sensor.
/// HIPAR externalTriggerEnabled/Turns the external power trigger on and off.
/// HIPAR level/Defines the trigger level for external power triggers. This control
/// HIPAR level/has effect only if External Trigger Enabled == On.
/// HIPAR hysteresis/Defines the trigger hysteresis for external power triggers.
/// HIPAR dropoutTime/Defines the time period that the input signal has to stay below the IF
/// HIPAR dropoutTime/power trigger level before the measurement starts. This control has
/// HIPAR dropoutTime/effect only if External Trigger Enabled == On.
/// HIPAR holdoffTime/Defines the trigger holdoff for external power triggers. This control
/// HIPAR holdoffTime/has effect only if External Trigger Enabled == On.
/// HIPAR slope/Selects the trigger condition for external power triggers. This
/// HIPAR slope/control has effect only if External Trigger Enabled == On.
ViStatus _VI_FUNC rsfpl_ConfigurePowerSensorExternalTrigger(ViSession instrSession,
                                                            ViInt32   sensor,
                                                            ViBoolean externalTriggerEnabled,
                                                            ViReal64  level,
                                                            ViReal64  hysteresis,
                                                            ViReal64  dropoutTime,
                                                            ViReal64  holdoffTime,
                                                            ViInt32   slope)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_POWER_SENSOR_EXTERNAL_TRIGGER_ENABLED,
            "PowerSensor", sensor - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sensor");
    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_POWER_SENSOR_EXTERNAL_TRIGGER_ENABLED, externalTriggerEnabled),
            3, "External Trigger Enabled");
    if (externalTriggerEnabled)
    {
        viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_POWER_SENSOR_EXTERNAL_TRIGGER_LEVEL, level),
                4, "Level");

        viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_POWER_SENSOR_EXTERNAL_TRIGGER_HYSTERESIS, hysteresis),
                5, "Hysteresis");

        viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_POWER_SENSOR_EXTERNAL_TRIGGER_DROPOUT_TIME, dropoutTime),
                6, "Dropout Time");

        viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_POWER_SENSOR_EXTERNAL_TRIGGER_HOLDOFF_TIME, holdoffTime),
                7, "Holdoff Time");

        viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_POWER_SENSOR_EXTERNAL_TRIGGER_SLOPE, slope),
                8, "Slope");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Zeroes the power sensor.
/// HIFN     Note that you have to disconnect the signals from the power sensor
/// HIFN  input before you start to zero the power sensor. Otherwise, results are
/// HIFN  invalid.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR sensor/Selects the sensor.
ViStatus _VI_FUNC rsfpl_PowerSensorZero(ViSession instrSession,
                                        ViInt32   sensor)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_POWER_SENSOR_ZERO,
            "PowerSensor", sensor - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sensor");
    viCheckParm(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_POWER_SENSOR_ZERO, NULL),
            3, "Power Sensor Zero");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the frequency of the power sensor.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR sensor/Selects the sensor.
/// HIPAR frequencyCoupling/Selects the frequency coupling for power sensor measurements.
/// HIPAR frequencyManual/Defines the frequency of the power sensor. This control is valid only
/// HIPAR frequencyManual/if Frequency Coupling == Off.
ViStatus _VI_FUNC rsfpl_ConfigurePowerSensorFrequency(ViSession instrSession,
                                                      ViInt32   sensor,
                                                      ViInt32   frequencyCoupling,
                                                      ViReal64  frequencyManual)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_POWER_SENSOR_FREQUENCY_COUPLING,
            "PowerSensor", sensor - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sensor");

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_POWER_SENSOR_FREQUENCY_COUPLING, frequencyCoupling),
            3, "Frequency Coupling");
    if (frequencyCoupling == RSFPL_VAL_PMET_COUPLING_OFF)
    {
        viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_POWER_SENSOR_FREQUENCY, frequencyManual),
                4, "Frequency Manual");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Includes or excludes the reference level offset of the analyzer for
/// HIFN  power sensor measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR sensor/Selects the sensor.
/// HIPAR useRefLevelOffsetEnabled/Includes or excludes the reference level offset of the analyzer for
/// HIPAR useRefLevelOffsetEnabled/power sensor measurements.
ViStatus _VI_FUNC rsfpl_ConfigurePowerSensorUseRefLevelOffsetEnabled(
                            ViSession instrSession,
                            ViInt32   sensor,
                            ViBoolean useRefLevelOffsetEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_POWER_SENSOR_USE_REF_LEVEL_OFFSET_ENABLED,
            "PowerSensor", sensor - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sensor");
    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_POWER_SENSOR_USE_REF_LEVEL_OFFSET_ENABLED, useRefLevelOffsetEnabled),
            3, "Use Ref Level Offset Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Takes the specified offset into account for the measured power. Only
/// HIFN  available if RSFPL_ATTR_POWER_SENSOR_USE_REF_LEVEL_OFFSET_ENABLED is
/// HIFN  off (see rsfpl_ConfigurePowerSensorUseRefLevelOffsetEnabled).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR sensor/Selects the sensor.
/// HIPAR sensorLevelOffset/Defines the offset to take into account for the measured power.
ViStatus _VI_FUNC rsfpl_ConfigurePowerSensorSensorLevelOffset(
                            ViSession instrSession,
                            ViInt32   sensor,
                            ViReal64  sensorLevelOffset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_POWER_SENSOR_USE_REF_LEVEL_OFFSET_ENABLED,
            "PowerSensor", sensor - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sensor");
    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_POWER_SENSOR_SENSOR_LEVEL_OFFSET, sensorLevelOffset),
            3, "Sensor Level Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the unit for absolute power sensor measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR sensor/Selects the sensor.
/// HIPAR absolutePowerUnit/Selects the unit for absolute power sensor measurements.
ViStatus _VI_FUNC rsfpl_ConfigurePowerSensorAbsolutePowerUnit(
                            ViSession instrSession,
                            ViInt32   sensor,
                            ViInt32   absolutePowerUnit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_POWER_SENSOR_ABSOLUTE_POWER_UNIT,
            "PowerSensor", sensor - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sensor");
    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_POWER_SENSOR_ABSOLUTE_POWER_UNIT, absolutePowerUnit),
            3, "Absolute Power Unit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the unit for relative power sensor measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR sensor/Selects the sensor.
/// HIPAR relativePowerUnit/Selects the unit for relative power sensor measurements.
ViStatus _VI_FUNC rsfpl_ConfigurePowerSensorRelativePowerUnit(
                            ViSession instrSession,
                            ViInt32   sensor,
                            ViInt32   relativePowerUnit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_POWER_SENSOR_RELATIVE_POWER_UNIT,
            "PowerSensor", sensor - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sensor");
    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_POWER_SENSOR_RELATIVE_POWER_UNIT, relativePowerUnit),
            3, "Relative Power Unit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures averaging for power sensor measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR sensor/Selects the sensor.
/// HIPAR averagingEnabled/Turns averaging for power sensor measurements on and off.
/// HIPAR numberOfReadings/Sets the number of power readings included in the averaging process of
/// HIPAR numberOfReadings/power sensor measurements.
ViStatus _VI_FUNC rsfpl_ConfigurePowerSensorAveraging(ViSession instrSession,
                                                      ViInt32   sensor,
                                                      ViBoolean averagingEnabled,
                                                      ViInt32   numberOfReadings)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_POWER_SENSOR_AVERAGING_ENABLED,
            "PowerSensor", sensor - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sensor");

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_POWER_SENSOR_AVERAGING_ENABLED, averagingEnabled),
            3, "Averaging Enabled");
    if (averagingEnabled)
    {
        viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_POWER_SENSOR_NUMBER_OF_READINGS, numberOfReadings),
                4, "Number Of Readings");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the duration of power sensor measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR sensor/Selects the sensor.
/// HIPAR measTime/Selects the duration of power sensor measurements.
ViStatus _VI_FUNC rsfpl_ConfigurePowerSensorMeasTime(ViSession instrSession,
                                                     ViInt32   sensor,
                                                     ViInt32   measTime)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_POWER_SENSOR_MEAS_TIME,
            "PowerSensor", sensor - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sensor");
    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_POWER_SENSOR_MEAS_TIME, measTime),
            3, "Meas Time");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the duty cycle correction.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR sensor/Selects the sensor.
/// HIPAR dutyCycleEnabled/Turns the duty cycle correction on and off.
/// HIPAR dutyCycleValue/Defines the duty cycle for the correction of pulse signals.
ViStatus _VI_FUNC rsfpl_ConfigurePowerSensorDutyCycle(ViSession instrSession,
                                                      ViInt32   sensor,
                                                      ViBoolean dutyCycleEnabled,
                                                      ViReal64  dutyCycleValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_POWER_SENSOR_DUTY_CYCLE_ENABLED,
            "PowerSensor", sensor - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sensor");
    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_POWER_SENSOR_DUTY_CYCLE_ENABLED, dutyCycleEnabled),
            3, "Duty Cycle Enabled");
    if (dutyCycleEnabled)
    {
        viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_POWER_SENSOR_DUTY_CYCLE_VALUE, dutyCycleValue),
                3, "Power Sensor Duty Cycle Value");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Initiates a power sensor measurement and queries the results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR sensor/Selects the sensor.
/// HIPAR measuredValue/Initiates a power sensor measurement and queries the results.
ViStatus _VI_FUNC rsfpl_PowerSensorRead(ViSession instrSession,
                                        ViInt32   sensor,
                                        ViReal64* measuredValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_POWER_SENSOR_READ,
            "PowerSensor", sensor - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sensor");

    viCheckParm(rsfpl_GetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_POWER_SENSOR_READ, measuredValue),
            3, "Measured Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the results of power sensor measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR sensor/Selects the sensor.
/// HIPAR measuredValue/Queries the results of power sensor measurements.
ViStatus _VI_FUNC rsfpl_PowerSensorFetch(ViSession instrSession,
                                         ViInt32   sensor,
                                         ViReal64* measuredValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_POWER_SENSOR_FETCH,
            "PowerSensor", sensor - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sensor");
    viCheckParm(rsfpl_GetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_POWER_SENSOR_FETCH, measuredValue),
            3, "Measured Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables or disables the internal generator. The generator signal is
/// HIFN  output at the GEN Output 50 Ohm connector on the front panel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Enables or disables the internal generator. The generator signal is
/// HIPAR enabled/output at the GEN Output 50 Ohm connector on the front panel.
ViStatus _VI_FUNC rsfpl_ConfigureInternalGeneratorState(ViSession instrSession,
                                                        ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_INTERNAL_GENERATOR_STATE, enabled),
            2, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Determines the type of signal provided by the internal generator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR usage/Determines the type of signal provided by the internal generator:
ViStatus _VI_FUNC rsfpl_ConfigureInternalGeneratorUsage(ViSession instrSession,
                                                        ViInt32   usage)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    switch (usage)
    {
    case RSFPL_VAL_INTERNAL_GENERATOR_USAGE_CW_SOURCE:
        checkErr(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_INTERNAL_GENERATOR_TRACKING_GENERATOR_STATE, VI_FALSE));
        break;
    case RSFPL_VAL_INTERNAL_GENERATOR_USAGE_TRACKING_GENERATOR:
        checkErr(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_INTERNAL_GENERATOR_TRACKING_GENERATOR_STATE, VI_TRUE));
        checkErr(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_INTERNAL_GENERATOR_POWER_MODE, RSFPL_VAL_GENERATOR_POWER_MODE_FIXED));
        break;
    case RSFPL_VAL_INTERNAL_GENERATOR_USAGE_POWER_SWEEP:
        checkErr(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_INTERNAL_GENERATOR_TRACKING_GENERATOR_STATE, VI_TRUE));
        checkErr(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_INTERNAL_GENERATOR_POWER_MODE, RSFPL_VAL_GENERATOR_POWER_MODE_SWEEP));
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, usage),
            2, "Usage");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures settings for using the internal generator as an independed
/// HIFN  CW source. See rsfpl_ConfigureInternalGeneratorUsage for usage
/// HIFN  settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR level/Defines the output power of the internal generator.
/// HIPAR offset/Defines a level offset for the external generator level. Thus, for
/// HIPAR offset/example, attenuators or amplifiers at the output of the external
/// HIPAR offset/generator can be taken into account for the setting.
/// HIPAR cwFrequency/Defines the frequency of the internal generator signal as an
/// HIPAR cwFrequency/independent CW source. The step size depends on the measurement mode.
ViStatus _VI_FUNC rsfpl_ConfigureInternalGeneratorIndependentCWSource(
                            ViSession instrSession,
                            ViReal64  level,
                            ViReal64  offset,
                            ViReal64  cwFrequency)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_INTERNAL_GENERATOR_LEVEL, level),
            2, "Level");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_INTERNAL_GENERATOR_LEVEL_OFFSET, offset),
            3, "Offset");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_INTERNAL_GENERATOR_CW_FREQUENCY, cwFrequency),
            4, "CW Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures settings for using the internal generator as a tracking
/// HIFN  generator. See rsfpl_ConfigureInternalGeneratorUsage for usage
/// HIFN  settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR level/Defines the output power of the internal generator.
/// HIPAR levelOffset/Defines a level offset for the external generator level. Thus, for
/// HIPAR levelOffset/example, attenuators or amplifiers at the output of the external
/// HIPAR levelOffset/generator can be taken into account for the setting.
ViStatus _VI_FUNC rsfpl_ConfigureInternalGeneratorTrackingGenerator(
                            ViSession instrSession,
                            ViReal64  level,
                            ViReal64  levelOffset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_INTERNAL_GENERATOR_LEVEL, level),
            2, "Level");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_INTERNAL_GENERATOR_LEVEL_OFFSET, levelOffset),
            3, "Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures settings for using the internal generator as a power sweep
/// HIFN  generator. See rsfpl_ConfigureInternalGeneratorUsage for usage
/// HIFN  settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR offset/Defines a level offset for the external generator level. Thus, for
/// HIPAR offset/example, attenuators or amplifiers at the output of the external
/// HIPAR offset/generator can be taken into account for the setting.
/// HIPAR psFrequency/Defines the frequency of the internal generator signal for a power
/// HIPAR psFrequency/sweep measurement. This frequency corresponds to the center frequency
/// HIPAR psFrequency/of the RF measurement.
/// HIPAR frequencyOffset/Defines an offset of the tracking generator signal to the analyzer
/// HIPAR frequencyOffset/frequency.
/// HIPAR levelStart/Defines the start power level of the internal generator signal for
/// HIPAR levelStart/power sweeps. This value must be lower than the stop level (see
/// HIPAR levelStart/RSFPL_ATTR_INTERNAL_GENERATOR_LEVEL_STOP).
/// HIPAR levelStop/Defines the end power level of the internal generator signal for power
/// HIPAR levelStop/sweeps. This value must be higher than the start level (see
/// HIPAR levelStop/RSFPL_ATTR_INTERNAL_GENERATOR_LEVEL_START).
ViStatus _VI_FUNC rsfpl_ConfigureInternalGeneratorPowerSweep(ViSession instrSession,
                                                             ViReal64  offset,
                                                             ViReal64  psFrequency,
                                                             ViReal64  frequencyOffset,
                                                             ViReal64  levelStart,
                                                             ViReal64  levelStop)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    // Stop must be > Start
    viCheckParm(RsCore_InvalidViReal64RangeMin(instrSession, levelStop, levelStart + DBL_EPSILON),
        6, "Level Stop");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_INTERNAL_GENERATOR_LEVEL_OFFSET, offset),
            2, "Offset");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_FREQUENCY_CENTER, psFrequency),
            3, "PS Frequency");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_INTERNAL_GENERATOR_FREQUENCY_OFFSET, frequencyOffset),
            4, "Frequency Offset");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_INTERNAL_GENERATOR_LEVEL_START, levelStart),
            5, "Level Start");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_INTERNAL_GENERATOR_LEVEL_STOP, levelStop),
            6, "Level Stop");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines an offset of the tracking generator signal to the analyzer
/// HIFN  frequency.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frequencyOffset/Defines an offset of the tracking generator signal to the analyzer
/// HIPAR frequencyOffset/frequency.
ViStatus _VI_FUNC rsfpl_ConfigureInternalGeneratorFrequencyOffset(
                            ViSession instrSession,
                            ViReal64  frequencyOffset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_INTERNAL_GENERATOR_FREQUENCY_OFFSET, frequencyOffset),
            2, "Frequency Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Initiates a reference measurement (calibration) using the transmission
/// HIFN  method. The reference measurement is the basis for the measurement
/// HIFN  normalization.
/// HIFN     To obtain a correct reference measurement, a complete sweep with
/// HIFN  synchronization to the end of the sweep must have been carried out.
/// HIFN  This is only possible in the single sweep mode.
/// HIFN     This command is only available if the Tracking Generator is active
/// HIFN  (see rsfpl_ConfigureInternalGeneratorState).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_InternalGeneratorCalibrateTransmission(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_INTERNAL_GENERATOR_CALIBRATION_METHOD, RSFPL_VAL_CALIBRATION_METHOD_TRANSMISSION));
    checkErr(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_HIDDEN_INTERNAL_GENERATOR_INITIATE_CALIBRATION, RSFPL_VAL_CALIBRATION_MODE_SHORT_OR_THROUGH));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Initiates a reference measurement (calibration) using the selected
/// HIFN  reflection method. The reference measurement is the basis for the
/// HIFN  measurement normalization.
/// HIFN     To obtain a correct reference measurement, a complete sweep with
/// HIFN  synchronization to the end of the sweep must have been carried out.
/// HIFN  This is only possible in the single sweep mode.
/// HIFN     This command is only available if the Tracking Generator is active
/// HIFN  (see rsfpl_ConfigureInternalGeneratorState).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR circuitType/Selects the circuit type for the calibration.
ViStatus _VI_FUNC rsfpl_InternalGeneratorCalibrateReflection(ViSession instrSession,
                                                             ViInt32   circuitType)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_INTERNAL_GENERATOR_CALIBRATION_METHOD, RSFPL_VAL_CALIBRATION_METHOD_REFLECTION));
    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_HIDDEN_INTERNAL_GENERATOR_INITIATE_CALIBRATION, circuitType),
            2, "Circuit Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Restores the measurement configuration used for calibration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_InternalGeneratorRecallCalibrationSettings(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_RECALL_CALIBRATION, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the calibration results save filepath and saves the calibration
/// HIFN  results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR saveFilePath/The calibration results save file path.
ViStatus _VI_FUNC rsfpl_InternalGeneratorSaveCalibrationSettings(
                            ViSession instrSession,
                            ViString  saveFilePath)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_SAVE_CALIBRATION, saveFilePath),
            2, "Save File Path");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the calibration results saved filepath.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR savedFilePath/Queries and the calibration results saved filepath.
ViStatus _VI_FUNC rsfpl_InternalGeneratorQueryCalibrationSaveFilePath(
                            ViSession instrSession,
                            ViInt32   bufferSize,
                            ViChar    savedFilePath[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViStringWithRealLen(instrSession, NULL,
        RSFPL_ATTR_CALIBRATION_SAVE_FILE_PATH,
        RS_VAL_DIRECT_USER_CALL, bufferSize, savedFilePath, NULL),
        2, "Saved File Path");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Uses the normalized measurement data to generate a transducer factor
/// HIFN  with up to 1001 points. The trace data is converted to a transducer
/// HIFN  with unit dB and stored in a file with the specified name and the
/// HIFN  suffix .trd under C:\ProgramData\Rohde-Schwarz\ZNL-FPL\trd. The
/// HIFN  frequency points are allocated in equidistant steps between start and
/// HIFN  stop frequency.
/// HIFN     The generated transducer factor can be further adapted using the
/// HIFN  commands described in Working with Transducers).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR name/Uses the normalized measurement data to generate a transducer factor
/// HIPAR name/with up to 1001 points. The trace data is converted to a transducer
/// HIPAR name/with unit dB and stored in a file with the specified name and the
/// HIPAR name/suffix .trd under C:\ProgramData\Rohde-Schwarz\ZNL-FPL\trd. The
/// HIPAR name/frequency points are allocated in equidistant steps between start and
/// HIPAR name/stop frequency.
ViStatus _VI_FUNC rsfpl_InternalGeneratorSaveAsTransducerFactor(
                            ViSession instrSession,
                            ViString  name)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_INTERNAL_GENERATOR_SAVE_AS_TRANSDUCER_FACTOR, name),
            2, "Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns the 28 V supply of the BNC connector labeled [noise source
/// HIFN  control] on the FPL on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR noiseSourceEnabled/Turns the 28 V supply of the BNC connector labeled [noise source
/// HIPAR noiseSourceEnabled/control] on the FPL on and off.
ViStatus _VI_FUNC rsfpl_ConfigureOutputNoiseSource(ViSession instrSession,
                                                   ViBoolean noiseSourceEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_OUTPUT_NOISE_SOURCE_ENABLED, noiseSourceEnabled),
            2, "Noise Source Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the type of signal available at the IF/VIDEO connector of the
/// HIFN  FPL.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR outputSource/Defines the type of signal available at the IF/VIDEO connector of the
/// HIPAR outputSource/FPL.
/// HIPAR frequency/Defines the frequency for the IF output of the FPL. The IF frequency
/// HIPAR frequency/of the signal is converted accordingly.
ViStatus _VI_FUNC rsfpl_ConfigureOutputSource(ViSession instrSession,
                                              ViInt32   outputSource,
                                              ViReal64  frequency)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_OUTPUT_SOURCE, outputSource),
            2, "Output Source");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_OUTPUT_FREQUENCY, frequency),
            3, "Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the control lines of the user ports for the AUX PORT
/// HIFN  connector. This 9-pole SUB-D male connector is located on the rear
/// HIFN  panel of the FPL. If enabled, the port is OUTPUT, if disabled, the port
/// HIFN  is INPUT.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR outputEnabled/Toggles the control lines of the user ports for the AUX PORT
/// HIPAR outputEnabled/connector. This 9-pole SUB-D male connector is located on the rear
/// HIPAR outputEnabled/panel of the FPL. If enabled, the port is OUTPUT, if disabled, the port
/// HIPAR outputEnabled/is INPUT.
/// HIPAR outputValue/Sets the control lines of the user ports.
ViStatus _VI_FUNC rsfpl_ConfigureOutputUserPortOutput(ViSession instrSession,
                                                      ViBoolean outputEnabled,
                                                      ViInt32   outputValue)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_OUTPUT_USER_PORT_OUTPUT_ENABLED, outputEnabled),
            2, "Output Enabled");
    if (outputEnabled)
    {
        viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_OUTPUT_USER_PORT_OUTPUT_VALUE, outputValue),
                3, " Output Value");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the signal polarity that indicates the trigger availability at
/// HIFN  the optional [AUX PORT] connector of the FPL.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR outputSignalPolarity/Defines the signal polarity that indicates the trigger availability at
/// HIPAR outputSignalPolarity/the optional [AUX PORT] connector of the FPL.
ViStatus _VI_FUNC rsfpl_ConfigureOutputSignalPolarity(ViSession instrSession,
                                                      ViInt32   outputSignalPolarity)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_OUTPUT_SIGNAL_POLARITY, outputSignalPolarity),
            2, "Output Signal Polarity");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the built-in loudspeaker for demodulated signals. This
/// HIFN  setting applies only to the current application.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Switches the built-in loudspeaker on or off for demodulated signals.
/// HIPAR enabled/This setting applies only to the current application.
/// HIPAR volume/Defines the volume of the built-in loudspeaker for demodulated
/// HIPAR volume/signals. This setting is maintained for all applications.
ViStatus _VI_FUNC rsfpl_ConfigureOutputLoudspeaker(ViSession instrSession,
                                                   ViBoolean enabled,
                                                   ViReal64  volume)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_OUTPUT_LOUDSPEAKER_ENABLED, enabled),
            2, "Enabled");
    if (enabled)
    {
        viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_OUTPUT_LOUDSPEAKER_VOLUME, volume),
                3, "Volume");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Temporarily disables the audio output via the built-in loudspeakers.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_OutputLoudspeakerMute(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_OUTPUT_LOUDSPEAKER_MUTE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the control lines of the user ports.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR userPortValue/Queries the control lines of the user ports.
ViStatus _VI_FUNC rsfpl_QueryUserPortValue(ViSession instrSession,
                                           ViInt32*  userPortValue)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_GetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_USER_PORT_VALUE, userPortValue),
            2, "User Port Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Initiates a measurement to determine and set the ideal settings for
/// HIFN  the current task automatically (only once for the current measurement).
/// HIFN     This includes:
/// HIFN     - Center frequency
/// HIFN     - Reference level
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_AdjustAll(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_ADJUST_ALL, NULL),
            2, "Adjust All");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the center frequency to the frequency with the highest signal
/// HIFN  level in the current frequency range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_AdjustFrequency(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_ADJUST_FREQUENCY, NULL),
            2, "Adjust Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Initiates a single (internal) measurement that evaluates and sets the
/// HIFN  ideal reference level for the current input data and measurement
/// HIFN  settings. This ensures that the settings of the RF attenuation and the
/// HIFN  reference level are optimally adjusted to the signal level without
/// HIFN  overloading the FPL or limiting the dynamic range by an S/N ratio that
/// HIFN  is too small.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_AdjustLevel(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_ADJUST_LEVEL, NULL),
            2, "Adjust Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  In order to determine the ideal reference level, the FPL performs a
/// HIFN  measurement on the current input data. Selects the way the FPL
/// HIFN  determines the length of the measurement .
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mode/In order to determine the ideal reference level, the FPL performs a
/// HIPAR mode/measurement on the current input data. Selects the way the FPL
/// HIPAR mode/determines the length of the measurement .
/// HIPAR manualLength/In order to determine the ideal reference level, the FPL performs a
/// HIPAR manualLength/measurement on the current input data. Defines the length of the
/// HIPAR manualLength/measurement if length of measurement mode is set to Manual.
ViStatus _VI_FUNC rsfpl_ConfigureAdjustLengthOfMeasurement(ViSession instrSession,
                                                           ViInt32   mode,
                                                           ViReal64  manualLength)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_ADJUST_LENGTH_OF_MEASUREMENT_MODE, mode),
            2, "Mode");
    if (mode == RSFPL_VAL_MANUAL)
    {
        viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_ADJUST_MANUAL_LENGTH_OF_MEASUREMENT, manualLength),
                3, "Manual Length");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  When the reference level is adjusted automatically, the internal
/// HIFN  attenuators and the preamplifier are also adjusted. In order to avoid
/// HIFN  frequent adaptation due to small changes in the input signal, you can
/// HIFN  define a hysteresis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR lower/This control defines a lower threshold the signal must fall below
/// HIPAR lower/(compared to the last measurement) before the reference level is
/// HIPAR lower/adapted automatically.
/// HIPAR upper/This control defines an upper threshold the signal must exceed
/// HIPAR upper/(compared to the last measurement) before the reference level is
/// HIPAR upper/adapted automatically.
ViStatus _VI_FUNC rsfpl_ConfigureAdjustLevelHysteresis(ViSession instrSession,
                                                       ViReal64  lower,
                                                       ViReal64  upper)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_ADJUST_LOWER_LEVEL_HYSTERESIS, lower),
            2, "Lower");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_ADJUST_UPPER_LEVEL_HYSTERESIS, upper),
            3, "Upper");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the behavior of the measurement when adjusting a setting
/// HIFN  automatically.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR waitForTriggerEnabled/Defines the behavior of the measurement when adjusting a setting
/// HIPAR waitForTriggerEnabled/automatically.
ViStatus _VI_FUNC rsfpl_ConfigureAdjustWaitForTriggerEnabled(ViSession instrSession,
                                                             ViBoolean waitForTriggerEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_ADJUST_WAIT_FOR_TRIGGER_ENABLED, waitForTriggerEnabled),
            2, "Wait For Trigger Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Matches the center frequency to the frequency of a marker.
/// HIFN     If you use the method in combination with a delta marker, that delta
/// HIFN  marker is turned into a normal marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR marker/Selects the marker.
ViStatus _VI_FUNC rsfpl_MarkerToCenterFrequency(ViSession instrSession,
                                                ViInt32   window,
                                                ViInt32   marker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_FREQUENCY_MARKER_TO_CENTER,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_FREQUENCY_MARKER_TO_CENTER,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap_marker),
            3, "Marker");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s",
        repCap_window, repCap_marker);

    viCheckParm(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_FREQUENCY_MARKER_TO_CENTER, NULL),
            3, "Frequency Marker To Center");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Restores the full span.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_ConfigureFrequencySpanFull(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_FREQUENCY_SPAN_FULL, NULL),
            2, "Frequency Span Full");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function sets the span to 0 Hz. The x-axis becomes the time axis
/// HIFN  with the grid lines corresponding to 1/10 of the current sweep time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_ConfigureFrequencyZeroSpan(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL,
        RSFPL_ATTR_FREQUENCY_SPAN, 0.0), 2, "Frequency Span");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configure step size.
/// HIFN     You can increase or decrease the center frequency quickly in fixed
/// HIFN  steps using the RSFPL_ATTR_FREQUENCY_CENTER_UP and
/// HIFN  RSFPL_ATTR_FREQUENCY_CENTER_DOWN.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frequencyStepSize/Defines the center frequency step size.
ViStatus _VI_FUNC rsfpl_ConfigureFrequencyStepSize(ViSession instrSession,
                                                   ViReal64  frequencyStepSize)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_FREQUENCY_STEP_SIZE_COUPLING_MODE, RSFPL_VAL_STEP_SIZE_COUPLING_OFF),
            1, "Step Size Coupling Mode");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_FREQUENCY_STEP_SIZE, frequencyStepSize),
            2, "Frequency Step Size");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures step size coupling.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR stepSizeCouplingMode/Couples and decouples the center frequency step size to the span or
/// HIPAR stepSizeCouplingMode/the resolution bandwidth.
/// HIPAR frequencyStepSizeFactor/Defines a step size factor if the center frequency step size is
/// HIPAR frequencyStepSizeFactor/coupled to the span or the resolution bandwidth.
ViStatus _VI_FUNC rsfpl_ConfigureFrequencyStepSizeCoupling(ViSession instrSession,
                                                           ViInt32   stepSizeCouplingMode,
                                                           ViReal64  frequencyStepSizeFactor)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_FREQUENCY_STEP_SIZE_COUPLING_MODE, stepSizeCouplingMode),
            2, "Step Size Coupling Mode");
    if (stepSizeCouplingMode == RSFPL_VAL_STEP_SIZE_COUPLING_SPAN)
    {
        viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_FREQUENCY_STEP_SIZE_FACTOR, frequencyStepSizeFactor),
                3, "Frequency Step Size Factor");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Matches the center frequency step size to the current marker
/// HIFN  frequency.
/// HIFN     Turns delta markers into normal markers.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR marker/Selects the marker.
ViStatus _VI_FUNC rsfpl_MarkerToStepSizeFrequency(ViSession instrSession,
                                                  ViInt32   window,
                                                  ViInt32   marker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_FREQUENCY_MARKER_TO_STEP_SIZE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_FREQUENCY_MARKER_TO_STEP_SIZE,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap_marker),
            3, "Marker");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s",
        repCap_window, repCap_marker);

    viCheckParm(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_FREQUENCY_MARKER_TO_STEP_SIZE, NULL),
            3, "Frequency Marker To Step Size");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Increases center frequency by step size, defined by
/// HIFN  RSFPL_ATTR_FREQUENCY_STEP_SIZE.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_FrequencyCenterUp(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_FREQUENCY_CENTER_UP, NULL),
            2, "Frequency Center Up");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Decreases center frequency by step size, defined by
/// HIFN  RSFPL_ATTR_FREQUENCY_STEP_SIZE.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_FrequencyCenterDown(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_FREQUENCY_CENTER_DOWN, NULL),
            2, "Frequency Center Down");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function toggles between linear and logarithmic display of the
/// HIFN  x-axis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR horizontalScale/Toggles between linear and logarithmic display of the x-axis.
ViStatus _VI_FUNC rsfpl_ConfigureHorizontalScale(ViSession instrSession,
                                                 ViInt32   window,
                                                 ViInt32   horizontalScale)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_HORIZONTAL_SCALE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_HORIZONTAL_SCALE, horizontalScale),
            3, "Horizontal Scale");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function toggles between linear and logarithmic display of the
/// HIFN  x-axis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR subwindow/Selects the subwindow.
/// HIPAR horizontalScale/Toggles between linear and logarithmic display of the x-axis.
ViStatus _VI_FUNC rsfpl_ConfigureSubwindowHorizontalScale(ViSession instrSession,
                                                          ViInt32   window,
                                                          ViInt32   subwindow,
                                                          ViInt32   horizontalScale)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_subwindow[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SUBWINDOW_HORIZONTAL_SCALE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SUBWINDOW_HORIZONTAL_SCALE,
            "Subwindow", subwindow , RS_REPCAP_BUF_SIZE, repCap_subwindow),
            3, "Subwindow");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap_window, repCap_subwindow);

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_HORIZONTAL_SCALE, horizontalScale),
            4, "Horizontal Scale");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures signal-tracking.
/// HIFN     With signal track activated, the maximum signal is determined after
/// HIFN  each frequency sweep and the center frequency is set to the frequency
/// HIFN  of this signal. Thus with drifting signals the center frequency follows
/// HIFN  the signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bandwidth/Defines the bandwidth around the center frequency within which the
/// HIPAR bandwidth/largest signal is searched.
/// HIPAR threshold/Defines the threshold level for the signal tracking process.
/// HIPAR trace/Defines the trace on which the largest signal is searched for in the
/// HIPAR trace/selected measurement window.
/// HIPAR signalTrackingEnabled/Turns signal tracking on and off.
ViStatus _VI_FUNC rsfpl_ConfigureSignalTracking(ViSession instrSession,
                                                ViBoolean signalTrackingEnabled,
                                                ViReal64  bandwidth,
                                                ViReal64  threshold,
                                                ViInt32   trace)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_SIGNAL_TRACKING_ENABLED, signalTrackingEnabled),
            2, "Signal Tracking Enabled");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_SIGNAL_TRACKING_BANDWIDTH, bandwidth),
            3, "Bandwidth");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_SIGNAL_TRACKING_THRESHOLD, threshold),
            4, "Threshold");

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_SIGNAL_TRACKING_TRACE, trace),
            5, "Trace");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Switches the labelling of the y-axis for frequency-based result
/// HIFN  diagrams. The frequency range itself is not changed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mode/Switches the labelling of the y-axis for frequency-based result
/// HIPAR mode/diagrams. The frequency range itself is not changed.
ViStatus _VI_FUNC rsfpl_ConfigureFrequencyAnnotationMode(ViSession instrSession,
                                                         ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_FREQUENCY_ANNOTATION_MODE, mode),
            2, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns the preamplifier on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR preamplifierEnabled/Turns the preamplifier on and off.
/// HIPAR gain/Selects the gain if the preamplifier is activated.
ViStatus _VI_FUNC rsfpl_ConfigurePreamplifier(ViSession instrSession,
                                              ViBoolean preamplifierEnabled,
                                              ViReal64  gain)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_PREAMPLIFIER_ENABLED, preamplifierEnabled),
            2, "Preamplifier Enabled");
    if (preamplifierEnabled)
    {
        viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_PREAMPLIFIER_GAIN, gain),
                4, "Gain");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the total attenuation for RF input.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR attenuationCouplingEnabled/Couples or decouples the attenuation to the reference level. Thus,
/// HIPAR attenuationCouplingEnabled/when the reference level is changed, the FPL determines the signal
/// HIPAR attenuationCouplingEnabled/level for optimal internal data processing and sets the required
/// HIPAR attenuationCouplingEnabled/attenuation accordingly.
/// HIPAR attenuation/Defines the total attenuation for RF input.
ViStatus _VI_FUNC rsfpl_ConfigureAttenuation(ViSession instrSession,
                                             ViBoolean attenuationCouplingEnabled,
                                             ViReal64  attenuation)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_ATTENUATION_COUPLING_ENABLED, attenuationCouplingEnabled),
            2, "Attenuation Coupling Enabled");
    if (!attenuationCouplingEnabled)
    {
        viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_ATTENUATION, attenuation),
                2, "Attenuation");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Matches the reference level to the power level of a marker.
/// HIFN     If you use the method in combination with a delta marker, that delta
/// HIFN  marker is turned into a normal marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR marker/Selects the marker.
ViStatus _VI_FUNC rsfpl_MarkerToReferenceLevel(ViSession instrSession,
                                               ViInt32   window,
                                               ViInt32   marker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, 16),
            3, "Marker");

    if (window != 0)
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:MARK%ld:FUNC:REF", window, marker);
    else
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:MARK%ld:FUNC:REF", marker);
    checkErr(rsfpl_WriteCommandWithOPCSync(instrSession, cmd, 0));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the unit of the y-axis.
/// HIFN     The unit applies to all power-based measurement windows with absolute
/// HIFN  values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR amplitudeUnit/Selects the unit of the y-axis.
ViStatus _VI_FUNC rsfpl_ConfigureAmplitudeUnit(ViSession instrSession,
                                               ViInt32   amplitudeUnit)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_AMPLITUDE_UNIT, amplitudeUnit),
            2, "Amplitude Unit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the reference level (for all traces in all windows).
/// HIFN     With a reference level offset other than 0, the value range of the
/// HIFN  reference level is modified by the offset.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR referenceLevel/Defines the reference level (for all traces in all windows).
ViStatus _VI_FUNC rsfpl_ConfigureReferenceLevel(ViSession instrSession,
                                                ViReal64  referenceLevel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_REFERENCE_LEVEL, referenceLevel),
            2, "Reference Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines a reference level offset (for all traces in all windows).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR referenceLevelOffset/Defines a reference level offset (for all traces in all windows).
ViStatus _VI_FUNC rsfpl_ConfigureReferenceLevelOffset(ViSession instrSession,
                                                      ViReal64  referenceLevelOffset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_REFERENCE_LEVEL_OFFSET, referenceLevelOffset),
            2, "Reference Level Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Automatic scaling of the y-axis is performed once, then switched off
/// HIFN  again (for all traces).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
ViStatus _VI_FUNC rsfpl_YAxisAutoscaleOnce(ViSession instrSession,
                                           ViInt32   window)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_Y_AXIS_AUTOSCALE_ONCE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");
    viCheckParm(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_Y_AXIS_AUTOSCALE_ONCE, NULL),
            3, "Y Axis Autoscale Once");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the display range of the y-axis (for all traces).
/// HIFN     Note that the command works only for a logarithmic scaling.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR subwindow/Selects the subwindow.
/// HIPAR range/Defines the display range of the y-axis (for all traces).
ViStatus _VI_FUNC rsfpl_ConfigureYAxisRange(ViSession instrSession,
                                            ViInt32   window,
                                            ViInt32   subwindow,
                                            ViReal64  range)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCapSw[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SUBWINDOW_Y_AXIS_RANGE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SUBWINDOW_Y_AXIS_RANGE,
            "Subwindow", subwindow , RS_REPCAP_BUF_SIZE, repCapSw),
            3, "Subwindow");
    strcat (repCap, ",");
    strcat (repCap, repCapSw);

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_SUBWINDOW_Y_AXIS_RANGE, range),
            4, "Y Axis Range");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Determines the grid spacing on the Y-axis for all diagrams, where
/// HIFN  possible. In spectrum displays, for example, this attribute is not
/// HIFN  available.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR subwindow/Selects the subwindow.
/// HIPAR gridSpacing/Determines the grid spacing on the Y-axis for all diagrams, where
/// HIPAR gridSpacing/possible. In spectrum displays, for example, this control is not
/// HIPAR gridSpacing/available.
ViStatus _VI_FUNC rsfpl_ConfigureYAxisGridSpacing(ViSession instrSession,
                                                  ViInt32   window,
                                                  ViInt32   subwindow,
                                                  ViReal64  gridSpacing)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCapSw[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SUBWINDOW_Y_AXIS_GRID_SPACING,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SUBWINDOW_Y_AXIS_GRID_SPACING,
            "Subwindow", subwindow , RS_REPCAP_BUF_SIZE, repCapSw),
            3, "Subwindow");
    strcat (repCap, ",");
    strcat (repCap, repCapSw);

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_SUBWINDOW_Y_AXIS_GRID_SPACING, gridSpacing),
            4, "Grid Spacing");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the reference value assigned to the reference position in the
/// HIFN  specified window. Separate reference values are maintained for the
/// HIFN  various displays.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR referenceValue/Defines the reference value assigned to the reference position in the
/// HIPAR referenceValue/specified window. Separate reference values are maintained for the
/// HIPAR referenceValue/various displays.
ViStatus _VI_FUNC rsfpl_ConfigureYAxisReferenceValue(ViSession instrSession,
                                                     ViInt32   window,
                                                     ViReal64  referenceValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_Y_AXIS_REFERENCE_VALUE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_Y_AXIS_REFERENCE_VALUE, referenceValue),
            3, "Reference Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the y-axis scaling for all traces.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR scaling/Selects the scaling of the y-axis for all traces.
/// HIPAR scalingMode/Selects the type of scaling of the y-axis (for all traces).
ViStatus _VI_FUNC rsfpl_ConfigureYAxisScaling(ViSession instrSession,
                                              ViInt32   window,
                                              ViInt32   scaling,
                                              ViInt32   scalingMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_Y_AXIS_SCALING_MODE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");
    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_Y_AXIS_SCALING, scaling),
            3, "Scaling");
    if (scaling != RSFPL_VAL_SCALING_LIN)
    {
        viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_Y_AXIS_SCALING_MODE, scalingMode),
                4, "Scaling Mode");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the y-axis scaling for all traces in a specified subwindow.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR subwindow/Selects the subwindow.
/// HIPAR scaling/Selects the scaling of the y-axis for all traces.
/// HIPAR scalingMode/Selects the type of scaling of the y-axis (for all traces).
ViStatus _VI_FUNC rsfpl_ConfigureSubwindowYAxisScaling(ViSession instrSession,
                                                       ViInt32   window,
                                                       ViInt32   subwindow,
                                                       ViInt32   scaling,
                                                       ViInt32   scalingMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_subwindow[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SUBWINDOW_Y_AXIS_SCALING,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SUBWINDOW_Y_AXIS_SCALING,
            "Subwindow", subwindow , RS_REPCAP_BUF_SIZE, repCap_subwindow),
            3, "Subwindow");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s",
        repCap_window, repCap_subwindow);

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_SUBWINDOW_Y_AXIS_SCALING, scaling),
            4, "Scaling");
    if (scaling != RSFPL_VAL_SCALING_LIN)
    {
        viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_SUBWINDOW_Y_AXIS_SCALING_MODE, scalingMode),
                5, "Scaling Mode");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the reference value assigned to the reference position in the
/// HIFN  specified window. Separate reference values are maintained for the
/// HIFN  various displays. Units and valid values are determined by the domain
/// HIFN  or spectrum.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR subwindow/Selects the subwindow.
/// HIPAR referenceValue/Defines the reference value assigned to the reference position in the
/// HIPAR referenceValue/specified window. Separate reference values are maintained for the
/// HIPAR referenceValue/various displays. Units and valid values are determined by the domain
/// HIPAR referenceValue/or spectrum.
ViStatus _VI_FUNC rsfpl_ConfigureSubwindowYAxisReferenceValue(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   subwindow,
                            ViReal64  referenceValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_subwindow[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SUBWINDOW_Y_AXIS_REFERENCE_VALUE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SUBWINDOW_Y_AXIS_REFERENCE_VALUE,
            "Subwindow", subwindow , RS_REPCAP_BUF_SIZE, repCap_subwindow),
            3, "Subwindow");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap_window, repCap_subwindow);

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_SUBWINDOW_Y_AXIS_REFERENCE_VALUE, referenceValue),
            4, "Reference Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the vertical position of the reference level on the display
/// HIFN  grid for all traces.
/// HIFN     The FPL adjusts the scaling of the y-axis accordingly.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR subwindow/Selects the subwindow.
/// HIPAR referenceLevelPosition/Defines the vertical position of the reference level on the display
/// HIPAR referenceLevelPosition/grid for all traces.
ViStatus _VI_FUNC rsfpl_ConfigureYAxisReferenceLevelPosition(ViSession instrSession,
                                                             ViInt32   window,
                                                             ViInt32   subwindow,
                                                             ViReal64  referenceLevelPosition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_subwindow[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SUBWINDOW_Y_AXIS_REFERENCE_LEVEL_POSITION,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SUBWINDOW_Y_AXIS_REFERENCE_LEVEL_POSITION,
            "Subwindow", subwindow , RS_REPCAP_BUF_SIZE, repCap_subwindow),
            3, "Subwindow");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap_window, repCap_subwindow);

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_SUBWINDOW_Y_AXIS_REFERENCE_LEVEL_POSITION, referenceLevelPosition),
            3, "Reference Level Position");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the resolution bandwidth and decouples the resolution
/// HIFN  bandwidth from the span.
/// HIFN     For statistics measurements, this attribute defines the demodulation
/// HIFN  bandwidth.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR rbw/Defines the resolution bandwidth and decouples the resolution
/// HIPAR rbw/bandwidth from the span.
ViStatus _VI_FUNC rsfpl_ConfigureRBW(ViSession instrSession,
                                     ViReal64  rbw)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_RBW, rbw),
            2, "RBW");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Couples and decouples the resolution bandwidth to the span.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR couplingEnabled/Couples and decouples the resolution bandwidth to the span.
/// HIPAR rbwToSpanRatio/Defines the ratio between the resolution bandwidth (Hz) and the span
/// HIPAR rbwToSpanRatio/(Hz).
ViStatus _VI_FUNC rsfpl_ConfigureRBWCoupling(ViSession instrSession,
                                             ViBoolean couplingEnabled,
                                             ViReal64  rbwToSpanRatio)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_RBW_AUTO_ENABLED, couplingEnabled),
            2, "Coupling Enabled");
    if (couplingEnabled)
    {
        viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_RBW_TO_SPAN_RATIO, rbwToSpanRatio),
                3, "RBW To Span Ratio");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the resolution filter type.
/// HIFN     When you change the filter type, the attribute selects the next larger
/// HIFN  filter bandwidth if the same bandwidth is unavailable for that filter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR rbwFilterType/Selects the resolution filter type.
ViStatus _VI_FUNC rsfpl_ConfigureRBWFilterType(ViSession instrSession,
                                               ViInt32   rbwFilterType)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_RBW_FILTER_TYPE, rbwFilterType),
            2, "RBW Filter Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the video bandwidth.
/// HIFN     Decouples the video bandwidth from the resolution bandwidths.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR vbw/Defines the video bandwidth.
ViStatus _VI_FUNC rsfpl_ConfigureVBW(ViSession instrSession,
                                     ViReal64  vbw)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_VBW, vbw),
            2, "VBW");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Couples and decouples the video bandwidth to the resolution bandwidth.
/// HIFN  This control has effect only if Coupling Enabled == On.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR couplingEnabled/Couples and decouples the video bandwidth to the resolution bandwidth.
/// HIPAR rbwToVBWRatio/Defines the coupling ratio of the resolution bandwidth to the video
/// HIPAR rbwToVBWRatio/bandwidth (RBW/VBW).
ViStatus _VI_FUNC rsfpl_ConfigureVBWCoupling(ViSession instrSession,
                                             ViBoolean couplingEnabled,
                                             ViInt32   rbwToVBWRatio)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_VBW_AUTO_ENABLED, couplingEnabled),
            2, "Coupling Enabled");
    if (couplingEnabled)
    {
        viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_RBW_TO_VBW_RATIO, rbwToVBWRatio),
                3, "RBW To VBW Ratio");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the position of the video filter in the signal path.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR vbwFilterMode/Selects the position of the video filter in the signal path.
ViStatus _VI_FUNC rsfpl_ConfigureVBWFilterMode(ViSession instrSession,
                                               ViInt32   vbwFilterMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_VBW_FILTER_MODE, vbwFilterMode),
            2, "VBW Filter Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Controls the sweep mode for an individual channel setup.
/// HIFN     Note that in single sweep mode, you can synchronize to the end of the
/// HIFN  measurement. In continuous sweep mode, synchronization to the end of
/// HIFN  the measurement is not possible. Thus, it is not recommended that you
/// HIFN  use continuous sweep mode in remote control, as results like trace data
/// HIFN  or markers are only valid after a single sweep end synchronization.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR sweepModeContinuousEnabled/Controls the sweep mode for an individual channel setup.
ViStatus _VI_FUNC rsfpl_ConfigureSweepModeContinuous(ViSession instrSession,
                                                     ViBoolean sweepModeContinuousEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_SWEEP_MODE_CONTINUOUS_ENABLED, sweepModeContinuousEnabled),
            2, "Sweep Mode Continuous Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the number of sweeps that the application uses to average
/// HIFN  traces.
/// HIFN     In case of continuous sweep mode, the application calculates the
/// HIFN  moving average over the average count.
/// HIFN     In case of single sweep mode, the application stops the measurement
/// HIFN  and calculates the average after the average count has been reached.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR sweepCount/Defines the number of sweeps that the application uses to average
/// HIPAR sweepCount/traces.
ViStatus _VI_FUNC rsfpl_ConfigureSweepCount(ViSession instrSession,
                                            ViInt32   sweepCount)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_SWEEP_COUNT, sweepCount),
            2, "Sweep Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the number of sweep points to analyze after a sweep.
/// HIFN     Note that the number of sweep points is limited to 10001 when
/// HIFN  measuring spurious emissions.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR sweepPoints/Defines the number of sweep points to analyze after a sweep.
ViStatus _VI_FUNC rsfpl_ConfigureSweepPoints(ViSession instrSession,
                                             ViInt32   sweepPoints)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_SWEEP_POINTS, sweepPoints),
            2, "Sweep Points");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the number of sweep points to analyze after a sweep.
/// HIFN     Note that the number of sweep points is limited to 10001 when
/// HIFN  measuring spurious emissions.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR sweepPoints/Defines the number of sweep points to analyze after a sweep.
ViStatus _VI_FUNC rsfpl_ConfigureWindowSweepPoints(ViSession instrSession,
                                                   ViInt32   window,
                                                   ViInt32   sweepPoints)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_WINDOW_SWEEP_POINTS,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_WINDOW_SWEEP_POINTS, sweepPoints),
            3, "Sweep Points");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  In FFT mode, several FFT analysis steps are required to cover the
/// HIFN  entire measurement span. The span which is covered by one FFT analysis
/// HIFN  step is called partial span. The partial span cannot be defined
/// HIFN  directly, but it can be optimized according to measurement
/// HIFN  requirements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR partialSpanOptimizationMode/In FFT mode, several FFT analysis steps are required to cover the
/// HIPAR partialSpanOptimizationMode/entire measurement span. The span which is covered by one FFT analysis
/// HIPAR partialSpanOptimizationMode/step is called partial span. The partial span cannot be defined
/// HIPAR partialSpanOptimizationMode/directly, but it can be optimized according to measurement
/// HIPAR partialSpanOptimizationMode/requirements.
ViStatus _VI_FUNC rsfpl_ConfigurePartialSpanOptimizationMode(ViSession instrSession,
                                                             ViInt32   partialSpanOptimizationMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_PARTIAL_SPAN_OPTIMIZATION_MODE, partialSpanOptimizationMode),
            2, "Partial Span Optimization Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the sweep time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR sweepTimeAutoEnabled/Couples and decouples the sweep time to the span and the resolution
/// HIPAR sweepTimeAutoEnabled/and video bandwidths.
/// HIPAR sweepTimeManual/Defines the sweep time. It automatically decouples the time from any
/// HIPAR sweepTimeManual/other settings.
ViStatus _VI_FUNC rsfpl_ConfigureSweepTime(ViSession instrSession,
                                           ViBoolean sweepTimeAutoEnabled,
                                           ViReal64  sweepTimeManual)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_SWEEP_TIME_AUTO_ENABLED, sweepTimeAutoEnabled),
            2, "Sweep Time Auto Enabled");
    if (!sweepTimeAutoEnabled)
    {
        viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_SWEEP_TIME, sweepTimeManual),
                2, "Sweep Time Manual");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This query returns the current number of started sweeps or
/// HIFN  measurements. Is only available if a sweep count value is defined and
/// HIFN  the instrument is in single sweep mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR currentNumberOfSweeps/This query returns the current number of started sweeps or
/// HIPAR currentNumberOfSweeps/measurements. Is only available if a sweep count value is defined and
/// HIPAR currentNumberOfSweeps/the instrument is in single sweep mode.
ViStatus _VI_FUNC rsfpl_QueryCurrentNumberOfSweeps(ViSession instrSession,
                                                   ViInt32*  currentNumberOfSweeps)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_GetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_CURRENT_NUMBER_OF_SWEEPS, currentNumberOfSweeps),
            2, "Current Number Of Sweeps");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Provides an estimation of the total time required to capture the data
/// HIFN  and process it. This time span may be considerably longer than the
/// HIFN  actual sweep time.
/// HIFN     Tip: To determine the necessary timeout for data capturing in a remote
/// HIFN  control program, double the estimated time and add 1 second.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR captureTime/Provides an estimation of the total time required to capture the data
/// HIPAR captureTime/and process it. This time span may be considerably longer than the
/// HIPAR captureTime/actual sweep time.
ViStatus _VI_FUNC rsfpl_QueryCaptureTime(ViSession instrSession,
                                         ViReal64* captureTime)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_GetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_CAPTURE_TIME, captureTime),
            2, "Capture Time");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the sweep type if you have turned on automatic selection of
/// HIFN  the sweep type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR sweepType/Queries the sweep type if you have turned on automatic selection of
/// HIPAR sweepType/the sweep type.
ViStatus _VI_FUNC rsfpl_QuerySweepType(ViSession instrSession,
                                       ViInt32*  sweepType)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_GetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_SWEEP_TYPE, sweepType),
            2, "Sweep Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the sweep type.
/// HIFN     The R&S FPL1000 automatically sets the optimal sweep type for the
/// HIFN  current measurement (sweep or FFT). The sweep type cannot be defined
/// HIFN  manually, except for EMI measurements. For measurements other than EMI,
/// HIFN  the sweep type is set to Auto
/// HIFN     automatically.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mode/Selects the sweep type.
ViStatus _VI_FUNC rsfpl_ConfigureSweepTypeMode(ViSession instrSession,
                                               ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_SWEEP_TYPE_MODE, mode),
            2, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the number of FFT subspans required to cover the entire
/// HIFN  measurement range (read-only).
/// HIFN     Only available in FFT sweep mode in the Spectrum application, and not
/// HIFN  for SEM, ACLR, or Spurious emissions measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fftSubspan/Returns the number of FFT subspans required to cover the entire
/// HIPAR fftSubspan/measurement range (read-only).
ViStatus _VI_FUNC rsfpl_QuerySweepFFTSubspan(ViSession instrSession,
                                             ViInt32*  fftSubspan)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_GetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_SWEEP_FFT_SUBSPAN, fftSubspan),
            2, "FFT Subspan");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the trigger source.
/// HIFN     Note on external triggers:
/// HIFN     If a measurement is configured to wait for an external trigger signal
/// HIFN  in a remote control program, remote control is blocked until the
/// HIFN  trigger is received and the program can continue. Make sure this
/// HIFN  situation is avoided in your remote control programs.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR triggerSource/Selects the trigger source.
ViStatus _VI_FUNC rsfpl_ConfigureTriggerSource(ViSession instrSession,
                                               ViInt32   triggerSource)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_TRIGGER_SOURCE, triggerSource),
            2, "Trigger Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the power level at the third intermediate frequency that must
/// HIFN  be exceeded to cause a trigger event.
/// HIFN     Note that any RF attenuation or preamplification is considered when
/// HIFN  the trigger level is analyzed. If defined, a reference level offset is
/// HIFN  also considered.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR triggerLevelIFP/Defines the power level at the third intermediate frequency that must
/// HIPAR triggerLevelIFP/be exceeded to cause a trigger event.
ViStatus _VI_FUNC rsfpl_ConfigureTriggerLevelIFP(ViSession instrSession,
                                                 ViReal64  triggerLevelIFP)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_TRIGGER_LEVEL_IFP, triggerLevelIFP),
            2, "Trigger Level IFP");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the magnitude the I/Q data must exceed to cause a trigger
/// HIFN  event.
/// HIFN     Note that any RF attenuation or preamplification is considered when
/// HIFN  the trigger level is analyzed. If defined, a reference level offset is
/// HIFN  also considered.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR triggerLevelIQPower/Defines the magnitude the I/Q data must exceed to cause a trigger
/// HIPAR triggerLevelIQPower/event.
ViStatus _VI_FUNC rsfpl_ConfigureTriggerLevelIQPower(ViSession instrSession,
                                                     ViReal64  triggerLevelIQPower)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_TRIGGER_LEVEL_IQ_POWER, triggerLevelIQPower),
            2, "Trigger Level IQ Power");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the time the input signal must stay below the trigger level
/// HIFN  before a trigger is detected again.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR triggerDropOutTime/Defines the time the input signal must stay below the trigger level
/// HIPAR triggerDropOutTime/before a trigger is detected again.
ViStatus _VI_FUNC rsfpl_ConfigureTriggerDropOutTime(ViSession instrSession,
                                                    ViReal64  triggerDropOutTime)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_TRIGGER_DROP_OUT_TIME, triggerDropOutTime),
            2, "Trigger Drop Out Time");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the time offset between the trigger event and the start of the
/// HIFN  sweep.
/// HIFN     A negative offset is possible for time domain measurements.
/// HIFN     For the trigger sources 'External' or 'IF Power', a common input
/// HIFN  signal is used for both trigger and gate. Therefore, changes to the
/// HIFN  gate delay will affect the trigger offset as well.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR triggerOffset/Defines the time offset between the trigger event and the start of the
/// HIPAR triggerOffset/sweep.
ViStatus _VI_FUNC rsfpl_ConfigureTriggerOffset(ViSession instrSession,
                                               ViReal64  triggerOffset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_TRIGGER_OFFSET, triggerOffset),
            2, "Trigger Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  For all trigger sources except time you can define whether triggering
/// HIFN  occurs when the signal rises to the trigger level or falls down to it.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR triggerSlope/For all trigger sources except time you can define whether triggering
/// HIPAR triggerSlope/occurs when the signal rises to the trigger level or falls down to it.
ViStatus _VI_FUNC rsfpl_ConfigureTriggerSlope(ViSession instrSession,
                                              ViInt32   triggerSlope)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_TRIGGER_SLOPE, triggerSlope),
            2, "Trigger Slope");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the trigger hysteresis, which is only available for 'IF Power'
/// HIFN  trigger sources.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR triggerHysteresis/Defines the trigger hysteresis, which is only available for 'IF Power'
/// HIPAR triggerHysteresis/trigger sources.
ViStatus _VI_FUNC rsfpl_ConfigureTriggerHysteresis(ViSession instrSession,
                                                   ViReal64  triggerHysteresis)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_TRIGGER_HYSTERESIS, triggerHysteresis),
            2, "Trigger Hysteresis");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the holding time before the next trigger event.
/// HIFN     Note that this attribute can be used for any trigger source, not just
/// HIFN  IF Power (despite the legacy keyword in SCPI command).
/// HIFN     Note: If you perform gated measurements in combination with the IF
/// HIFN  Power trigger, the instrument ignores the holding time for frequency
/// HIFN  sweep, FFT sweep, zero span and I/Q data measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR triggerHoldoff/Defines the holding time before the next trigger event.
ViStatus _VI_FUNC rsfpl_ConfigureTriggerHoldoff(ViSession instrSession,
                                                ViReal64  triggerHoldoff)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_TRIGGER_HOLDOFF, triggerHoldoff),
            2, "Trigger Holdoff");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the level the external signal must exceed to cause a trigger
/// HIFN  event.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR triggerPort/Selects the trigger port.
/// HIPAR externalTriggerLevel/Defines the level the external signal must exceed to cause a trigger
/// HIPAR externalTriggerLevel/event.
ViStatus _VI_FUNC rsfpl_ConfigureExternalTriggerLevel(ViSession instrSession,
                                                      ViInt32   triggerPort,
                                                      ViReal64  externalTriggerLevel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_EXTERNAL_TRIGGER_LEVEL,
            "TriggerPort", triggerPort , RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger Port");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_EXTERNAL_TRIGGER_LEVEL, externalTriggerLevel),
            3, "External Trigger Level");

Error:
    (void)RsCore_UnlockSession(instrSession);

    return error;
}

/// HIFN  Configures gated measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR gateEnabled/Turns gated measurements on and off.
/// HIPAR source/Selects the signal source for gated measurements.
/// HIPAR mode/Selects the way gated measurements are triggered. This control has
/// HIPAR mode/effect only if Gate Enabled == On.
/// HIPAR delay/Defines the delay time between the gate signal and the continuation of
/// HIPAR delay/the measurement.
/// HIPAR length/Defines the gate length. The gate length can only be set
/// HIPAR gatePolarity/Selects the polarity of an external gate signal.
ViStatus _VI_FUNC rsfpl_ConfigureGate(ViSession instrSession,
                                      ViBoolean gateEnabled,
                                      ViInt32   source,
                                      ViInt32   mode,
                                      ViReal64  delay,
                                      ViReal64  length,
                                      ViInt32   gatePolarity)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_GATE_ENABLED, gateEnabled),
            2, "Gate Enabled");

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_GATE_SOURCE, source),
            3, "Source");

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_GATE_MODE, mode),
            4, "Mode");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_GATE_DELAY, delay),
            5, "Delay");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_GATE_LENGTH, length),
            6, "Length");

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_GATE_POLARITY, gatePolarity),
            7, "Polarity");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/This control selects marker.
/// HIPAR markerEnabled/Turns markers on and off. If the corresponding marker number is
/// HIPAR markerEnabled/currently active as a delta marker, it is turned into a normal marker.
/// HIPAR trace/Selects the trace the marker is positioned on.
ViStatus _VI_FUNC rsfpl_ConfigureMarker(ViSession instrSession,
                                        ViInt32   window,
                                        ViInt32   marker,
                                        ViBoolean markerEnabled,
                                        ViInt32   trace)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_MARKER_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_MARKER_ENABLED,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap_marker),
            3, "Marker");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s",
        repCap_window, repCap_marker);

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_MARKER_ENABLED, markerEnabled),
            4, "Marker Enabled");

    if (markerEnabled)
    {
        viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_MARKER_ASSIGNED_TRACE, trace),
                5, "Trace");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the horizontal position and the marker amplitude
/// HIFN  level of the selected marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/This control selects marker.
/// HIPAR markerPosition/Queries marker position on the x-axis.
/// HIPAR markerAmplitude/Queries the position of a marker on the y-axis.
ViStatus _VI_FUNC rsfpl_QueryMarker(ViSession instrSession,
                                    ViInt32   window,
                                    ViInt32   marker,
                                    ViReal64* markerPosition,
                                    ViReal64* markerAmplitude)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_MARKER_POSITION,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_MARKER_POSITION,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap_marker),
            3, "Marker");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s",
        repCap_window, repCap_marker);

    viCheckParm(rsfpl_GetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_MARKER_POSITION, markerPosition),
            4, "Marker Position");

    viCheckParm(rsfpl_GetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_MARKER_RESULT, markerAmplitude),
            5, "Marker Amplitude");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns off all markers.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_AllMarkersOff(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_ALL_MARKERS_OFF, NULL),
            2, "All Markers Off");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configure general settings of all markers in all windows.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR markerTableDisplayMode/Turns the marker table on and off.
/// HIPAR markerInfoEnabled/Turns the marker information in all diagrams on and off.
/// HIPAR markerStepsize/Selects the marker step size mode for all markers in all windows.
ViStatus _VI_FUNC rsfpl_ConfigureMarkerGeneralSettings(ViSession instrSession,
                                                       ViInt32   markerTableDisplayMode,
                                                       ViBoolean markerInfoEnabled,
                                                       ViInt32   markerStepsize)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_MARKER_TABLE_DISPLAY_MODE, markerTableDisplayMode),
            2, "Marker Table Display Mode");

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_MARKER_INFO_ENABLED, markerInfoEnabled),
            3, "Marker Info Enabled");

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_MARKER_STEPSIZE, markerStepsize),
            4, "Marker Stepsize");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Links the specified marker in all displays of the specified type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR markerLinking/Links the specified marker in all displays of the specified type.
ViStatus _VI_FUNC rsfpl_ConfigureMarkerLinking(ViSession instrSession,
                                               ViInt32   markerLinking)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_MARKER_LINKING, markerLinking),
            2, "Marker Linking");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a marker to a specific coordinate on the x-axis.
/// HIFN     If necessary, the command activates the marker.
/// HIFN     If the marker has been used as a delta marker, the command turns it
/// HIFN  into a normal marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/This control selects marker.
/// HIPAR markerPosition/Moves a marker to a specific coordinate on the x-axis.
ViStatus _VI_FUNC rsfpl_ConfigureMarkerPosition(ViSession instrSession,
                                                ViInt32   window,
                                                ViInt32   marker,
                                                ViReal64  markerPosition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_MARKER_POSITION,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_MARKER_POSITION,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap_marker),
            3, "Marker");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s",
        repCap_window, repCap_marker);

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_MARKER_POSITION, markerPosition),
            4, "Marker Position");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Links normal marker <m1> to any active normal marker <m2>.
/// HIFN     If you change the horizontal position of marker <m2>, marker <m1>
/// HIFN  changes its horizontal position to the same value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR firstMarker/This control selects first marker.
/// HIPAR secondMarker/This control selects second marker.
/// HIPAR linkEnabled/Link enabled/disabled.
ViStatus _VI_FUNC rsfpl_LinkToAnotherMarker(ViSession instrSession,
                                            ViInt32   window,
                                            ViInt32   firstMarker,
                                            ViInt32   secondMarker,
                                            ViBoolean linkEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, firstMarker, 1, 16),
            3, "First Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, secondMarker, 1, 16),
            4, "Second Marker");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:MARK%ld:LINK:TO:MARK%ld %s", window, firstMarker, secondMarker, linkEnabled ? "ON" : "OFF");
    checkErr(rsfpl_WriteCommandWithOPCSync(instrSession, cmd, 0));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Links the normal source marker <ms> to any active delta destination
/// HIFN  marker <md>.
/// HIFN     If you change the horizontal position of marker <md>, marker <ms>
/// HIFN  changes its horizontal position to the same value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR firstMarker/This control selects first marker.
/// HIPAR secondMarker/This control selects second marker.
/// HIPAR linkEnabled/Link enabled/disabled.
ViStatus _VI_FUNC rsfpl_LinkToAnotherDeltaMarker(ViSession instrSession,
                                                 ViInt32   window,
                                                 ViInt32   firstMarker,
                                                 ViInt32   secondMarker,
                                                 ViBoolean linkEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, firstMarker, 1, 16),
            3, "First Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, secondMarker, 1, 16),
            4, "Second Marker");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:MARK%ld:LINK:TO:DELT%ld %s", window, firstMarker, secondMarker, linkEnabled ? "ON" : "OFF");
    checkErr(rsfpl_WriteCommandWithOPCSync(instrSession, cmd, 0));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns the suppression of the local oscillator during automatic marker
/// HIFN  positioning on and off (for all markers in all windows).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR excludeLOEnabled/Turns the suppression of the local oscillator during automatic marker
/// HIPAR excludeLOEnabled/positioning on and off (for all markers in all windows).
ViStatus _VI_FUNC rsfpl_ConfigureMarkerExcludeLOEnabled(ViSession instrSession,
                                                        ViBoolean excludeLOEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_MARKER_EXCLUDE_LO_ENABLED, excludeLOEnabled),
            2, "Exclude LO Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the peak excursion (for all markers in all windows).
/// HIFN     The peak excursion sets the requirements for a peak to be detected
/// HIFN  during a peak search.
/// HIFN     The unit depends on the measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR peakExcursion/Defines the peak excursion (for all markers in all windows).
ViStatus _VI_FUNC rsfpl_ConfigureMarkerPeakExcursion(ViSession instrSession,
                                                     ViReal64  peakExcursion)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_MARKER_PEAK_EXCURSION, peakExcursion),
            2, "Peak Excursion");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns marker search limits on and off for all markers in all windows.
/// HIFN     If you perform a measurement in the time domain, this attribute limits
/// HIFN  the range of the trace to be analyzed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchLimitsEnabled/Turns marker search limits on and off for all markers in all windows.
/// HIPAR searchLimitLeft/Defines the left limit of the marker search range for all markers in
/// HIPAR searchLimitLeft/all windows.
/// HIPAR searchLimitRight/Defines the right limit of the marker search range for all markers in
/// HIPAR searchLimitRight/all windows.
ViStatus _VI_FUNC rsfpl_ConfigureMarkerSearchLimits(ViSession instrSession,
                                                    ViBoolean searchLimitsEnabled,
                                                    ViReal64  searchLimitLeft,
                                                    ViReal64  searchLimitRight)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_MARKER_SEARCH_LIMITS_ENABLED, searchLimitsEnabled),
            2, "Search Limits Enabled");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_MARKER_SEARCH_LIMIT_LEFT, searchLimitLeft),
            3, "Search Limit Left");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_MARKER_SEARCH_LIMIT_RIGHT, searchLimitRight),
            4, "Search Limit Right");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Adjusts the marker search range to the zoom area for all markers in
/// HIFN  all windows.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchZoomLimitsEnabled/Adjusts the marker search range to the zoom area for all markers in
/// HIPAR searchZoomLimitsEnabled/all windows.
ViStatus _VI_FUNC rsfpl_ConfigureMarkerSearchZoomLimitsEnabled(
                            ViSession instrSession,
                            ViBoolean searchZoomLimitsEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_MARKER_SEARCH_ZOOM_LIMITS_ENABLED, searchZoomLimitsEnabled),
            2, "Search Zoom Limits Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Turns a threshold for the marker peak search on and off (for all
/// HIFN  markers in all windows).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR thresholdEnabled/Turns a threshold for the marker peak search on and off (for all
/// HIPAR thresholdEnabled/markers in all windows).
/// HIPAR thresholdValue/Defines a threshold level for the marker peak search (for all markers
/// HIPAR thresholdValue/in all windows).  This control has effect only if Threshold Enabled ==
/// HIPAR thresholdValue/On.
ViStatus _VI_FUNC rsfpl_ConfigureMarkerSearchThreshold(ViSession instrSession,
                                                       ViBoolean thresholdEnabled,
                                                       ViReal64  thresholdValue)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_MARKER_SEARCH_THRESHOLD_ENABLED, thresholdEnabled),
            2, "Threshold Enabled");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_MARKER_SEARCH_THRESHOLD, thresholdValue),
            3, "Threshold Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function deactivates the search range limits.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_DeactivateAllSearchLimits(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_MARKER_SEARCH_LIMITS_ENABLED, VI_FALSE),
            2, "Deactivate Search Limits");

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_MARKER_SEARCH_THRESHOLD_ENABLED, VI_FALSE),
            3, "Deactivate Search Threshold");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns an automatic marker peak search for a trace minimum and maximum
/// HIFN  on and off. The FPL performs the peak search after each sweep.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR autoMaxPeakSearchEnabled/Turns an automatic marker peak search for a trace maximum on and off.
/// HIPAR autoMaxPeakSearchEnabled/The FPL performs the peak search after each sweep.
/// HIPAR autoMinPeakSearchEnabled/Turns an automatic marker peak search for a trace minimum on and off.
/// HIPAR autoMinPeakSearchEnabled/The FPL performs the peak search after each sweep.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/This control selects marker.
ViStatus _VI_FUNC rsfpl_ConfigureMarkerAutoPeakSearch(ViSession instrSession,
                                                      ViInt32   window,
                                                      ViInt32   marker,
                                                      ViBoolean autoMaxPeakSearchEnabled,
                                                      ViBoolean autoMinPeakSearchEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_MARKER_AUTO_MAX_PEAK_SEARCH_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_MARKER_AUTO_MAX_PEAK_SEARCH_ENABLED,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap_marker),
            3, "Marker");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s",
        repCap_window, repCap_marker);

    if (autoMinPeakSearchEnabled)
    {
        viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_MARKER_AUTO_MIN_PEAK_SEARCH_ENABLED, autoMinPeakSearchEnabled),
                4, "Auto Min Peak Search Enabled");
    }
    else
    {
        viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_MARKER_AUTO_MAX_PEAK_SEARCH_ENABLED, autoMaxPeakSearchEnabled),
                5, "Auto Max Peak Search Enabled");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function specifies the type of marker search and performs the
/// HIFN  search.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/This control selects marker.
/// HIPAR markerSearch/Specifies the type of marker search type.
ViStatus _VI_FUNC rsfpl_MarkerSearch(ViSession instrSession,
                                     ViInt32   window,
                                     ViInt32   marker,
                                     ViInt32   markerSearch)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_MARKER_MAX_PEAK,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_MARKER_MAX_PEAK,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap_marker),
            3, "Marker");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s",
        repCap_window, repCap_marker);

    switch (markerSearch)
    {
    case RSFPL_VAL_MARKER_SEARCH_HIGHEST:
        viCheckParm(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_MARKER_MAX_PEAK, NULL),
                4, "Marker Search");
        break;
    case RSFPL_VAL_MARKER_SEARCH_MINIMUM:
        viCheckParm(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_MARKER_MIN_PEAK, NULL),
                4, "Marker Search");
        break;
    case RSFPL_VAL_MARKER_SEARCH_NEXT_PEAK:
        viCheckParm(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_MARKER_MAX_PEAK_NEXT, NULL),
                4, "Marker Search");
        break;
    case RSFPL_VAL_MARKER_SEARCH_NEXT_PEAK_LEFT:
        viCheckParm(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_MARKER_MAX_PEAK_LEFT, NULL),
                4, "Marker Search");
        break;
    case RSFPL_VAL_MARKER_SEARCH_NEXT_PEAK_RIGHT:
        viCheckParm(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_MARKER_MAX_PEAK_RIGHT, NULL),
                4, "Marker Search");
        break;
    case RSFPL_VAL_MARKER_SEARCH_NEXT_MINIMUM:
        viCheckParm(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_MARKER_MIN_PEAK_NEXT, NULL),
                4, "Marker Search");
        break;
    case RSFPL_VAL_MARKER_SEARCH_NEXT_MINIMUM_LEFT:
        viCheckParm(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_MARKER_MIN_PEAK_LEFT, NULL),
                4, "Marker Search");
        break;
    case RSFPL_VAL_MARKER_SEARCH_NEXT_MINIMUM_RIGHT:
        viCheckParm(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_MARKER_MIN_PEAK_RIGHT, NULL),
                4, "Marker Search");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, markerSearch), 4, "Marker Search");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the position of a marker on the y-axis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/This control selects marker.
/// HIPAR markerAmplitude/Configures the position of a marker on the y-axis.
ViStatus _VI_FUNC rsfpl_QueryMarkerAmplitude(ViSession instrSession,
                                             ViInt32   window,
                                             ViInt32   marker,
                                             ViReal64* amplitude)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_MARKER_AMPLITUDE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_MARKER_AMPLITUDE,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap_marker),
            3, "Marker");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s",
        repCap_window, repCap_marker);

    checkErr(rsfpl_GetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_MARKER_AMPLITUDE, amplitude));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This attribute sets a marker to a particular probability value. You
/// HIFN  can query the corresponding level with RSFPL_ATTR_MARKER_POSITION.
/// HIFN  Using the attribute turns delta markers into normal markers.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/This control selects marker.
/// HIPAR markerProbability/This control sets a marker to a particular probability value. You can
/// HIPAR markerProbability/query the corresponding level with RSFPL_ATTR_MARKER_POSITION. Using
/// HIPAR markerProbability/the control turns delta markers into normal markers.
ViStatus _VI_FUNC rsfpl_ConfigureMarkerProbability(ViSession instrSession,
                                                   ViInt32   window,
                                                   ViInt32   marker,
                                                   ViReal64  markerProbability)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_MARKER_PROBABILITY,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_MARKER_PROBABILITY,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap_marker),
            3, "Marker");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s",
        repCap_window, repCap_marker);

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_MARKER_PROBABILITY, markerProbability),
            3, "Marker Probability");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the factor or magnitude of the marker zoom. Marker zoom is
/// HIFN  enabled or disabled with RSFPL_ATTR_DISPLAY_SINGLE_ZOOM_ENABLED.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/This control selects marker.
/// HIPAR markerZoomFactor/Defines the factor or magnitude of the marker zoom. Marker zoom is
/// HIPAR markerZoomFactor/enabled or disabled with RSFPL_ATTR_DISPLAY_SINGLE_ZOOM_ENABLED.
ViStatus _VI_FUNC rsfpl_ConfigureMarkerZoomFactor(ViSession instrSession,
                                                  ViInt32   window,
                                                  ViInt32   marker,
                                                  ViReal64  markerZoomFactor)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_MARKER_ZOOM_FACTOR,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_MARKER_ZOOM_FACTOR,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap_marker),
            3, "Marker");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap_window, repCap_marker);

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_MARKER_ZOOM_FACTOR, markerZoomFactor),
            4, "Marker Zoom Factor");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the marker frequency counter resolution and turns
/// HIFN  the marker frequency counter on/off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/This control selects marker.
/// HIPAR signalCount/Turns the frequency counter at the marker position on and off.
/// HIPAR resolution/Specifies the frequency counter resolution. This control has effect
/// HIPAR resolution/only if Frequency Counter Enabled == On.
ViStatus _VI_FUNC rsfpl_ConfigureMarkerFrequencyCounter(ViSession instrSession,
                                                        ViInt32   window,
                                                        ViInt32   marker,
                                                        ViBoolean signalCount,
                                                        ViReal64  resolution)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SIGNAL_COUNT_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SIGNAL_COUNT_ENABLED,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap_marker),
            3, "Marker");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s",
        repCap_window, repCap_marker);

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_SIGNAL_COUNT_ENABLED, signalCount),
            4, "Signal Count");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_SIGNAL_COUNT_RESOLUTION, resolution),
            5, "Resolution");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the result of the frequency counter for the
/// HIFN  indicated marker. Before the command, the frequency counter should be
/// HIFN  switched on and a complete measurement performed to obtain a valid
/// HIFN  count result. Therefore, a single sweep with synchronization must be
/// HIFN  performed between switching on the frequency counter and querying the
/// HIFN  count result.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/This control selects marker.
/// HIPAR window/This control selects the measurement window.
/// HIPAR counterFrequency/Returns the result of the frequency counter for the indicated marker.
ViStatus _VI_FUNC rsfpl_QueryFrequencyCounterResult(ViSession instrSession,
                                                    ViInt32   window,
                                                    ViInt32   marker,
                                                    ViReal64* counterFrequency)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SIGNAL_COUNT_FREQUENCY,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SIGNAL_COUNT_FREQUENCY,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap_marker),
            3, "Marker");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s",
        repCap_window, repCap_marker);

    viCheckParm(rsfpl_GetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_SIGNAL_COUNT_FREQUENCY, counterFrequency),
            4, "Counter Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns a peak search on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/This control selects marker.
/// HIPAR window/This control selects the measurement window.
/// HIPAR peakListEnabled/Turns a peak search on and off.
ViStatus _VI_FUNC rsfpl_ConfigureMarkerPeakListEnabled(ViSession instrSession,
                                                       ViInt32   window,
                                                       ViInt32   marker,
                                                       ViBoolean peakListEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_MARKER_PEAK_SEARCH_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_MARKER_PEAK_SEARCH_ENABLED,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap_marker),
            3, "Marker");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap_window, repCap_marker);

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_MARKER_PEAK_SEARCH_ENABLED, peakListEnabled),
            4, "Peak List Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures parameters of marker peak list function.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/This control selects marker.
/// HIPAR window/This control selects the measurement window.
/// HIPAR sortMode/Selects the order in which the results of a peak search are returned.
/// HIPAR maximumNumberOfPeaks/Defines the maximum number of peaks that the FPL looks for during a
/// HIPAR maximumNumberOfPeaks/peak search.
/// HIPAR labelsEnabled/Turns labels for peaks found during a peak search on and off.
ViStatus _VI_FUNC rsfpl_ConfigureMarkerPeakList(ViSession instrSession,
                                                ViInt32   window,
                                                ViInt32   marker,
                                                ViInt32   sortMode,
                                                ViInt32   maximumNumberOfPeaks,
                                                ViBoolean labelsEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_MARKER_PEAK_LIST_SORT_MODE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_MARKER_PEAK_LIST_SORT_MODE,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap_marker),
            3, "Marker");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s",
        repCap_window, repCap_marker);

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_MARKER_PEAK_LIST_SORT_MODE, sortMode),
            4, "Sort Mode");

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_MARKER_PEAK_LIST_MAXIMUM_NUMBER_OF_PEAKS, maximumNumberOfPeaks),
            5, "Maximum Number Of Peaks");

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_MARKER_PEAK_LIST_LABELS_ENABLED, labelsEnabled),
            6, "Labels Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Initiates a peak search. Defines the number of peaks to find during
/// HIFN  the search.
/// HIFN     Note that the actual number of peaks found during the search also
/// HIFN  depends on the peak excursion.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR numberOfPeaks/Initiates a peak search. Defines the number of peaks to find during
/// HIPAR numberOfPeaks/the search.
/// HIPAR marker/This control selects marker.
/// HIPAR window/This control selects the measurement window.
ViStatus _VI_FUNC rsfpl_MarkerPeakListSearchPeaks(ViSession instrSession,
                                                  ViInt32   window,
                                                  ViInt32   marker,
                                                  ViInt32   numberOfPeaks)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_MARKER_PEAK_LIST_PEAKS,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_MARKER_PEAK_LIST_PEAKS,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap_marker),
            3, "Marker");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s",
        repCap_window, repCap_marker);

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_MARKER_PEAK_LIST_PEAKS, numberOfPeaks),
            4, "Number Of Peaks");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the number of peaks that have been found during a peak search.
/// HIFN     The actual number of peaks that have been found may differ from the
/// HIFN  number of peaks you have set to be found because of the peak excursion.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR numberOfFoundPeaks/Queries the number of peaks that have been found during a peak search.
/// HIPAR marker/This control selects marker.
/// HIPAR window/This control selects the measurement window.
ViStatus _VI_FUNC rsfpl_QueryMarkerPeakListNumberOfFoundPeaks(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   marker,
                            ViInt32*  numberOfFoundPeaks)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_MARKER_PEAK_LIST_NUMBER_OF_FOUND_PEAKS,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_MARKER_PEAK_LIST_NUMBER_OF_FOUND_PEAKS,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap_marker),
            3, "Marker");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s",
        repCap_window, repCap_marker);

    viCheckParm(rsfpl_GetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_MARKER_PEAK_LIST_NUMBER_OF_FOUND_PEAKS, numberOfFoundPeaks),
            4, "Number Of Found Peaks");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the position of the peaks on the x-axis.
/// HIFN     The order depends on the sort order that has been set.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/This control selects marker.
/// HIPAR arraySize/This control defines the size of peak list array. It should correspond
/// HIPAR arraySize/with requested peak list count.
/// HIPAR window/This control selects the measurement window.
/// HIPAR peakList/This control returns the list of X values of the maxima found.
ViStatus _VI_FUNC rsfpl_QueryMarkerPeakListXPositions(ViSession instrSession,
                                                      ViInt32   window,
                                                      ViInt32   marker,
                                                      ViUInt32  arraySize,
                                                      ViReal64  peakList[],
                                                      ViUInt32* numberOfResults)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, 16),
            3, "Marker");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:MARK%ld:FUNC:FPE:X?", window, marker);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, peakList, (ViInt32*)numberOfResults));
    checkErr(rsfpl_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the position of the peaks on the y-axis.
/// HIFN     The order depends on the sort order that has been set
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR arraySize/This control defines the size of peak list array. It should correspond
/// HIPAR arraySize/with requested peak list count.
/// HIPAR peakList/This control returns the list of Y values of the maxima found.
/// HIPAR marker/This control selects marker.
/// HIPAR window/This control selects the measurement window.
ViStatus _VI_FUNC rsfpl_QueryMarkerPeakListYPositions(ViSession instrSession,
                                                      ViInt32   window,
                                                      ViInt32   marker,
                                                      ViUInt32  arraySize,
                                                      ViReal64  peakList[],
                                                      ViUInt32* numberOfResults)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, 16),
            3, "Marker");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:MARK%ld:FUNC:FPE:Y?", window, marker);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, peakList, (ViInt32*)numberOfResults));
    checkErr(rsfpl_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns the noise measurement at the marker position on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/This control selects marker.
/// HIPAR window/This control selects the measurement window.
/// HIPAR measurementMarkerEnabled/Turns the noise measurement at the marker position on and off.
ViStatus _VI_FUNC rsfpl_ConfigureNoiseMeasurementMarker(ViSession instrSession,
                                                        ViInt32   window,
                                                        ViInt32   marker,
                                                        ViBoolean noiseMeasurementEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_NOISE_MEASUREMENT_MARKER_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_NOISE_MEASUREMENT_MARKER_ENABLED,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap_marker),
            3, "Marker");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s",
        repCap_window, repCap_marker);

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_NOISE_MEASUREMENT_MARKER_ENABLED, noiseMeasurementEnabled),
            4, "Noise Measurement Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Removes all noise markers in the specified window.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
ViStatus _VI_FUNC rsfpl_NoiseMeasurementMarkerAllOff(ViSession instrSession,
                                                     ViInt32   window)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_NOISE_MEASUREMENT_MARKER_ALL_OFF,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_NOISE_MEASUREMENT_MARKER_ALL_OFF,
            "Marker", 0 , RS_REPCAP_BUF_SIZE, repCap_marker));
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s",
        repCap_window, repCap_marker);

    viCheckParm(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_NOISE_MEASUREMENT_MARKER_ALL_OFF, NULL),
            2, "Window");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the result of the noise measurement.
/// HIFN     To get a valid result, you have to perform a complete measurement with
/// HIFN  synchronization to the end of the measurement before reading out the
/// HIFN  result. This is only possible for single sweep mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/This control selects marker.
/// HIPAR window/This control selects the measurement window.
/// HIPAR measurementMarkerResult/Queries the result of the noise measurement.
ViStatus _VI_FUNC rsfpl_QueryNoiseMeasurementMarkerResult(ViSession instrSession,
                                                          ViInt32   marker,
                                                          ViInt32   window,
                                                          ViReal64* measurementResult)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_NOISE_MEASUREMENT_MARKER_RESULT,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_NOISE_MEASUREMENT_MARKER_RESULT,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap_marker),
            3, "Marker");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s",
        repCap_window, repCap_marker);

    viCheckParm(rsfpl_GetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_NOISE_MEASUREMENT_MARKER_RESULT, measurementResult),
            4, "Measurement Marker Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns the phase noise measurement at the marker position on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/This control selects marker.
/// HIPAR phaseNoiseMarkerEnabled/Turns the phase noise measurement at the marker position on and off.
ViStatus _VI_FUNC rsfpl_ConfigurePhaseNoiseMeasurementMarker(ViSession instrSession,
                                                             ViInt32   window,
                                                             ViInt32   marker,
                                                             ViBoolean phaseNoiseMarkerEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_PHASE_NOISE_MEASUREMENT_MARKER_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_PHASE_NOISE_MEASUREMENT_MARKER_ENABLED,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap_marker),
            3, "Marker");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s",
        repCap_window, repCap_marker);

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_PHASE_NOISE_MEASUREMENT_MARKER_ENABLED, phaseNoiseMarkerEnabled),
            4, "Phase Noise Marker Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Removes all phase noise markers in the specified window.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_PhaseNoiseMeasurementMarkerAllOff(ViSession instrSession,
                                                          ViInt32   window)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_PHASE_NOISE_MEASUREMENT_MARKER_ALL_OFF,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");
    viCheckParm(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_PHASE_NOISE_MEASUREMENT_MARKER_ALL_OFF, NULL),
            2, "Phase Noise Measurement Marker All Off");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the result of a phase noise measurement.
/// HIFN     If necessary, the attribute activates the measurement first.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR phaseNoiseResult/Queries the result of a phase noise measurement.
/// HIPAR marker/This control selects marker.
/// HIPAR window/This control selects the measurement window.
ViStatus _VI_FUNC rsfpl_QueryPhaseNoiseMeasurementMarkerResult(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   marker,
                            ViReal64* phaseNoiseResult)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_PHASE_NOISE_MEASUREMENT_MARKER_RESULT,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_PHASE_NOISE_MEASUREMENT_MARKER_RESULT,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap_marker),
            3, "Marker");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s",
        repCap_window, repCap_marker);

    viCheckParm(rsfpl_GetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_PHASE_NOISE_MEASUREMENT_MARKER_RESULT, phaseNoiseResult),
            4, "Phase Noise Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns markers for band power measurements on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/This control selects marker.
/// HIPAR window/This control selects the measurement window.
/// HIPAR bandPowerMarkerEnabled/Turns markers for band power measurements on and off.
/// HIPAR mode/Selects the way the results for a band power marker are displayed.
/// HIPAR mode/This control has effect only if Band Power Marker Enabled == On.
/// HIPAR span/Defines the bandwidth around the marker position.
ViStatus _VI_FUNC rsfpl_ConfigureBandPowerMarker(ViSession instrSession,
                                                 ViInt32   window,
                                                 ViInt32   marker,
                                                 ViBoolean bandPowerMarkerEnabled,
                                                 ViInt32   mode,
                                                 ViReal64  span)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_BAND_POWER_MARKER_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_BAND_POWER_MARKER_ENABLED,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap_marker),
            3, "Marker");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s",
        repCap_window, repCap_marker);

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_BAND_POWER_MARKER_ENABLED, bandPowerMarkerEnabled),
            4, "Band Power Marker Enabled");

    if (bandPowerMarkerEnabled)
    {
        viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_BAND_POWER_MARKER_MODE, mode),
                5, "mode");

        viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_BAND_POWER_MARKER_SPAN, span),
                6, "Span");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the results of the band power measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/This control selects marker.
/// HIPAR window/This control selects the measurement window.
/// HIPAR bandPowerMarkerResult/Queries the results of the band power measurement.
ViStatus _VI_FUNC rsfpl_QueryBandPowerMarkerResult(ViSession instrSession,
                                                   ViInt32   window,
                                                   ViInt32   marker,
                                                   ViReal64* bandPowerMarkerResult)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_BAND_POWER_MARKER_RESULT,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_BAND_POWER_MARKER_RESULT,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap_marker),
            3, "Marker");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s",
        repCap_window, repCap_marker);

    viCheckParm(rsfpl_GetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_BAND_POWER_MARKER_RESULT, bandPowerMarkerResult),
            4, "Band Power Marker Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Removes all band power markers in the specified window.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
ViStatus _VI_FUNC rsfpl_BandPowerMarkerAllOff(ViSession instrSession,
                                              ViInt32   window)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_BAND_POWER_MARKER_ALL_OFF,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_BAND_POWER_MARKER_ALL_OFF,
            "Marker", 0 , RS_REPCAP_BUF_SIZE, repCap_marker));
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s",
        repCap_window, repCap_marker);

    checkErr(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_BAND_POWER_MARKER_ALL_OFF, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the n dB Down marker function.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR ndBDownEnabled/Turns the n dB Down marker function on and off.
/// HIPAR ndBDownDistance/Defines the distance of the n dB down markers to the reference marker.
ViStatus _VI_FUNC rsfpl_ConfigurendBDown(ViSession instrSession,
                                         ViBoolean ndBDownEnabled,
                                         ViReal64  ndBDownDistance)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_N_DB_DOWN_ENABLED, ndBDownEnabled),
            4, "N dB Down Enabled");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_N_DB_DOWN_DISTANCE, ndBDownDistance),
            5, "N dB Down Distance");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the distance of the n dB down markers from each other.
/// HIFN     To get a valid result, you have to perform a complete measurement with
/// HIFN  synchronization to the end of the measurement before reading out the
/// HIFN  result. This is only possible for single sweep mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR ndBDownResult/Queries the distance of the n dB down markers from each other.
ViStatus _VI_FUNC rsfpl_QueryndBDownResult(ViSession instrSession,
                                           ViReal64* ndBDownResult)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_GetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_N_DB_DOWN_RESULT, ndBDownResult),
            4, "n dBDown Marker Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the position of the n dB down markers on the x-axis when
/// HIFN  measuring in the frequency domain.
/// HIFN     To get a valid result, you have to perform a complete measurement with
/// HIFN  synchronization to the end of the measurement before reading out the
/// HIFN  result. This is only possible for single sweep mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR left/Absolute frequency of the n dB marker to the left of the reference
/// HIPAR left/marker in Hz.
/// HIPAR right/Absolute frequency of the n dB marker to the right of the reference
/// HIPAR right/marker in Hz.
ViStatus _VI_FUNC rsfpl_QueryndBDownFrequencies(ViSession instrSession,
                                                ViReal64* left,
                                                ViReal64* right)
{
    ViStatus  error = VI_SUCCESS;
    ViReal64* result = NULL;
    ViInt32   returnedValues = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_QueryAsciiViReal64Array(instrSession, "CALC:MARK:FUNC:NDBD:FREQ?", &result, &returnedValues));
    checkErr(rsfpl_CheckStatus(instrSession));

    *left = result[0];
    *right = result[1];

Error:
    if (result) free(result);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the Q factor of n dB down measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR ndBDownQFactor/Queries the Q factor of n dB down measurements.
ViStatus _VI_FUNC rsfpl_QueryndBDownQFactor(ViSession instrSession,
                                            ViReal64* ndBDownQFactor)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_GetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_N_DB_DOWN_Q_FACTOR, ndBDownQFactor),
            4, "n dBDown Q Factor");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the position of the n dB down markers on the x-axis when
/// HIFN  measuring in the time domain.
/// HIFN     To get a valid result, you have to perform a complete measurement with
/// HIFN  synchronization to the end of the measurement before reading out the
/// HIFN  result. This is only possible for single sweep mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR left/Absolute position in time of the n dB marker to the left of the
/// HIPAR left/reference marker in seconds.
/// HIPAR right/Absolute position in time of the n dB marker to the right of the
/// HIPAR right/reference marker in seconds.
ViStatus _VI_FUNC rsfpl_QueryndBDownTime(ViSession instrSession,
                                         ViReal64* left,
                                         ViReal64* right)
{
    ViStatus  error = VI_SUCCESS;
    ViReal64* result = NULL;
    ViInt32   returnedValues;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_QueryAsciiViReal64Array(instrSession, "CALC:MARK:FUNC:NDBD:TIME?", &result, &returnedValues));

    *left = result[0];
    *right = result[1];

Error:
    if (result) free(result);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns the audio demodulator on and off when the measurement reaches a
/// HIFN  marker position.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/This control selects marker.
/// HIPAR window/This control selects the measurement window.
/// HIPAR demodulationEnabled/Turns the audio demodulator on and off when the measurement reaches a
/// HIPAR demodulationEnabled/marker position.
/// HIPAR demodulationMode/Selects the demodulation mode at the marker position. This control has
/// HIPAR demodulationMode/effect only if Demodulation Enabled == On.
ViStatus _VI_FUNC rsfpl_ConfigureMarkerDemodulation(ViSession instrSession,
                                                    ViInt32   window,
                                                    ViInt32   marker,
                                                    ViBoolean demodulationEnabled,
                                                    ViInt32   demodulationMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_MARKER_DEMODULATION_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_MARKER_DEMODULATION_ENABLED,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap_marker),
            3, "Marker");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s",
        repCap_window, repCap_marker);

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_MARKER_DEMODULATION_ENABLED, demodulationEnabled),
            4, "Demodulation Enabled");

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_MARKER_DEMODULATION_MODE, demodulationMode),
            5, "Demodulation Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns continuous demodulation of the signal at the marker position in
/// HIFN  the frequency domain on and off.
/// HIFN     In the time domain continuous demodulation is always on.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/This control selects marker.
/// HIPAR window/This control selects the measurement window.
/// HIPAR continuousEnabled/Turns continuous demodulation of the signal at the marker position in
/// HIPAR continuousEnabled/the frequency domain on and off.
ViStatus _VI_FUNC rsfpl_ConfigureMarkerDemodulationContinuous(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   marker,
                            ViBoolean continuousEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_MARKER_DEMODULATION_CONTINUOUS_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_MARKER_DEMODULATION_CONTINUOUS_ENABLED,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap_marker),
            3, "Marker");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap_window, repCap_marker);

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_MARKER_DEMODULATION_CONTINUOUS_ENABLED, continuousEnabled),
            4, "Continuous Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines for how long the the signal at the marker position is
/// HIFN  demodulated.
/// HIFN     In the time domain continuous demodulation is always on.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR stopTime/Defines for how long the the signal at the marker position is
/// HIPAR stopTime/demodulated.
/// HIPAR marker/This control selects marker.
/// HIPAR window/This control selects the measurement window.
ViStatus _VI_FUNC rsfpl_ConfigureMarkerDemodulationStopTime(ViSession instrSession,
                                                            ViInt32   window,
                                                            ViInt32   marker,
                                                            ViReal64  stopTime)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_MARKER_DEMODULATION_STOP_TIME,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_MARKER_DEMODULATION_STOP_TIME,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap_marker),
            3, "Marker");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s",
        repCap_window, repCap_marker);

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_MARKER_DEMODULATION_STOP_TIME, stopTime),
            4, "Stop Time");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures selective demodulation at the marker position.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/This control selects marker.
/// HIPAR window/This control selects the measurement window.
/// HIPAR enabled/Turns selective demodulation at the marker position on and off.
/// HIPAR selectiveThreshold/Defines the threshold for selective demodulation.
ViStatus _VI_FUNC rsfpl_ConfigureSelectiveDemodulation(ViSession instrSession,
                                                       ViBoolean selectiveEnabled,
                                                       ViReal64  selectiveThreshold)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_MARKER_DEMODULATION_SELECTIVE_ENABLED, selectiveEnabled),
            4, "Selective Enabled");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_MARKER_DEMODULATION_SELECTIVE_THRESHOLD, selectiveThreshold),
            5, "Selective Threshold");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Positions a marker on a particular frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/This control selects marker.
/// HIPAR window/This control selects the measurement window.
/// HIPAR spectrogramMarkerFrame/Positions a marker on a particular frame.
ViStatus _VI_FUNC rsfpl_ConfigureSpectrogramMarkerFramePosition(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   marker,
                            ViInt32   spectrogramMarkerFrame)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SPECTROGRAM_MARKER_FRAME,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SPECTROGRAM_MARKER_FRAME,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap_marker),
            3, "Marker");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s",
        repCap_window, repCap_marker);

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_SPECTROGRAM_MARKER_FRAME, spectrogramMarkerFrame),
            4, "Spectrogram Marker Frame");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the marker search area for all spectrogram markers in the
/// HIFN  channel setup.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/This control selects marker.
/// HIPAR window/This control selects the measurement window.
/// HIPAR searchArea/Defines the marker search area for all spectrogram markers in the
/// HIPAR searchArea/channel setup.
ViStatus _VI_FUNC rsfpl_ConfigureSpectrogramMarkerSearchArea(ViSession instrSession,
                                                             ViInt32   searchArea)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_SPECTROGRAM_MARKER_SEARCH_AREA, searchArea),
            4, "Search Area");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function specifies the type of marker search and performs the
/// HIFN  search.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/This control selects marker.
/// HIPAR markerSearch/Specifies the type of marker search type.
ViStatus _VI_FUNC rsfpl_SpectrogramMarkerSearch(ViSession instrSession,
                                                ViInt32   window,
                                                ViInt32   marker,
                                                ViInt32   markerSearch)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SPECTROGRAM_MARKER_MAX_PEAK,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SPECTROGRAM_MARKER_MAX_PEAK,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap_marker),
            3, "Marker");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s",
        repCap_window, repCap_marker);

    switch (markerSearch)
    {
    case RSFPL_VAL_SGR_MARKER_SEARCH_HIGHEST:
        viCheckParm(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_SPECTROGRAM_MARKER_MAX_PEAK, NULL),
                4, "Marker Search");
        break;
    case RSFPL_VAL_SGR_MARKER_SEARCH_MINIMUM:
        viCheckParm(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_SPECTROGRAM_MARKER_MIN_PEAK, NULL),
                4, "Marker Search");
        break;
    case RSFPL_VAL_SGR_MARKER_SEARCH_NEXT_PEAK:
        viCheckParm(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_SPECTROGRAM_MARKER_MAX_PEAK_NEXT, NULL),
                4, "Marker Search");
        break;
    case RSFPL_VAL_SGR_MARKER_SEARCH_NEXT_PEAK_ABOVE:
        viCheckParm(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_SPECTROGRAM_MARKER_MAX_PEAK_ABOVE, NULL),
                4, "Marker Search");
        break;
    case RSFPL_VAL_SGR_MARKER_SEARCH_NEXT_PEAK_BELOW:
        viCheckParm(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_SPECTROGRAM_MARKER_MAX_PEAK_BELOW, NULL),
                4, "Marker Search");
        break;
    case RSFPL_VAL_SGR_MARKER_SEARCH_NEXT_MINIMUM:
        viCheckParm(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_SPECTROGRAM_MARKER_MIN_PEAK_NEXT, NULL),
                4, "Marker Search");
        break;
    case RSFPL_VAL_SGR_MARKER_SEARCH_NEXT_MINIMUM_ABOVE:
        viCheckParm(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_SPECTROGRAM_MARKER_MIN_PEAK_ABOVE, NULL),
                4, "Marker Search");
        break;
    case RSFPL_VAL_SGR_MARKER_SEARCH_NEXT_MINIMUM_BELOW:
        viCheckParm(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_SPECTROGRAM_MARKER_MIN_PEAK_BELOW, NULL),
                4, "Marker Search");
        break;
    case RSFPL_VAL_SGR_MARKER_SEARCH_LEVEL_MINIMUM:
        viCheckParm(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_SPECTROGRAM_MARKER_MIN_LEVEL, NULL),
                4, "Marker Search");
        break;
    case RSFPL_VAL_SGR_MARKER_SEARCH_LEVEL_MAXIMUM:
        viCheckParm(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_SPECTROGRAM_MARKER_MAX_LEVEL, NULL),
                4, "Marker Search");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, markerSearch), 4, "Marker Search");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the I/Q data file to be used as input for further
/// HIFN  measurements. The I/Q data must have a specific format (.iq.tar).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR filename/String containing the path and name of the source file. The file
/// HIPAR filename/extension is *.iq.tar.
/// HIPAR analysisBW/Optionally: The analysis bandwidth to be used by the measurement. The
/// HIPAR analysisBW/bandwidth must be smaller than or equal to the bandwidth of the data
/// HIPAR analysisBW/that was stored in the file.
ViStatus _VI_FUNC rsfpl_SelectIQDataFile(ViSession instrSession,
                                         ViString  filename,
                                         ViReal64  analysisBW)
{
    ViStatus error = VI_SUCCESS;
    ViChar   param[RS_MAX_MESSAGE_LEN] = "";

    checkErr(RsCore_LockSession(instrSession));

    if (analysisBW > 0.0)
    {
        snprintf(param, RS_MAX_MESSAGE_LEN, "'%s',%.12lG", filename, analysisBW);
    }
    else
    {
        snprintf(param, RS_MAX_MESSAGE_LEN, "'%s'", filename);
    }

    checkErr(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_IQ_ANALYZER_SELECT_IQ_DATA_FILE_RAWSTRING, param));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables and configures EMI measurement marker functionality.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Turns the EMI measurement marker functionality on and off.
/// HIPAR dwellTime/Defines the dwell time during the final measurement.
ViStatus _VI_FUNC rsfpl_ConfigureEMIMeasurement(ViSession instrSession,
                                                ViBoolean enabled,
                                                ViReal64  dwellTime)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_EMI_MEASUREMENT_ENABLED, enabled),
            2, "Enabled");

    if (enabled)
    {
        viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_EMI_MEASUREMENT_DWELL_TIME, dwellTime),
            3, "Dwell Time");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  The commands required to control a LISN. This feature requires the
/// HIFN  optional additional interfaces (R&S FPL1-B5).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR lisnType/Turns automatic control of a LISN on and off. It also selects the type
/// HIPAR lisnType/of network.
/// HIPAR phase/Selects one LISN phase to be measured.
/// HIPAR highPassFilter/Turns the 150 kHz highpass filter for the ENV216 network on and off.
ViStatus _VI_FUNC rsfpl_ConfigureEMILISNVNetwork(ViSession instrSession,
                                                 ViInt32   LISNType,
                                                 ViInt32   phase,
                                                 ViBoolean highPassFilter)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_EMI_LISN_V_NETWORK_TYPE, LISNType),
            2, "LISN Type");

    switch (LISNType)
    {
    case RSFPL_VAL_EMI_VNET_TYPE_OFF:
        break;
    case RSFPL_VAL_EMI_VNET_TYPE_216:
        viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_EMI_LISN_V_NETWORK_HIGH_PASS_FILTER, highPassFilter),
                4, "High Pass Filter");
        break;
    default:
        viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_EMI_LISN_V_NETWORK_PHASE, phase),
                3, "Phase");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Initiates a marker peak search.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/Selects the marker.
ViStatus _VI_FUNC rsfpl_EMIMeasurementMarkerPeakSearch(ViSession instrSession,
                                                       ViInt32   window,
                                                       ViInt32   marker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_EMI_MEASUREMENT_MARKER_PEAK_SEARCH,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_EMI_MEASUREMENT_MARKER_PEAK_SEARCH,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap_marker),
            3, "Marker");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s",
        repCap_window, repCap_marker);

    checkErr(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_EMI_MEASUREMENT_MARKER_PEAK_SEARCH, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the detector for a specific marker during the final
/// HIFN  measurement. If the marker is not yet active, the command also turns
/// HIFN  the marker on.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/Selects the marker.
/// HIPAR detector/Selects the detector for a specific marker during the final
/// HIPAR detector/measurement. If the marker is not yet active, the command also turns
/// HIPAR detector/the marker on.
ViStatus _VI_FUNC rsfpl_ConfigureEMIMeasurementMarkerDetector(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   marker,
                            ViInt32   detector)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_EMI_MEASUREMENT_MARKER_DETECTOR,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_EMI_MEASUREMENT_MARKER_DETECTOR,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap_marker),
            3, "Marker");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s",
        repCap_window, repCap_marker);

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_EMI_MEASUREMENT_MARKER_DETECTOR, detector),
            4, "Detector");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the result of the EMI measurement at the marker position.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/Selects the marker.
/// HIPAR result/Queries the result of the EMI measurement at the marker position.
ViStatus _VI_FUNC rsfpl_QueryEMIMeasurementMarkerResult(ViSession instrSession,
                                                        ViInt32   window,
                                                        ViInt32   marker,
                                                        ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_EMI_MEASUREMENT_MARKER_RESULT,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_EMI_MEASUREMENT_MARKER_RESULT,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap_marker),
            3, "Marker");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s",
        repCap_window, repCap_marker);

    checkErr(rsfpl_GetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_EMI_MEASUREMENT_MARKER_RESULT, result));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the condition of a marker position in relation to a certain
/// HIFN  limit line.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/Selects the marker.
/// HIPAR limitLine/Selects the limit line.
/// HIPAR limitCondition/Returns the condition of a marker position in relation to a certain
/// HIPAR limitCondition/limit line.
ViStatus _VI_FUNC rsfpl_QueryEMIMeasurementMarkerLimitCondition(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   marker,
                            ViInt32   limitLine,
                            ViInt32*  limitCondition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_limit[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_EMI_MEASUREMENT_MARKER_LIMIT_CONDITION,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_EMI_MEASUREMENT_MARKER_LIMIT_CONDITION,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap_marker),
            3, "Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_EMI_MEASUREMENT_MARKER_LIMIT_CONDITION,
            "LimitLine", limitLine - 1, RS_REPCAP_BUF_SIZE, repCap_limit),
            4, "Limit Line");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s,%s",
        repCap_window, repCap_marker, repCap_limit);

    checkErr(rsfpl_GetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_EMI_MEASUREMENT_MARKER_LIMIT_CONDITION, limitCondition));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the vertical distance from the marker position to the limit
/// HIFN  line. The unit is dB. If the marker has been assigned to a different
/// HIFN  trace than the limit line, or if no limit ine is defined for the marker
/// HIFN  position, the command returns -200.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/Selects the marker.
/// HIPAR limitLine/Selects the limit line.
/// HIPAR verticalDistance/Queries the vertical distance from the marker position to the limit
/// HIPAR verticalDistance/line. The unit is dB. If the marker has been assigned to a different
/// HIPAR verticalDistance/trace than the limit line, or if no limit ine is defined for the marker
/// HIPAR verticalDistance/position, the command returns -200.
ViStatus _VI_FUNC rsfpl_QueryEMIMeasurementMarkerLimitVerticalDistance(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   marker,
                            ViInt32   limitLine,
                            ViReal64* verticalDistance)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_limit[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_EMI_MEASUREMENT_MARKER_LIMIT_VERTICAL_DISTANCE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_EMI_MEASUREMENT_MARKER_LIMIT_VERTICAL_DISTANCE,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap_marker),
            3, "Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_EMI_MEASUREMENT_MARKER_LIMIT_VERTICAL_DISTANCE,
            "LimitLine", limitLine - 1, RS_REPCAP_BUF_SIZE, repCap_limit),
            4, "Limit Line");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s,%s",
        repCap_window, repCap_marker, repCap_limit);

    checkErr(rsfpl_GetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_EMI_MEASUREMENT_MARKER_LIMIT_VERTICAL_DISTANCE, verticalDistance));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Initiates a delta marker peak search.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR deltaMarker/Selects the delta marker.
ViStatus _VI_FUNC rsfpl_EMIMeasurementDeltaMarkerPeakSearch(ViSession instrSession,
                                                            ViInt32   window,
                                                            ViInt32   deltaMarker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_EMI_MEASUREMENT_DELTA_MARKER_PEAK_SEARCH,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_EMI_MEASUREMENT_DELTA_MARKER_PEAK_SEARCH,
            "Marker", deltaMarker - 1, RS_REPCAP_BUF_SIZE, repCap_marker),
            3, "Delta Marker");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap_window,
        repCap_marker);

    checkErr(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_EMI_MEASUREMENT_DELTA_MARKER_PEAK_SEARCH, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the detector for a specific marker during the final
/// HIFN  measurement. If the marker is not yet active, the command also turns
/// HIFN  the marker on.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR deltaMarker/Selects the delta marker.
/// HIPAR detector/Selects the detector for a specific marker during the final
/// HIPAR detector/measurement. If the marker is not yet active, the command also turns
/// HIPAR detector/the marker on.
ViStatus _VI_FUNC rsfpl_ConfigureEMIMeasurementDeltaMarkerDetector(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   deltaMarker,
                            ViInt32   detector)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_EMI_MEASUREMENT_DELTA_MARKER_DETECTOR,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_EMI_MEASUREMENT_DELTA_MARKER_DETECTOR,
            "Marker", deltaMarker - 1, RS_REPCAP_BUF_SIZE, repCap_marker),
            3, "Delta Marker");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap_window,
        repCap_marker);

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_EMI_MEASUREMENT_DELTA_MARKER_DETECTOR, detector),
            4, "Detector");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the result of the EMI measurement at the marker position.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR deltaMarker/Selects the delta marker.
/// HIPAR result/Queries the result of the EMI measurement at the marker position.
ViStatus _VI_FUNC rsfpl_QueryEMIMeasurementDeltaMarkerResult(ViSession instrSession,
                                                             ViInt32   window,
                                                             ViInt32   deltaMarker,
                                                             ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_EMI_MEASUREMENT_DELTA_MARKER_RESULT,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_EMI_MEASUREMENT_DELTA_MARKER_RESULT,
            "Marker", deltaMarker - 1, RS_REPCAP_BUF_SIZE, repCap_marker),
            3, "Delta Marker");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap_window,
        repCap_marker);

    checkErr(rsfpl_GetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_EMI_MEASUREMENT_DELTA_MARKER_RESULT, result));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the condition of a marker position in relation to a certain
/// HIFN  limit line.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR deltaMarker/Selects the delta marker.
/// HIPAR limitLine/Selects the limit line.
/// HIPAR limitCondition/Queries the condition of a marker position in relation to a certain
/// HIPAR limitCondition/limit line.
ViStatus _VI_FUNC rsfpl_QueryEMIMeasurementDeltaMarkerLimitCondition(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   deltaMarker,
                            ViInt32   limitLine,
                            ViInt32*  limitCondition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_limit[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_EMI_MEASUREMENT_DELTA_MARKER_LIMIT_CONDITION,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_EMI_MEASUREMENT_DELTA_MARKER_LIMIT_CONDITION,
            "Marker", deltaMarker - 1, RS_REPCAP_BUF_SIZE, repCap_marker),
            3, "Delta Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_EMI_MEASUREMENT_DELTA_MARKER_LIMIT_CONDITION,
            "LimitLine", limitLine - 1, RS_REPCAP_BUF_SIZE, repCap_limit),
            4, "Limit Line");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s,%s",
        repCap_window, repCap_marker, repCap_limit);

    checkErr(rsfpl_GetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_EMI_MEASUREMENT_DELTA_MARKER_LIMIT_CONDITION, limitCondition));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the vertical distance from the marker position to the limit
/// HIFN  line. The unit is dB. If the marker has been assigned to a different
/// HIFN  trace than the limit line, or if no limit ine is defined for the marker
/// HIFN  position, the command returns -200.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR deltaMarker/Selects the delta marker.
/// HIPAR limitLine/Selects the limit line.
/// HIPAR verticalDistance/Queries the vertical distance from the marker position to the limit
/// HIPAR verticalDistance/line. The unit is dB. If the marker has been assigned to a different
/// HIPAR verticalDistance/trace than the limit line, or if no limit ine is defined for the marker
/// HIPAR verticalDistance/position, the command returns -200.
ViStatus _VI_FUNC rsfpl_QueryEMIMeasurementDeltaMarkerLimitVerticalDistance(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   deltaMarker,
                            ViInt32   limitLine,
                            ViReal64* verticalDistance)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_limit[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_EMI_MEASUREMENT_DELTA_MARKER_LIMIT_VERTICAL_DISTANCE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_EMI_MEASUREMENT_DELTA_MARKER_LIMIT_VERTICAL_DISTANCE,
            "Marker", deltaMarker - 1, RS_REPCAP_BUF_SIZE, repCap_marker),
            3, "Delta Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_EMI_MEASUREMENT_DELTA_MARKER_LIMIT_VERTICAL_DISTANCE,
            "LimitLine", limitLine - 1, RS_REPCAP_BUF_SIZE, repCap_limit),
            4, "Limit Line");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s,%s",
        repCap_window, repCap_marker, repCap_limit);

    checkErr(rsfpl_GetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_EMI_MEASUREMENT_DELTA_MARKER_LIMIT_VERTICAL_DISTANCE, verticalDistance));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function enables the active deltamarker on the specified trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/This control selects marker.
/// HIPAR state/Turns delta markers on and off.
/// HIPAR trace/Selects the trace a delta marker is positioned on.
ViStatus _VI_FUNC rsfpl_ConfigureDeltaMarker(ViSession instrSession,
                                             ViInt32   window,
                                             ViInt32   marker,
                                             ViBoolean state,
                                             ViInt32   trace)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_DELTA_MARKER_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_DELTA_MARKER_ENABLED,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap_marker),
            3, "Marker");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s",
        repCap_window, repCap_marker);

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_DELTA_MARKER_ENABLED, state),
            4, "State");

    if (state)
    {
        viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_DELTA_MARKER_ASSIGNED_TRACE, trace),
                5, "Trace");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines whether the position of a delta marker is provided as an
/// HIFN  absolute value or relative to a reference marker.
/// HIFN     Note that when the position of a delta marker is queried, the result
/// HIFN  is always an absolute value!
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR deltaMarkerMode/Defines whether the position of a delta marker is provided as an
/// HIPAR deltaMarkerMode/absolute value or relative to a reference marker.
ViStatus _VI_FUNC rsfpl_ConfigureDeltaMarkerMode(ViSession instrSession,
                                                 ViInt32   window,
                                                 ViInt32   deltaMarkerMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_DELTA_MARKER_MODE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_DELTA_MARKER_MODE, deltaMarkerMode),
            5, "Delta Marker Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Turns off all delta markers.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_AllDeltaMarkersOff(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_ALL_DELTA_MARKERS_OFF, NULL),
            2, "All Delta Markers Off");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function specifies the type of marker search and performs the
/// HIFN  search.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/This control selects marker.
/// HIPAR markerSearch/Specifies the type of marker search type.
ViStatus _VI_FUNC rsfpl_DeltaMarkerSearch(ViSession instrSession,
                                          ViInt32   window,
                                          ViInt32   marker,
                                          ViInt32   markerSearch)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_MARKER_MAX_PEAK,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_MARKER_MAX_PEAK,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap_marker),
            3, "Marker");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap_window, repCap_marker);

    switch (markerSearch)
    {
    case RSFPL_VAL_MARKER_SEARCH_HIGHEST:
        viCheckParm(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_MARKER_MAX_PEAK, NULL),
                4, "Marker Search");
        break;
    case RSFPL_VAL_MARKER_SEARCH_MINIMUM:
        viCheckParm(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_MARKER_MIN_PEAK, NULL),
                4, "Marker Search");
        break;
    case RSFPL_VAL_MARKER_SEARCH_NEXT_PEAK:
        viCheckParm(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_MARKER_MAX_PEAK_NEXT, NULL),
                4, "Marker Search");
        break;
    case RSFPL_VAL_MARKER_SEARCH_NEXT_PEAK_LEFT:
        viCheckParm(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_MARKER_MAX_PEAK_LEFT, NULL),
                4, "Marker Search");
        break;
    case RSFPL_VAL_MARKER_SEARCH_NEXT_PEAK_RIGHT:
        viCheckParm(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_MARKER_MAX_PEAK_RIGHT, NULL),
                4, "Marker Search");
        break;
    case RSFPL_VAL_MARKER_SEARCH_NEXT_MINIMUM:
        viCheckParm(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_MARKER_MIN_PEAK_NEXT, NULL),
                4, "Marker Search");
        break;
    case RSFPL_VAL_MARKER_SEARCH_NEXT_MINIMUM_LEFT:
        viCheckParm(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_MARKER_MIN_PEAK_LEFT, NULL),
                4, "Marker Search");
        break;
    case RSFPL_VAL_MARKER_SEARCH_NEXT_MINIMUM_RIGHT:
        viCheckParm(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_MARKER_MIN_PEAK_RIGHT, NULL),
                4, "Marker Search");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, markerSearch), 4, "Marker Search");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the relative position of a delta marker on the y-axis and
/// HIFN  x-axis.
/// HIFN     If necessary, the attribute activates the delta marker first.
/// HIFN     To get a valid result, you have to perform a complete measurement with
/// HIFN  synchronization to the end of the measurement before reading out the
/// HIFN  result. This is only possible for single sweep mode.
/// HIFN     The unit depends on the application of the attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/This control selects marker.
/// HIPAR deltaMarkerRelativePosition/Queries the relative position of a delta marker on the x-axis.
/// HIPAR deltaMarkerAmplitude/Queries the relative position of a delta marker on the y-axis.
ViStatus _VI_FUNC rsfpl_QueryDeltaMarker(ViSession instrSession,
                                         ViInt32   window,
                                         ViInt32   marker,
                                         ViReal64* deltaMarkerRelativePosition,
                                         ViReal64* deltaMarkerAmplitude)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_DELTA_MARKER_RESULT,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_DELTA_MARKER_RESULT,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap_marker),
            3, "Marker");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s",
        repCap_window, repCap_marker);

    viCheckParm(rsfpl_GetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_DELTA_MARKER_RESULT, deltaMarkerRelativePosition),
            4, "Delta Marker Relative Position");

    viCheckParm(rsfpl_GetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_DELTA_MARKER_RELATIVE_POSITION, deltaMarkerAmplitude),
            5, "Delta Marker Amplitude");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a delta marker to a particular coordinate on the x-axis.
/// HIFN     If necessary, the attribute activates the delta marker and positions a
/// HIFN  reference marker to the peak power.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/This control selects marker.
/// HIPAR window/This control selects the measurement window.
/// HIPAR deltaMarkerPosition/Moves a delta marker to a particular coordinate on the x-axis.
ViStatus _VI_FUNC rsfpl_ConfigureDeltaMarkerPosition(ViSession instrSession,
                                                     ViInt32   window,
                                                     ViInt32   marker,
                                                     ViReal64  deltaMarkerPosition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_DELTA_MARKER_POSITION,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_DELTA_MARKER_POSITION,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap_marker),
            3, "Marker");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s",
        repCap_window, repCap_marker);

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_DELTA_MARKER_POSITION, deltaMarkerPosition),
            4, "Delta Marker Position");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Links delta marker <m1> to any active normal marker <m2>.
/// HIFN     If you change the horizontal position of marker <m2>, delta marker
/// HIFN  <m1> changes its horizontal position to the same value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR linkEnabled/Links delta marker <m1> to any active normal marker <m2>.
/// HIPAR window/This control selects the measurement window.
/// HIPAR firstMarker/This control selects first marker.
/// HIPAR secondMarker/This control selects second marker.
ViStatus _VI_FUNC rsfpl_DeltaMarkerLinkToAnotherMarker(ViSession instrSession,
                                                       ViInt32   window,
                                                       ViInt32   firstMarker,
                                                       ViInt32   secondMarker,
                                                       ViBoolean linkEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, firstMarker, 1, 16),
            3, "First Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, secondMarker, 1, 16),
            4, "Second Marker");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:DELT%ld:LINK:TO:MARK%ld %s", window, firstMarker, secondMarker, linkEnabled ? "ON" : "OFF");
    checkErr(rsfpl_WriteCommandWithOPCSync(instrSession, cmd, 0));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Links the delta source marker <ms> to any active destination delta
/// HIFN  marker <md>.
/// HIFN     If you change the horizontal position of marker <md>, marker <ms>
/// HIFN  changes its horizontal position to the same value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR firstMarker/This control selects first marker.
/// HIPAR secondMarker/This control selects second marker.
/// HIPAR linkEnabled/Links the delta source marker <ms> to any active destination delta
/// HIPAR linkEnabled/marker <md>.
ViStatus _VI_FUNC rsfpl_DeltaMarkerLinkToAnotherDeltaMarker(ViSession instrSession,
                                                            ViInt32   window,
                                                            ViInt32   firstMarker,
                                                            ViInt32   secondMarker,
                                                            ViBoolean linkEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, firstMarker, 1, 16),
            3, "First Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, secondMarker, 1, 16),
            4, "Second Marker");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:DELT%ld:LINK:TO:DELT%ld %s", window, firstMarker, secondMarker, linkEnabled ? "ON" : "OFF");
    checkErr(rsfpl_WriteCommandWithOPCSync(instrSession, cmd, 0));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Links delta marker <m> to marker 1.
/// HIFN     If you change the horizontal position (x-value) of marker 1, delta
/// HIFN  marker <m> changes its horizontal position to the same value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/This control selects marker.
/// HIPAR window/This control selects the measurement window.
/// HIPAR deltaMarkerLinkToMarker1/Links delta marker <m> to marker 1.
ViStatus _VI_FUNC rsfpl_DeltaMarkerLinkToMarker1(ViSession instrSession,
                                                 ViInt32   window,
                                                 ViInt32   marker,
                                                 ViBoolean deltaMarkerLinkToMarker1)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_DELTA_MARKER_LINK_TO_MARKER_1,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_DELTA_MARKER_LINK_TO_MARKER_1,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap_marker),
            3, "Marker");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s",
        repCap_window, repCap_marker);

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_DELTA_MARKER_LINK_TO_MARKER_1, deltaMarkerLinkToMarker1),
            4, "Delta Marker Link To Marker1");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects a reference marker for a delta marker other than marker 1.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/This control selects marker.
/// HIPAR fixedReferenceEnabled/Enables/disables a fixed reference..
/// HIPAR deltaMarkerReference/Selects a reference marker for a delta marker other than marker 1.
/// HIPAR deltaMarkerReference/This control has effect only if  Fixed Reference Enabled == Off.
ViStatus _VI_FUNC rsfpl_ConfigureDeltaMarkerReference(ViSession instrSession,
                                                      ViInt32   window,
                                                      ViInt32   marker,
                                                      ViBoolean fixedReferenceEnabled,
                                                      ViInt32   deltaMarkerReference)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_FIXED_REFERENCE_MARKER_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_FIXED_REFERENCE_MARKER_ENABLED,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap_marker),
            3, "Marker");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s",
        repCap_window, repCap_marker);

    if (!fixedReferenceEnabled)
    {
        viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_DELTA_MARKER_REFERENCE, deltaMarkerReference),
                5, "Delta Marker Reference");
    }
    else
    {
        viCheckParm(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_DELTA_MARKER_REFERENCE_FIXED, NULL),
                5, "Delta Marker Reference");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates or deactivates a marker that defines a fixed reference point
/// HIFN  for relative marker analysis.
/// HIFN     If necessary, it activates a marker and positions it on the peak
/// HIFN  power.
/// HIFN     Subsequently, you can change the coordinates of the fixed reference
/// HIFN  independent of the marker. The fixed reference is independent of the
/// HIFN  trace and is applied to all active delta markers.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/This control selects marker.
/// HIPAR window/This control selects the measurement window.
/// HIPAR fixedReferenceEnabled/Activates or deactivates a marker that defines a fixed reference point
/// HIPAR fixedReferenceEnabled/for relative marker analysis.
ViStatus _VI_FUNC rsfpl_ConfigureFixedReferenceMarkerEnabled(ViSession instrSession,
                                                             ViInt32   window,
                                                             ViInt32   marker,
                                                             ViBoolean fixedReferenceEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_FIXED_REFERENCE_MARKER_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_FIXED_REFERENCE_MARKER_ENABLED,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap_marker),
            3, "Marker");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s",
        repCap_window, repCap_marker);

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_FIXED_REFERENCE_MARKER_ENABLED, fixedReferenceEnabled),
            4, "Fixed Reference Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Moves the fixed reference marker to the peak power.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/This control selects marker.
/// HIPAR window/This control selects the measurement window.
ViStatus _VI_FUNC rsfpl_FixedReferenceToMaxPeak(ViSession instrSession,
                                                ViInt32   marker,
                                                ViInt32   window)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_FIXED_REFERENCE_MARKER_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_FIXED_REFERENCE_MARKER_ENABLED,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap_marker),
            3, "Marker");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s",
        repCap_window, repCap_marker);

    viCheckParm(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_FIXED_REFERENCE_MARKER_MAX_PEAK, NULL),
            2, "Fixed Reference To Max Peak");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines reference point.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/This control selects marker.
/// HIPAR window/This control selects the measurement window.
/// HIPAR referencePointHorizontal/Defines the horizontal position of the fixed delta marker reference
/// HIPAR referencePointHorizontal/point. The coordinates of the reference may be anywhere in the diagram.
/// HIPAR referencePointVertical/Defines the vertical position of the fixed delta marker reference
/// HIPAR referencePointVertical/point. The coordinates of the reference may be anywhere in the diagram.
/// HIPAR referencePointLevelOffset/Defines a level offset for the fixed delta marker reference point.
ViStatus _VI_FUNC rsfpl_ConfigureFixedReferencePoint(ViSession instrSession,
                                                     ViInt32   window,
                                                     ViInt32   marker,
                                                     ViReal64  referencePointHorizontal,
                                                     ViReal64  referencePointVertical,
                                                     ViReal64  referencePointLevelOffset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_FIXED_REFERENCE_MARKER_REFERENCE_POINT_HORIZONTAL,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_FIXED_REFERENCE_MARKER_REFERENCE_POINT_HORIZONTAL,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap_marker),
            3, "Marker");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap_window, repCap_marker);

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_FIXED_REFERENCE_MARKER_REFERENCE_POINT_HORIZONTAL, referencePointHorizontal),
            4, "Reference Point Horizontal");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_FIXED_REFERENCE_MARKER_REFERENCE_POINT_VERTICAL, referencePointVertical),
            5, "Reference Point Vertical");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_FIXED_REFERENCE_MARKER_REFERENCE_POINT_LEVEL_OFFSET, referencePointLevelOffset),
            6, "Reference Point Level Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Turns the phase noise measurement at the delta marker position on and
/// HIFN  off.
/// HIFN     The reference marker for phase noise measurements is either a normal
/// HIFN  marker or a fixed reference. If necessary, the attribute turns on the
/// HIFN  reference marker.
/// HIFN     The correction values for the bandwidth and the log amplifier are
/// HIFN  taken into account in the measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/This control selects marker.
/// HIPAR window/This control selects the measurement window.
/// HIPAR phaseNoiseEnabled/Turns the phase noise measurement at the delta marker position on and
/// HIPAR phaseNoiseEnabled/off.
ViStatus _VI_FUNC rsfpl_ConfigurePhaseNoiseMeasurementDeltaMarkerEnabled(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   marker,
                            ViBoolean phaseNoiseEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_PHASE_NOISE_MEASUREMENT_DELTA_MARKER_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_PHASE_NOISE_MEASUREMENT_DELTA_MARKER_ENABLED,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap_marker),
            3, "Marker");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s",
        repCap_window, repCap_marker);

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_PHASE_NOISE_MEASUREMENT_DELTA_MARKER_ENABLED, phaseNoiseEnabled),
            4, "Phase Noise Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Turns an automatic peak search for the fixed reference marker at the
/// HIFN  end of a sweep on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/This control selects marker.
/// HIPAR window/This control selects the measurement window.
/// HIPAR autoPeakSearchEnabled/Turns an automatic peak search for the fixed reference marker at the
/// HIPAR autoPeakSearchEnabled/end of a sweep on and off.
ViStatus _VI_FUNC rsfpl_ConfigurePhaseNoiseMeasurementDeltaMarkerAutoPeakSearchEnabled(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   marker,
                            ViBoolean autoPeakSearchEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_PHASE_NOISE_MEASUREMENT_DELTA_MARKER_AUTO_PEAK_SEARCH_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_PHASE_NOISE_MEASUREMENT_DELTA_MARKER_AUTO_PEAK_SEARCH_ENABLED,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap_marker),
            3, "Marker");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s",
        repCap_window, repCap_marker);

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_PHASE_NOISE_MEASUREMENT_DELTA_MARKER_AUTO_PEAK_SEARCH_ENABLED, autoPeakSearchEnabled),
            4, "Auto Peak Search Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the result of a phase noise measurement.
/// HIFN     If necessary, the attribute activates the measurement first.
/// HIFN     Is only available in the Spectrum application.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/This control selects marker.
/// HIPAR window/This control selects the measurement window.
/// HIPAR deltaMarkerResult/Queries the result of a phase noise measurement.
ViStatus _VI_FUNC rsfpl_QueryPhaseNoiseMeasurementDeltaMarkerResult(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   marker,
                            ViReal64* deltaMarkerResult)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_PHASE_NOISE_MEASUREMENT_DELTA_MARKER_RESULT,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_PHASE_NOISE_MEASUREMENT_DELTA_MARKER_RESULT,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap_marker),
            3, "Marker");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s",
        repCap_window, repCap_marker);

    viCheckParm(rsfpl_GetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_PHASE_NOISE_MEASUREMENT_DELTA_MARKER_RESULT, deltaMarkerResult),
            4, "Delta Marker Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures delta markers for band power measurements.
/// HIFN     If neccessary, also turns on a reference marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/This control selects marker.
/// HIPAR window/This control selects the measurement window.
/// HIPAR bandPowerEnabled/Turns delta markers for band power measurements on and off.
/// HIPAR bandPowerMode/Selects the way the results for a band power delta marker are
/// HIPAR bandPowerMode/displayed.
/// HIPAR span/Defines the bandwidth around the delta marker position.
ViStatus _VI_FUNC rsfpl_ConfigureBandPowerDeltaMarker(ViSession instrSession,
                                                      ViInt32   window,
                                                      ViInt32   marker,
                                                      ViBoolean bandPowerEnabled,
                                                      ViInt32   bandPowerMode,
                                                      ViReal64  span)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_BAND_POWER_DELTA_MARKER_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_BAND_POWER_DELTA_MARKER_ENABLED,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap_marker),
            3, "Marker");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s",
        repCap_window, repCap_marker);

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_BAND_POWER_DELTA_MARKER_ENABLED, bandPowerEnabled),
            4, "Band Power Enabled");

    if (bandPowerEnabled)
    {
        viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_BAND_POWER_DELTA_MARKER_MODE, bandPowerMode),
                5, "Band Power Mode");

        viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_BAND_POWER_DELTA_MARKER_SPAN, span),
                6, "Span");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the results of the band power measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/This control selects marker.
/// HIPAR window/This control selects the measurement window.
/// HIPAR bandPowerResult/Queries the results of the band power measurement.
ViStatus _VI_FUNC rsfpl_QueryBandPowerDeltaMarkerResult(ViSession instrSession,
                                                        ViInt32   window,
                                                        ViInt32   marker,
                                                        ViReal64* bandPowerResult)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_BAND_POWER_DELTA_MARKER_RESULT,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_BAND_POWER_DELTA_MARKER_RESULT,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap_marker),
            3, "Marker");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s",
        repCap_window, repCap_marker);

    viCheckParm(rsfpl_GetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_BAND_POWER_DELTA_MARKER_RESULT, bandPowerResult),
            4, "Band Power Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Positions a delta marker on a particular frame. The frame is relative
/// HIFN  to the position of marker 1.
/// HIFN     The attribute is available for the spectrogram.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/This control selects marker.
/// HIPAR window/This control selects the measurement window.
/// HIPAR spectrogramDeltaMarkerFrame/Positions a delta marker on a particular frame. The frame is relative
/// HIPAR spectrogramDeltaMarkerFrame/to the position of marker 1.
ViStatus _VI_FUNC rsfpl_ConfigureSpectrogramDeltaMarkerFrame(ViSession instrSession,
                                                             ViInt32   window,
                                                             ViInt32   marker,
                                                             ViInt32   spectrogramDeltaMarkerFrame)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SPECTROGRAM_DELTA_MARKER_FRAME,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SPECTROGRAM_DELTA_MARKER_FRAME,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap_marker),
            3, "Marker");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s",
        repCap_window, repCap_marker);

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_SPECTROGRAM_DELTA_MARKER_FRAME, spectrogramDeltaMarkerFrame),
            4, "Spectrogram Delta Marker Frame");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the marker search area for all spectrogram markers in the
/// HIFN  channel setup.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR deltaMarkerSearchArea/Defines the marker search area for all spectrogram markers in the
/// HIPAR deltaMarkerSearchArea/channel setup.
ViStatus _VI_FUNC rsfpl_ConfigureSpectrogramDeltaMarkerSearchArea(
                            ViSession instrSession,
                            ViInt32   deltaMarkerSearchArea)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_SPECTROGRAM_DELTA_MARKER_SEARCH_AREA, deltaMarkerSearchArea),
            4, "Delta Marker Search Area");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function specifies the type of marker search and performs the
/// HIFN  search.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/This control selects marker.
/// HIPAR markerSearch/Specifies the type of marker search type.
ViStatus _VI_FUNC rsfpl_SpectrogramDeltaMarkerSearch(ViSession instrSession,
                                                     ViInt32   window,
                                                     ViInt32   marker,
                                                     ViInt32   markerSearch)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SPECTROGRAM_DELTA_MARKER_MAX_PEAK,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SPECTROGRAM_DELTA_MARKER_MAX_PEAK,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap_marker),
            3, "Marker");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s",
        repCap_window, repCap_marker);

    switch (markerSearch)
    {
    case RSFPL_VAL_SGR_MARKER_SEARCH_HIGHEST:
        viCheckParm(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_SPECTROGRAM_DELTA_MARKER_MAX_PEAK, NULL),
                4, "Marker Search");
        break;
    case RSFPL_VAL_SGR_MARKER_SEARCH_MINIMUM:
        viCheckParm(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_SPECTROGRAM_DELTA_MARKER_MIN_PEAK, NULL),
                4, "Marker Search");
        break;
    case RSFPL_VAL_SGR_MARKER_SEARCH_NEXT_PEAK:
        viCheckParm(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_SPECTROGRAM_DELTA_MARKER_MAX_PEAK_NEXT, NULL),
                4, "Marker Search");
        break;
    case RSFPL_VAL_SGR_MARKER_SEARCH_NEXT_PEAK_ABOVE:
        viCheckParm(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_SPECTROGRAM_DELTA_MARKER_MAX_PEAK_ABOVE, NULL),
                4, "Marker Search");
        break;
    case RSFPL_VAL_SGR_MARKER_SEARCH_NEXT_PEAK_BELOW:
        viCheckParm(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_SPECTROGRAM_DELTA_MARKER_MAX_PEAK_BELOW, NULL),
                4, "Marker Search");
        break;
    case RSFPL_VAL_SGR_MARKER_SEARCH_NEXT_MINIMUM:
        viCheckParm(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_SPECTROGRAM_DELTA_MARKER_MIN_PEAK_NEXT, NULL),
                4, "Marker Search");
        break;
    case RSFPL_VAL_SGR_MARKER_SEARCH_NEXT_MINIMUM_ABOVE:
        viCheckParm(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_SPECTROGRAM_DELTA_MARKER_MIN_PEAK_ABOVE, NULL),
                4, "Marker Search");
        break;
    case RSFPL_VAL_SGR_MARKER_SEARCH_NEXT_MINIMUM_BELOW:
        viCheckParm(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_SPECTROGRAM_DELTA_MARKER_MIN_PEAK_BELOW, NULL),
                4, "Marker Search");
        break;
    case RSFPL_VAL_SGR_MARKER_SEARCH_LEVEL_MINIMUM:
        viCheckParm(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_SPECTROGRAM_DELTA_MARKER_MIN_LEVEL, NULL),
                4, "Marker Search");
        break;
    case RSFPL_VAL_SGR_MARKER_SEARCH_LEVEL_MAXIMUM:
        viCheckParm(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_SPECTROGRAM_DELTA_MARKER_MAX_LEVEL, NULL),
                4, "Marker Search");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, markerSearch), 4, "Marker Search");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the trace.
/// HIFN     The measurement continues in the background.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR subwindow/Selects the subwindow.
/// HIPAR trace/Selects the trace.
/// HIPAR traceMode/Selects the trace mode. If necessary, the selected trace is also
/// HIPAR traceMode/activated.
ViStatus _VI_FUNC rsfpl_ConfigureTrace(ViSession instrSession,
                                       ViInt32   window,
                                       ViInt32   subwindow,
                                       ViInt32   trace,
                                       ViInt32   traceMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_subwindow[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_trace[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SUBWINDOW_TRACE_MODE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SUBWINDOW_TRACE_MODE,
            "Subwindow", subwindow , RS_REPCAP_BUF_SIZE, repCap_subwindow),
            3, "Subwindow");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SUBWINDOW_TRACE_MODE,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCap_trace),
            4, "Trace");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s,%s",
        repCap_window, repCap_subwindow, repCap_trace);

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_SUBWINDOW_TRACE_MODE, traceMode),
            4, "Trace Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures trace detector.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR trace/Selects the trace.
/// HIPAR traceDetectorAutoEnabled/Couples and decouples the detector to the trace mode.
/// HIPAR traceDetector/Defines the trace detector to be used for trace analysis. This control
/// HIPAR traceDetector/has effect only if Trace Detector Auto Enabled == Off.
ViStatus _VI_FUNC rsfpl_ConfigureTraceDetector(ViSession instrSession,
                                               ViInt32   window,
                                               ViInt32   trace,
                                               ViBoolean traceDetectorAutoEnabled,
                                               ViInt32   traceDetector)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_trace[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_TRACE_DETECTOR_AUTO_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_TRACE_DETECTOR_AUTO_ENABLED,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCap_trace),
            3, "Trace");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s",
        repCap_window, repCap_trace);

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_TRACE_DETECTOR_AUTO_ENABLED, traceDetectorAutoEnabled),
            4, "Trace Detector Auto Enabled");

    if (!traceDetectorAutoEnabled)
    {
        viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_TRACE_DETECTOR, traceDetector),
                5, "Trace Detector");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns averaging for a particular trace in a particular window on and
/// HIFN  off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR trace/Selects the trace.
/// HIPAR traceAverageEnabled/Turns averaging for a particular trace in a particular window on and
/// HIPAR traceAverageEnabled/off.
ViStatus _VI_FUNC rsfpl_ConfigureTraceAverageEnabled(ViSession instrSession,
                                                     ViInt32   window,
                                                     ViInt32   trace,
                                                     ViBoolean traceAverageEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_trace[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_TRACE_AVERAGE_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_TRACE_AVERAGE_ENABLED,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCap_trace),
            3, "Trace");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s",
        repCap_window, repCap_trace);

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_TRACE_AVERAGE_ENABLED, traceAverageEnabled),
            4, "Trace Average Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the number of sweeps that the application uses to average
/// HIFN  traces.
/// HIFN     In case of continuous sweep mode, the application calculates the
/// HIFN  moving average over the average count.
/// HIFN     In case of single sweep mode, the application stops the measurement
/// HIFN  and calculates the average after the average count has been reached.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR traceAverageCount/Defines the number of sweeps that the application uses to average
/// HIPAR traceAverageCount/traces.
ViStatus _VI_FUNC rsfpl_ConfigureTraceAverageCount(ViSession instrSession,
                                                   ViInt32   traceAverageCount)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_TRACE_AVERAGE_COUNT, traceAverageCount),
            2, "Trace Average Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the trace averaging mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR traceAverageMode/Selects the trace averaging mode.
ViStatus _VI_FUNC rsfpl_ConfigureTraceAverageMode(ViSession instrSession,
                                                  ViInt32   window,
                                                  ViInt32   traceAverageMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_TRACE_AVERAGE_MODE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");
    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_TRACE_AVERAGE_MODE, traceAverageMode),
            3, "Trace Average Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures trace smoothing for a particular trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR traceSmoothingEnabled/Turns trace smoothing for a particular trace on and off.
/// HIPAR window/This control selects the measurement window.
/// HIPAR trace/Selects the trace.
/// HIPAR apertureValue/Defines the degree (aperture) of the trace smoothing. This control has
/// HIPAR apertureValue/effect only if Trace Smoothing Enabled == On.
ViStatus _VI_FUNC rsfpl_ConfigureTraceSmoothing(ViSession instrSession,
                                                ViInt32   window,
                                                ViInt32   trace,
                                                ViBoolean traceSmoothingEnabled,
                                                ViInt32   apertureValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_trace[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_TRACE_SMOOTHING_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_TRACE_SMOOTHING_ENABLED,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCap_trace),
            3, "Trace");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s",
        repCap_window, repCap_trace);

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_TRACE_SMOOTHING_ENABLED, traceSmoothingEnabled),
            4, "Trace Smoothing Enabled");
    if (traceSmoothingEnabled)
    {
        viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_TRACE_SMOOTHING_VALUE, apertureValue),
                5, "Aperture Value");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns an automatic reset of a trace on and off after a parameter has
/// HIFN  changed.
/// HIFN     The reset works for trace modes min hold, max hold and average.
/// HIFN     Note that the attribute has no effect if critical parameters like the
/// HIFN  span have been changed to avoid invalid measurement results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR trace/Selects the trace.
/// HIPAR traceHoldEnabled/Turns an automatic reset of a trace on and off after a parameter has
/// HIPAR traceHoldEnabled/changed.
ViStatus _VI_FUNC rsfpl_ConfigureTraceHoldEnabled(ViSession instrSession,
                                                  ViInt32   window,
                                                  ViInt32   trace,
                                                  ViBoolean traceHoldEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_trace[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_TRACE_HOLD_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_TRACE_HOLD_ENABLED,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCap_trace),
            3, "Trace");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap_window, repCap_trace);

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_TRACE_HOLD_ENABLED, traceHoldEnabled),
            3, "Trace Hold Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Copies data from one trace to another.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR destinationTrace/Selects the destination trace.
/// HIPAR sourceTrace/Selects the source trace.
ViStatus _VI_FUNC rsfpl_CopyTrace(ViSession instrSession,
                                  ViInt32   window,
                                  ViInt32   destinationTrace,
                                  ViInt32   sourceTrace)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, destinationTrace, 1, 6),
            3, "Destination Trace");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, sourceTrace, 1, 6),
            4, "Source Trace");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRAC%ld:COPY TRACE%ld,TRACE%ld", window, destinationTrace, sourceTrace);
    checkErr(rsfpl_WriteCommandWithOPCSync(instrSession, cmd, 0));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures subtraction of traces.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR position/Defines the position of the trace resulting from the mathematical
/// HIPAR position/operation.
/// HIPAR traceMathEnabled/Turns the trace mathematics on and off.
/// HIPAR mode/Selects the way the FPL calculates trace mathematics.
ViStatus _VI_FUNC rsfpl_ConfigureSubtractTraces(ViSession instrSession,
                                                ViInt32   window,
                                                ViReal64  position,
                                                ViBoolean traceMathEnabled,
                                                ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_TRACE_MATHEMATICS_POSITION,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");
    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_TRACE_MATHEMATICS_POSITION, position),
            3, "Position");

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_TRACE_MATHEMATICS_ENABLED, traceMathEnabled),
            4, "Trace Math Enabled");

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_TRACE_MATHEMATICS_MODE, mode),
            5, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This method subtracts the array elements of trace defined in argument
/// HIFN  Trace 2 from Trace 1 of the instrument and stores the result in the
/// HIFN  Trace 1.
/// HIFN
/// HIFN     Destination Trace = TRACE1 - {trace 2}
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR trace2/Selects the trace for subtraction.
ViStatus _VI_FUNC rsfpl_SubtractTraces(ViSession instrSession,
                                       ViInt32   window,
                                       ViInt32   trace2)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 1, 6),
            3, "Trace 2");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:MATH (TRACE1-TRACE%ld)", window, trace2);
    checkErr(rsfpl_WriteCommandWithOPCSync(instrSession, cmd, 0));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Applies predefined, commonly required trace settings to the selected
/// HIFN  window.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR trace/Selects the trace.
/// HIPAR presetTraces/Applies predefined, commonly required trace settings to the selected
/// HIPAR presetTraces/window.
ViStatus _VI_FUNC rsfpl_ApplyWindowPresetTraces(ViSession instrSession,
                                                ViInt32   window,
                                                ViInt32   trace,
                                                ViInt32   presetTraces)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_trace[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_DISPLAY_WINDOW_PRESET_TRACES,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_DISPLAY_WINDOW_PRESET_TRACES,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCap_trace),
            3, "Trace");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap_window, repCap_trace);

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_DISPLAY_WINDOW_PRESET_TRACES, presetTraces),
            4, "Preset Traces");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns a trace on and off. The measurement continues in the background.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR subwindow/This control selects the measurement subwindow.
/// HIPAR trace/Selects the trace.
/// HIPAR enabled/Turns a trace on and off. The measurement continues in the background.
ViStatus _VI_FUNC rsfpl_ConfigureSubwindowTraceEnabled(ViSession instrSession,
                                                       ViInt32   window,
                                                       ViInt32   subwindow,
                                                       ViInt32   trace,
                                                       ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_subwindow[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_trace[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SUBWINDOW_TRACE_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SUBWINDOW_TRACE_ENABLED,
            "Subwindow", subwindow , RS_REPCAP_BUF_SIZE, repCap_subwindow),
            3, "Subwindow");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SUBWINDOW_TRACE_ENABLED,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCap_trace),
            4, "Trace");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s,%s",
        repCap_window, repCap_subwindow, repCap_trace);

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_SUBWINDOW_TRACE_ENABLED, enabled),
            5, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns an automatic reset of a trace on and off after a parameter has
/// HIFN  changed.
/// HIFN     The reset works for trace modes min hold, max hold and average.
/// HIFN     Note that this has no effect if critical parameters like the
/// HIFN  span have been changed to avoid invalid measurement results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR subwindow/This control selects the measurement subwindow.
/// HIPAR trace/Selects the trace.
/// HIPAR holdEnabled/Turns an automatic reset of a trace on and off after a parameter has
/// HIPAR holdEnabled/changed.
ViStatus _VI_FUNC rsfpl_ConfigureSubwindowTraceHold(ViSession instrSession,
                                                    ViInt32   window,
                                                    ViInt32   subwindow,
                                                    ViInt32   trace,
                                                    ViBoolean holdEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_subwindow[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_trace[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SUBWINDOW_TRACE_HOLD_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SUBWINDOW_TRACE_HOLD_ENABLED,
            "Subwindow", subwindow , RS_REPCAP_BUF_SIZE, repCap_subwindow),
            3, "Subwindow");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SUBWINDOW_TRACE_HOLD_ENABLED,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCap_trace),
            4, "Trace");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s,%s",
        repCap_window, repCap_subwindow, repCap_trace);

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_SUBWINDOW_TRACE_HOLD_ENABLED, holdEnabled),
            5, "Hold Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures trace smoothing for a particular trace. If enabled, the
/// HIFN  trace is smoothed by the smoothing value specified.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR subwindow/This control selects the measurement subwindow.
/// HIPAR trace/Selects the trace.
/// HIPAR smoothingEnabled/Turns trace smoothing for a particular trace on and off. If enabled,
/// HIPAR smoothingEnabled/the trace is smoothed by the value specified using aperture value.
/// HIPAR smoothingValue/Defines the degree (aperture) of the trace smoothing.
ViStatus _VI_FUNC rsfpl_ConfigureSubwindowTraceSmoothing(ViSession instrSession,
                                                         ViInt32   window,
                                                         ViInt32   subwindow,
                                                         ViInt32   trace,
                                                         ViBoolean smoothingEnabled,
                                                         ViInt32   smoothingValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_subwindow[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_trace[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SUBWINDOW_TRACE_SMOOTHING_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SUBWINDOW_TRACE_SMOOTHING_ENABLED,
            "Subwindow", subwindow , RS_REPCAP_BUF_SIZE, repCap_subwindow),
            3, "Subwindow");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SUBWINDOW_TRACE_SMOOTHING_ENABLED,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCap_trace),
            4, "Trace");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s,%s",
        repCap_window, repCap_subwindow, repCap_trace);

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_SUBWINDOW_TRACE_SMOOTHING_ENABLED, smoothingEnabled),
            5, "Smoothing Enabled");

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_SUBWINDOW_TRACE_SMOOTHING_VALUE, smoothingValue),
            6, "Smoothing Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Applies predefined, commonly required trace settings to the selected
/// HIFN  subwindow.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR subwindow/This control selects the measurement subwindow.
/// HIPAR trace/Selects the trace.
/// HIPAR presetTraces/Applies predefined, commonly required trace settings to the selected
/// HIPAR presetTraces/subwindow.
ViStatus _VI_FUNC rsfpl_ApplySubwindowPresetTraces(ViSession instrSession,
                                                   ViInt32   window,
                                                   ViInt32   subwindow,
                                                   ViInt32   trace,
                                                   ViInt32   presetTraces)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_subwindow[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_trace[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_DISPLAY_SUBWINDOW_PRESET_TRACES,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_DISPLAY_SUBWINDOW_PRESET_TRACES,
            "Subwindow", subwindow , RS_REPCAP_BUF_SIZE, repCap_subwindow),
            3, "Subwindow");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_DISPLAY_SUBWINDOW_PRESET_TRACES,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCap_trace),
            4, "Trace");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s,%s", repCap_window, repCap_subwindow, repCap_trace);

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_DISPLAY_SUBWINDOW_PRESET_TRACES, presetTraces),
            5, "Preset Traces");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns the spectrogram on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR spectrogramEnabled/Turns the spectrogram on and off.
ViStatus _VI_FUNC rsfpl_ConfigureSpectrogramEnabled(ViSession instrSession,
                                                    ViBoolean spectrogramEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_SPECTROGRAM_ENABLED, spectrogramEnabled),
            2, "Spectrogram Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates or deactivates a 3-dimensional spectrogram for the selected
/// HIFN  result display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR 3dEnabled/Activates or deactivates a 3-dimensional spectrogram for the selected
/// HIPAR 3dEnabled/result display.
ViStatus _VI_FUNC rsfpl_ConfigureSpectrogram3DEnabled(ViSession instrSession,
                                                      ViBoolean _3DEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_SPECTROGRAM_3D_ENABLED, _3DEnabled),
            2, "3D Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates and deactivates the time stamp.
/// HIFN     If the time stamp is active, some commands do not address frames as
/// HIFN  numbers, but as (relative) time values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR spectrogramTimeStampEnabled/Activates and deactivates the time stamp.
ViStatus _VI_FUNC rsfpl_ConfigureSpectrogramTimeStampEnabled(ViSession instrSession,
                                                             ViBoolean spectrogramTimeStampEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_SPECTROGRAM_TIME_STAMP_ENABLED, spectrogramTimeStampEnabled),
            2, "Spectrogram Time Stamp Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Time stamp is off: Selects a specific frame for further analysis. The
/// HIFN  range depends on the history depth.
/// HIFN     Time stamp is on: Selects a frame via its time stamp. The number is
/// HIFN  the distance to frame 0 in seconds. The range depends on the history
/// HIFN  depth.
/// HIFN     The attribute is available if no measurement is running or after a
/// HIFN  single sweep has ended.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR spectrogramSelectedFrame/Time stamp is off: Selects a specific frame for further analysis. The
/// HIPAR spectrogramSelectedFrame/range depends on the history depth.
ViStatus _VI_FUNC rsfpl_ConfigureSpectrogramSelectedFrame(ViSession instrSession,
                                                          ViInt32   spectrogramSelectedFrame)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_SPECTROGRAM_SELECTED_FRAME, spectrogramSelectedFrame),
            2, "Spectrogram Selected Frame");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the number of frames to be stored in the FPL memory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR spectrogramHistoryDepth/Defines the number of frames to be stored in the FPL memory.
ViStatus _VI_FUNC rsfpl_ConfigureSpectrogramHistoryDepth(ViSession instrSession,
                                                         ViInt32   spectrogramHistoryDepth)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_SPECTROGRAM_HISTORY_DEPTH, spectrogramHistoryDepth),
            2, "Spectrogram History Depth");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures spectrogram sweep.
/// HIFN     This setting applies to all spectrograms in the channel setup.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR continousSweepEnabled/Determines whether the results of the last measurement are deleted
/// HIPAR continousSweepEnabled/before starting a new measurement in single sweep mode.
/// HIPAR numberOfFrames/Defines the number of frames to be recorded in a single sweep.
ViStatus _VI_FUNC rsfpl_ConfigureSpectrogramSweep(ViSession instrSession,
                                                  ViBoolean continousSweepEnabled,
                                                  ViInt32   numberOfFrames)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_SPECTROGRAM_CONTINOUS_SWEEP_ENABLED, continousSweepEnabled),
            2, "Continous Sweep Enabled");
    if (!continousSweepEnabled)
    {
        viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_SPECTROGRAM_NUMBER_OF_FRAMES, numberOfFrames),
                3, "Number Of Frames");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the number of frames to be recorded in a single sweep.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR spectrogramNumberOfFrames/Determines whether the results of the last measurement are deleted
ViStatus _VI_FUNC rsfpl_SpectrogramNumberOfFrames(ViSession instrSession,
                                                  ViInt32   spectrogramNumberOfFrames)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_SPECTROGRAM_NUMBER_OF_FRAMES, spectrogramNumberOfFrames),
            2, "Spectrogram Number Of Frames");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the starting time of the current frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR seconds/Number of seconds that have passed since 01.01.1970 till the frame
/// HIPAR seconds/start.
/// HIPAR nanoseconds/Number of nanoseconds that have passed in addition to the <Seconds>
/// HIPAR nanoseconds/since 01.01.1970 till the frame start.
/// HIPAR reserved1/Reserved for future use.
/// HIPAR reserved2/Reserved for future use.
ViStatus _VI_FUNC rsfpl_QuerySpectrogramTimeStampData(ViSession instrSession,
                                                      ViInt32*  seconds,
                                                      ViInt32*  nanoseconds,
                                                      ViInt32*  reserved1,
                                                      ViInt32*  reserved2)
{
    ViStatus error = VI_SUCCESS;
             reserved1;
             reserved2;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsfpl_QueryViString(instrSession,
        "CALC:SGR:TST:DATA? CURR",
        RS_MAX_MESSAGE_BUF_SIZE,
        gsCommand,
        NULL));
    (void)sscanf(gsCommand, "%ld,%ld", seconds, nanoseconds);

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the starting time of the frames.
/// HIFN     The return values consist of four values for each frame. If the
/// HIFN  Spectrogram is empty, the method returns '0,0,0,0'. The times are given
/// HIFN  as delta values, which simplifies evaluating relative results; however,
/// HIFN  you can also calculate the absolute date and time as displayed on the
/// HIFN  screen.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR seconds/Number of seconds that have passed since 01.01.1970 till the frame
/// HIPAR seconds/start.
/// HIPAR nanoseconds/Number of nanoseconds that have passed in addition to the <Seconds>
/// HIPAR nanoseconds/since 01.01.1970 till the frame start.
/// HIPAR reserved1/Reserved for future use. Can be NULL.
/// HIPAR reserved2/Reserved for future use. Can be NULL.
ViStatus _VI_FUNC rsfpl_QuerySpectrogramTimeStampDataAll(ViSession instrSession,
                                                         ViUInt32  arraySize,
                                                         ViInt32   seconds[],
                                                         ViInt32   nanoseconds[],
                                                         ViInt32   reserved1[],
                                                         ViInt32   reserved2[])
{
    ViStatus error = VI_SUCCESS;
    ViChar*  buffer = NULL;
    ViChar*  p2uff;
    ViUInt32 i = 0;
             reserved1;
             reserved2;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_QueryViStringUnknownLength(instrSession, "CALC:SGR:TST:DATA? ALL", &buffer));
    checkErr(rsfpl_CheckStatus(instrSession));

    p2uff = strtok(buffer, ",");
    do
    {
        seconds[i] = (ViInt32)RsCore_Convert_String_To_ViInt32(p2uff);
        p2uff = strtok(NULL, ",");
        nanoseconds[i] = (ViInt32)RsCore_Convert_String_To_ViInt32(p2uff);
        (void)strtok(NULL, ",");
        (void)strtok(NULL, ",\n");
        p2uff = strtok(NULL, ",\n");
        i++;
    } while ((p2uff != NULL) && (i < arraySize));

Error:
    if (buffer) free(buffer);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Resets the spectrogram and clears the history buffer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_SpectrogramClear(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_SPECTROGRAM_CLEAR, NULL),
            2, "Spectrogram Clear");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the state and size of spectrograms.
/// HIFN     The function is available for result displays that support
/// HIFN  spectrograms.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR spectrogramLayout/Selects the state and size of spectrograms.
ViStatus _VI_FUNC rsfpl_ConfigureSpectrogramLayout(ViSession instrSession,
                                                   ViInt32   window,
                                                   ViInt32   spectrogramLayout)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SPECTROGRAM_LAYOUT,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");
    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_SPECTROGRAM_LAYOUT, spectrogramLayout),
            3, "Spectrogram Layout");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the spectrum diagram trace on which the spectrogram is based.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trace/Selects the spectrum diagram trace on which the spectrogram is based.
ViStatus _VI_FUNC rsfpl_ConfigureSpectrogramSelectTrace(ViSession instrSession,
                                                        ViInt32   window,
                                                        ViInt32   trace)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SPECTROGRAM_SELECT_TRACE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_SPECTROGRAM_SELECT_TRACE, trace),
            3, "Trace");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Restores the original color map.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
ViStatus _VI_FUNC rsfpl_ColorMapRestoreDefault(ViSession instrSession,
                                               ViInt32   window)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_COLOR_MAP_RESTORE_DEFAULT,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_COLOR_MAP_RESTORE_DEFAULT, NULL),
            3, "Color Map Restore Default");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the color scheme.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR colorMapScheme/Selects the color scheme.
ViStatus _VI_FUNC rsfpl_ConfigureColorMapScheme(ViSession instrSession,
                                                ViInt32   window,
                                                ViInt32   colorMapScheme)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_COLOR_MAP_SCHEME,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_COLOR_MAP_SCHEME, colorMapScheme),
            3, "Color Map Scheme");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the the color map.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR startingPoint/Defines the starting point of the color map.
/// HIPAR window/This control selects the measurement window.
/// HIPAR endPoint/Defines the end point of the color map.
/// HIPAR curveShape/Defines the shape and focus of the color curve for the spectrogram
/// HIPAR curveShape/result display.
ViStatus _VI_FUNC rsfpl_ConfigureColorMap(ViSession instrSession,
                                          ViInt32   window,
                                          ViReal64  startingPoint,
                                          ViReal64  endPoint,
                                          ViReal64  curveShape)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_COLOR_MAP_STARTING_POINT,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");
    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_COLOR_MAP_STARTING_POINT, startingPoint),
            3, "Starting Point");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_COLOR_MAP_END_POINT, endPoint),
            4, "End Point");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_COLOR_MAP_CURVE_SHAPE, curveShape),
            5, "Curve Shape");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This command sets the focus on the selected result display window.
/// HIFN  This window is then the active window. Use this command to select the
/// HIFN  window before querying trace data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
ViStatus _VI_FUNC rsfpl_WindowSetFocus(ViSession instrSession,
                                       ViInt32   window)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_DISPLAY_WINDOW_SET_FOCUS,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    checkErr(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_DISPLAY_WINDOW_SET_FOCUS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This command sets the focus on the selected result display window and
/// HIFN  subwindow. This subwindow is then the active window. Use this command
/// HIFN  to select the subwindow before querying trace data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR subwindow/This control selects the measurement subwindow.
ViStatus _VI_FUNC rsfpl_SubwindowSetFocus(ViSession instrSession,
                                          ViInt32   window,
                                          ViInt32   subwindow)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_subwindow[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_DISPLAY_SUBWINDOW_SET_FOCUS,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_DISPLAY_SUBWINDOW_SET_FOCUS,
            "Subwindow", subwindow , RS_REPCAP_BUF_SIZE, repCap_subwindow),
            3, "Subwindow");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap_window, repCap_subwindow);

    checkErr(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_DISPLAY_SUBWINDOW_SET_FOCUS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the display settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR statusBarEnabled/Turns the status bar on and off.
/// HIPAR softkeyBarEnabled/Turns the softkey bar on and off.
/// HIPAR toolbarEnabled/Turns the toolbar on or off.
/// HIPAR touchEnabled/Controls the touch screen functionality.
/// HIPAR channelInformationEnabled/Hides or displays the channel bar information.
/// HIPAR xAxisLabelEnabled/Turns the label of the x-axis on and off.
/// HIPAR dateAndTimeEnabled/Adds or removes the date and time from the display.
/// HIPAR frontPanelKeysEnabled/Includes or excludes the front panel keys when working with the remote
/// HIPAR frontPanelKeysEnabled/desktop.
ViStatus _VI_FUNC rsfpl_ConfigureDisplay(ViSession instrSession,
                                         ViBoolean statusBarEnabled,
                                         ViBoolean softkeyBarEnabled,
                                         ViBoolean toolbarEnabled,
                                         ViInt32   touchEnabled,
                                         ViBoolean channelInformationEnabled,
                                         ViBoolean xAxisLabelEnabled,
                                         ViBoolean dateAndTimeEnabled,
                                         ViBoolean frontPanelKeysEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_DISPLAY_STATUS_BAR_ENABLED, statusBarEnabled),
            2, "Status Bar Enabled");

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_DISPLAY_SOFTKEY_BAR_ENABLED, softkeyBarEnabled),
            3, "Softkey Bar Enabled");

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_DISPLAY_TOOLBAR_ENABLED, toolbarEnabled),
            4, "Toolbar Enabled");

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_DISPLAY_TOUCH_ENABLED, touchEnabled),
            5, "Touch Enabled");

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_DISPLAY_CHANNEL_BAR_INFORMATION_ENABLED, channelInformationEnabled),
            6, "Channel Information Enabled");

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_DISPLAY_X_AXIS_LABEL_ENABLED, xAxisLabelEnabled),
            7, "X Axis Label Enabled");

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_DISPLAY_DATE_AND_TIME_ENABLED, dateAndTimeEnabled),
            8, "Date And Time Enabled");

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_DISPLAY_FRONT_PANEL_KEYS_ENABLED, frontPanelKeysEnabled),
            9, "Front Panel Keys Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Changes the brightness of the display in eight steps.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR displayBrightness/Changes the brightness of the display in eight steps.
ViStatus _VI_FUNC rsfpl_ConfigureDisplayBrightness(ViSession instrSession,
                                                   ViInt32   displayBrightness)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_DISPLAY_BRIGHTNESS, displayBrightness),
            2, "Display Brightness");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the time and date format.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR dateAndTimeFormat/Selects the time and date format.
ViStatus _VI_FUNC rsfpl_ConfigureDisplayDateAndTimeFormat(ViSession instrSession,
                                                          ViInt32   dateAndTimeFormat)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_DISPLAY_DATE_AND_TIME_FORMAT, dateAndTimeFormat),
            2, "Date And Time Format");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the time and date format.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR displayTemporarySize/Maximizes the size of the selected result display window temporarily.
ViStatus _VI_FUNC rsfpl_ConfigureDisplayTemporarySize(ViSession instrSession,
                                                      ViInt32   displayTemporarySize)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_DISPLAY_TEMPORARY_SIZE, displayTemporarySize),
            2, "Display Temporary Size");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Determines which tab is displayed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR displayFormat/Determines which tab is displayed.
ViStatus _VI_FUNC rsfpl_ConfigureDisplayFormat(ViSession instrSession,
                                               ViInt32   displayFormat)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_DISPLAY_FORMAT, displayFormat),
            2, "Display Format");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries all available display themes.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR arraySize/Defines the memory size allocated for Catalog.
/// HIPAR catalog/Queries all available display themes.
/// HIPAR returnedValues/Returns number of returned values.
ViStatus _VI_FUNC rsfpl_QueryColorsAndThemesCatalog(ViSession instrSession,
                                                    ViUInt32  arraySize,
                                                    ViChar    catalog[],
                                                    ViUInt32* returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsfpl_QueryViString(instrSession, "DISP:THEM:CAT?", arraySize, catalog, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the display theme.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR andThemesSelectedTheme/Selects the display theme.
ViStatus _VI_FUNC rsfpl_ConfigureColorsAndThemesSelectedTheme(
                            ViSession instrSession,
                            ViString  selectedTheme)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_COLORS_AND_THEMES_SELECTED_THEME, selectedTheme),
            2, "Selected Theme");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Resets the color scheme for the display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR color/Selects the color.
ViStatus _VI_FUNC rsfpl_ResetColorScheme(ViSession instrSession,
                                         ViInt32   color)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, color, 1, 4),
            2, "Color");
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DISP:CMAP:DEF%ld", color);
    checkErr(rsfpl_WriteCommandWithOPCSync(instrSession, cmd, 0));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Adds a window to the display in the active channel setup.
/// HIFN     Is always used as a query so that you immediately obtain the name of
/// HIFN  the new window as a result.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR windowName/String containing the name of the existing window the new window is
/// HIPAR windowName/inserted next to.
/// HIPAR direction/Direction the new window is added relative to the existing window.
/// HIPAR windowType/Type of result display (evaluation method) you want to add.
ViStatus _VI_FUNC rsfpl_LayoutAddWindow(ViSession instrSession,
                                        ViString  windowName,
                                        ViInt32   direction,
                                        ViInt32   windowType,
                                        ViUInt32  arraySize,
                                        ViChar    returnedWindowName[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   response[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2direction;
    ViChar*  p2type;

    if (direction == RSFPL_VAL_DIRECTION_REPLACE)
    {
        checkErr(rsfpl_LayoutReplaceWindow(instrSession, windowName, windowType));
    }
    else
    {


        checkErr(RsCore_LockSession(instrSession));

        viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsFpl_rngLayoutDirection, direction, &p2direction),
            3, "Direction");
        viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsFpl_rngLayoutWindowType, windowType, &p2type),
            4, "Window Type");

        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:ADD? '%s',%s,%s", windowName, p2direction, p2type);
        checkErr(RsCore_QueryViString(instrSession, cmd, response));
        checkErr(rsfpl_CheckStatus(instrSession));

        if (returnedWindowName != NULL)
        {
            RsCore_TrimString(response, RS_VAL_TRIM_WHITESPACES_AND_ALL_QUOTES);
            strncpy(returnedWindowName, response, arraySize);
        }
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Replaces the window type (for example from 'Diagram' to 'Result
/// HIFN  Summary') of an already existing window in the active channel setup
/// HIFN  while keeping its position, index and window name.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR windowName/String containing the name of the existing window the new window is
/// HIPAR windowName/inserted next to.
/// HIPAR windowType/Type of result display (evaluation method) you want to replace.
ViStatus _VI_FUNC rsfpl_LayoutReplaceWindow(ViSession instrSession,
                                            ViString  windowName,
                                            ViInt32   windowType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2type;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsFpl_rngLayoutWindowType, windowType, &p2type),
        3, "Window Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:REPL '%s',%s", windowName, p2type);
    checkErr(rsfpl_WriteCommandWithOPCSync(instrSession, cmd, 0));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves a window in the active channel setup.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR sourceWindowName/String containing the name of an existing window that is to be moved.
/// HIPAR destinationWindowName/String containing the name of an existing window the selected window
/// HIPAR destinationWindowName/is placed next to or replaces.
/// HIPAR direction/Direction the new window is added relative to the existing window.
ViStatus _VI_FUNC rsfpl_LayoutMoveWindow(ViSession instrSession,
                                         ViString  sourceWindowName,
                                         ViString  destinationWindowName,
                                         ViInt32   direction)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2direction;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsFpl_rngLayoutDirection, direction, &p2direction),
        3, "Direction");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:MOVE '%s','%s',%s", sourceWindowName,
        destinationWindowName,
        p2direction);
    checkErr(rsfpl_WriteCommandWithOPCSync(instrSession, cmd, 0));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Removes a window from the display in the active channel setup.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Removes a window from the display in the active channel setup.
ViStatus _VI_FUNC rsfpl_LayoutRemoveWindow(ViSession instrSession,
                                           ViString  window)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:REM '%s'", window);
    checkErr(rsfpl_WriteCommandWithOPCSync(instrSession, cmd, 0));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the name and index of all active windows in the active channel
/// HIFN  setup from top left to bottom right. The result is a comma-separated
/// HIFN  list of values for each window, with the syntax:
/// HIFN     <WindowName_1>,<WindowIndex_1>..<WindowName_n>,<WindowIndex_n>
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR catalog/Queries all available display themes.
/// HIPAR arraySize/Defines the memory size allocated for Catalog.
/// HIPAR returnedValues/Returns number of returned values.
ViStatus _VI_FUNC rsfpl_QueryLayoutWindowList(ViSession instrSession,
                                              ViUInt32  arraySize,
                                              ViChar    catalog[],
                                              ViUInt32* returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsfpl_QueryViString(instrSession, "LAY:CAT?", arraySize, catalog, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the index of a particular display window in the active channel
/// HIFN  setup.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR windowName/String containing the name of the existing window the new window is
/// HIPAR windowName/inserted next to.
/// HIPAR index/Queries the index of a particular display window in the active channel
/// HIPAR index/setup.
ViStatus _VI_FUNC rsfpl_LayoutIdentifyWindow(ViSession instrSession,
                                             ViString  windowName,
                                             ViInt32*  index)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:IDEN? '%s'", windowName);
    checkErr(rsfpl_QueryViInt32(instrSession, cmd, index));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This method changes the position of a splitter and thus controls the
/// HIFN  size of the windows on each side of the splitter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window1/String containing the number or name of the first window the splitter
/// HIPAR window1/controls.
/// HIPAR window2/String containing the number or name of the second window the splitter
/// HIPAR window2/controls.
/// HIPAR position/New vertical or horizontal position of the splitter in percent of the
/// HIPAR position/screen area (without channel and status bar and softkey menu). The
/// HIPAR position/point of origin (x = 0%, y = 0%) is in the lower left corner of the
/// HIPAR position/screen. The end point (x = 100%, y = 100%) is in the upper right corner
/// HIPAR position/of the screen.
ViStatus _VI_FUNC rsfpl_ConfigureLayoutSplitterPosition(ViSession instrSession,
                                                        ViString  window1,
                                                        ViString  window2,
                                                        ViInt32   position)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:SPL '%s','%s',%ld", window1, window2, position);
    checkErr(rsfpl_WriteCommandWithOPCSync(instrSession, cmd, 0));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries or defines the window type of the window specified by the
/// HIFN  index <n>. The window type determines which results are displayed. For
/// HIFN  a list of possible window types see LAYout:ADD[:WINDow]?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR type/Queries or defines the window type of the window specified by the
/// HIPAR type/index <n>. The window type determines which results are displayed. For
/// HIPAR type/a list of possible window types see LAYout:ADD[:WINDow]?
ViStatus _VI_FUNC rsfpl_ConfigureLayoutWindowType(ViSession instrSession,
                                                  ViInt32   window,
                                                  ViString  type)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_LAYOUT_WINDOW_TYPE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_LAYOUT_WINDOW_TYPE, type),
            3, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns the zoom on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR displaySingleZoomEnabled/Turns the zoom on and off.
ViStatus _VI_FUNC rsfpl_ConfigureDisplaySingleZoomEnabled(ViSession instrSession,
                                                          ViInt32   window,
                                                          ViBoolean displaySingleZoomEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_DISPLAY_SINGLE_ZOOM_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");
    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_DISPLAY_SINGLE_ZOOM_ENABLED, displaySingleZoomEnabled),
            3, "Display Single Zoom Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the zoom area.
/// HIFN     To define a zoom area, you first have to turn the zoom on. The lower
/// HIFN  left corner is the origin of coordinate system. The upper right corner
/// HIFN  is the end point of the system.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR lowerLeftX/Diagram coordinates in % of the complete diagram that define the zoom
/// HIPAR lowerLeftX/area. Lower left corner X coordinate.
/// HIPAR lowerLeftY/Diagram coordinates in % of the complete diagram that define the zoom
/// HIPAR lowerLeftY/area. Lower left corner Y coordinate.
/// HIPAR upperRightX/Diagram coordinates in % of the complete diagram that define the zoom
/// HIPAR upperRightX/area. Upper right corner X coordinate.
/// HIPAR upperRightY/Diagram coordinates in % of the complete diagram that define the zoom
/// HIPAR upperRightY/area. Upper right corner Y coordinate.
ViStatus _VI_FUNC rsfpl_ConfigureSingleZoom(ViSession instrSession,
                                            ViInt32   window,
                                            ViInt32   lowerLeftX,
                                            ViInt32   lowerLeftY,
                                            ViInt32   upperRightX,
                                            ViInt32   upperRightY)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, lowerLeftX, 0, 100),
            3, "Lower Left X");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, lowerLeftY, 0, 100),
            4, "Lower Left Y");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, upperRightX, 0, 100),
            5, "Upper Right X");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, upperRightY, 0, 100),
            6, "Upper Right Y");

    if (window == 0)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DISP:ZOOM:AREA %ld,%ld,%ld,%ld", lowerLeftX,
            lowerLeftY,
            upperRightX,
            upperRightY);
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DISP:WIND%ld:ZOOM:AREA %ld,%ld,%ld,%ld", window,
            lowerLeftX,
            lowerLeftY,
            upperRightX,
            upperRightY);
    }

    checkErr(rsfpl_WriteCommandWithOPCSync(instrSession, cmd, 0));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns the zoom on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR subwindow/This control selects the measurement subwindow.
/// HIPAR displaySingleZoomEnabled/Turns the zoom on and off.
ViStatus _VI_FUNC rsfpl_ConfigureSubwindowDisplaySingleZoomEnabled(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   subwindow,
                            ViBoolean displaySingleZoomEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_subwindow[RS_REPCAP_BUF_SIZE] = "";

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SUBWINDOW_DISPLAY_SINGLE_ZOOM_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SUBWINDOW_DISPLAY_SINGLE_ZOOM_ENABLED,
            "Subwindow", subwindow , RS_REPCAP_BUF_SIZE, repCap_subwindow),
            3, "Subwindow");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap_window,
        repCap_subwindow);


    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_SUBWINDOW_DISPLAY_SINGLE_ZOOM_ENABLED, displaySingleZoomEnabled),
            4, "Display Single Zoom Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the zoom area. To define a zoom area, you first have to turn
/// HIFN  the zoom on. The lower left corner is the origin of coordinate system.
/// HIFN  The upper right corner is the end point of the system.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR subwindow/This control selects the measurement subwindow.
/// HIPAR lowerLeftX/Diagram coordinates in % of the complete diagram that define the zoom
/// HIPAR lowerLeftX/area. Lower left corner X coordinate.
/// HIPAR lowerLeftY/Diagram coordinates in % of the complete diagram that define the zoom
/// HIPAR lowerLeftY/area. Lower left corner Y coordinate.
/// HIPAR upperRightX/Diagram coordinates in % of the complete diagram that define the zoom
/// HIPAR upperRightX/area. Upper right corner X coordinate.
/// HIPAR upperRightY/Diagram coordinates in % of the complete diagram that define the zoom
/// HIPAR upperRightY/area. Upper right corner Y coordinate.
ViStatus _VI_FUNC rsfpl_ConfigureSubwindowSingleZoom(ViSession instrSession,
                                                     ViInt32   window,
                                                     ViInt32   subwindow,
                                                     ViInt32   lowerLeftX,
                                                     ViInt32   lowerLeftY,
                                                     ViInt32   upperRightX,
                                                     ViInt32   upperRightY)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViInt32  windowCount = 16;
    ViInt32  subwindowCount = 64;
    ViChar   windowCmd[RS_REPCAP_BUF_SIZE] = "";
    ViChar   subwindowCmd[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Window", &windowCount));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, windowCount - 1),
        2, "Window");
    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Subwindow", &subwindowCount));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subwindow, 0, subwindowCount - 1),
        3, "Subwindow");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, lowerLeftX, 0, 100),
            4, "Lower Left X");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, lowerLeftY, 0, 100),
            5, "Lower Left Y");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, upperRightX, 0, 100),
            6, "Upper Right X");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, upperRightY, 0, 100),
            7, "Upper Right Y");

    if (window > 0)
        snprintf(windowCmd, RS_REPCAP_BUF_SIZE, ":WIND%ld", window);
    if (subwindow > 0)
        snprintf(subwindowCmd, RS_REPCAP_BUF_SIZE, ":SUBW%ld", subwindow);

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DISP%s%s:ZOOM:AREA %ld,%ld,%ld,%ld",
            windowCmd, subwindowCmd, lowerLeftX, lowerLeftY, upperRightX, upperRightY);
    checkErr(rsfpl_WriteCommandWithOPCSync(instrSession, cmd, 0));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns the multiple zoom on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR multipleZoomEnabled/Turns the multiple zoom on and off.
ViStatus _VI_FUNC rsfpl_ConfigureMultipleZoomEnabled(ViSession instrSession,
                                                     ViInt32   window,
                                                     ViInt32   zoomWindow,
                                                     ViBoolean multipleZoomEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_zoom[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_MULTIPLE_ZOOM_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_MULTIPLE_ZOOM_ENABLED,
            "ZoomWindow", zoomWindow - 1, RS_REPCAP_BUF_SIZE, repCap_zoom),
            3, "Zoom Window");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s",
        repCap_window, repCap_zoom);

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_MULTIPLE_ZOOM_ENABLED, multipleZoomEnabled),
            3, "Multiple Zoom Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the zoom area for a multiple zoom.
/// HIFN     To define a zoom area, you first have to turn the zoom on. The lower
/// HIFN  left corner is the origin of coordinate system. The upper right corner
/// HIFN  is the end point of the system.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR lowerLeftX/Diagram coordinates in % of the complete diagram that define the zoom
/// HIPAR lowerLeftX/area. Lower left corner X coordinate.
/// HIPAR lowerLeftY/Diagram coordinates in % of the complete diagram that define the zoom
/// HIPAR lowerLeftY/area. Lower left corner Y coordinate.
/// HIPAR upperRightX/Diagram coordinates in % of the complete diagram that define the zoom
/// HIPAR upperRightX/area. Upper right corner X coordinate.
/// HIPAR upperRightY/Diagram coordinates in % of the complete diagram that define the zoom
/// HIPAR upperRightY/area. Upper right corner Y coordinate.
ViStatus _VI_FUNC rsfpl_MultipleZoom(ViSession instrSession,
                                     ViInt32   window,
                                     ViInt32   zoomWindow,
                                     ViInt32   lowerLeftX,
                                     ViInt32   lowerLeftY,
                                     ViInt32   upperRightX,
                                     ViInt32   upperRightY)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, zoomWindow, 1, 4),
            3, "Zoom Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, lowerLeftX, 0, 100),
            4, "Lower Left X");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, lowerLeftY, 0, 100),
            5, "Lower Left Y");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, upperRightX, 0, 100),
            6, "Upper Right X");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, upperRightY, 0, 100),
            7, "Upper Right Y");

    if (window == 0)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DISP:ZOOM:MULT:%ld:AREA %ld,%ld,%ld,%ld", zoomWindow,
            lowerLeftX,
            lowerLeftY,
            upperRightX,
            upperRightY);
    }

    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DISP:WIND%ld:ZOOM:MULT%ld:AREA %ld,%ld,%ld,%ld",
                window, zoomWindow, lowerLeftX, lowerLeftY, upperRightX, upperRightY);
    }

    checkErr(rsfpl_WriteCommandWithOPCSync(instrSession, cmd, 0));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns the multiple zoom on and off for a specified subwindow.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR subwindow/This control selects the measurement subwindow.
/// HIPAR multipleZoomEnabled/Turns the multiple zoom on and off.
ViStatus _VI_FUNC rsfpl_ConfigureSubwindowMultipleZoomEnabled(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   subwindow,
                            ViInt32   zoomWindow,
                            ViBoolean multipleZoomEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_subwindow[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_zoom[RS_REPCAP_BUF_SIZE] = "";

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SUBWINDOW_MULTIPLE_ZOOM_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SUBWINDOW_MULTIPLE_ZOOM_ENABLED,
            "Subwindow", subwindow , RS_REPCAP_BUF_SIZE, repCap_subwindow),
            3, "Subwindow");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SUBWINDOW_MULTIPLE_ZOOM_ENABLED,
            "ZoomWindow", zoomWindow - 1, RS_REPCAP_BUF_SIZE, repCap_zoom),
            4, "Zoom Window");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s,%s", repCap_window,
        repCap_subwindow, repCap_zoom);


    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_SUBWINDOW_MULTIPLE_ZOOM_ENABLED, multipleZoomEnabled),
            5, "Multiple Zoom Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the zoom area. To define a zoom area, you first have to turn
/// HIFN  the zoom on. The lower left corner is the origin of coordinate system.
/// HIFN  The upper right corner is the end point of the system.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR subwindow/This control selects the measurement subwindow.
/// HIPAR zoomWindow/This control selects the zoom window.
/// HIPAR lowerLeftX/Diagram coordinates in % of the complete diagram that define the zoom
/// HIPAR lowerLeftX/area. Lower left corner X coordinate.
/// HIPAR lowerLeftY/Diagram coordinates in % of the complete diagram that define the zoom
/// HIPAR lowerLeftY/area. Lower left corner Y coordinate.
/// HIPAR upperRightX/Diagram coordinates in % of the complete diagram that define the zoom
/// HIPAR upperRightX/area. Upper right corner X coordinate.
/// HIPAR upperRightY/Diagram coordinates in % of the complete diagram that define the zoom
/// HIPAR upperRightY/area. Upper right corner Y coordinate.
ViStatus _VI_FUNC rsfpl_SubwindowMultipleZoom(ViSession instrSession,
                                              ViInt32   window,
                                              ViInt32   subwindow,
                                              ViInt32   zoomWindow,
                                              ViInt32   lowerLeftX,
                                              ViInt32   lowerLeftY,
                                              ViInt32   upperRightX,
                                              ViInt32   upperRightY)
{
    ViStatus error = VI_SUCCESS;
    ViInt32  windowCount = 17;
    ViInt32  subwindowCount = 65;
    ViInt32  zoomWindowMax = 4;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar   windowCmd[RS_REPCAP_BUF_SIZE] = "";
    ViChar   subwindowCmd[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Window", &windowCount));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, windowCount - 1),
        2, "Window");
    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Subwindow", &subwindowCount));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subwindow, 0, subwindowCount - 1),
        3, "Subwindow");
    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "ZoomWindow", &zoomWindowMax));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, zoomWindow, 1, zoomWindowMax),
            4, "Zoom Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, lowerLeftX, 0, 100),
            5, "Lower Left X");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, lowerLeftY, 0, 100),
            6, "Lower Left Y");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, upperRightX, 0, 100),
            7, "Upper Right X");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, upperRightY, 0, 100),
            8, "Upper Right Y");

    if (window > 0)
        snprintf(windowCmd, RS_REPCAP_BUF_SIZE, ":WIND%ld", window);
    if (subwindow > 0)
        snprintf(subwindowCmd, RS_REPCAP_BUF_SIZE, ":SUBW%ld", subwindow);

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DISP%s%s:ZOOM:MULT%ld:AREA %ld,%ld,%ld,%ld", windowCmd, subwindowCmd, zoomWindow, lowerLeftX, lowerLeftY, upperRightX, upperRightY);
    checkErr(rsfpl_WriteCommandWithOPCSync(instrSession, cmd, 0));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures a frequency line.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR line/This control selects the display line.
/// HIPAR displayLineEnabled/Turns a frequency line on and off.
/// HIPAR displayLinePosition/Defines the position of a frequency line.
ViStatus _VI_FUNC rsfpl_ConfigureFrequencyDisplayLine(ViSession instrSession,
                                                      ViInt32   window,
                                                      ViInt32   line,
                                                      ViBoolean displayLineEnabled,
                                                      ViReal64  displayLinePosition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_line[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_FREQUENCY_DISPLAY_LINE_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_FREQUENCY_DISPLAY_LINE_ENABLED,
            "DisplayLine", line - 1, RS_REPCAP_BUF_SIZE, repCap_line),
            3, "Line");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s",
        repCap_window, repCap_line);

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_FREQUENCY_DISPLAY_LINE_ENABLED, displayLineEnabled),
            4, "Display Line Enabled");
    if (displayLineEnabled)
    {
        viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_FREQUENCY_DISPLAY_LINE_POSITION, displayLinePosition),
                5, "Display Line Position");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures a time line.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR line/This control selects the display line.
/// HIPAR displayLineEnabled/Turns a time line on and off.
/// HIPAR displayLinePosition/Defines the position of a time line.
ViStatus _VI_FUNC rsfpl_ConfigureTimeDisplayLine(ViSession instrSession,
                                                 ViInt32   window,
                                                 ViInt32   line,
                                                 ViBoolean displayLineEnabled,
                                                 ViReal64  displayLinePosition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_line[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_TIME_DISPLAY_LINE_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_TIME_DISPLAY_LINE_ENABLED,
            "DisplayLine", line - 1, RS_REPCAP_BUF_SIZE, repCap_line),
            3, "Line");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s",
        repCap_window, repCap_line);

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_TIME_DISPLAY_LINE_ENABLED, displayLineEnabled),
            4, "Display Line Enabled");
    if (displayLineEnabled)
    {
        viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_TIME_DISPLAY_LINE_POSITION, displayLinePosition),
                5, "Display Line Position");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures a power line.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR line/This control selects the display line.
/// HIPAR displayLineEnabled/Turns a power line on and off.
/// HIPAR displayLinePosition/Defines the position of a power line.
ViStatus _VI_FUNC rsfpl_ConfigurePowerDisplayLine(ViSession instrSession,
                                                  ViInt32   window,
                                                  ViInt32   line,
                                                  ViBoolean displayLineEnabled,
                                                  ViReal64  displayLinePosition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_line[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_POWER_DISPLAY_LINE_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_POWER_DISPLAY_LINE_ENABLED,
            "DisplayLine", line - 1, RS_REPCAP_BUF_SIZE, repCap_line),
            3, "Line");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap_window, repCap_line);

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_POWER_DISPLAY_LINE_ENABLED, displayLineEnabled),
            4, "Display Line Enabled");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_POWER_DISPLAY_LINE_POSITION, displayLinePosition),
            5, "Display Line Position");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures a display line.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR line/This control selects the display line.
/// HIPAR displayLineEnabled/Turns a display line on and off.
/// HIPAR displayLinePosition/Defines the (vertical) position of a display line.
ViStatus _VI_FUNC rsfpl_ConfigureHorizontalDisplayLine(ViSession instrSession,
                                                       ViInt32   window,
                                                       ViInt32   line,
                                                       ViBoolean displayLineEnabled,
                                                       ViReal64  displayLinePosition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_line[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_HORIZONTAL_DISPLAY_LINE_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_HORIZONTAL_DISPLAY_LINE_ENABLED,
            "DisplayLine", line - 1, RS_REPCAP_BUF_SIZE, repCap_line),
            3, "Line");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s",
        repCap_window, repCap_line);

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_HORIZONTAL_DISPLAY_LINE_ENABLED, displayLineEnabled),
            4, "Display Line Enabled");

    if (displayLineEnabled)
    {
        viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_HORIZONTAL_DISPLAY_LINE_POSITION, displayLinePosition),
                5, "Display Line Position");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function creates new limit or selects existing one and configures
/// HIFN  basic parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR limit/This control selects limit line.
/// HIPAR name/Selects a limit line that already exists or defines a name for a new
/// HIPAR name/limit line.
/// HIPAR domain/Selects the domain of the limit line.
/// HIPAR comment/Defines a comment for a limit line.
/// HIPAR deleteExistingLimitLine/Removes specified limit line before new is created.
ViStatus _VI_FUNC rsfpl_CreateLimitLine(ViSession instrSession,
                                        ViInt32   limit,
                                        ViString  name,
                                        ViInt32   domain,
                                        ViString  comment,
                                        ViBoolean deleteExistingLimitLine)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_LIMIT_LINE_NAME,
            "LimitLine", limit - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Limit");
    viCheckParm(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_LIMIT_LINE_NAME, name),
            3, "Name");

    if (deleteExistingLimitLine)
    {
        viCheckParm(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_LIMIT_LINE_DELETE, NULL),
                3, "Limit Line Delete");
    }

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_LIMIT_LINE_CONTROL_DOMAIN, domain),
            4, "Domain");

    viCheckParm(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_LIMIT_LINE_COMMENT, comment),
            5, "Comment");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns the limit check for a specific trace on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trace/Selects the trace.
/// HIPAR limit/This control selects limit line.
/// HIPAR linesAssignTraceEnabled/Turns the limit check for a specific trace on and off.
ViStatus _VI_FUNC rsfpl_ConfigureLimitLinesAssignTraceEnabled(
                            ViSession instrSession,
                            ViInt32   limit,
                            ViInt32   trace,
                            ViBoolean assignTraceEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_trace[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_LIMIT_LINES_ASSIGN_TRACE_ENABLED,
            "LimitLine", limit - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Limit");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_LIMIT_LINES_ASSIGN_TRACE_ENABLED,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCap_trace),
            3, "Trace");

    (void)strncat(repCap, ",", RS_REPCAP_BUF_SIZE - strlen(repCap) - 1);
    (void)strncat(repCap, repCap_trace, RS_REPCAP_BUF_SIZE - strlen(repCap) - 1);

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_LIMIT_LINES_ASSIGN_TRACE_ENABLED, assignTraceEnabled),
            4, "Assign Trace Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures specified limit line.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR limit/This control selects limit.
/// HIPAR type/Specifies a limit line to be either an upper or lower type limit line.
/// HIPAR type/An upper line will be used as the maximum allowable value when
/// HIPAR type/comparing with the data. A lower limit line defines the minimum value.
/// HIPAR units/Defines the unit of a limit line. If you select dB as the limit line
/// HIPAR units/unit, the control automatically turns the limit line into a relative
/// HIPAR units/limit line.
/// HIPAR xAxisSpacing/Selects linear or logarithmic interpolation for the calculation of
/// HIPAR xAxisSpacing/limit lines from one horizontal point to the next.
/// HIPAR yAxisInterpolation/Selects linear or logarithmic interpolation for the upper or lower
/// HIPAR yAxisInterpolation/limit line.
/// HIPAR xAxisScaling/Selects the horizontal limit line scaling.
/// HIPAR yAxisScaling/Selects the vertical limit line scaling.
/// HIPAR margin/Defines an area around a limit line where limit check violations are
/// HIPAR margin/still tolerated.
/// HIPAR threshold/Defines a threshold for relative limit lines.
ViStatus _VI_FUNC rsfpl_ConfigureLimitLine(ViSession instrSession,
                                           ViInt32   limit,
                                           ViInt32   type,
                                           ViInt32   units,
                                           ViInt32   xAxisSpacing,
                                           ViInt32   yAxisInterpolation,
                                           ViInt32   xAxisScaling,
                                           ViInt32   yAxisScaling,
                                           ViReal64  margin,
                                           ViReal64  threshold)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_LIMIT_LINE_UNITS,
            "LimitLine", limit - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Limit");

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_LIMIT_LINE_UNITS, units),
            4, "Units");

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_LIMIT_LINE_CONTROL_SPACING, xAxisSpacing),
            5, "X Axis Spacing");

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_LIMIT_LINE_CONTROL_SCALING, xAxisScaling),
            7, "Scaling");

    switch (type)
    {
    case RSFPL_VAL_LIMIT_LOWER:
        viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_LIMIT_LINE_LOWER_SPACING, yAxisInterpolation),
                6, "Y Axis Interpolation");

        viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_LIMIT_LINE_LOWER_SCALING, yAxisScaling),
                8, "Y Axis Scaling");

        viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_LIMIT_LINE_LOWER_MARGIN, margin),
                9, "Margin");

        viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_LIMIT_LINE_LOWER_THRESHOLD, threshold),
                10, "Threshold");
        break;
    case RSFPL_VAL_LIMIT_UPPER:
        viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_LIMIT_LINE_UPPER_SPACING, yAxisInterpolation),
                6, "Y Axis Interpolation");

        viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_LIMIT_LINE_UPPER_SCALING, yAxisScaling),
                8, "Y Axis Scaling");

        viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_LIMIT_LINE_UPPER_MARGIN, margin),
                9, "Margin");

        viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_LIMIT_LINE_UPPER_THRESHOLD, threshold),
                10, "Threshold");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, type), 3, "Type");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function defines specified limit line.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR limit/This control selects limit.
/// HIPAR type/Specifies a limit line to be either an upper or lower type limit line.
/// HIPAR type/An upper line will be used as the maximum allowable value when
/// HIPAR type/comparing with the data. A lower limit line defines the minimum value.
/// HIPAR count/Specifies number of values in X Axis and Amplitude arrays.
/// HIPAR xAxis/Specifies the frequency or time value for a limit point.
/// HIPAR amplitude/Specifies the amplitude value for the current limit point.
ViStatus _VI_FUNC rsfpl_DefineLimitLine(ViSession instrSession,
                                        ViInt32   limit,
                                        ViInt32   type,
                                        ViUInt32  count,
                                        ViReal64  xAxis[],
                                        ViReal64  amplitude[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViString limTypeArr[] = { "LOW", "UPP", NULL };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, limit, 1, 8),
            2, "Limit");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, type, 0, 1),
            3, "Type");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, xAxis),
        5, "X Axis");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, amplitude),
        6, "Amplitude");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:LIM%ld:CONT ", limit);
    checkErr(RsCore_WriteAsciiViReal64Array(instrSession, cmd, xAxis, count));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:LIM%ld:%s ", limit, limTypeArr[type]);
    checkErr(RsCore_WriteAsciiViReal64Array(instrSession, cmd, amplitude, count));

    checkErr(rsfpl_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns the limit check for a specific limit line on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR limit/This control selects limit.
/// HIPAR limitLineCheckEnabled/Turns the limit check for a specific limit line on and off.
ViStatus _VI_FUNC rsfpl_ConfigureLimitLineCheckEnabled(ViSession instrSession,
                                                       ViInt32   limit,
                                                       ViBoolean limitLineCheckEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_LIMIT_LINE_CHECK_ENABLED,
            "LimitLine", limit - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Limit");

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_LIMIT_LINE_CHECK_ENABLED, limitLineCheckEnabled),
            3, "Limit Line Check Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns an upper limit line on and off.
/// HIFN     Before you can use the function, you have to select a limit line.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR limit/This control selects limit.
/// HIPAR type/Specifies a limit line to be either an upper or lower type limit line.
/// HIPAR type/An upper line will be used as the maximum allowable value when
/// HIPAR type/comparing with the data. A lower limit line defines the minimum value.
/// HIPAR limitEnabled/Turns the limit line on and off.
ViStatus _VI_FUNC rsfpl_ConfigureLimitLineEnabled(ViSession instrSession,
                                                  ViInt32   limit,
                                                  ViInt32   type,
                                                  ViBoolean limitEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_LIMIT_LINE_UPPER_ENABLED,
            "LimitLine", limit - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Limit");

    switch (type)
    {
    case RSFPL_VAL_LIMIT_LOWER:
        viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_LIMIT_LINE_LOWER_ENABLED, limitEnabled),
                4, "Limit Enabled");
        break;
    case RSFPL_VAL_LIMIT_UPPER:
        viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_LIMIT_LINE_UPPER_ENABLED, limitEnabled),
                4, "Limit Enabled");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, type), 3, "Type");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function moves specified limit line.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR limit/This control selects limit.
/// HIPAR type/Specifies a limit line to be type to be moved.
/// HIPAR method/This control selects the method of moving.
/// HIPAR offsetValue/Defines the move value.
ViStatus _VI_FUNC rsfpl_MoveLimitLine(ViSession instrSession,
                                      ViInt32   limit,
                                      ViInt32   type,
                                      ViInt32   method,
                                      ViReal64  offsetValue)
{
    ViStatus                               error = VI_SUCCESS;
    ViInt32                                attributes[3][2] = {
    {RSFPL_ATTR_LIMIT_LINE_LOWER_OFFSET,   RSFPL_ATTR_LIMIT_LINE_LOWER_SHIFT},
    {RSFPL_ATTR_LIMIT_LINE_UPPER_OFFSET,   RSFPL_ATTR_LIMIT_LINE_UPPER_SHIFT},
    {RSFPL_ATTR_LIMIT_LINE_CONTROL_OFFSET, RSFPL_ATTR_LIMIT_LINE_SHIFT}
                                           };
    ViChar                                 repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, type, 0, 2),
            3, "Type");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, method, 0, 1),
            4, "Method");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_LIMIT_LINE_CONTROL_OFFSET,
            "LimitLine", limit - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Limit");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, attributes[type][method], offsetValue),
            5, "Offset Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Copies a limit line.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR limit/This control selects limit.
/// HIPAR copyTo/This control selects target limit.
ViStatus _VI_FUNC rsfpl_CopyLimitLine(ViSession instrSession,
                                      ViInt32   limit,
                                      ViInt32   copyTo)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_LIMIT_LINE_COPY,
            "LimitLine", limit - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Limit");

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_LIMIT_LINE_COPY, copyTo),
            3, "Copy To");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Deletes a limit line.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR limit/This control selects limit line.
ViStatus _VI_FUNC rsfpl_DeleteLimitLine(ViSession instrSession,
                                        ViInt32   limit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_LIMIT_LINE_DELETE,
            "LimitLine", limit - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Limit");

    viCheckParm(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_LIMIT_LINE_DELETE, NULL),
            3, "Limit Line Delete");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Deletes the result of the current limit check.
/// HIFN     The method works on all limit lines in all measurement windows at the
/// HIFN  same time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR limit/This control selects limit line.
ViStatus _VI_FUNC rsfpl_DeleteLimitResults(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_LIMIT_DELETE_RESULTS, NULL),
            2, "Limit Delete Results");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the names of all active limit lines.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR catalog/Queries names of all active limit lines.
/// HIPAR arraySize/Defines the memory size allocated for Catalog.
/// HIPAR returnedValues/Returns number of returned values.
ViStatus _VI_FUNC rsfpl_QueryLimitNamesOfActiveLimitLines(ViSession instrSession,
                                                          ViUInt32  arraySize,
                                                          ViChar    catalog[],
                                                          ViUInt32* returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsfpl_QueryViString(instrSession, "CALC:LIM:ACT?", arraySize, catalog, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Loads the limit line from the selected file in .CSV format.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR limit/This control selects limit line.
/// HIPAR fileName/Loads the limit line from the selected file in .CSV format.
ViStatus _VI_FUNC rsfpl_LoadLimitLine(ViSession instrSession,
                                      ViString  fileName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_LOAD_LIMIT_LINE, fileName),
            2, "Load Limit Line");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Exports limit line data to an ASCII (CSV) file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fileName/String containing the path and name of the target file.
/// HIPAR limitLineName/Name of the limit line to be exported.
ViStatus _VI_FUNC rsfpl_ExportLimitLine(ViSession instrSession,
                                        ViString  fileName,
                                        ViString  limitLineName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:STOR:LIM '%s','%s'", fileName, limitLineName);
    checkErr(rsfpl_WriteCommandWithOPCSync(instrSession, cmd, 0));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the default storage device used by all file system commands.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR selectedStorageDevice/Selects the default storage device used by all file system commands.
ViStatus _VI_FUNC rsfpl_ConfigureDataSelectedStorageDevice(ViSession instrSession,
                                                           ViString  selectedStorageDevice)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_DATA_SELECTED_STORAGE_DEVICE, selectedStorageDevice),
            2, "Selected Storage Device");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Changes the current directory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR selectedDirectory/Changes the current directory.
ViStatus _VI_FUNC rsfpl_ConfigureDataSelectedDirectory(ViSession instrSession,
                                                       ViString  selectedDirectory)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_DATA_SELECTED_DIRECTORY, selectedDirectory),
            2, "Selected Directory");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Creates a new directory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR directory/Creates a new directory.
ViStatus _VI_FUNC rsfpl_DataCreateDirectory(ViSession instrSession,
                                            ViString  directory)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_DATA_CREATE_DIRECTORY, directory),
            2, "Directory");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the contents of a particular directory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR directory/String containing the path and directory
/// HIPAR contents/Returns the contents of a particular directory.
/// HIPAR arraySize/Defines the memory size allocated for Content.
/// HIPAR returnedValues/Returns number of returned values.
ViStatus _VI_FUNC rsfpl_QueryDataDirectoryContents(ViSession instrSession,
                                                   ViString  directory,
                                                   ViUInt32  arraySize,
                                                   ViChar    contents[],
                                                   ViUInt32* returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:CAT? '%s'", directory);
    checkErr(rsfpl_QueryViString(instrSession, cmd, arraySize, contents, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the contents of a particular directory with additional
/// HIFN  information about the files.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR directory/String containing the path and directory
/// HIPAR contents/Returns the contents of a particular directory.
/// HIPAR arraySize/Defines the memory size allocated for Content.
/// HIPAR returnedValues/Returns number of returned values.
ViStatus _VI_FUNC rsfpl_QueryDataDirectoryAdditionalContents(ViSession instrSession,
                                                             ViString  directory,
                                                             ViUInt32  arraySize,
                                                             ViChar    contents[],
                                                             ViUInt32* returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:CAT:LONG? '%s'", directory);
    checkErr(rsfpl_QueryViString(instrSession, cmd, arraySize, contents, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Deletes the indicated directory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR directory/Deletes the indicated directory.
ViStatus _VI_FUNC rsfpl_DataDeleteDirectory(ViSession instrSession,
                                            ViString  directory)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_DATA_DELETE_DIRECTORY, directory),
            2, "Directory");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Has several purposes, depending on the context it is used in.
/// HIFN     It creates a new and empty file. It defines the file name for
/// HIFN  screenshots taken with RSFPL_ATTR_HCOPY_PRINT. Note that you have to
/// HIFN  route the printer output to a file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR dataFileName/Has several purposes, depending on the context it is used in.
ViStatus _VI_FUNC rsfpl_ConfigureDataFileName(ViSession instrSession,
                                              ViString  dataFileName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_DATA_FILE_NAME, dataFileName),
            2, "Data File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines a comment for the stored settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR comment/Defines a comment for the stored settings.
ViStatus _VI_FUNC rsfpl_ConfigureDataFileNameComment(ViSession instrSession,
                                                     ViString  comment)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_DATA_FILE_NAME_COMMENT, comment),
            2, "Comment");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the decimal separator for data exported in ASCII format.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR decimalSeparator/Selects the decimal separator for data exported in ASCII format.
ViStatus _VI_FUNC rsfpl_ConfigureDataDecimalSeparator(ViSession instrSession,
                                                      ViInt32   decimalSeparator)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_DATA_DECIMAL_SEPARATOR, decimalSeparator),
            2, "Decimal Separator");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines how the x-values of the trace are determined in the frequency
/// HIFN  domain.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR distribution/Defines how the x-values of the trace are determined in the frequency
/// HIPAR distribution/domain.
ViStatus _VI_FUNC rsfpl_ConfigureDataXValueDistribution(ViSession instrSession,
                                                        ViInt32   distribution)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_DATA_X_VALUE_DISTRIBUTION, distribution),
            2, "Distribution");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This method copies one or more files to another directory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR sourceFile/String containing the path and file name of the source file.
/// HIPAR destinationFile/String containing the path and name of the target file.
ViStatus _VI_FUNC rsfpl_DataCopyFile(ViSession instrSession,
                                     ViString  sourceFile,
                                     ViString  destinationFile)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:COPY '%s','%s'", sourceFile, destinationFile);
    checkErr(rsfpl_WriteCommandWithOPCSync(instrSession, cmd, 0));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This method moves a file to another directory.
/// HIFN
/// HIFN     This method also renames the file if you define a new name in the
/// HIFN  target directory.
/// HIFN
/// HIFN     If you do not include a path for <NewFileName>, the method just
/// HIFN  renames the file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR sourceFile/String containing the path and file name of the source file.
/// HIPAR newFileName/String containing the path and name of the target file.
ViStatus _VI_FUNC rsfpl_DataMoveFile(ViSession instrSession,
                                     ViString  sourceFile,
                                     ViString  newFileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:MOVE '%s','%s'", sourceFile, newFileName);
    checkErr(rsfpl_WriteCommandWithOPCSync(instrSession, cmd, 0));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Deletes a file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fileName/Deletes a file.
ViStatus _VI_FUNC rsfpl_DataDeleteFile(ViSession instrSession,
                                       ViString  fileName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_DATA_DELETE, fileName),
            2, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Maps a drive to a server or server directory of the network.
/// HIFN     Note that you have to allow sharing for a server or folder in
/// HIFN  Microsoft networks first.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR drive/String containing the drive name or path of the directory you want to
/// HIPAR drive/map.
/// HIPAR hostname/String containing the host name of the computer or the IP address and
/// HIPAR hostname/the share name of the drive.
/// HIPAR username/String containing a user name in the network.
/// HIPAR password/String containing the password corresponding to the <UserName>.
/// HIPAR reconnect/Reconnects at logon with the same user name.
ViStatus _VI_FUNC rsfpl_DataNetworkMapDrive(ViSession instrSession,
                                            ViString  drive,
                                            ViString  hostname,
                                            ViString  username,
                                            ViString  password,
                                            ViBoolean reconnect)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:NETW:MAP '%s','%s','%s','%s',%s", drive,
        hostname,
        username,
        password,
        reconnect ? "ON" : "OFF");
    checkErr(rsfpl_WriteCommandWithOPCSync(instrSession, cmd, 0));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Disconnects a network drive.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR drive/String containing the drive name.
/// HIPAR force/Determines whether disconnection is forced or not.
ViStatus _VI_FUNC rsfpl_DataNetworkDisconnectDrive(ViSession instrSession,
                                                   ViString  drive,
                                                   ViBoolean force)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:NETW:DISC '%s',%s", drive,
        force ? "ON" : "OFF");
    checkErr(rsfpl_WriteCommandWithOPCSync(instrSession, cmd, 0));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns a list of unused network drives.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR arraySize/Defines the memory size allocated for Unused Drives.
/// HIPAR unusedDrives/Returns a list of unused network drives.
/// HIPAR returnedValues/Returns number of returned values.
ViStatus _VI_FUNC rsfpl_QueryDataNetworkUnusedDrives(ViSession instrSession,
                                                     ViUInt32  arraySize,
                                                     ViChar    unusedDrives[],
                                                     ViUInt32* returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsfpl_QueryViString(instrSession, "MMEM:NETW:UNUS?", arraySize, unusedDrives, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns a list of all network drives in use.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR arraySize/Defines the memory size allocated for Used Drives.
/// HIPAR usedDrives/Returns a list of all network drives in use.
/// HIPAR returnedValues/Returns number of returned values.
ViStatus _VI_FUNC rsfpl_QueryDataNetworkUsedDrives(ViSession instrSession,
                                                   ViUInt32  arraySize,
                                                   ViChar    usedDrives[],
                                                   ViUInt32* returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsfpl_QueryViString(instrSession, "MMEM:NETW:USED?", arraySize, usedDrives, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines whether the data from the entire instrument or only from the
/// HIFN  current channel setup is stored with the subsequent instrument settings
/// HIFN  methods and attributes.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR settingsMode/Defines whether the data from the entire instrument or only from the
/// HIPAR settingsMode/current channel setup is stored with the subsequent instrument settings
/// HIPAR settingsMode/methods and controls.
ViStatus _VI_FUNC rsfpl_ConfigureDataInstrumentSettingsMode(ViSession instrSession,
                                                            ViInt32   settingsMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_DATA_INSTRUMENT_SETTINGS_MODE, settingsMode),
            2, "Settings Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Deletes all instrument configuration files in the current directory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_DataInstrumentSettingsDeleteAll(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_DATA_INSTRUMENT_SETTINGS_DELETE_ALL, NULL),
            2, "Data Instrument Settings Delete All");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Deletes an instrument configuration file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fileName/String containing the path and name of the file to delete. The string
/// HIPAR fileName/may or may not contain the file's extension.
ViStatus _VI_FUNC rsfpl_DataInstrumentSettingsDelete(ViSession instrSession,
                                                     ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:CLE:STAT 1,'%s'", fileName);
    checkErr(rsfpl_WriteCommandWithOPCSync(instrSession, cmd, 0));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Restores an instrument configuration and defines that configuration as
/// HIFN  the default state.
/// HIFN     The default state is restored after a preset or after you turn on the
/// HIFN  instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fileName/String containing the path and name of the configuration file.
ViStatus _VI_FUNC rsfpl_DataInstrumentSettingsStartupRecall(ViSession instrSession,
                                                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:LOAD:AUTO 1,'%s'", fileName);
    checkErr(rsfpl_WriteCommandWithOPCSync(instrSession, cmd, 0));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines whether the channel setups that will be loaded with the
/// HIFN  subsequent MMEM:LOAD:STAT command
/// HIFN  (RSFPL_FUNC_DATA_INSTRUMENT_SETTINGS_RECALL) will replace the current
/// HIFN  channel setup or activate a new channel setup.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR settingsRecallMode/Defines whether the channel setups that will be loaded with the
/// HIPAR settingsRecallMode/subsequent MMEM:LOAD:STAT command
/// HIPAR settingsRecallMode/(RSFPL_FUNC_DATA_INSTRUMENT_SETTINGS_RECALL) will replace the current
/// HIPAR settingsRecallMode/channel setup or activate a new channel setup.
ViStatus _VI_FUNC rsfpl_ConfigureDataInstrumentSettingsRecallMode(
                            ViSession instrSession,
                            ViInt32   settingsRecallMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_DATA_INSTRUMENT_SETTINGS_RECALL_MODE, settingsRecallMode),
            2, "Settings Recall Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Restores and activates the instrument configuration stored in a *.dfl
/// HIFN  file.
/// HIFN     Note that files with other formats cannot be loaded with this method.
/// HIFN     The contents that are reloaded from the file are defined by the last
/// HIFN  selection made either in the Save/Recall dialogs (manual operation) or
/// HIFN  through the Items To Store attributes (remote operation; the settings
/// HIFN  are identical in both cases).
/// HIFN     By default, the selection is limited to the HW Settings. The selection
/// HIFN  is not reset by Preset or *RST.
/// HIFN     As a consequence, the results of a SCPI script using the Recall method
/// HIFN  without a previous Items To Store attribute may vary, depending on
/// HIFN  previous actions in the GUI or in previous scripts, even if the script
/// HIFN  starts with the *RST command.
/// HIFN     It is therefore recommended that you use the appropriate Items To
/// HIFN  Store attribute before using this method.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fileName/String containing the path and name of the file to load. The string
/// HIPAR fileName/may or may not include the file's extension.
ViStatus _VI_FUNC rsfpl_DataInstrumentSettingsRecall(ViSession instrSession,
                                                     ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:LOAD:STAT 1,'%s'", fileName);
    checkErr(rsfpl_WriteCommandWithOPCSync(instrSession, cmd, 0));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Saves the current instrument configuration in a *.dfl file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fileName/String containing the path and name of the target file. The file
/// HIPAR fileName/extension is .dfl.
ViStatus _VI_FUNC rsfpl_DataInstrumentSettingsSave(ViSession instrSession,
                                                   ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:STOR:STAT 1,'%s'", fileName);
    checkErr(rsfpl_WriteCommandWithOPCSync(instrSession, cmd, 0));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Saves the current instrument configuration in a *.dfl file.
/// HIFN     The file name depends on the one you have set with Save method. Adds a
/// HIFN  consecutive number to the file name.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_DataInstrumentSettingsSaveNext(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_DATA_INSTRUMENT_SETTINGS_SAVE_NEXT, NULL),
            2, "Data Instrument Settings Save Next");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the current settings as the only item to store to and load
/// HIFN  from a configuration file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_ItemsToStoreInstrumentDefault(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_ITEMS_TO_STORE_INSTRUMENT_DEFAULT, NULL),
            2, "Items To Store Instrument Default");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Includes all items when storing or loading a configuration file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_ItemsToStoreInstrumentAll(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_ITEMS_TO_STORE_INSTRUMENT_ALL, NULL),
            2, "Items To Store Instrument All");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Does not include any of the following items when storing or loading a
/// HIFN  configuration file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_ItemsToStoreInstrumentNone(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_ITEMS_TO_STORE_INSTRUMENT_NONE, NULL),
            2, "Items To Store Instrument None");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures which information shall be part of stored configuration
/// HIFN  file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR hwSettingsEnabled/Includes or excludes measurement (hardware) settings when storing or
/// HIPAR hwSettingsEnabled/loading a configuration file.
/// HIPAR limitLinesEnabled/Includes or excludes all limit lines (active and inactive) when
/// HIPAR limitLinesEnabled/storing or loading a configuration file.
/// HIPAR spectrogramDataEnabled/Includes or excludes spectrogram data when storing or loading a
/// HIPAR spectrogramDataEnabled/configuration file.
/// HIPAR traceDataEnabled/Includes or excludes trace data when storing or loading a
/// HIPAR traceDataEnabled/configuration file.
/// HIPAR transducerFactorsEnabled/Includes or excludes transducer factors when storing or loading a
/// HIPAR transducerFactorsEnabled/configuration file.
ViStatus _VI_FUNC rsfpl_ConfigureItemsToStoreInstrument(ViSession instrSession,
                                                        ViBoolean hwSettingsEnabled,
                                                        ViBoolean limitLinesEnabled,
                                                        ViBoolean spectrogramDataEnabled,
                                                        ViBoolean traceDataEnabled,
                                                        ViBoolean transducerFactorsEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_ITEMS_TO_STORE_INSTRUMENT_HW_SETTINGS_ENABLED, hwSettingsEnabled),
            2, "HW Settings Enabled");

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_ITEMS_TO_STORE_INSTRUMENT_LIMIT_LINES_ENABLED, limitLinesEnabled),
            3, "Limit Lines Enabled");

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_ITEMS_TO_STORE_INSTRUMENT_SPECTROGRAM_DATA_ENABLED, spectrogramDataEnabled),
            4, "Spectrogram Data Enabled");

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_ITEMS_TO_STORE_INSTRUMENT_TRACE_DATA_ENABLED, traceDataEnabled),
            5, "Trace Data Enabled");

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_ITEMS_TO_STORE_INSTRUMENT_TRANSDUCER_FACTORS_ENABLED, transducerFactorsEnabled),
            6, "Transducer Factors Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Includes or excludes source calibration data for an optional external
/// HIFN  generator when storing or loading a configuration file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR sourceCalibrationData/Includes or excludes source calibration data for an optional external
/// HIPAR sourceCalibrationData/generator when storing or loading a configuration file.
ViStatus _VI_FUNC rsfpl_ConfigureItemsToStoreInstrumentSourceCalibrationData(
                            ViSession instrSession,
                            ViBoolean sourceCalibrationData)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_ITEMS_TO_STORE_INSTRUMENT_SOURCE_CALIBRATION_DATA, sourceCalibrationData),
            2, "Source Calibration Data");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the current settings as the only item to store to and load
/// HIFN  from a configuration file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_ItemsToStoreChannelDefault(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_ITEMS_TO_STORE_CHANNEL_DEFAULT, NULL),
            2, "Items To Store Channel Default");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Includes all items when storing or loading a configuration file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_ItemsToStoreChannelAll(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_ITEMS_TO_STORE_CHANNEL_ALL, NULL),
            2, "Items To Store Channel All");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Does not include any of the following items when storing or loading a
/// HIFN  configuration file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_ItemsToStoreChannelNone(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_ITEMS_TO_STORE_CHANNEL_NONE, NULL),
            2, "Items To Store Channel None");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures which information shall be part of stored configuration
/// HIFN  file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR hwSettingsEnabled/Includes or excludes measurement (hardware) settings when storing or
/// HIPAR hwSettingsEnabled/loading a configuration file.
/// HIPAR limitLinesEnabled/Includes or excludes all limit lines (active and inactive) when
/// HIPAR limitLinesEnabled/storing or loading a configuration file.
/// HIPAR spectrogramDataEnabled/Includes or excludes spectrogram data when storing or loading a
/// HIPAR spectrogramDataEnabled/configuration file.
/// HIPAR traceDataEnabled/Includes or excludes trace data when storing or loading a
/// HIPAR traceDataEnabled/configuration file.
/// HIPAR transducerFactorsEnabled/Includes or excludes transducer factors when storing or loading a
/// HIPAR transducerFactorsEnabled/configuration file.
ViStatus _VI_FUNC rsfpl_ConfigureItemsToStoreChannel(ViSession instrSession,
                                                     ViBoolean hwSettingsEnabled,
                                                     ViBoolean limitLinesEnabled,
                                                     ViBoolean spectrogramDataEnabled,
                                                     ViBoolean traceDataEnabled,
                                                     ViBoolean transducerFactorsEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_ITEMS_TO_STORE_CHANNEL_HW_SETTINGS_ENABLED, hwSettingsEnabled),
            2, "HW Settings Enabled");

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_ITEMS_TO_STORE_CHANNEL_LIMIT_LINES_ENABLED, limitLinesEnabled),
            3, "Limit Lines Enabled");

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_ITEMS_TO_STORE_CHANNEL_SPECTROGRAM_DATA_ENABLED, spectrogramDataEnabled),
            4, "Spectrogram Data Enabled");

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_ITEMS_TO_STORE_CHANNEL_TRACE_DATA_ENABLED, traceDataEnabled),
            5, "Trace Data Enabled");

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_ITEMS_TO_STORE_CHANNEL_TRANSDUCER_FACTORS_ENABLED, transducerFactorsEnabled),
            6, "Transducer Factors Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Includes or excludes source calibration data for an optional external
/// HIFN  generator when storing or loading a configuration file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR sourceCalibrationData/Includes or excludes source calibration data for an optional external
/// HIPAR sourceCalibrationData/generator when storing or loading a configuration file.
ViStatus _VI_FUNC rsfpl_ConfigureItemsToStoreChannelSourceCalibrationData(
                            ViSession instrSession,
                            ViBoolean sourceCalibrationData)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_ITEMS_TO_STORE_CHANNEL_SOURCE_CALIBRATION_DATA, sourceCalibrationData),
            2, "Source Calibration Data");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Aborts a running hardcopy output.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_HcopyAbort(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_HCOPY_ABORT, NULL),
            2, "Hcopy Abort");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the output of the HCOPy[:IMMediate<1|2>] command.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mode/Selects the output of the HCOPy[:IMMediate<1|2>] command.
ViStatus _VI_FUNC rsfpl_HcopyMode(ViSession instrSession,
                                  ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_HCOPY_MODE, mode),
            2, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Initiates a print job.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_HcopyPrint(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_HCOPY_PRINT, NULL),
            2, "Hcopy Print");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Initiates a print job. Adds a consecutive number to the file name.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_HcopyPrintNext(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_HCOPY_PRINT_NEXT, NULL),
            2, "Hcopy Print Next");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Determines the content included in the printout.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR contentType/Determines the type of content included in the printout.
/// HIPAR comment/Defines an optional comment to be included in the printout of the
/// HIPAR comment/display. Maximum 120 characters are allowed. Up to 60 characters fit in
/// HIPAR comment/one line. In the first line, a manual line-feed can be forced at any
/// HIPAR comment/point by entering @.
/// HIPAR logoEnabled/Activates/deactivates the printout of the Rohde & Schwarz company logo
/// HIPAR logoEnabled/at the top of each page.
/// HIPAR pageCountEnabled/Includes or excludes the page number for printouts consisting of
/// HIPAR pageCountEnabled/multiple pages.
/// HIPAR dateAndTimeEnabled/Includes or excludes the time and date in the printout.
ViStatus _VI_FUNC rsfpl_ConfigureHcopyContent(ViSession instrSession,
                                              ViInt32   contentType,
                                              ViString  comment,
                                              ViBoolean logoEnabled,
                                              ViBoolean pageCountEnabled,
                                              ViBoolean dateAndTimeEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_HCOPY_CONTENT_TYPE, contentType),
            2, "Content Type");

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_HCOPY_COMMENT, comment),
            3, "Comment");

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_HCOPY_LOGO_ENABLED, logoEnabled),
            4, "Logo Enabled");

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_HCOPY_PAGE_COUNT_ENABLED, pageCountEnabled),
            5, "Page Count Enabled");

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_HCOPY_DATE_AND_TIME_ENABLED, dateAndTimeEnabled),
            6, "Date And Time Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the windows to be included in the printout.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channelName/String containing the name of the channel setup.
/// HIPAR windowName/String containing the name of the existing window.
/// HIPAR windowEnabled/Defines whether window is included in printout.
ViStatus _VI_FUNC rsfpl_ConfigureHcopyWindowPrintout(ViSession instrSession,
                                                     ViString  channelName,
                                                     ViString  windowName,
                                                     ViBoolean windowEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "HCOP:PAGE:WIND1:STAT '%s','%s',%s", channelName, windowName, windowEnabled ? "ON" : "OFF");
    checkErr(rsfpl_WriteCommandWithOPCSync(instrSession, cmd, 0));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects all windows of the specified channel setup to be included in
/// HIFN  the printout.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channelName/String containing the name of the channel setup.
/// HIPAR channelEnabled/Defines whether window is included in printout.
ViStatus _VI_FUNC rsfpl_ConfigureHcopyChannelPrintout(ViSession instrSession,
                                                      ViString  channelName,
                                                      ViBoolean channelEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "HCOP:PAGE:WIND1:CHAN:STAT '%s',%s", channelName, channelEnabled ? "ON" : "OFF");
    checkErr(rsfpl_WriteCommandWithOPCSync(instrSession, cmd, 0));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the destination of a print job.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR destination/Selects the destination of a print job.
ViStatus _VI_FUNC rsfpl_ConfigureHcopyDestination(ViSession instrSession,
                                                  ViInt32   destination)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_HCOPY_DESTINATION, destination),
            2, "Destination");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the printer that processes jobs sent by the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR selectedPrinter/Selects the printer that processes jobs sent by the instrument.
ViStatus _VI_FUNC rsfpl_ConfigureHcopySelectedPrinter(ViSession instrSession,
                                                      ViString  selectedPrinter)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_HCOPY_SELECTED_PRINTER, selectedPrinter),
            2, "Selected Printer");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the file format for a print job.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR hcopyFileFormat/Selects the file format for a print job.
ViStatus _VI_FUNC rsfpl_HcopyFileFormat(ViSession instrSession,
                                        ViInt32   fileFormat)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_HCOPY_FILE_FORMAT, fileFormat),
            2, "File Format");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the name of the first available printer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR arraySize/Defines the memory size allocated for First Printer.
/// HIPAR firstPrinter/Queries the name of the first available printer.
/// HIPAR returnedValues/Returns number of returned values.
ViStatus _VI_FUNC rsfpl_QueryHcopyFirstPrinter(ViSession instrSession,
                                               ViUInt32  arraySize,
                                               ViChar    firstPrinter[],
                                               ViUInt32* returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_GetAttributeViString(instrSession, NULL, RSFPL_ATTR_HCOPY_FIRST_PRINTER,
            arraySize, firstPrinter),
            2, "First Printer");
    *returnedValues = (ViUInt32)strlen(firstPrinter);

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the name of available printers.
/// HIFN     You have to use RSFPL_ATTR_HCOPY_FIRST_PRINTER/FirstPrinter for this
/// HIFN  attribute to work properly.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR arraySize/Defines the memory size allocated for Next Printer.
/// HIPAR nextPrinter/Queries the name of available printers.
/// HIPAR returnedValues/Returns number of returned values.
ViStatus _VI_FUNC rsfpl_QueryHcopyNextPrinter(ViSession instrSession,
                                              ViUInt32  arraySize,
                                              ViChar    nextPrinter[],
                                              ViUInt32* returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_GetAttributeViString(instrSession, NULL, RSFPL_ATTR_HCOPY_NEXT_PRINTER,
            arraySize, nextPrinter),
            2, "Next Printer");
    *returnedValues = (ViUInt32)strlen(nextPrinter);

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the page orientation of the printout.
/// HIFN     The attribute is only available if the output device is a printer or a
/// HIFN  PDF file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR pageOrientation/Selects the page orientation of the printout.
ViStatus _VI_FUNC rsfpl_ConfigureHcopyPageOrientation(ViSession instrSession,
                                                      ViInt32   pageOrientation)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_HCOPY_PAGE_ORIENTATION, pageOrientation),
            2, "Page Orientation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines how many windows are displayed on a single page of the
/// HIFN  printout for Windows content type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR windowsPerPage/Defines how many windows are displayed on a single page of the
/// HIPAR windowsPerPage/printout for Windows content type.
ViStatus _VI_FUNC rsfpl_ConfigureHcopyWindowsPerPage(ViSession instrSession,
                                                     ViInt32   windowsPerPage)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_HCOPY_WINDOWS_PER_PAGE, windowsPerPage),
            2, "Windows Per Page");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Determines the scaling of the windows in the printout.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR scaleToFitEnabled/Determines the scaling of the windows in the printout.
ViStatus _VI_FUNC rsfpl_ConfigureHcopyPageScaleToFitEnabled(ViSession instrSession,
                                                            ViBoolean scaleToFitEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_HCOPY_PAGE_SCALE_TO_FIT_ENABLED, scaleToFitEnabled),
            2, "Scale To Fit Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the page margin.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR unit/Defines the unit in which the margins for the printout page are
/// HIPAR unit/configured.
/// HIPAR bottom/Defines the margin at the bottom of the printout page on which no
/// HIPAR bottom/elements are printed. The margins are defined according to
/// HIPAR bottom/RSFPL_ATTR_HCOPY_PAGE_MARGIN_UNIT.
/// HIPAR left/Defines the margin at the left side of the printout page on which no
/// HIPAR left/elements are printed. The margins are defined according to
/// HIPAR left/RSFPL_ATTR_HCOPY_PAGE_MARGIN_UNIT.
/// HIPAR right/Defines the margin at the right side of the printout page on which no
/// HIPAR right/elements are printed. The margins are defined according to
/// HIPAR right/RSFPL_ATTR_HCOPY_PAGE_MARGIN_UNIT.
/// HIPAR top/Defines the margin at the top of the printout page on which no
/// HIPAR top/elements are printed. The margins are defined according to
/// HIPAR top/RSFPL_ATTR_HCOPY_PAGE_MARGIN_UNIT.
ViStatus _VI_FUNC rsfpl_ConfigureHcopyPageMargin(ViSession instrSession,
                                                 ViInt32   unit,
                                                 ViReal64  bottom,
                                                 ViReal64  left,
                                                 ViReal64  right,
                                                 ViReal64  top)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_HCOPY_PAGE_MARGIN_UNIT, unit),
            2, "Unit");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_HCOPY_PAGE_MARGIN_BOTTOM, bottom),
            3, "Bottom");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_HCOPY_PAGE_MARGIN_LEFT, left),
            4, "Left");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_HCOPY_PAGE_MARGIN_RIGHT, right),
            5, "Right");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_HCOPY_PAGE_MARGIN_TOP, top),
            6, "Top");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns color printing on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR colorPrintingEnabled/Turns color printing on and off.
ViStatus _VI_FUNC rsfpl_ConfigureHcopyColorPrinting(ViSession instrSession,
                                                    ViBoolean colorPrintingEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_HCOPY_COLOR_PRINTING_ENABLED, colorPrintingEnabled),
            2, "Color Printing Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Resets the color scheme for the display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR color/Selects the color.
ViStatus _VI_FUNC rsfpl_ResetHcopyColorScheme(ViSession instrSession,
                                              ViInt32   color)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DISP:CMAP:DEF%ld", color);
    checkErr(rsfpl_WriteCommandWithOPCSync(instrSession, cmd, 0));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Determines the format of the ASCII file to be imported or exported.
/// HIFN  Depending on the external program in which the data file was created or
/// HIFN  will be evaluated, a comma-separated list (CSV) or a plain data format
/// HIFN  (DAT) file may be required.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fileFormat/Determines the format of the ASCII file to be imported or exported.
/// HIPAR fileFormat/Depending on the external program in which the data file was created or
/// HIPAR fileFormat/will be evaluated, a comma-separated list (CSV) or a plain data format
/// HIPAR fileFormat/(DAT) file may be required.
ViStatus _VI_FUNC rsfpl_ConfigureDataResultsFileFormat(ViSession instrSession,
                                                       ViInt32   fileFormat)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_DATA_RESULTS_FILE_FORMAT, fileFormat),
            2, "File Format");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  If enabled, additional instrument and measurement settings are
/// HIFN  included in the header of the export file for result data. If disabled,
/// HIFN  only the pure result data from the selected traces and tables is
/// HIFN  exported.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR headerEnabled/If enabled, additional instrument and measurement settings are
/// HIPAR headerEnabled/included in the header of the export file for result data. If disabled,
/// HIPAR headerEnabled/only the pure result data from the selected traces and tables is
/// HIPAR headerEnabled/exported.
ViStatus _VI_FUNC rsfpl_ConfigureDataResultsHeaderEnabled(ViSession instrSession,
                                                          ViBoolean headerEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_DATA_RESULTS_HEADER_ENABLED, headerEnabled),
            2, "Header Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the data to be included in a data export file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR resultsIncludedDataExport/Selects the data to be included in a data export file.
ViStatus _VI_FUNC rsfpl_ConfigureDataResultsIncludedDataExport(
                            ViSession instrSession,
                            ViInt32   resultsIncludedDataExport)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_DATA_RESULTS_INCLUDED_DATA_EXPORT, resultsIncludedDataExport),
            2, "Results Included Data Export");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the data to be included in a data import file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR resultsIncludedDataImport/Selects the data to be included in a data import file.
ViStatus _VI_FUNC rsfpl_ConfigureDataResultsIncludedDataImport(
                            ViSession instrSession,
                            ViInt32   resultsIncludedDataImport)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_DATA_RESULTS_INCLUDED_DATA_IMPORT, resultsIncludedDataImport),
            2, "Results Included Data Import");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Exports the marker peak list available for spurious emission
/// HIFN  measurements to a file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fileName/Exports the marker peak list available for spurious emission
/// HIPAR fileName/measurements to a file.
ViStatus _VI_FUNC rsfpl_DataResultsExportMarkerPeakListSpurious(
                            ViSession instrSession,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_DATA_RESULTS_EXPORT_MARKER_PEAK_LIST_SPURIOUS, fileName),
            2, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Exports trace data from the specified window to an ASCII file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR trace/Defines the trace on which the largest signal is searched for in the
/// HIPAR trace/selected measurement window.
/// HIPAR fileName/String containing the path and name of the target file.
ViStatus _VI_FUNC rsfpl_DataResultsExportTrace(ViSession instrSession,
                                               ViInt32   window,
                                               ViInt32   trace,
                                               ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trace, 0, 6),
            3, "Trace");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:STOR%ld:TRAC %ld,'%s'", window, trace, fileName);
    checkErr(rsfpl_WriteCommandWithOPCSync(instrSession, cmd, 0));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This command imports trace data from the specified window to an ASCII
/// HIFN  file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR trace/Number of the trace to be stored. This parameter is ignored if
/// HIPAR trace/RSFPL_ATTR_DATA_RESULTS_INCLUDED_DATA_IMPORT is set to ALL
/// HIPAR trace/(rsfpl_ConfigureDataResultsIncludedDataImport).
/// HIPAR fileName/String containing the path and name of the target file.
ViStatus _VI_FUNC rsfpl_DataResultsImportTrace(ViSession instrSession,
                                               ViInt32   window,
                                               ViInt32   trace,
                                               ViChar    fileName[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViInt32  windowCount = 17;
    ViInt32  traceMax = 6;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Window", &windowCount));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, windowCount - 1),
        2, "Window");
    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Trace", &traceMax));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trace, 1, traceMax),
            3, "Trace");

    snprintf(cmd, RS_MAX_MESSAGE_LEN, "MMEM:LOAD%ld:TRAC %ld,'%s'", window, trace, fileName);

    checkErr(rsfpl_WriteCommandWithOPCSync(instrSession, cmd, 0));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Exports the SEM and spurious emission list evaluation to a file.
/// HIFN     The file format is *.dat.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR fileName/Exports the SEM and spurious emission list evaluation to a file.
ViStatus _VI_FUNC rsfpl_DataResultsExportSpuriousEmissionList(
                            ViSession instrSession,
                            ViInt32   window,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_Y_AXIS_AUTOSCALE_ONCE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");
    viCheckParm(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_DATA_RESULTS_EXPORT_SPURIOUS_EMISSION_LIST, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Exports the marker peak list to a file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR fileName/Exports the marker peak list to a file.
ViStatus _VI_FUNC rsfpl_DataResultsExportMarkerPeakList(ViSession instrSession,
                                                        ViInt32   window,
                                                        ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_DATA_RESULTS_EXPORT_MARKER_PEAK_LIST,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");
    viCheckParm(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_DATA_RESULTS_EXPORT_MARKER_PEAK_LIST, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Exports spectrogram data to an ASCII file.
/// HIFN     The file contains the data for every frame in the history buffer. The
/// HIFN  data corresponding to a particular frame begins with information about
/// HIFN  the frame number and the time that frame was recorded.
/// HIFN     Note that, depending on the size of the history buffer, the process of
/// HIFN  exporting the data can take a while.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR fileName/Exports spectrogram data to an ASCII file.
ViStatus _VI_FUNC rsfpl_DataResultsExportSpectrogramData(ViSession instrSession,
                                                         ViInt32   window,
                                                         ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_DATA_RESULTS_EXPORT_SPECTROGRAM_DATA,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");
    viCheckParm(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_DATA_RESULTS_EXPORT_SPECTROGRAM_DATA, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures general test report settings. Use
/// HIFN  rsfpl_ConfigureDataFileName to configure the name of the report and the
/// HIFN  location where the report file is to be saved.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR pageFormat/Selects the size of the test report document.
/// HIPAR fileFormat/File format for the test report.
/// HIPAR printerFriendlyColors/Turns the use of printer friendly colors on and off.
/// HIPAR includeDate/Includes or excludes date and time from the test report.
/// HIPAR includePageCount/Includes or excludes page number from the test report.
ViStatus _VI_FUNC rsfpl_ConfigureTestReportTemplateGeneralSettings(
                            ViSession instrSession,
                            ViInt32   pageFormat,
                            ViInt32   fileFormat,
                            ViBoolean printerFriendlyColors,
                            ViBoolean includeDate,
                            ViBoolean includePageCount)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_TEST_REPORT_TEMPLATE_PAGE_FORMAT, pageFormat),
            2, "Page Format");

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_HCOPY_FILE_FORMAT, fileFormat),
            3, "File Format");

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_TEST_REPORT_TEMPLATE_PRINTER_FRIENDLY_COLORS_ENABLED, printerFriendlyColors),
            4, "Printer Friendly Colors");

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_TEST_REPORT_TEMPLATE_INCLUDE_DATE, includeDate),
            5, "Include Date");

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_TEST_REPORT_TEMPLATE_INCLUDE_PAGE_COUNT, includePageCount),
            6, "Include Page Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the location where the measurement data sets for the report
/// HIFN  are stored until the report is created.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR path/Defines the location where the measurement data sets for the report
/// HIPAR path/are stored until the report is created.
ViStatus _VI_FUNC rsfpl_ConfigureTestReportTemplateRawDataStorage(
                            ViSession instrSession,
                            ViString  path)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_TEST_REPORT_TEMPLATE_RAW_DATA_STORAGE, path),
            2, "Path");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures and defines header lines in the document.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR headerLine/Selects the header line to configure.
/// HIPAR visibility/Selects how often the items in the report header are displayed in the
/// HIPAR visibility/document.
/// HIPAR title/Defines a custom name for one of the items part of the report header.
/// HIPAR value/Optionally defines a descriptive text for one of the items part of the
/// HIPAR value/report header.
ViStatus _VI_FUNC rsfpl_ConfigureTestReportTemplateTitlePageLine(
                            ViSession instrSession,
                            ViInt32   headerLine,
                            ViInt32   visibility,
                            ViString  title,
                            ViString  value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_TEST_REPORT_TEMPLATE_TITLE_PAGE_LINE_VISIBILITY,
            "HeaderLine", headerLine - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Header Line");

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_TEST_REPORT_TEMPLATE_TITLE_PAGE_LINE_VISIBILITY, visibility),
            3, "Visibility");

    if (visibility != RSFPL_VAL_TEST_REPORT_ITEM_VISIBILITY_NEVER)
    {
        viCheckParm(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_TEST_REPORT_TEMPLATE_TITLE_PAGE_LINE_TITLE, title),
                4, "Title");

        viCheckParm(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_TEST_REPORT_TEMPLATE_TITLE_PAGE_LINE_VALUE, value),
                5, "Value");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects a graphic (for example a company logo) that is shown at the
/// HIFN  top of each page in the test report and the conditions under which the
/// HIFN  picture is shown.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR visibility/Selects how often the logo is displayed in the document.
/// HIPAR logoPath/Selects a graphic (for example a company logo) that is shown at the
/// HIPAR logoPath/top of each page in the test report.
ViStatus _VI_FUNC rsfpl_ConfigureTestReportTemplateLogo(ViSession instrSession,
                                                        ViInt32   visibility,
                                                        ViString  logoPath)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_TEST_REPORT_TEMPLATE_TITLE_PAGE_LOGO_VISIBILITY, visibility),
            2, "Visibility");

    if (visibility != RSFPL_VAL_TEST_REPORT_ITEM_VISIBILITY_NEVER)
    {
        viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_TEST_REPORT_TEMPLATE_TITLE_PAGE_LOGO, logoPath),
                3, "Logo Path");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the type of information that a test report is made up out of.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channelType/Defines the channel type that the selection applies to.
/// HIPAR arraySize/The size of the items array.
/// HIPAR items/An array that defines the items you want to include in the test
/// HIPAR items/report.
ViStatus _VI_FUNC rsfpl_ConfigureTestReportTemplateSelectedItems(
                            ViSession instrSession,
                            ViInt32   channelType,
                            ViInt32   arraySize,
                            ViInt32   items[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   param[RS_MAX_MESSAGE_LEN] = "";
    ViPChar  p2buf = param;
    ViPChar  p2cmdString;

    checkErr(RsCore_LockSession(instrSession));

    if (channelType != RSFPL_VAL_TEST_REPORT_CHANNEL_ACTIVE)
    {
        viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession,
            &RsFpl_rngTestReportChannelType,
            channelType, &p2cmdString),
            2, "Channel Type");

        p2buf += snprintf(p2buf, RS_MAX_MESSAGE_LEN, "%s,'", p2cmdString);
    }
    else
    {
        p2buf += snprintf(p2buf, RS_MAX_MESSAGE_LEN, "'");
    }

    for (int i = 0; i < arraySize; i++)
    {
        viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession,
            &RsFpl_rngTestReportItems,
            items[i], &p2cmdString),
            4, "Items");
        if (i == 0)
        {
            p2buf += snprintf(p2buf, RS_MAX_MESSAGE_LEN - strlen(param) - 1, "%s", p2cmdString);
        }
        else
        {
            p2buf += snprintf(p2buf, RS_MAX_MESSAGE_LEN - strlen(param) - 1, ",%s", p2cmdString);
        }
    }

    p2buf += snprintf(p2buf, RS_MAX_MESSAGE_LEN - strlen(param) - 1, "'");

    checkErr(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_TEST_REPORT_TEMPLATE_SELECTED_ITEMS, param));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the type of information that a test report is made up out of.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channelType/Defines the channel type that the selection applies to. Leave blank
/// HIPAR channelType/for the currently active channel type.
/// HIPAR items/A comma-separated list which defines the items you want to include in
/// HIPAR items/the test report.
ViStatus _VI_FUNC rsfpl_ConfigureTestReportTemplateSelectedItemsString(
                            ViSession instrSession,
                            ViString  channelType,
                            ViString  items)
{
    ViStatus error = VI_SUCCESS;
    ViChar   param[RS_MAX_MESSAGE_LEN] = "";

    checkErr(RsCore_LockSession(instrSession));

    if ((channelType == NULL) || (strlen(channelType) == 0))
    {
        snprintf(param, RS_MAX_MESSAGE_LEN, "'%s'", items);
    }
    else
    {
        snprintf(param, RS_MAX_MESSAGE_LEN, "%s,'%s'", channelType, items);
    }

    checkErr(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_TEST_REPORT_TEMPLATE_SELECTED_ITEMS, param));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the test report templates available in the default report
/// HIFN  directory (and its subdirectories).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR templateCatalogue/Queries the test report templates available in the default report
/// HIPAR templateCatalogue/directory (and its subdirectories).
/// HIPAR bytesRead/Returns the number of bytes read from the instrument. If this is
/// HIPAR bytesRead/larger than the allocated buffer size, not all data wa returned.
/// HIPAR bytesRead/Consider reallocating the buffer and calling again.
ViStatus _VI_FUNC rsfpl_QueryTestReportTemplateCatalogue(ViSession instrSession,
                                                         ViInt32   bufferSize,
                                                         ViChar    templateCatalogue[],
                                                         ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViStringWithRealLen(instrSession, NULL,
        RSFPL_ATTR_TEST_REPORT_TEMPLATE_CATALOGUE,
        RS_VAL_DIRECT_USER_CALL, bufferSize, templateCatalogue, bytesRead),
        3, "Template Catalogue");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Saves a test report template in XML format.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR templateName/String containing the name of the template. The .xml file extension is
/// HIPAR templateName/added automatically.
ViStatus _VI_FUNC rsfpl_SaveTestReportTemplate(ViSession instrSession,
                                               ViString  templateName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_HIDDEN_TEST_REPORT_TEMPLATE_SAVE, templateName),
            2, "Template");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Loads a test report template.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR templateName/String containing the name of the template.
ViStatus _VI_FUNC rsfpl_LoadTestReportTemplate(ViSession instrSession,
                                               ViString  templateName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_HIDDEN_TEST_REPORT_TEMPLATE_LOAD, templateName),
            2, "Template");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Deletes a test report template.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR templateName/String containing the name of the template.
ViStatus _VI_FUNC rsfpl_DeleteTestReportTemplate(ViSession instrSession,
                                                 ViString  templateName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_HIDDEN_TEST_REPORT_TEMPLATE_DELETE, templateName),
            2, "Template");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Restores the default configuration of the test report regarding the
/// HIFN  information that is part of the report. It also restores the default
/// HIFN  names of the measurement information titles.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_TestReportRestoreDefaults(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_TEST_REPORT_RESTORE_DEFAULTS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Creates a new dataset for a new test report. Creating a new test
/// HIFN  report deletes all previously saved datasets. The current measurement
/// HIFN  results are added as the first dataset to the new report.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_TestReportNew(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_TEST_REPORT_NEW, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Adds the current measurement results to the test report.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_TestReportAppend(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_TEST_REPORT_APPEND, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Deletes one of the datasets that are currently part of a test report.
/// HIFN  Note that the command only deletes datasets as a whole (= complete
/// HIFN  chapters). Deleting individual items of a dataset is not possible.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR dataset/Data set to delete from the test report.
ViStatus _VI_FUNC rsfpl_TestReportRemoveSet(ViSession instrSession,
                                            ViInt32   dataset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_HIDDEN_TEST_REPORT_REMOVE_SET, dataset),
            2, "Dataset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Removes all existing datasets from the test report.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_TestReportRemoveAllSets(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_TEST_REPORT_REMOVE_ALL_SETS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures or disables the report title page.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR includeTitlePage/Includes or excludes the title page from the test report.
/// HIPAR title/Defines the title for the test report as shown on its title page.
/// HIPAR abstract/Optionally defines the description of the test report as shown on its
/// HIPAR abstract/title page.
ViStatus _VI_FUNC rsfpl_ConfigureTestReportTitlePage(ViSession instrSession,
                                                     ViBoolean includeTitlePage,
                                                     ViString  title,
                                                     ViString  abstract)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_TEST_REPORT_TEMPLATE_INCLUDE_TITLE_PAGE, includeTitlePage),
            2, "Include Title Page");

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_TEST_REPORT_TITLE, title),
            3, "Title");
    if (abstract == NULL)
    {
        viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_TEST_REPORT_DESCRIPTION, ""),
                4, "Abstract");
    }
    else
    {
        viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_TEST_REPORT_DESCRIPTION, abstract),
                4, "Abstract");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Includes or excludes the complete set of measurement information from
/// HIFN  the test report.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR includeInformation/Includes or excludes the complete set of measurement information from
/// HIPAR includeInformation/the test report.
ViStatus _VI_FUNC rsfpl_ConfigureTestReportIncludeUserDefinedInformation(
                            ViSession instrSession,
                            ViBoolean includeInformation)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_TEST_REPORT_INCLUDE_USER_DEFINED_INFORMATION, includeInformation),
            2, "Include Information");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the selected information to be included in the test report for
/// HIFN  the currently active channel. This list is set for the current template
/// HIFN  with rsfpl_ConfigureTestReportTemplateSelectedItems or
/// HIFN  rsfpl_ConfigureTestReportTemplateSelectedItemsString.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR testReportItemList/Queries the selected information to be included in the test report for
/// HIPAR testReportItemList/the currently active channel.
/// HIPAR bytesRead/Returns the number of bytes read from the instrument. If this is
/// HIPAR bytesRead/larger than the allocated buffer size, not all data wa returned.
/// HIPAR bytesRead/Consider reallocating the buffer and calling again.
ViStatus _VI_FUNC rsfpl_QueryTestReportItemList(ViSession instrSession,
                                                ViInt32   bufferSize,
                                                ViChar    testReportItemList[],
                                                ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViStringWithRealLen(instrSession, NULL,
        RSFPL_ATTR_TEST_REPORT_ITEM_LIST,
        RS_VAL_DIRECT_USER_CALL, bufferSize,
        testReportItemList, bytesRead),
        2, "String");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Saves the test report with the selected items to the selected Report
/// HIFN  Path with the selected File Type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR filetype/File format for the test report.
/// HIPAR reportPath/Filename for the test report.
ViStatus _VI_FUNC rsfpl_TestReportSave(ViSession instrSession,
                                       ViInt32   fileType,
                                       ViString  reportPath)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_HCOPY_MODE, RSFPL_VAL_HCOPY_MODE_REPORT));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_HCOPY_FILE_FORMAT, fileType),
            2, "File Type");

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_DATA_FILE_NAME, reportPath),
            3, "Report Path");
    checkErr(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_HCOPY_PRINT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries all transducer factors stored on the instrument.
/// HIFN     After general data for the transducer storage directory, data for the
/// HIFN  individual files is listed.
/// HIFN     The result is a comma-separated list of values with the following
/// HIFN  syntax:
/// HIFN     <UsedMem>,<FreeMem>,<FileSize>,<FileName>[,<FileSize>,<FileName>]
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR catalog/Queries all available transducer factors.
/// HIPAR arraySize/Defines the memory size allocated for Catalog.
/// HIPAR returnedValues/Returns number of returned values.
ViStatus _VI_FUNC rsfpl_QueryTransducerFactorCatalog(ViSession instrSession,
                                                     ViUInt32  arraySize,
                                                     ViChar    catalog[],
                                                     ViUInt32* returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_GetAttributeViString(instrSession, NULL, RSFPL_ATTR_TRANSDUCER_FACTOR_CATALOG,
            arraySize, catalog),
            2, "Catalog");

    *returnedValues = (ViUInt32)strlen(catalog);

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects a transducer factor.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR selectedTransducerFactor/Selects a transducer factor.
ViStatus _VI_FUNC rsfpl_ConfigureSelectedTransducerFactor(ViSession instrSession,
                                                          ViString  selectedTransducerFactor)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_SELECTED_TRANSDUCER_FACTOR, selectedTransducerFactor),
            2, "Selected Transducer Factor");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures selected transducer factor.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR scaling/Selects the frequency scaling of the transducer factor.
/// HIPAR unit/Selects the unit of the transducer factor.
/// HIPAR comment/Defines the comment for the selected transducer factor.
ViStatus _VI_FUNC rsfpl_ConfigureTransducerFactor(ViSession instrSession,
                                                  ViInt32   scaling,
                                                  ViInt32   unit,
                                                  ViString  comment)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_TRANSDUCER_FACTOR_SCALING, scaling),
            2, "Scaling");

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_TRANSDUCER_FACTOR_UNIT, unit),
            3, "Unit");

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_TRANSDUCER_FACTOR_COMMENT, comment),
            4, "Comment");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures transducer factors for specific trace points. A set of
/// HIFN  transducer factors defines an interpolated transducer line and can be
/// HIFN  stored on the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR arraySize/This parameter determines the size of each array in Frequency and
/// HIPAR arraySize/Level parameters.
/// HIPAR frequency/This array defines the frequency part (in Hz) of the test points for
/// HIPAR frequency/the selected transducer factor. The frequencies must be in ascending
/// HIPAR frequency/order.
/// HIPAR level/This array defines the level part of the test points for the selected
/// HIPAR level/transducer factor. The level values are sent as dimensionless numbers;
/// HIPAR level/the unit is specified by control Unit.
ViStatus _VI_FUNC rsfpl_TransducerFactorWriteData(ViSession instrSession,
                                                  ViUInt32  arraySize,
                                                  ViReal64  frequency[],
                                                  ViReal64  level[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_WriteAsciiViReal64ArraysInterleaved(instrSession, "SENS:CORR:TRAN:DATA ", arraySize, frequency, level, NULL, NULL, NULL, NULL));
    checkErr(rsfpl_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns the selected transducer factor on or off.
/// HIFN     Before you can use the attribute, you have to select a transducer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR transducerFactorEnabled/Turns the selected transducer factor on or off.
ViStatus _VI_FUNC rsfpl_ConfigureTransducerFactorEnabled(ViSession instrSession,
                                                         ViBoolean transducerFactorEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_TRANSDUCER_FACTOR_ENABLED, transducerFactorEnabled),
            2, "Transducer Factor Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns an automatic adjustment of the reference level to the transducer
/// HIFN  on and off.
/// HIFN     Before you can use the attribute, you have to select and turn on a
/// HIFN  transducer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR levelAutoAdjustEnabled/Turns an automatic adjustment of the reference level to the transducer
/// HIPAR levelAutoAdjustEnabled/on and off.
ViStatus _VI_FUNC rsfpl_ConfigureTransducerFactorRefLevelAutoAdjustEnabled(
                            ViSession instrSession,
                            ViBoolean levelAutoAdjustEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_TRANSDUCER_FACTOR_REF_LEVEL_AUTO_ADJUST_ENABLED, levelAutoAdjustEnabled),
            2, "Level Auto Adjust Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This attribute loads the transducer factor from the selected file in
/// HIFN  .CSV format.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR transducerFactorImport/This control loads the transducer factor from the selected file in
/// HIPAR transducerFactorImport/.CSV format.
ViStatus _VI_FUNC rsfpl_TransducerFactorImport(ViSession instrSession,
                                               ViString  transducerFactorImport)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_TRANSDUCER_FACTOR_IMPORT, transducerFactorImport),
            2, "Transducer Factor Import");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Exports transducer factor data to an ASCII (CSV) file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fileName/String containing the path and name of the target file.
/// HIPAR transducerName/Name of the transducer factor to be exported.
ViStatus _VI_FUNC rsfpl_TransducerFactorExport(ViSession instrSession,
                                               ViString  fileName,
                                               ViString  transducerName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:STOR:TFAC '%s','%s'", fileName, transducerName);
    checkErr(rsfpl_WriteCommandWithOPCSync(instrSession, cmd, 0));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Deletes the currently selected transducer factor.
/// HIFN     Before you can use the attribute/method, you have to select a
/// HIFN  transducer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_DeleteTransducerFactor(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_DELETE_TRANSDUCER_FACTOR, NULL),
            2, "Delete Transducer Factor");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the reference oscillator.
/// HIFN     If you want to select the external reference, it must be connected to
/// HIFN  the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR referenceOscillatorSource/Selects the reference oscillator.
ViStatus _VI_FUNC rsfpl_ConfigureReferenceOscillatorSource(ViSession instrSession,
                                                           ViInt32   referenceOscillatorSource)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_GENERAL_REFERENCE_OSCILLATOR_SOURCE, referenceOscillatorSource),
            2, "Reference Oscillator Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns the output of a reference signal on the corresponding connector
/// HIFN  ('Ref Output') on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR output100MHzEnabled/Turns the output of a reference signal on the corresponding connector
/// HIPAR output100MHzEnabled/('Ref Output') on and off.
ViStatus _VI_FUNC rsfpl_ConfigureReferenceOscillator100MHzEnabled(
                            ViSession instrSession,
                            ViBoolean output100MHzEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_GENERAL_REFERENCE_OSCILLATOR_OUTPUT_100_MHZ_ENABLED, output100MHzEnabled),
            2, "Output 100MHz Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns logging of remote commands on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR remoteLoggingEnabled/Turns logging of remote commands on and off.
ViStatus _VI_FUNC rsfpl_ConfigureRemoteLoggingEnabled(ViSession instrSession,
                                                      ViBoolean remoteLoggingEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_GENERAL_REMOTE_LOGGING_ENABLED, remoteLoggingEnabled),
            2, "Remote Logging Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures display of instrument in remote control mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR displayUpdateEnabled/Turns the display during remote operation on and off.
/// HIPAR displayErrorEnabled/The error display during remote operation on and off.
ViStatus _VI_FUNC rsfpl_ConfigureRemoteDisplay(ViSession instrSession,
                                               ViBoolean displayUpdateEnabled,
                                               ViBoolean displayErrorEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_GENERAL_DISPLAY_UPDATE_ENABLED, displayUpdateEnabled),
            2, "Display Update Enabled");

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_GENERAL_DISPLAY_ERROR_ENABLED, displayErrorEnabled),
            2, "Display Error Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines whether the 'Display Update'
/// HIFN  (RSFPL_ATTR_GENERAL_DISPLAY_UPDATE_ENABLED) function remains available
/// HIFN  in remote operation or not.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR lockEnabled/Defines whether the 'Display Update'
/// HIPAR lockEnabled/(RSFPL_ATTR_GENERAL_DISPLAY_UPDATE_ENABLED) function remains available
/// HIPAR lockEnabled/in remote operation or not.
ViStatus _VI_FUNC rsfpl_ConfigureRemoteDisplayLockEnabled(ViSession instrSession,
                                                          ViBoolean lockEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_GENERAL_DISPLAY_LOCK_ENABLED, lockEnabled),
            2, "Lock Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates the local lockout (remote control) or returns to the local
/// HIFN  mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR generalLocalLockoutEnabled/Activates the local lockout (remote control) or returns to the local
/// HIPAR generalLocalLockoutEnabled/mode.
ViStatus _VI_FUNC rsfpl_ConfigureLocalLockout(ViSession instrSession,
                                              ViBoolean generalLocalLockoutEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_GENERAL_LOCAL_LOCKOUT_ENABLED, generalLocalLockoutEnabled),
            2, "General Local Lockout Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the language of the software-defined interface elements (such
/// HIFN  as softkeys, dialog boxes, diagram texts etc.).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR displayLanguage/Defines the language of the software-defined interface elements (such
/// HIPAR displayLanguage/as softkeys, dialog boxes, diagram texts etc.).
ViStatus _VI_FUNC rsfpl_ConfigureDisplayLanguage(ViSession instrSession,
                                                 ViInt32   displayLanguage)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_GENERAL_DISPLAY_LANGUAGE, displayLanguage),
            2, "Display Language");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the date and time on the instrument.
// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR year/The current year.
/// HIPAR month/The current month.
/// HIPAR day/The current day.
/// HIPAR hour/The current hour.
/// HIPAR minute/The current minute.
/// HIPAR second/The current second.
ViStatus _VI_FUNC rsfpl_ConfigureSystemDateTime(ViSession instrSession,
                                                ViInt32   year,
                                                ViInt32   month,
                                                ViInt32   day,
                                                ViInt32   hour,
                                                ViInt32   minute,
                                                ViInt32   second)
{
    ViStatus error = VI_SUCCESS;
    ViChar   param[RS_MAX_SHORT_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, year, 1),
        2, "Year");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, month, 1, 12),
            3, "Month");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, day, 1, 31),
            4, "Day");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, hour, 0, 23),
            5, "Hour");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, minute, 0, 59),
            6, "Minute");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, second, 0, 60), // 60 allows for leap seconds
        7, "Second");

    snprintf(param, RS_MAX_SHORT_MESSAGE_BUF_SIZE, "%ld,%ld,%ld", year, month, day);
    checkErr(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_GENERAL_SYSTEM_DATE, param));

    snprintf(param, RS_MAX_SHORT_MESSAGE_BUF_SIZE, "%ld,%ld,%ld", hour, minute, second);
    checkErr(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_GENERAL_SYSTEM_TIME, param));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the date and time on the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR year/The instrument's current system year.
/// HIPAR month/The instrument's current system month.
/// HIPAR day/The instrument's current system day.
/// HIPAR hour/The instrument's current system hour.
/// HIPAR minute/The instrument's current system minute.
/// HIPAR second/The instrument's current system second.
ViStatus _VI_FUNC rsfpl_QuerySystemDateTime(ViSession instrSession,
                                            ViInt32*  year,
                                            ViInt32*  month,
                                            ViInt32*  day,
                                            ViInt32*  hour,
                                            ViInt32*  minute,
                                            ViInt32*  second)
{
    ViStatus error = VI_SUCCESS;
    ViChar   response[RS_MAX_SHORT_MESSAGE_BUF_SIZE] = "";
    ViPInt32 results1 = NULL;
    ViPInt32 results2 = NULL;
    ViInt32  resCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsfpl_GetAttributeViString(instrSession, NULL,
        RSFPL_ATTR_GENERAL_SYSTEM_DATE,
        RS_MAX_SHORT_MESSAGE_BUF_SIZE,
        response));
    checkErr(RsCore_Convert_String_To_ViInt32Array(instrSession, response, &results1, &resCount));
    if (resCount != 3)
    {
        viCheckErrElab(RS_ERROR_UNEXPECTED_RESPONSE, "Unexpected response from instrument.");
    }
    if (year) *year = results1[0];
    if (month) *month = results1[1];
    if (day) *day = results1[2];

    resCount = 0;
    checkErr(rsfpl_GetAttributeViString(instrSession, NULL,
        RSFPL_ATTR_GENERAL_SYSTEM_TIME,
        RS_MAX_SHORT_MESSAGE_BUF_SIZE,
        response));
    checkErr(RsCore_Convert_String_To_ViInt32Array(instrSession, response, &results2, &resCount));
    if (resCount != 3)
    {
        viCheckErrElab(RS_ERROR_UNEXPECTED_RESPONSE, "Unexpected response from instrument.");
    }
    if (hour) *hour = results2[0];
    if (minute) *minute = results2[1];
    if (second) *second = results2[2];

Error:
    if (results1) free(results1);
    if (results2) free(results2);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Resets the response to the REV? query for the revision number to the
/// HIFN  factory default.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_GeneralResetRevisionToFactoryDefault(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_GENERAL_RESET_REVISION_TO_FACTORY_DEFAULT, NULL),
            2, "General Reset Revision To Factory Default");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Resets the query to *IDN? to its default value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_GeneralResetIDNQueryToFactoryDefault(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_GENERAL_RESET_IDN_QUERY_TO_FACTORY_DEFAULT, NULL),
            2, "General Reset IDN Query To Factory Default");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This attribute  returns '1' for the factory default state and '0'  if
/// HIFN  the ID string is changed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR factoryDefaultIDN/This control  returns '1' for the factory default state and '0'  if
/// HIPAR factoryDefaultIDN/the ID string is changed.
ViStatus _VI_FUNC rsfpl_QueryFactoryDefaultIDN(ViSession instrSession,
                                               ViInt32*  factoryDefaultIDN)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsfpl_GetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_GENERAL_GET_IDN_STRING_FACTORY, factoryDefaultIDN));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the response to *IDN?.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR idnQueryString/Defines the response to *IDN?.
ViStatus _VI_FUNC rsfpl_ConfigureIDNQueryString(ViSession instrSession,
                                                ViString  idnQueryString)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_GENERAL_IDN_QUERY_STRING, idnQueryString),
            2, "IDN Query String");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the LXI settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR arraySize/Defines the memory size allocated for LXI Info.
/// HIPAR lxiInfo/Queries the LXI settings.
/// HIPAR returnedValues/Returns number of returned values.
ViStatus _VI_FUNC rsfpl_QueryLXIInfo(ViSession instrSession,
                                     ViUInt32  arraySize,
                                     ViChar    lxiInfo[],
                                     ViUInt32* returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_GetAttributeViString(instrSession, NULL, RSFPL_ATTR_GENERAL_LXI_INFO,
            arraySize, lxiInfo),
            3, "LXI Info");

    *returnedValues = (ViUInt32)strlen(lxiInfo);

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Resets the LAN configuration as required by the LXI standard. The
/// HIFN  command also resets the LXI password and instrument description.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_LXILANReset(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_GENERAL_LXI_LAN_RESET, NULL),
            2, "LXI LAN Reset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the LXI instrument description.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR lxiInstrumentDescription/Defines the LXI instrument description.
ViStatus _VI_FUNC rsfpl_ConfigureLXIInstrumentDescription(ViSession instrSession,
                                                          ViString  lxiInstrumentDescription)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_GENERAL_LXI_INSTRUMENT_DESCRIPTION, lxiInstrumentDescription),
            2, "LXI Instrument Description");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the LXI password.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR lxiPassword/Defines the LXI password.
ViStatus _VI_FUNC rsfpl_ConfigureLXIPassword(ViSession instrSession,
                                             ViString  lxiPassword)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_GENERAL_LXI_PASSWORD, lxiPassword),
            2, "LXI Password");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Deletes all contents of the 'System Messages' table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_ClearInstrumentErrors(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_SYSTEM_CLEAR_ERRORS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function deletes all contents of the 'Remote Errors' table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_ClearRemoteErrors(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_SYSTEM_CLEAR_REMOTE_ERRORS_TABLE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the error messages that occurred during remote operation.
/// HIFN  Note: The remote error list is automatically cleared when the R&S FPL
/// HIFN  is shut down.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Sets the size of Errors string.
/// HIPAR errorTable/Returns the error message string read from the instrument's error
/// HIPAR errorTable/message queue.
/// HIPAR returnCount/Count of bytes read from instrument. Might be larger than Buffer Size,
/// HIPAR returnCount/indicating that Buffer Size was not sufficient to accomodate
/// HIPAR returnCount/instrument's response.
ViStatus _VI_FUNC rsfpl_QueryRemoteErrors(ViSession instrSession,
                                          ViInt32   bufferSize,
                                          ViChar    errorTable[],
                                          ViInt32*  returnCount)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, bufferSize, 0),
        2, "Buffer Size");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, errorTable),
        3, "Null address for Error Table buffer");

    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession,
        "SYST:ERR:LIST? REM", bufferSize, errorTable, returnCount));

    checkErr(rsfpl_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the BIOS version of the CPU board.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR arraySize/Defines the memory size allocated for System BIOS Info.
/// HIPAR systemBIOSInfo/Queries the BIOS version of the CPU board.
/// HIPAR returnedValues/Returns number of returned values.
ViStatus _VI_FUNC rsfpl_QuerySystemBIOSInfo(ViSession instrSession,
                                            ViUInt32  arraySize,
                                            ViChar    systemBIOSInfo[],
                                            ViUInt32* returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_GetAttributeViString(instrSession, NULL, RSFPL_ATTR_SYSTEM_BIOS_INFO,
            arraySize, systemBIOSInfo),
            2, "System BIOS Info");

    *returnedValues = (ViUInt32)strlen(systemBIOSInfo);

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries hardware information.
/// HIFN
/// HIFN     String containing the following information for every hardware
/// HIFN  component.
/// HIFN     component: name of the hardware component
/// HIFN     serial#: serial number of the component
/// HIFN     order#: order number of the component
/// HIFN     model: model of the component
/// HIFN     code: code of the component
/// HIFN     revision: revision of the component
/// HIFN     subrevision: subrevision of the component
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR arraySize/Defines the memory size allocated for System HW Info.
/// HIPAR systemHWInfo/Queries hardware information.
/// HIPAR returnedValues/Returns number of returned values.
ViStatus _VI_FUNC rsfpl_QuerySystemHWInfo(ViSession instrSession,
                                          ViUInt32  arraySize,
                                          ViChar    systemHWInfo[],
                                          ViUInt32* returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_GetAttributeViString(instrSession, NULL, RSFPL_ATTR_SYSTEM_HW_INFO,
            arraySize, systemHWInfo),
            2, "System HW Info");

    *returnedValues = (ViUInt32)strlen(systemHWInfo);

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries information about the hardware and software components.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR arraySize/Defines the memory size allocated for System Version Info.
/// HIPAR systemVersionInfo/Queries information about the hardware and software components.
/// HIPAR returnedValues/Returns number of returned values.
ViStatus _VI_FUNC rsfpl_QuerySystemVersionInfo(ViSession instrSession,
                                               ViUInt32  arraySize,
                                               ViChar    systemVersionInfo[],
                                               ViUInt32* returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_GetAttributeViString(instrSession, NULL, RSFPL_ATTR_SYSTEM_HW_INFO,
            arraySize, systemVersionInfo),
            2, "System Version Info");

    *returnedValues = (ViUInt32)strlen(systemVersionInfo);

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Shuts down the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_SystemShutdown(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_GENERAL_SHUTDOWN, NULL),
            2, "System Shutdown");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the current charge state of the optional battery packs.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR systemBatteryLevel/Queries the current charge state of the optional battery packs.
ViStatus _VI_FUNC rsfpl_QuerySystemBatteryLevel(ViSession instrSession,
                                                ViReal64* systemBatteryLevel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsfpl_GetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_SYSTEM_BATTERY_LEVEL, systemBatteryLevel));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Reboots the instrument, including the operating system.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_SystemReboot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_GENERAL_REBOOT, NULL),
            2, "System Reboot");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Calibration query
/// HIFN     Initiates a calibration of the instrument and subsequently queries the
/// HIFN  calibration status. Responses > 0 indicate errors.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR calibrate/Calibration query
ViStatus _VI_FUNC rsfpl_Calibrate(ViSession instrSession,
                                  ViInt32*  calibrate)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_GetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_CALIBRATE, calibrate),
            2, "Calibrate");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Initiates a calibration (self-alignment) routine and queries if
/// HIFN  calibration was successful.
/// HIFN     During the acquisition of correction data the instrument does not
/// HIFN  accept any remote control commands.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR timeout/Sets the timeout value in ms.
/// HIPAR calibrationSuccessful/Initiates a calibration (self-alignment) routine and queries if
/// HIPAR calibrationSuccessful/calibration was successful.
ViStatus _VI_FUNC rsfpl_CalibrateAll(ViSession  instrSession,
                                     ViUInt32   timeout,
                                     ViBoolean* calibrationSuccessful)
{
    ViStatus error = VI_SUCCESS;
    ViUInt32 oldTimeout = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetVisaTimeout(instrSession, &oldTimeout));
    viCheckParm(RsCore_SetVisaTimeout(instrSession, timeout),
        2, "Timeout");

    viCheckParm(rsfpl_GetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_CALIBRATE_ALL, calibrationSuccessful),
            3, "Calibration Successful");

Error:
    if (oldTimeout > 0)
        RsCore_SetVisaTimeout(instrSession, oldTimeout);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the results collected during calibration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR calibrationResult/Returns the results collected during calibration.
ViStatus _VI_FUNC rsfpl_QueryCalibrationResult(ViSession instrSession,
                                               ViUInt32  arraySize,
                                               ViChar    calibrationResult[],
                                               ViUInt32* returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_GetAttributeViString(instrSession, NULL, RSFPL_ATTR_CALIBRATION_RESULT,
            arraySize, calibrationResult),
            3, "Calibration Result");

    *returnedValues = (ViUInt32)strlen(calibrationResult);

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the frequency of the calibration signal.
/// HIFN     Possible frequencies of the calibration signal are fix.
/// HIFN     If you define a frequency that is not available, the instrument uses
/// HIFN  the next available frequency. Example: a frequency of 20 MHz is rounded
/// HIFN  up to the next available frequency (25 MHz).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR calibrationSignalFrequency/Defines the frequency of the calibration signal.
ViStatus _VI_FUNC rsfpl_ConfigureCalibrationSignalFrequency(ViSession instrSession,
                                                            ViReal64  calibrationSignalFrequency)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_CALIBRATION_SIGNAL_FREQUENCY, calibrationSignalFrequency),
            2, "Calibration Signal Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the bandwidth of the calibration signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR calibrationSignalBandwidth/Selects the bandwidth of the calibration signal.
ViStatus _VI_FUNC rsfpl_ConfigureCalibrationSignalBandwidth(ViSession instrSession,
                                                            ViInt32   calibrationSignalBandwidth)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_CALIBRATION_SIGNAL_BANDWIDTH, calibrationSignalBandwidth),
            2, "Calibration Signal Bandwidth");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates or deactivates the use of an internal calibration signal as
/// HIFN  input for the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR calibrationInputSource/Activates or deactivates the use of an internal calibration signal as
/// HIPAR calibrationInputSource/input for the instrument.
ViStatus _VI_FUNC rsfpl_ConfigureCalibrationInputSource(ViSession instrSession,
                                                        ViInt32   calibrationInputSource)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_CALIBRATION_INPUT_SOURCE, calibrationInputSource),
            2, "Calibration Input Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the frequency of the calibration signal for R&S FPL1000 models
/// HIFN  43 MHz and higher. Only takes effect if a microwave calibration signal
/// HIFN  is selected for input (RSFPL_ATTR_CALIBRATION_INPUT_SOURCE).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR calibrationSignalFrequency/Defines the frequency of the calibration signal for R&S FPL1000 models
/// HIPAR calibrationSignalFrequency/43 MHz and higher. Only takes effect if a microwave calibration signal
/// HIPAR calibrationSignalFrequency/is selected for input (RSFPL_ATTR_CALIBRATION_INPUT_SOURCE).
ViStatus _VI_FUNC rsfpl_ConfigureCalibrationSignalFrequency43MHzPlus(
                            ViSession instrSession,
                            ViReal64  calibrationSignalFrequency)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_CALIBRATION_SIGNAL_FREQUENCY_43_MHZ_PLUS, calibrationSignalFrequency),
            2, "Calibration Signal Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the calibration frequency for frequencies greater than 7 GHz.
/// HIFN  Only takes effect if a microwave calibration signal is selected for
/// HIFN  input (RSFPL_ATTR_CALIBRATION_INPUT_SOURCE).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR calibrationSignalFrequency/Sets the calibration frequency for frequencies greater than 7 GHz.
/// HIPAR calibrationSignalFrequency/Only takes effect if a microwave calibration signal is selected for
/// HIPAR calibrationSignalFrequency/input (RSFPL_ATTR_CALIBRATION_INPUT_SOURCE).
ViStatus _VI_FUNC rsfpl_ConfigureCalibrationSignalFrequency7GHzPlus(
                            ViSession instrSession,
                            ViReal64  calibrationSignalFrequency)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_CALIBRATION_SIGNAL_FREQUENCY_7_GHZ_PLUS, calibrationSignalFrequency),
            2, "Calibration Signal Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the frequency of the internal broadband calibration signal to
/// HIFN  be used for IF filter calibration. Only available if the bandwidth
/// HIFN  extension option R&S FPL1-B160 is installed. Before you can use the
/// HIFN  command, you have to feed in a calibration signal with
/// HIFN  RSFPL_ATTR_CALIBRATION_INPUT_SOURCE.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR calibrationSignalFrequency/Defines the frequency of the internal broadband calibration signal to
/// HIPAR calibrationSignalFrequency/be used for IF filter calibration. Only available if the bandwidth
/// HIPAR calibrationSignalFrequency/extension option R&S FPL1-B160 is installed. Before you can use the
/// HIPAR calibrationSignalFrequency/command, you have to feed in a calibration signal with
/// HIPAR calibrationSignalFrequency/RSFPL_ATTR_CALIBRATION_INPUT_SOURCE.
ViStatus _VI_FUNC rsfpl_ConfigureCalibrationSignalWidebandFrequency(
                            ViSession instrSession,
                            ViReal64  calibrationSignalFrequency)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_CALIBRATION_SIGNAL_WIDEBAND_FREQUENCY, calibrationSignalFrequency),
            2, "Calibration Signal Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the distance of the peaks of the microwave calibration signal
/// HIFN  for calibration of the YIG filter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR peakDistance/Selects the distance of the peaks of the microwave calibration signal
/// HIPAR peakDistance/for calibration of the YIG filter.
ViStatus _VI_FUNC rsfpl_ConfigureCalibrationSignalPeakDistance(
                            ViSession instrSession,
                            ViInt32   peakDistance)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_CALIBRATION_SIGNAL_PEAK_DISTANCE, peakDistance),
            2, "Peak Distance");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This attribute queries the current frontend temperature of the
/// HIFN  instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frontendTemperature/This control queries the current frontend temperature of the
/// HIPAR frontendTemperature/instrument.
ViStatus _VI_FUNC rsfpl_QueryFrontendTemperature(ViSession instrSession,
                                                 ViReal64* frontendTemperature)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_GetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_FRONTEND_TEMPERATURE, frontendTemperature),
            2, "Frontend Temperature");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This attribute reads the results of the selftest out of the
/// HIFN  instrument. The lines of the result table are output as string data
/// HIFN  separated by commas.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR arraySize/Defines the memory size allocated for Self Test Results.
/// HIPAR selfTestResults/This control reads the results of the selftest out of the instrument.
/// HIPAR selfTestResults/The lines of the result table are output as string data separated by
/// HIPAR selfTestResults/commas.
/// HIPAR returnedValues/Returns number of returned values.
ViStatus _VI_FUNC rsfpl_QuerySelfTestResults(ViSession instrSession,
                                             ViUInt32  arraySize,
                                             ViChar    selfTestResults[],
                                             ViUInt32* returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsfpl_QueryViString(instrSession, "DIAG:SERV:STES:RES?", arraySize, selfTestResults, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Restores the calibration results or configuration settings. This
/// HIFN  calibration measurement is application dependent, e.g. noise
/// HIFN  calibration, and is independent of the overall calibration state of the
/// HIFN  analyzer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_RecallCalibration(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_RECALL_CALIBRATION, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the calibration results save filepath and saves the calibration
/// HIFN  results or configuration settings. This calibration measurement is
/// HIFN  application dependent, e.g. noise calibration, and is independent of
/// HIFN  the overall calibration state of the analyzer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR saveCalibration/Sets the calibration results save filepath and saves the calibration
/// HIPAR saveCalibration/results or configuration settings. This calibration measurement is
/// HIPAR saveCalibration/application dependent, e.g. noise calibration, and is independent of
/// HIPAR saveCalibration/the overall calibration state of the analyzer.
ViStatus _VI_FUNC rsfpl_SaveCalibration(ViSession instrSession,
                                        ViString  filePath)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_SAVE_CALIBRATION, filePath),
            2, "File Path");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the calibration save file path. This calibration measurement
/// HIFN  is application dependent, e.g. noise calibration, and is independent of
/// HIFN  the overall calibration state of the analyzer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR filePath/Returns the calibration save file path.
ViStatus _VI_FUNC rsfpl_QueryCalibrationSaveFilePath(ViSession instrSession,
                                                     ViInt32   bufferSize,
                                                     ViChar    filePath[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViStringWithRealLen(instrSession, NULL,
                                                       RSFPL_ATTR_CALIBRATION_SAVE_FILE_PATH,
                                                       RS_VAL_DIRECT_USER_CALL, bufferSize, filePath, NULL),
                2, "File Path");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Starts a service function.
/// HIFN     The service functions are available after you have entered the level 1
/// HIFN  or level 2 system password.
/// HIFN     The ID of the service function is made up out of five numbers,
/// HIFN  separated by a point.
/// HIFN     - function group number
/// HIFN     - board number
/// HIFN     - function number
/// HIFN     - parameter 1 (see the Service Manual)
/// HIFN     - parameter 2 (see the Service Manual)
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serviceFunction/Starts a service function.
ViStatus _VI_FUNC rsfpl_StartServiceFunction(ViSession instrSession,
                                             ViString  serviceFunction)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_SERVICE_FUNCTION, serviceFunction),
            2, "Service Function");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the results of the most recent service function you have used.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR arraySize/Defines the memory size allocated for Last Result.
/// HIPAR lastResult/Queries the results of the most recent service function you have used.
/// HIPAR returnedValues/Returns number of returned values.
ViStatus _VI_FUNC rsfpl_QueryServiceFunctionLastResult(ViSession instrSession,
                                                       ViUInt32  arraySize,
                                                       ViChar    lastResult[],
                                                       ViUInt32* returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_GetAttributeViString(instrSession, NULL, RSFPL_ATTR_SERVICE_FUNCTION_LAST_RESULT,
            arraySize, lastResult),
            3, "Last Results");

    *returnedValues = (ViUInt32)strlen(lastResult);

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Deletes the results of the most recent service function you have used.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_ServiceFunctionDeleteLastResult(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_SERVICE_FUNCTION_DELETE_LAST_RESULT, NULL),
            2, "Service Function Delete Last Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Saves the results of the most recent service function you have used.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fileName/Saves the results of the most recent service function you have used.
ViStatus _VI_FUNC rsfpl_SaveServiceFunctionResult(ViSession instrSession,
                                                  ViString  fileName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_SERVICE_FUNCTION_RESULT_FILE_NAME, fileName),
            2, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Creates a *.zip file with important support information. The *.zip
/// HIFN  file contains the system configuration information ('device
/// HIFN  footprint'), the current eeprom data and a screenshot of the screen
/// HIFN  display (if available).
/// HIFN     As a result of this command, the created file name (including the
/// HIFN  drive and path) is returned.
/// HIFN     If you contact the Rohde&Schwarz support to get help for a certain
/// HIFN  problem, send this file to the support in order to identify and solve
/// HIFN  the problem faster.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR arraySize/Defines the memory size allocated for File Path.
/// HIPAR filePath/Creates a *.zip file with important support information. The *.zip
/// HIPAR filePath/file contains the system configuration information ('device
/// HIPAR filePath/footprint'), the current eeprom data and a screenshot of the screen
/// HIPAR filePath/display (if available).
/// HIPAR returnedValues/Returns number of returned values.
ViStatus _VI_FUNC rsfpl_QueryServiceSystemInformationFilePath(
                            ViSession instrSession,
                            ViUInt32  arraySize,
                            ViChar    filePath[],
                            ViUInt32* returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_GetAttributeViString(instrSession, NULL, RSFPL_ATTR_SERVICE_SYSTEM_INFORMATION_FILE_PATH,
            arraySize, filePath),
            3, "File Path");

    *returnedValues = (ViUInt32)strlen(filePath);

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Clears any previously provided password and returns to the most
/// HIFN  restrictive service level.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_ServiceFunctionResetPassword(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_SERVICE_FUNCTION_RESET_PASSWORD, NULL),
            2, "Service Function Reset Password");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Provides a password for subsequent service functions.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serviceFunctionPassword/Provides a password for subsequent service functions.
ViStatus _VI_FUNC rsfpl_ProvideServiceFunctionPassword(ViSession instrSession,
                                                       ViString  serviceFunctionPassword)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_SERVICE_FUNCTION_PASSWORD, serviceFunctionPassword),
            2, "Service Function Password");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This method initiates a signal acquisition based on the present
/// HIFN  instrument configuration. It then waits for the acquisition to
/// HIFN  complete, and returns the trace as an array of amplitude values. The
/// HIFN  amplitude array returns data that represent the amplitude of the
/// HIFN  signals obtained by sweeping from the start frequency to the stop
/// HIFN  frequency (in frequency domain, in time domain the amplitude array is
/// HIFN  ordered from beginning of sweep to end).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR trace/Selects the type of result to be returned.
/// HIPAR maximumTime/Pass the maximum length of time in which to allow the read waveform
/// HIPAR maximumTime/operation to complete.
/// HIPAR arrayLength/Number of array points requested.
/// HIPAR actualPoints/Number of points actually returned in the Amplitude array and X Array
/// HIPAR actualPoints/array.
/// HIPAR amplitude/A user allocated buffer into which the trace amplitudes is stored.
ViStatus _VI_FUNC rsfpl_ReadYTrace(ViSession instrSession,
                                   ViInt32   window,
                                   ViInt32   trace,
                                   ViUInt32  maximumTime,
                                   ViUInt32  arrayLength,
                                   ViUInt32* actualPoints,
                                   ViReal64  amplitude[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   trString[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trace - 1, RSFPL_VAL_TRACE_RESULT_TRACE1, RSFPL_VAL_TRACE_RESULT_SPECTROGRAM), 3, "Trace");

    checkErr(rsfpl_InitiateSynchronized(instrSession, maximumTime));

    snprintf(trString, RS_MAX_MESSAGE_BUF_SIZE, "%s", gsc_read_y_trace_arr[trace - 1]);

    checkErr(rsfpl_dataReadTrace(instrSession, window, trString, arrayLength, amplitude, (ViInt32*)actualPoints));

    checkErr(rsfpl_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This method returns the trace the spectrum analyzer acquires. The
/// HIFN  trace is from a previously initiated acquisition. The user calls the
/// HIFN  Initiate function to start an acquisition. The user calls the
/// HIFN  Acquisition Status function to determine when the acquisition is
/// HIFN  complete.
/// HIFN     The user may call the Read Y Trace function instead of the Initiate
/// HIFN  function. This method starts an acquisition, waits for the acquisition
/// HIFN  to complete, and returns the trace in one function call.
/// HIFN     The Amplitude array returns data that represents the amplitude of the
/// HIFN  signals obtained by sweeping from the start frequency to the stop
/// HIFN  frequency (in frequency domain, in time domain the amplitude array is
/// HIFN  ordered from beginning of sweep to end). The Amplitude Units attribute
/// HIFN  determines the units of the points in the Amplitude array.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR trace/Selects the type of result to be returned.
/// HIPAR arrayLength/Number of array points requested.
/// HIPAR actualPoints/Number of points actually returned in the array.
/// HIPAR amplitude/A user allocated buffer into which the trace amplitudes is stored.
ViStatus _VI_FUNC rsfpl_FetchYTrace(ViSession instrSession,
                                    ViInt32   window,
                                    ViInt32   trace,
                                    ViUInt32  arrayLength,
                                    ViUInt32* actualPoints,
                                    ViReal64  amplitude[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   trString[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trace - 1, RSFPL_VAL_TRACE_RESULT_TRACE1, RSFPL_VAL_TRACE_RESULT_SPECTROGRAM), 3, "Trace");

    snprintf(trString, RS_MAX_MESSAGE_BUF_SIZE, "%s", gsc_read_y_trace_arr[trace - 1]);

    checkErr(rsfpl_dataReadTrace(instrSession, window, trString, arrayLength, amplitude, (ViInt32*)actualPoints));

    checkErr(rsfpl_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This method queries the previously acquired trace data for the
/// HIFN  specified trace from the memory. As an offset and number of sweep
/// HIFN  points to be retrieved can be specified, the trace data can be
/// HIFN  retrieved in smaller portions, making the function faster than the
/// HIFN  ReadY method. This is useful if only specific parts of the trace data
/// HIFN  are of interest.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR trace/Selects the trace.
/// HIPAR offsetSweepPoint/The offset in sweep points related to the start of the measurement at
/// HIPAR offsetSweepPoint/which data retrieval is to start.
/// HIPAR numberOfSweeps/Number of sweep points to be retrieved from the trace.
/// HIPAR arrayLength/Number of array points requested.
/// HIPAR actualPoints/Number of points actually returned in the array.
/// HIPAR amplitude/A user allocated buffer into which the trace amplitudes is stored.
ViStatus _VI_FUNC rsfpl_FetchYTracePrevious(ViSession instrSession,
                                            ViInt32   window,
                                            ViInt32   trace,
                                            ViInt32   offsetSweepPoint,
                                            ViInt32   numberOfSweeps,
                                            ViUInt32  arrayLength,
                                            ViUInt32* actualPoints,
                                            ViReal64  amplitude[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trace, 1, 6),
            3, "Trace");

    if (window != 0)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":FORM REAL,32;:TRAC%ld:MEM? TRACE%ld,%ld,%ld", window, trace, offsetSweepPoint, numberOfSweeps);
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":FORM REAL,32;:TRAC:MEM? TRACE%ld,%ld,%ld", trace, offsetSweepPoint, numberOfSweeps);
    }

    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arrayLength, amplitude, (ViInt32*)actualPoints));
    checkErr(rsfpl_CheckStatus(instrSession));

Error:
    (void)RsCore_Write(instrSession, "FORM ASC");
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the horizontal trace data for each sweep point in the
/// HIFN  specified window, for example the frequency in frequency domain or the
/// HIFN  time in time domain measurements.
/// HIFN     This is especially useful for traces with non-equidistant x-values,
/// HIFN  e.g. for SEM or Spurious Emissions measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR trace/Selects the trace for Detector definition.
/// HIPAR arrayLength/Number of array points requested.
/// HIPAR actualPoints/Number of points actually returned in the array.
/// HIPAR x/A user allocated buffer into which the trace X values is stored.
ViStatus _VI_FUNC rsfpl_FetchXTrace(ViSession instrSession,
                                    ViInt32   window,
                                    ViInt32   trace,
                                    ViUInt32  arrayLength,
                                    ViUInt32* actualPoints,
                                    ViReal64  x[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trace, 1, 6),
            3, "Trace");

    if (window == 0)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FORM REAL,32;:TRAC:DATA:X? TRACE%ld", trace);
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FORM REAL,32;:TRAC%ld:DATA:X? TRACE%ld", window, trace);
    }

    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arrayLength, x, (ViInt32*)actualPoints));

    checkErr(rsfpl_CheckStatus(instrSession));

Error:
    (void)RsCore_Write(instrSession, "FORM ASC");
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the result of a limit check in the specified window.
/// HIFN     Note that for SEM measurements, the limit line repeated capability is
/// HIFN  irrelevant, as only one specific SEM limit line is checked for the
/// HIFN  currently relevant power class.
/// HIFN     To get a valid result, you have to perform a complete measurement with
/// HIFN  synchronization to the end of the measurement before reading out the
/// HIFN  result. This is only possible for single sweep mode.
/// HIFN     The result of the limit check responds with 0 for PASS, 1 for FAIL.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR limit/This control selects limit.
/// HIPAR state/The result of the limit check responds with 0 for PASS, 1 for FAIL,
/// HIPAR state/and 2 for MARGIN.
ViStatus _VI_FUNC rsfpl_GetLimitCheckResult(ViSession instrSession,
                                            ViInt32   window,
                                            ViInt32   limit,
                                            ViInt32*  state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_limit[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_LIMIT_LINE_CHECK_RESULT,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_LIMIT_LINE_CHECK_RESULT,
            "LimitLine", limit - 1, RS_REPCAP_BUF_SIZE, repCap_limit),
            3, "Limit");

    (void)strncat(repCap, ",", RS_REPCAP_BUF_SIZE - strlen(repCap) - 1);
    (void)strncat(repCap, repCap_limit, RS_REPCAP_BUF_SIZE - strlen(repCap) - 1);

    viCheckParm(rsfpl_GetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_LIMIT_LINE_CHECK_RESULT, state),
            4, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Starts a (single) new measurement.
/// HIFN     With sweep count or average count > 0, this means a restart of the
/// HIFN  corresponding number of measurements. With trace mode MAXHold, MINHold
/// HIFN  and AVERage, the previous results are reset on restarting the
/// HIFN  measurement.
/// HIFN     In single sweep mode, synchronization to the end of the indicated
/// HIFN  number of measurements is achieved with the command *OPC. In
/// HIFN  continuous-sweep mode, synchronization to the sweep end is not possible
/// HIFN  since the overall measurement never ends.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR maximumTime/Pass the maximum length of time in which to allow the read waveform
/// HIPAR maximumTime/operation to complete.
ViStatus _VI_FUNC rsfpl_InitiateSynchronized(ViSession instrSession,
                                             ViUInt32  maximumTime)
{
    ViStatus error = VI_SUCCESS;
    ViUInt32 old_timeout = 0;

    checkErr(RsCore_LockSession(instrSession));

    if (maximumTime != 0)
    {
        checkErr(rsfpl_GetTimeout(instrSession, &old_timeout));
        checkErr(rsfpl_SetTimeout(instrSession, maximumTime));

        checkErr(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_INITIATE_SYNCHRONIZED, NULL));
    }
    else
    {
        checkErr(rsfpl_Initiate(instrSession));
    }

Error:
    if (old_timeout > 0)
        rsfpl_SetTimeout(instrSession, old_timeout);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Starts a (single) new measurement.
/// HIFN     With sweep count or average count > 0, this means a restart of the
/// HIFN  corresponding number of measurements. With trace mode MAXHold, MINHold
/// HIFN  and AVERage, the previous results are reset on restarting the
/// HIFN  measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_Initiate(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsfpl_SetAttributeViString(instrSession, "", RSFPL_ATTR_INITIATE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Aborts the measurement in the current channel setup and resets the
/// HIFN  trigger system.
/// HIFN     Note on blocked remote control programs:
/// HIFN     If a sequential command cannot be completed, for example because a
/// HIFN  triggered sweep never receives a trigger, the remote control program
/// HIFN  will never finish and the remote channel to the FPL is blocked for
/// HIFN  further commands. In this case, you must interrupt processing on the
/// HIFN  remote channel first in order to abort the measurement.
/// HIFN     To do so, send a 'Device Clear' command from the control instrument to
/// HIFN  the FPL on a parallel channel to clear all currently active remote
/// HIFN  channels.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_Abort(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsfpl_SetAttributeViString(instrSession, "", RSFPL_ATTR_ABORT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Restarts a (single) measurement that has been stopped (using Abort) or
/// HIFN  finished in single sweep mode.
/// HIFN     The measurement is restarted at the beginning, not where the previous
/// HIFN  measurement was stopped.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR maximumTime/Pass the maximum length of time in which to allow the read waveform
/// HIPAR maximumTime/operation to complete.
ViStatus _VI_FUNC rsfpl_ContinueMeasurement(ViSession instrSession,
                                            ViUInt32  maximumTime)
{
    ViStatus error = VI_SUCCESS;
    ViUInt32 old_timeout = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsfpl_GetTimeout(instrSession, &old_timeout));
    checkErr(rsfpl_SetTimeout(instrSession, maximumTime));

    checkErr(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_CONTINUE_MEASUREMENT, NULL));

Error:
    if (old_timeout > 0)
        rsfpl_SetTimeout(instrSession, old_timeout);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Triggers all actions waiting for a trigger event. In particular, *TRG
/// HIFN  generates a manual trigger signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR maximumTime/Pass the maximum length of time in which to allow the read waveform
/// HIPAR maximumTime/operation to complete.
ViStatus _VI_FUNC rsfpl_SendSWTrigger(ViSession instrSession,
                                      ViUInt32  maximumTime)
{
    ViStatus error = VI_SUCCESS;
    ViUInt32 old_timeout = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsfpl_GetTimeout(instrSession, &old_timeout));
    checkErr(rsfpl_SetTimeout(instrSession, maximumTime));

    checkErr(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_SEND_SW_TRIGGER, NULL));

Error:
    if (old_timeout > 0)
        rsfpl_SetTimeout(instrSession, old_timeout);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the spurious emission and spectrum emission mask measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementMode/Selects the spurious emission and spectrum emission mask measurements.
ViStatus _VI_FUNC rsfpl_ConfigureMeasurementMode(ViSession instrSession,
                                                 ViInt32   measurementMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_MEASUREMENT_MODE, measurementMode),
            2, "Measurement Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the power measurement results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR reserved/This control is reserved.
/// HIPAR powerMeasurement/Selects the ACP (adjacent channel power) standard.
/// HIPAR arraySize/Defines the memory size allocated for Results.
/// HIPAR results/Returns the measured values:
/// HIPAR returnedValues/Returns number of returned values.
ViStatus _VI_FUNC rsfpl_QueryPowerResults(ViSession instrSession,
                                          ViInt32   reserved,
                                          ViInt32   powerMeasurement,
                                          ViUInt32  arraySize,
                                          ViReal64  results[],
                                          ViUInt32* returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, powerMeasurement, RSFPL_VAL_MEAS_POW_ACP, RSFPL_VAL_MEAS_POW_COB),
            3, "Power Measurement");
    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, arraySize, 1),
        4, "Array Size");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, results),
        5, "Results");

    reserved;
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:MARK:FUNC:POW:RES? %s", gsc_PowerMeasurementArr[powerMeasurement]);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, results, (ViInt32*)returnedValues));
    checkErr(rsfpl_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns a power measurement off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_DisablePowerMeasurement(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_DISABLE_POWER_MEASUREMENT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the trace channel power measurements are performed on and
/// HIFN  trace display mode.
/// HIFN     For the measurement to work, the corresponding trace has to be active.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trace/Selects the trace channel power measurements are performed on.
/// HIPAR displayMode/Selects the trace display mode for power measurements.
ViStatus _VI_FUNC rsfpl_ConfigurePowerMeasurementTrace(ViSession instrSession,
                                                       ViInt32   trace,
                                                       ViInt32   displayMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_POWER_MEASUREMENT_TRACE, trace),
            2, "Trace");

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_POWER_MEASUREMENT_TRACE_DISPLAY_MODE, displayMode),
            3, "Display Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Determines the ideal span, bandwidths and detector for the current
/// HIFN  power measurement.
/// HIFN     To get a valid result, you have to perform a complete measurement with
/// HIFN  synchronization to the end of the measurement before reading out the
/// HIFN  result. This is only possible for single sweep mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Determines the ideal span, bandwidths and detector for the current
/// HIPAR measurement/power measurement.
ViStatus _VI_FUNC rsfpl_PowerMeasurementAdjustSettings(ViSession instrSession,
                                                       ViInt32   measurement)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_POWER_MEASUREMENT_ADJUST_SETTINGS, measurement),
            2, "Measurement");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Determines the ideal reference level for the current measurement.
/// HIFN     This automatic routine makes sure that the that the signal power level
/// HIFN  does not overload the FPL or limit the dynamic range by too small a S/N
/// HIFN  ratio.
/// HIFN     To determine the best reference level, the FPL aborts current
/// HIFN  measurements and performs a series of test sweeps. After it has
/// HIFN  finished the test, it continues with the actual measurement.
/// HIFN     To get a valid result, you have to perform a complete sweep with
/// HIFN  synchronization to the sweep end. This is only possible in single sweep
/// HIFN  mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_PowerMeasurementsFindReference(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_POWER_MEASUREMENTS_FIND_REFERENCE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects a power measurement and turns the measurement on.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Determines the ideal span, bandwidths and detector for the current
/// HIPAR measurement/power measurement.
ViStatus _VI_FUNC rsfpl_PowerMeasurementSelectAndEnable(ViSession instrSession,
                                                        ViInt32   measurement)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_POWER_MEASUREMENT_SELECT_AND_ENABLE, measurement),
            2, "Measurement");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the results of power measurements.
/// HIFN     To get a valid result, you have to perform a complete measurement with
/// HIFN  synchronization to the end of the measurement before reading out the
/// HIFN  result. This is only possible for single sweep mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR resultType/Select result type.
/// HIPAR resultValue/Returns result value.
ViStatus _VI_FUNC rsfpl_QueryPowerMeasurementResult(ViSession instrSession,
                                                    ViInt32   resultType,
                                                    ViReal64* resultValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2pwtype;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, resultType, RSFPL_VAL_POWER_RESULT_CHANNEL, RSFPL_VAL_POWER_RESULT_PPOWER),
            3, "Result Type");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, resultValue),
        3, "Results");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsFpl_rngPowerMeasurementResultType, resultType, &p2pwtype),
        3, "Power Measurement");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:MARK:FUNC:POW:RES? %s", p2pwtype);
    checkErr(rsfpl_QueryViReal64(instrSession, cmd, resultValue));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables or disables the compression point measurement.
/// HIFN     This measurement is only available if the optional internal generator
/// HIFN  is installed (R&S FPL1-B9) and active, and only in the Spectrum
/// HIFN  application.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Enables or disables the compression point measurement.
ViStatus _VI_FUNC rsfpl_ConfigureCompressionPointMeasurementEnabled(
                            ViSession instrSession,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_COMPRESSION_POINT_MEASUREMENT_ENABLED, enabled),
            2, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the reference value for the compression point measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mode/Determines how the reference value for the compression point
/// HIPAR mode/measurement is defined.
/// HIPAR value/Defines the reference value used to calculate the compression point
/// HIPAR value/for manual reference mode.
ViStatus _VI_FUNC rsfpl_ConfigureCompressionPointMeasurementReference(
                            ViSession instrSession,
                            ViInt32   mode,
                            ViReal64  value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_COMPRESSION_POINT_MEASUREMENT_REFERENCE_MODE, mode),
            2, "Mode");

    if (mode == RSFPL_VAL_MANUAL)
    {
        viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_COMPRESSION_POINT_MEASUREMENT_REFERENCE_VALUE, value),
                3, "Value");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables and configures the calculation of the compression points.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR 1dBPointEnabled/Enables or disables the calculation of the 1dB compression point.
/// HIPAR 3dBPointEnabled/Enables or disables the calculation of the 3 dB compression point.
/// HIPAR ndBPointEnabled/Enables or disables the calculation of the N dB compression point. Set
/// HIPAR ndBPointEnabled/the value of N in the next parameter.
/// HIPAR ndBValue/Defines the compression value for the n&nbsp;dB compression point (see
/// HIPAR ndBValue/RSFPL_ATTR_COMPRESSION_POINT_MEASUREMENT_N_DB_POINT_ENABLED).
ViStatus _VI_FUNC rsfpl_ConfigureCompressionPointMeasurementSelectCompressionPoints(
                            ViSession instrSession,
                            ViBoolean _1DBPointEnabled,
                            ViBoolean _3DBPointEnabled,
                            ViBoolean nDBPointEnabled,
                            ViReal64  nDBValue)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_COMPRESSION_POINT_MEASUREMENT_1_DB_POINT_ENABLED, _1DBPointEnabled),
            2, "1 dB Point Enabled");

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_COMPRESSION_POINT_MEASUREMENT_3_DB_POINT_ENABLED, _3DBPointEnabled),
            3, "2 dB Point Enabled");

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_COMPRESSION_POINT_MEASUREMENT_N_DB_POINT_ENABLED, nDBPointEnabled),
            4, "N dB Point Enabled");

    if (nDBPointEnabled)
    {
        viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_COMPRESSION_POINT_MEASUREMENT_N_DB_POINT_VALUE, nDBValue),
                5, "N dB Value");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the results of the 1 dB compression point measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/The size of the allocated Results buffer
/// HIPAR results/Queries the results of the 1 dB compression point measurement.
ViStatus _VI_FUNC rsfpl_QueryCompressionPointMeasurement1dBResult(
                            ViSession instrSession,
                            ViInt32   bufferSize,
                            ViReal64  results[])
{
    ViStatus  error = VI_SUCCESS;
    ViPChar   response = NULL;
    ViPReal64 outDblArray = NULL;
    ViInt32   outArraySize = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeViStringUnknownLen(instrSession, NULL,
        RSFPL_ATTR_QUERY_COMPRESSION_POINT_MEASUREMENT_1_DB_RESULT,
        RS_VAL_DIRECT_USER_CALL,
        &response));
    checkErr(RsCore_Convert_String_To_ViReal64Array(instrSession, response, &outDblArray, &outArraySize));
    if (outArraySize > 0)
    {
        checkErr(RsCore_CopyToUserBufferViReal64Array(results, bufferSize, outDblArray, outArraySize));
    }
    else
    {
        viCheckErrElab(RS_ERROR_UNEXPECTED_RESPONSE, "Unexpected response from instrument.");
    }

Error:
    if (response) free(response);
    if (outDblArray) free(outDblArray);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the results of the 3 dB compression point measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/The size of the allocated Results buffer
/// HIPAR results/Queries the results of the 3 dB compression point measurement.
ViStatus _VI_FUNC rsfpl_QueryCompressionPointMeasurement3dBResult(
                            ViSession instrSession,
                            ViInt32   bufferSize,
                            ViReal64  results[])
{
    ViStatus  error = VI_SUCCESS;
    ViPChar   response = NULL;
    ViPReal64 outDblArray = NULL;
    ViInt32   outArraySize = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeViStringUnknownLen(instrSession, NULL,
        RSFPL_ATTR_QUERY_COMPRESSION_POINT_MEASUREMENT_3_DB_RESULT,
        RS_VAL_DIRECT_USER_CALL,
        &response));
    checkErr(RsCore_Convert_String_To_ViReal64Array(instrSession, response, &outDblArray, &outArraySize));
    if (outArraySize > 0)
    {
        checkErr(RsCore_CopyToUserBufferViReal64Array(results, bufferSize, outDblArray, outArraySize));
    }
    else
    {
        viCheckErrElab(RS_ERROR_UNEXPECTED_RESPONSE, "Unexpected response from instrument.");
    }

Error:
    if (response) free(response);
    if (outDblArray) free(outDblArray);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the results of the N dB compression point measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/The size of the allocated Results buffer
/// HIPAR results/Queries the results of the N dB compression point measurement.
ViStatus _VI_FUNC rsfpl_QueryCompressionPointMeasurementNdBResult(
                            ViSession instrSession,
                            ViInt32   bufferSize,
                            ViReal64  results[])
{
    ViStatus  error = VI_SUCCESS;
    ViPChar   response = NULL;
    ViPReal64 outDblArray = NULL;
    ViInt32   outArraySize = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeViStringUnknownLen(instrSession, NULL,
        RSFPL_ATTR_QUERY_COMPRESSION_POINT_MEASUREMENT_N_DB_RESULT,
        RS_VAL_DIRECT_USER_CALL,
        &response));
    checkErr(RsCore_Convert_String_To_ViReal64Array(instrSession, response, &outDblArray, &outArraySize));
    if (outArraySize > 0)
    {
        checkErr(RsCore_CopyToUserBufferViReal64Array(results, bufferSize, outDblArray, outArraySize));
    }
    else
    {
        viCheckErrElab(RS_ERROR_UNEXPECTED_RESPONSE, "Unexpected response from instrument.");
    }

Error:
    if (response) free(response);
    if (outDblArray) free(outDblArray);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the way the FPL displays the power of adjacent channels. You
/// HIFN  need at least one adjacent channel for the attribute to work.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channelPowerDisplayMode/Selects the way the FPL displays the power of adjacent channels. You
/// HIPAR channelPowerDisplayMode/need at least one adjacent channel for the control to work.
ViStatus _VI_FUNC rsfpl_ConfigureChannelPowerDisplayMode(ViSession instrSession,
                                                         ViInt32   channelPowerDisplayMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_CHANNEL_POWER_DISPLAY_MODE, channelPowerDisplayMode),
            2, "Channel Power Display Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the way the FPL returns results for power measurements. If
/// HIFN  enabled, result is returned as channel power density in dBm/Hz,
/// HIFN  otherwise as channel power in dBm.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR powerAsDensityEnabled/Selects the way the FPL returns results for power measurements. If
/// HIPAR powerAsDensityEnabled/enabled, result is returned as channel power density in dBm/Hz,
/// HIPAR powerAsDensityEnabled/otherwise as channel power in dBm.
ViStatus _VI_FUNC rsfpl_ConfigureChannelPowerAsDensityEnabled(
                            ViSession instrSession,
                            ViBoolean powerAsDensityEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_CHANNEL_POWER_AS_DENSITY_ENABLED, powerAsDensityEnabled),
            2, "Channel Power As Density Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the number of transmission channels.
/// HIFN     The attribute works for measurements in the frequency domain.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR numberOfTransmissionChannels/Defines the number of transmission channels.
ViStatus _VI_FUNC rsfpl_ConfigureChannelPowerNumberOfTransmissionChannels(
                            ViSession instrSession,
                            ViInt32   numberOfTransmissionChannels)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_POWER_CHANNEL_NUMBER_OF_TRANSMISSION_CHANNELS, numberOfTransmissionChannels),
            2, "Number Of Transmission Channels");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the number of pairs of adjacent and alternate channels.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channelPowerChannelPairs/Defines the number of pairs of adjacent and alternate channels.
ViStatus _VI_FUNC rsfpl_ConfigureChannelPowerChannelPairs(ViSession instrSession,
                                                          ViInt32   channelPowerChannelPairs)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_CHANNEL_POWER_CHANNEL_PAIRS, channelPowerChannelPairs),
            2, "Channel Power Channel Pairs");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the weighting filters for all channels.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR filterEnabled/Turns the weighting filters for all channels on and off.
/// HIPAR filterValue/Defines the alpha value for the weighting filter for all channels.
/// HIPAR filterValue/This control has effect only if Filter Enabled == On.
ViStatus _VI_FUNC rsfpl_ConfigureChannelPowerWeightingFilterAll(
                            ViSession instrSession,
                            ViBoolean filterEnabled,
                            ViReal64  filterValue)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_CHANNEL_POWER_WEIGHTING_FILTER_ENABLED_ALL, filterEnabled),
            2, "Filter Enabled");

    if (filterEnabled)
    {
        viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_CHANNEL_POWER_WEIGHTING_FILTER_VALUE_ALL, filterValue),
                2, "Filter Value");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns the limit check for ACLR measurements on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR limitCheckEnabled/Turns the limit check for ACLR measurements on and off.
ViStatus _VI_FUNC rsfpl_ConfigureChannelPowerACLRLimitCheckEnabled(
                            ViSession instrSession,
                            ViBoolean limitCheckEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_CHANNEL_POWER_ACLR_LIMIT_CHECK_ENABLED, limitCheckEnabled),
            2, "Limit Check Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Loads a measurement configuration
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR limitCheckEnabled/Selects the configuration
ViStatus _VI_FUNC rsfpl_ConfigureACLRSelectedMeasurementConfiguration(
                            ViSession instrSession,
                            ViInt32   configuration)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_ACLR_SELECTED_MEASUREMENT_CONFIGURATION, configuration),
            2, "Configuration");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries all files containing ACLR standards.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR arraySize/This parameter determines the size of array Configurations Catalog.
/// HIPAR configurationsCatalog/Queries all files containing ACLR standards.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsfpl_QueryACLRMeasurementConfigurationsCatalog(
                            ViSession instrSession,
                            ViUInt32  arraySize,
                            ViChar    configurationsCatalog[],
                            ViUInt32* returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsfpl_QueryViString(instrSession, "CALC:MARK:FUNC:POW:STAN:CAT?", arraySize, configurationsCatalog, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Loads a measurement configuration.
/// HIFN     The measurement configuration for power measurements consists of
/// HIFN  weighting filter, channel bandwidth and spacing, resolution and video
/// HIFN  bandwidth, detector and sweep time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR configurationName/Loads a measurement configuration.
ViStatus _VI_FUNC rsfpl_ACLRMeasurementConfigurationsLoad(ViSession instrSession,
                                                          ViString  configurationName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_ACLR_MEASUREMENT_CONFIGURATIONS_LOAD, configurationName),
            2, "Configurations Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Saves the current ACLR measurement configuration as a new ACLR
/// HIFN  standard.
/// HIFN     The measurement configuration for power measurements consists of
/// HIFN  weighting filter, channel bandwidth and spacing, resolution and video
/// HIFN  bandwidth, detector and sweep time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR configurationName/Saves the current ACLR measurement configuration as a new ACLR
/// HIPAR configurationName/standard.
ViStatus _VI_FUNC rsfpl_ACLRMeasurementConfigurationsSave(ViSession instrSession,
                                                          ViString  configurationName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_ACLR_MEASUREMENT_CONFIGURATIONS_SAVE, configurationName),
            2, "Configurations Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Deletes a file containing an ACLR standard.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR configurationName/Deletes a file containing an ACLR standard.
ViStatus _VI_FUNC rsfpl_ACLRMeasurementConfigurationsDelete(ViSession instrSession,
                                                            ViString  configurationName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_ACLR_MEASUREMENT_CONFIGURATIONS_DELETE, configurationName),
            2, "Configurations Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the transmission channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channelNumber/Sets the channel number.
/// HIPAR name/Defines a name for a transmission channel.
/// HIPAR bandwidth/Defines the channel bandwidth of the transmission channels.
/// HIPAR spacing/Defines the distance between transmission channels.
ViStatus _VI_FUNC rsfpl_ConfigureChannelPowerTxChannel(ViSession instrSession,
                                                       ViInt32   channelNumber,
                                                       ViString  name,
                                                       ViReal64  bandwidth,
                                                       ViReal64  spacing)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_CHANNEL_POWER_TX_CHANNEL_NAME,
            "Channel", channelNumber - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel Number");

    viCheckParm(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_CHANNEL_POWER_TX_CHANNEL_NAME, name),
            3, "Name");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_CHANNEL_POWER_TX_CHANNEL_BANDWIDTH, bandwidth),
            4, "Bandwidth");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_CHANNEL_POWER_TX_CHANNEL_SPACING, spacing),
            5, "Spacing");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the weighting filter for a transmission channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channelNumber/Sets the channel number.
/// HIPAR filterEnabled/Turns the weighting filter for a transmission channel on and off.
/// HIPAR filterValue/Defines the roll-off factor for the transmission channel weighting
/// HIPAR filterValue/filter. This control has effect only if Filter Enabled == On.
ViStatus _VI_FUNC rsfpl_ConfigureChannelPowerTxChannelWeightingFilter(
                            ViSession instrSession,
                            ViInt32   channelNumber,
                            ViBoolean filterEnabled,
                            ViReal64  filterValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_CHANNEL_POWER_TX_CHANNEL_WEIGHTING_FILTER_ENABLED,
            "Channel", channelNumber - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel Number");

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_CHANNEL_POWER_TX_CHANNEL_WEIGHTING_FILTER_ENABLED, filterEnabled),
            3, "Filter Enabled");

    if (filterEnabled)
    {
        viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_CHANNEL_POWER_TX_CHANNEL_WEIGHTING_FILTER_VALUE, filterValue),
                4, "Filter Value");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the adjacent channels.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR name/Defines a name for the adjacent channel.
/// HIPAR bandwidth/Defines the channel bandwidth of the adjacent channels.
/// HIPAR spacing/Defines the distance from transmission channel to adjacent channel.
ViStatus _VI_FUNC rsfpl_ConfigureChannelPowerAdjacentChannel(ViSession instrSession,
                                                             ViString  name,
                                                             ViReal64  bandwidth,
                                                             ViReal64  spacing)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_CHANNEL_POWER_ADJACENT_CHANNEL_NAME, name),
            2, "Name");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_CHANNEL_POWER_ADJACENT_CHANNEL_BANDWIDTH, bandwidth),
            3, "Bandwidth");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_CHANNEL_POWER_ADJACENT_CHANNEL_SPACING, spacing),
            4, "Spacing");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the weighting filter for the adjacent channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR filterEnabled/Turns the weighting filter for the adjacent channel on and off.
/// HIPAR filterValue/Defines the roll-off factor for the adjacent channel weighting filter.
/// HIPAR filterValue/This control has effect only if Filter Enabled == On.
ViStatus _VI_FUNC rsfpl_ConfigureChannelPowerAdjacentChannelWeightingFilter(
                            ViSession instrSession,
                            ViBoolean filterEnabled,
                            ViReal64  filterValue)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_CHANNEL_POWER_ADJACENT_CHANNEL_WEIGHTING_FILTER_ENABLED, filterEnabled),
            3, "Filter Enabled");

    if (filterEnabled)
    {
        viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_CHANNEL_POWER_ADJACENT_CHANNEL_WEIGHTING_FILTER_VALUE, filterValue),
                4, "Filter Value");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the absolute limit of the adjacent channels.
/// HIFN     If you have defined an absolute limit as well as a relative limit, the
/// HIFN  FPL uses the lower value for the limit check.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR limitCheckEnabled/Turns the absolute limit check for the adjacent channels on and off.
/// HIPAR lowerLimit/The limit of the lower adjacent channel.
/// HIPAR upperLimit/The limit of the upper adjacent channel.
ViStatus _VI_FUNC rsfpl_ConfigureChannelPowerAdjacentLimitAbsolute(
                            ViSession instrSession,
                            ViBoolean limitCheckEnabled,
                            ViReal64  lowerLimit,
                            ViReal64  upperLimit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViReal64Range(instrSession, lowerLimit, -200.0, 200.0), 3, "Lower Limit");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, upperLimit, -200.0, 200.0), 4, "Upper Limit");

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_CHANNEL_POWER_ADJACENT_LIMIT_ABSOLUTE_CHECK_ENABLED, limitCheckEnabled),
            2, "Limit Check Enabled");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:LIM:ACP:ACH:ABS %.12lG,%.12lG", lowerLimit, upperLimit);
    checkErr(rsfpl_WriteCommandWithOPCSync(instrSession, cmd, 0));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the relative limit check for the adjacent channels.
/// HIFN     You have to activate the general ACLR limit check before using this
/// HIFN  method.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR lowerEnabled/Enables/disables lower limit check.
/// HIPAR upperEnabled/Enables/disables upper limit check.
/// HIPAR lowerLimit/The limit of the lower adjacent channel.
/// HIPAR upperLimit/The limit of the upper adjacent channel.
ViStatus _VI_FUNC rsfpl_ConfigureChannelPowerAdjacentLimitRelative(
                            ViSession instrSession,
                            ViBoolean lowerEnabled,
                            ViBoolean upperEnabled,
                            ViReal64  lowerLimit,
                            ViReal64  upperLimit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViReal64Range(instrSession, lowerLimit, 0.0, 100.0), 4, "Lower Limit");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, upperLimit, 0.0, 100.0), 5, "Upper Limit");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:LIM:ACP:ACH:STAT %s,%s", lowerEnabled ? "ON" : "OFF", upperEnabled ? "ON" : "OFF");
    checkErr(rsfpl_WriteCommandWithOPCSync(instrSession, cmd, 0));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:LIM:ACP:ACH %.12lG,%.12lG", lowerLimit, upperLimit);
    checkErr(rsfpl_WriteCommandWithOPCSync(instrSession, cmd, 0));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the state of the limit check for the adjacent channels in an
/// HIFN  ACLR measurement.
/// HIFN     To get a valid result, you have to perform a complete measurement with
/// HIFN  synchronization to the end of the measurement before reading out the
/// HIFN  result. This is only possible for single sweep mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR lower/Result for the lower adjacent channel.
/// HIPAR upper/Result for the upper adjacent channel.
ViStatus _VI_FUNC rsfpl_QueryChannelPowerAdjacentLimitCheckResult(
                            ViSession instrSession,
                            ViInt32*  lower,
                            ViInt32*  upper)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  buffer = NULL;
    ViChar*  p2buf;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, lower),
        2, "Lower");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, upper),
        3, "Upper");

    checkErr(RsCore_QueryViStringUnknownLength(instrSession, "CALC:LIM:ACP:ACH:RES?", &buffer));

    p2buf = strtok(buffer, ",");

    if (p2buf != NULL)
    {
        checkErr(RsCore_GetIndexOfToken(gsc_ResultsString, ',', p2buf, lower, NULL));
    }

    p2buf = strtok(NULL, ",\n");

    if (p2buf != NULL)
    {
        checkErr(RsCore_GetIndexOfToken(gsc_ResultsString, ',', p2buf, upper, NULL));
    }

Error:
    if (buffer) free(buffer);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures alternate channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channelNumber/Sets the alternate channel number.
/// HIPAR channelName/Defines a name for an alternate channel.
/// HIPAR bandwidth/Defines the channel bandwidth of the alternate channels.
/// HIPAR spacing/Defines the distance from transmission channel to alternate channels.
ViStatus _VI_FUNC rsfpl_ConfigureChannelPowerAlternateChannel(
                            ViSession instrSession,
                            ViInt32   channelNumber,
                            ViString  channelName,
                            ViReal64  bandwidth,
                            ViReal64  spacing)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channelNumber, 1, 11),
            2, "Channel Number");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_CHANNEL_POWER_ALTERNATE_CHANNEL_NAME,
            "AlternateChannel", channelNumber - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel Number");

    viCheckParm(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_CHANNEL_POWER_ALTERNATE_CHANNEL_NAME, channelName),
            3, "Channel Name");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_CHANNEL_POWER_ALTERNATE_CHANNEL_BANDWIDTH, bandwidth),
            4, "Bandwidth");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_CHANNEL_POWER_ALTERNATE_CHANNEL_SPACING, spacing),
            5, "Spacing");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the weighting filter for the alternate channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channelNumber/Sets the alternate channel number.
/// HIPAR filterEnabled/Turns the weighting filter for an alternate channel on and off.
/// HIPAR filterValue/Defines the roll-off factor for the alternate channel weighting
/// HIPAR filterValue/filter. This control has effect only if Filter Enabled == On.
ViStatus _VI_FUNC rsfpl_ConfigureChannelPowerAlternateChannelWeightingFilter(
                            ViSession instrSession,
                            ViInt32   channelNumber,
                            ViBoolean filterEnabled,
                            ViReal64  filterValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channelNumber, 1, 11),
            2, "Channel Number");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_CHANNEL_POWER_ALTERNATE_CHANNEL_WEIGHTING_FILTER_ENABLED,
            "AlternateChannel", channelNumber - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel Number");

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_CHANNEL_POWER_ALTERNATE_CHANNEL_WEIGHTING_FILTER_ENABLED, filterEnabled),
            3, "Filter Enabled");

    if (filterEnabled)
    {
        viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_CHANNEL_POWER_ALTERNATE_CHANNEL_WEIGHTING_FILTER_VALUE, filterValue),
                4, "Filter Value");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the absolute limit check for the alternate channels.
/// HIFN     You have to activate the general ACLR limit check before using this
/// HIFN  method.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channelNumber/Sets the alternate channel number.
/// HIPAR lowerEnabled/Enables/disables lower limit check.
/// HIPAR upperEnabled/Enables/disables upper limit check.
/// HIPAR lowerLimit/The limit of the lower alternate channel.
/// HIPAR upperLimit/The limit of the lower alternate channel.
ViStatus _VI_FUNC rsfpl_ConfigureChannelPowerAlternateLimitAbsolute(
                            ViSession instrSession,
                            ViInt32   channelNumber,
                            ViBoolean lowerEnabled,
                            ViBoolean upperEnabled,
                            ViReal64  lowerLimit,
                            ViReal64  upperLimit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channelNumber, 1, 11),
            2, "Channel Number");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, lowerLimit, -200.0, 200.0), 5, "Lower Limit");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, upperLimit, -200.0, 200.0), 6, "Upper Limit");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:LIM:ACP:ALT%ld:ABS:STAT %s,%s", channelNumber, lowerEnabled ? "ON" : "OFF", upperEnabled ? "ON" : "OFF");
    checkErr(rsfpl_WriteCommandWithOPCSync(instrSession, cmd, 0));

    checkErr(rsfpl_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:LIM:ACP:ALT%ld:ABS %.12lG,%.12lG", channelNumber, lowerLimit, upperLimit);
    checkErr(rsfpl_WriteCommandWithOPCSync(instrSession, cmd, 0));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the relative limit check for the alternate channels.
/// HIFN     You have to activate the general ACLR limit check before using this
/// HIFN  method.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channelNumber/Sets the alternate channel number.
/// HIPAR lowerEnabled/Enables/disables lower limit check.
/// HIPAR upperEnabled/Enables/disables upper limit check.
/// HIPAR lowerLimit/The limit of the lower alternate channel.
/// HIPAR upperLimit/The limit of the upper alternate channel.
ViStatus _VI_FUNC rsfpl_ConfigureChannelPowerAlternateLimitRelative(
                            ViSession instrSession,
                            ViInt32   channelNumber,
                            ViBoolean lowerEnabled,
                            ViBoolean upperEnabled,
                            ViReal64  lowerLimit,
                            ViReal64  upperLimit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channelNumber, 1, 11),
            2, "Channel Number");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, lowerLimit, 0.0, 100.0), 5, "Lower Limit");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, upperLimit, 0.0, 100.0), 6, "Upper Limit");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:LIM:ACP:ALT%ld:STAT %s,%s", channelNumber, lowerEnabled ? "ON" : "OFF", upperEnabled ? "ON" : "OFF");
    checkErr(rsfpl_WriteCommandWithOPCSync(instrSession, cmd, 0));

    checkErr(rsfpl_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:LIM:ACP:ALT%ld %.12lG,%.12lG", channelNumber, lowerLimit, upperLimit);
    checkErr(rsfpl_WriteCommandWithOPCSync(instrSession, cmd, 0));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the state of the limit check for the adjacent or alternate
/// HIFN  channels in an ACLR measurement.
/// HIFN     To get a valid result, you have to perform a complete measurement with
/// HIFN  synchronization to the end of the measurement before reading out the
/// HIFN  result. This is only possible for single sweep mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channelNumber/Sets the alternate channel number.
/// HIPAR lower/Result for the lower alternate channel.
/// HIPAR upper/Result for the upper alternate channel.
ViStatus _VI_FUNC rsfpl_QueryChannelPowerAlternateLimitCheckResult(
                            ViSession instrSession,
                            ViInt32   channelNumber,
                            ViInt32*  lower,
                            ViInt32*  upper)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   response[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2buf;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channelNumber, 1, 11),
            2, "Channel Number");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, lower),
        2, "Lower");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, upper),
        3, "Upper");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:LIM:ACP:ALT%ld:RES?", channelNumber);
    checkErr(RsCore_QueryViString(instrSession, cmd, response));

    p2buf = strtok(response, ",");

    if (p2buf != NULL)
    {
        checkErr(RsCore_GetIndexOfToken(gsc_ResultsString, ',', p2buf, lower, NULL));
    }

    p2buf = strtok(NULL, ",\n");

    if (p2buf != NULL)
    {
        checkErr(RsCore_GetIndexOfToken(gsc_ResultsString, ',', p2buf, upper, NULL));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the channel power as the reference for relative ACLR
/// HIFN  measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_ReferenceChannelSetCPReference(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_REFERENCE_CHANNEL_SET_CP_REFERENCE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the reference channel for relative measurements.
/// HIFN     You need at least one channel for the attribute to work.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channelNumberAutoType/Selects the reference channel for relative measurements.
ViStatus _VI_FUNC rsfpl_ConfigureReferenceChannelNumberAutoType(
                            ViSession instrSession,
                            ViInt32   channelNumberAutoType)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_REFERENCE_CHANNEL_NUMBER_AUTO_TYPE, channelNumberAutoType),
            2, "Channel Number Auto Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines a reference channel for relative ACLR measurements.
/// HIFN     You need at least one channel for the attribute to work.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR referenceChannelNumber/Defines a reference channel for relative ACLR measurements.
ViStatus _VI_FUNC rsfpl_ConfigureReferenceChannelNumber(ViSession instrSession,
                                                        ViInt32   referenceChannelNumber)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_REFERENCE_CHANNEL_NUMBER, referenceChannelNumber),
            2, "Reference Channel Number");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the percentage of the total power that defines the occupied
/// HIFN  bandwidth.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR percentOfPowerBandwidth/Selects the percentage of the total power that defines the occupied
/// HIPAR percentOfPowerBandwidth/bandwidth.
ViStatus _VI_FUNC rsfpl_ConfigurePercentOfPowerBandwidth(ViSession instrSession,
                                                         ViReal64  percentOfPowerBandwidth)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_PERCENT_OF_POWER_BANDWIDTH, percentOfPowerBandwidth),
            2, "Percent Of Power Bandwidth");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Initiates a Spectrum Emission Mask measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_SEMInitiate(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_SEM_INITIATE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the number of sub blocks in the SEM measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR semNumberOfSubBlocks/Defines the number of sub blocks in the SEM measurement.
ViStatus _VI_FUNC rsfpl_ConfigureSEMNumberOfSubBlocks(ViSession instrSession,
                                                      ViInt32   semNumberOfSubBlocks)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_SEM_NUMBER_OF_SUB_BLOCKS, semNumberOfSubBlocks),
            2, "SEM Number Of Sub Blocks");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Restores the default configurations of predefined SEM standards.
/// HIFN     Note that the command will overwrite customized standards that have
/// HIFN  the same name as predefined standards.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_SEMConfigurationsRestoreDefault(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_SEM_CONFIGURATIONS_RESTORE_DEFAULT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Loads a measurement configuration.
/// HIFN     Standard definitions are stored in an xml file. The default directory
/// HIFN  for SEM standards is
/// HIFN  C:\Users\Public\Documents\Rohde-Schwarz\Analyzer\sem_std.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR configurationName/Loads a measurement configuration.
ViStatus _VI_FUNC rsfpl_SEMConfigurationsLoad(ViSession instrSession,
                                              ViString  configurationName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_SEM_CONFIGURATIONS_LOAD, configurationName),
            2, "Configuration Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Saves the current SEM measurement configuration.
/// HIFN     Standard definitions are stored in an xml file. The default directory
/// HIFN  for SEM standards is
/// HIFN  C:\Users\Public\Documents\Rohde-Schwarz\Analyzer\sem_std.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR configurationName/Saves the current SEM measurement configuration.
ViStatus _VI_FUNC rsfpl_SEMConfigurationsSave(ViSession instrSession,
                                              ViString  configurationName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_SEM_CONFIGURATIONS_SAVE, configurationName),
            2, "Configurations Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Restores the predefined limit lines for the selected Spectrum Emission
/// HIFN  Mask standard.
/// HIFN     All modifications made to the predefined limit lines are lost and the
/// HIFN  factory-set values are restored.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_SEMConfigurationsRestoreLimits(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_SEM_CONFIGURATIONS_RESTORE_LIMITS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the center frequency of the selected sub block in a Multi-SEM
/// HIFN  measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR subBlock/Defines the number of sub block.
/// HIPAR frequency/Defines the center frequency of the selected sub block in a Multi-SEM
/// HIPAR frequency/measurement.
ViStatus _VI_FUNC rsfpl_ConfigureSEMSubBlockFrequency(ViSession instrSession,
                                                      ViInt32   subBlock,
                                                      ViReal64  frequency)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subBlock, 1, 3),
            2, "Sub Block");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SEM_SUB_BLOCK_FREQUENCY,
            "Subblock", subBlock - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sub Block");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_SEM_SUB_BLOCK_FREQUENCY, frequency),
            3, "Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns high speed mode for SEM measurements on and off.
/// HIFN     For SEM with multiple sub blocks, fast SEM is not available. If more
/// HIFN  than one sub block is defined and a standard is loaded which contains
/// HIFN  an active fast SEM setting, this setting is disabled.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR subBlock/Defines the number of sub block.
/// HIPAR sweepListFastSEMEnabled/Turns high speed mode for SEM measurements on and off.
ViStatus _VI_FUNC rsfpl_ConfigureSweepListFastSEMEnabled(ViSession instrSession,
                                                         ViInt32   subBlock,
                                                         ViBoolean sweepListFastSEMEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subBlock, 1, 3),
            2, "Sub Block");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SWEEP_LIST_FAST_SEM_ENABLED,
            "Subblock", subBlock - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sub Block");

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_SWEEP_LIST_FAST_SEM_ENABLED, sweepListFastSEMEnabled),
            2, "Sweep List Fast SEM Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables or disables symmetrical configuration of the range settings.
/// HIFN  Any changes to the range settings in active Symmetrical Setup mode lead
/// HIFN  to symmetrical changes in the other ranges (where possible).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR subBlock/Defines the number of sub block.
/// HIPAR enabled/Enables or disables symmetrical configuration of the range settings.
/// HIPAR enabled/Any changes to the range settings in active Symmetrical Setup mode lead
/// HIPAR enabled/to symmetrical changes in the other ranges (where possible). In
/// HIPAR enabled/particular, this means:
ViStatus _VI_FUNC rsfpl_ConfigureSweepListSymmetricalSetupEnabled(
                            ViSession instrSession,
                            ViInt32   subBlock,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SWEEP_LIST_SYMMETRICAL_SETUP_ENABLED,
            "Subblock", subBlock - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sub Block");

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_SWEEP_LIST_SYMMETRICAL_SETUP_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the number of ranges in the sweep list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR subBlock/Defines the number of sub block.
/// HIPAR sweepListNumberOfRanges/Queries the number of ranges in the sweep list.
ViStatus _VI_FUNC rsfpl_QuerySweepListNumberOfRanges(ViSession instrSession,
                                                     ViInt32   subBlock,
                                                     ViInt32*  sweepListNumberOfRanges)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subBlock, 1, 3),
            2, "Sub Block");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SWEEP_LIST_NUMBER_OF_RANGES,
            "Subblock", subBlock - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sub Block");

    viCheckParm(rsfpl_GetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_SWEEP_LIST_NUMBER_OF_RANGES, sweepListNumberOfRanges),
            3, "Sweep List Number Of Ranges");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the results of power measurements.
/// HIFN     To get a valid result, you have to perform a complete measurement with
/// HIFN  synchronization to the end of the measurement before reading out the
/// HIFN  result. This is only possible for single sweep mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR subBlock/Defines the number of sub block.
/// HIPAR resultType/Select result type.
/// HIPAR arraySize/Defines the memory size allocated for Results.
/// HIPAR results/Returns the measured values:
/// HIPAR returnedValues/Returns number of returned values.
ViStatus _VI_FUNC rsfpl_QuerySEMPowerMeasurementResult(ViSession instrSession,
                                                       ViInt32   subBlock,
                                                       ViInt32   resultType,
                                                       ViUInt32  arraySize,
                                                       ViReal64  results[],
                                                       ViUInt32* returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  p2measType;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subBlock, 1, 3),
            2, "Sub Block");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, resultType, RSFPL_VAL_POWER_RESULT_CHANNEL, RSFPL_VAL_POWER_RESULT_ACLR),
            3, "Result Type");
    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, arraySize, 1),
        4, "Array Size");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, results),
        5, "Results");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsFpl_rngPowerMeasurementResultType, resultType, &p2measType),
        3, "Power Measurement");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:MARK:FUNC:POW%ld:RES? %s", subBlock, p2measType);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, results, (ViInt32*)returnedValues));
    checkErr(rsfpl_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Which limit line is to be used for an SEM measurement depends on the
/// HIFN  power class the input signal power belongs to. Defines wether the power
/// HIFN  class is determined automatically or manually.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR subBlock/Defines the number of sub block.
/// HIPAR semPowerClassesMode/Which limit line is to be used for an SEM measurement depends on the
/// HIPAR semPowerClassesMode/power class the input signal power belongs to. Defines wether the power
/// HIPAR semPowerClassesMode/class is determined automatically or manually.
ViStatus _VI_FUNC rsfpl_ConfigureSEMPowerClassesMode(ViSession instrSession,
                                                     ViInt32   subBlock,
                                                     ViInt32   semPowerClassesMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subBlock, 1, 3),
            2, "Sub Block");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SEM_POWER_CLASSES_MODE,
            "Subblock", subBlock - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sub Block");

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_SEM_POWER_CLASSES_MODE, semPowerClassesMode),
            3, "SEM Power Classes Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the number of power classes to be defined.
/// HIFN     Must be executed before any new power class values can be defined.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR subBlock/Defines the number of sub block.
/// HIPAR numberOfClasses/Sets the number of power classes to be defined.
ViStatus _VI_FUNC rsfpl_ConfigureSEMPowerClassesNumberOfClasses(
                            ViSession instrSession,
                            ViInt32   subBlock,
                            ViInt32   numberOfClasses)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subBlock, 1, 3),
            2, "Sub Block");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SEM_POWER_CLASSES_NUMBER_OF_CLASSES,
            "Subblock", subBlock - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sub Block");

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_SEM_POWER_CLASSES_NUMBER_OF_CLASSES, numberOfClasses),
            3, "Number Of Classes");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates the manual limit line selection as and specifies the
/// HIFN  expected power as a value. Depending on the entered value, the
/// HIFN  associated predefined limit lines is selected.
/// HIFN     The power class to be used is not defined directly, but via the
/// HIFN  expected power. The power class is not re-assigned to the input signal
/// HIFN  power dynamically, but only once when the command is executed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR subBlock/Defines the number of sub block.
/// HIPAR manualLimitLine/Activates the manual limit line selection as and specifies the
/// HIPAR manualLimitLine/expected power as a value. Depending on the entered value, the
/// HIPAR manualLimitLine/associated predefined limit lines is selected.
ViStatus _VI_FUNC rsfpl_ConfigureSEMPowerClassesManualLimitLine(
                            ViSession instrSession,
                            ViInt32   subBlock,
                            ViReal64  manualLimitLine)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subBlock, 1, 3),
            2, "Sub Block");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SEM_POWER_CLASSES_MANUAL_LIMIT_LINE,
            "Subblock", subBlock - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sub Block");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_SEM_POWER_CLASSES_MANUAL_LIMIT_LINE, manualLimitLine),
            3, "Manual Limit Line");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets up to 4 power classes in one step.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR subBlock/Defines the number of sub block.
/// HIPAR arraySize/Defines the number of elements in Ranges.
/// HIPAR ranges/Defines up to 4 range maximums. The array should have 3 elements.
ViStatus _VI_FUNC rsfpl_ConfigureSEMPowerClassesRangeDefinition(
                            ViSession instrSession,
                            ViInt32   subBlock,
                            ViUInt32  arraySize,
                            ViReal64  ranges[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subBlock, 1, 3),
            2, "Sub Block");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:LIM:ESP%ld:LIM ", subBlock);
    checkErr(RsCore_WriteAsciiViReal64Array(instrSession, cmd, ranges, arraySize));
    checkErr(rsfpl_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Inserts a new SEM range and updates the range numbers accordingly.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR subBlock/Defines the number of sub block.
/// HIPAR range/Defines the number of range.
/// HIPAR insertMode/Inserts a new SEM range and updates the range numbers accordingly.
ViStatus _VI_FUNC rsfpl_SweepListRangeInsert(ViSession instrSession,
                                             ViInt32   subBlock,
                                             ViInt32   range,
                                             ViInt32   insertMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_range[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subBlock, 1, 3),
            2, "Sub Block");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, range, 1, 30),
            3, "Range");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SWEEP_LIST_RANGE_INSERT,
            "Subblock", subBlock - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sub Block");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SWEEP_LIST_RANGE_INSERT,
            "Range", range - 1, RS_REPCAP_BUF_SIZE, repCap_range),
            3, "Range");

    (void)strncat(repCap, ",", RS_REPCAP_BUF_SIZE - strlen(repCap) - 1);
    (void)strncat(repCap, repCap_range, RS_REPCAP_BUF_SIZE - strlen(repCap) - 1);

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_SWEEP_LIST_RANGE_INSERT, insertMode),
            4, "Insert Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Removes a range from the sweep list.
/// HIFN     Note that
/// HIFN     - you cannot delete the reference range
/// HIFN     - a minimum of three ranges is mandatory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR subBlock/Defines the number of sub block.
/// HIPAR range/Defines the number of range.
ViStatus _VI_FUNC rsfpl_SweepListRangeDelete(ViSession instrSession,
                                             ViInt32   subBlock,
                                             ViInt32   range)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_range[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subBlock, 1, 3),
            2, "Sub Block");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, range, 1, 30),
            3, "Range");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SWEEP_LIST_RANGE_DELETE,
            "Subblock", subBlock - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sub Block");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SWEEP_LIST_RANGE_DELETE,
            "Range", range - 1, RS_REPCAP_BUF_SIZE, repCap_range),
            3, "Range");

    (void)strncat(repCap, ",", RS_REPCAP_BUF_SIZE - strlen(repCap) - 1);
    (void)strncat(repCap, repCap_range, RS_REPCAP_BUF_SIZE - strlen(repCap) - 1);

    checkErr(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_SWEEP_LIST_RANGE_DELETE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the start and stop frequency of a SEM range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR subBlock/Defines the number of sub block.
/// HIPAR range/Defines the number of range.
/// HIPAR start/Defines the start frequency of a SEM range.
/// HIPAR stop/Defines the stop frequency of a SEM range.
ViStatus _VI_FUNC rsfpl_ConfigureSweepListRangeStartStop(ViSession instrSession,
                                                         ViInt32   subBlock,
                                                         ViInt32   range,
                                                         ViReal64  start,
                                                         ViReal64  stop)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_range[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subBlock, 1, 3),
            2, "Sub Block");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, range, 1, 30),
            3, "Range");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SWEEP_LIST_RANGE_START,
            "Subblock", subBlock - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sub Block");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SWEEP_LIST_RANGE_START,
            "Range", range - 1, RS_REPCAP_BUF_SIZE, repCap_range),
            3, "Range");

    (void)strncat(repCap, ",", RS_REPCAP_BUF_SIZE - strlen(repCap) - 1);
    (void)strncat(repCap, repCap_range, RS_REPCAP_BUF_SIZE - strlen(repCap) - 1);

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_SWEEP_LIST_RANGE_START, start),
            4, "Start");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_SWEEP_LIST_RANGE_STOP, stop),
            5, "Stop");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the filter type for an SEM range.
/// HIFN     In case of high speed measurements, the filter has to be identical for
/// HIFN  all ranges.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR subBlock/Defines the number of sub block.
/// HIPAR range/Defines the number of range.
/// HIPAR filterType/Selects the filter type for an SEM range.
ViStatus _VI_FUNC rsfpl_ConfigureSweepListRangeFilterType(ViSession instrSession,
                                                          ViInt32   subBlock,
                                                          ViInt32   range,
                                                          ViInt32   filterType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_range[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subBlock, 1, 3),
            2, "Sub Block");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, range, 1, 30),
            3, "Range");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SWEEP_LIST_RANGE_FILTER_TYPE,
            "Subblock", subBlock - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sub Block");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SWEEP_LIST_RANGE_FILTER_TYPE,
            "Range", range - 1, RS_REPCAP_BUF_SIZE, repCap_range),
            3, "Range");

    (void)strncat(repCap, ",", RS_REPCAP_BUF_SIZE - strlen(repCap) - 1);
    (void)strncat(repCap, repCap_range, RS_REPCAP_BUF_SIZE - strlen(repCap) - 1);

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_SWEEP_LIST_RANGE_FILTER_TYPE, filterType),
            4, "Filter Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the resolution bandwidth for a SEM range.
/// HIFN     In case of high speed measurements, the resolution bandwidth has to be
/// HIFN  identical for all ranges.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR subBlock/Defines the number of sub block.
/// HIPAR range/Defines the number of range.
/// HIPAR rbw/Defines the resolution bandwidth for a SEM range.
ViStatus _VI_FUNC rsfpl_ConfigureSweepListRangeRBW(ViSession instrSession,
                                                   ViInt32   subBlock,
                                                   ViInt32   range,
                                                   ViReal64  rbw)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_range[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subBlock, 1, 3),
            2, "Sub Block");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, range, 1, 30),
            3, "Range");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SWEEP_LIST_RANGE_RBW,
            "Subblock", subBlock - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sub Block");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SWEEP_LIST_RANGE_RBW,
            "Range", range - 1, RS_REPCAP_BUF_SIZE, repCap_range),
            3, "Range");

    (void)strncat(repCap, ",", RS_REPCAP_BUF_SIZE - strlen(repCap) - 1);
    (void)strncat(repCap, repCap_range, RS_REPCAP_BUF_SIZE - strlen(repCap) - 1);

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_SWEEP_LIST_RANGE_RBW, rbw),
            4, "RBW");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the video bandwidth for a SEM range.
/// HIFN     In case of high speed measurements, the video bandwidth has to be
/// HIFN  identical for all ranges.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR subBlock/Defines the number of sub block.
/// HIPAR range/Defines the number of range.
/// HIPAR vbw/Defines the video bandwidth for a SEM range.
ViStatus _VI_FUNC rsfpl_ConfigureSweepListRangeVBW(ViSession instrSession,
                                                   ViInt32   subBlock,
                                                   ViInt32   range,
                                                   ViReal64  vbw)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_range[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subBlock, 1, 3),
            2, "Sub Block");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, range, 1, 30),
            3, "Range");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SWEEP_LIST_RANGE_VBW,
            "Subblock", subBlock - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sub Block");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SWEEP_LIST_RANGE_VBW,
            "Range", range - 1, RS_REPCAP_BUF_SIZE, repCap_range),
            3, "Range");

    (void)strncat(repCap, ",", RS_REPCAP_BUF_SIZE - strlen(repCap) - 1);
    (void)strncat(repCap, repCap_range, RS_REPCAP_BUF_SIZE - strlen(repCap) - 1);

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_SWEEP_LIST_RANGE_VBW, vbw),
            4, "VBW");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the sweep time for a SEM range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR subBlock/Defines the number of sub block.
/// HIPAR range/Defines the number of range.
/// HIPAR sweepTimeAutoEnabled/Turns automatic selection of the sweep time for a SEM range on and
/// HIPAR sweepTimeAutoEnabled/off.
/// HIPAR sweepTimeManual/Defines the sweep time for a SEM range.
ViStatus _VI_FUNC rsfpl_ConfigureSweepListRangeSweepTime(ViSession instrSession,
                                                         ViInt32   subBlock,
                                                         ViInt32   range,
                                                         ViBoolean sweepTimeAutoEnabled,
                                                         ViReal64  sweepTimeManual)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_range[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subBlock, 1, 3),
            2, "Sub Block");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, range, 1, 30),
            3, "Range");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SWEEP_LIST_RANGE_SWEEP_TIME_AUTO_ENABLED,
            "Subblock", subBlock - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sub Block");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SWEEP_LIST_RANGE_SWEEP_TIME_AUTO_ENABLED,
            "Range", range - 1, RS_REPCAP_BUF_SIZE, repCap_range),
            3, "Range");

    (void)strncat(repCap, ",", RS_REPCAP_BUF_SIZE - strlen(repCap) - 1);
    (void)strncat(repCap, repCap_range, RS_REPCAP_BUF_SIZE - strlen(repCap) - 1);

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_SWEEP_LIST_RANGE_SWEEP_TIME_AUTO_ENABLED, sweepTimeAutoEnabled),
            4, "Sweep Time Auto Enabled");

    if (!sweepTimeAutoEnabled)
    {
        viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_SWEEP_LIST_RANGE_SWEEP_TIME_MANUAL, sweepTimeManual),
                5, "Sweep Time Manual");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the reference level for a SEM range.
/// HIFN     In case of high speed measurements, the reference level has to be
/// HIFN  identical for all ranges.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR subBlock/Defines the number of sub block.
/// HIPAR range/Defines the number of range.
/// HIPAR referenceLevel/Defines the reference level for a SEM range.
ViStatus _VI_FUNC rsfpl_ConfigureSweepListRangeReferenceLevel(
                            ViSession instrSession,
                            ViInt32   subBlock,
                            ViInt32   range,
                            ViReal64  referenceLevel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_range[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subBlock, 1, 3),
            2, "Sub Block");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, range, 1, 30),
            3, "Range");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SWEEP_LIST_RANGE_REFERENCE_LEVEL,
            "Subblock", subBlock - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sub Block");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SWEEP_LIST_RANGE_REFERENCE_LEVEL,
            "Range", range - 1, RS_REPCAP_BUF_SIZE, repCap_range),
            3, "Range");

    (void)strncat(repCap, ",", RS_REPCAP_BUF_SIZE - strlen(repCap) - 1);
    (void)strncat(repCap, repCap_range, RS_REPCAP_BUF_SIZE - strlen(repCap) - 1);

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_SWEEP_LIST_RANGE_REFERENCE_LEVEL, referenceLevel),
            4, "Reference Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the input attenuation for a SEM range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR subBlock/Defines the number of sub block.
/// HIPAR range/Defines the number of range.
/// HIPAR attenuationAutoEnabled/Turns automatic selection of the input attenuation for a SEM range on
/// HIPAR attenuationAutoEnabled/and off.
/// HIPAR attenuationManual/Defines the input attenuation for a SEM range.
ViStatus _VI_FUNC rsfpl_ConfigureSweepListRangeAttenuation(ViSession instrSession,
                                                           ViInt32   subBlock,
                                                           ViInt32   range,
                                                           ViBoolean attenuationAutoEnabled,
                                                           ViReal64  attenuationManual)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_range[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subBlock, 1, 3),
            2, "Sub Block");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, range, 1, 30),
            3, "Range");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SWEEP_LIST_RANGE_ATTENUATION_AUTO_ENABLED,
            "Subblock", subBlock - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sub Block");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SWEEP_LIST_RANGE_ATTENUATION_AUTO_ENABLED,
            "Range", range - 1, RS_REPCAP_BUF_SIZE, repCap_range),
            3, "Range");

    (void)strncat(repCap, ",", RS_REPCAP_BUF_SIZE - strlen(repCap) - 1);
    (void)strncat(repCap, repCap_range, RS_REPCAP_BUF_SIZE - strlen(repCap) - 1);

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_SWEEP_LIST_RANGE_ATTENUATION_AUTO_ENABLED, attenuationAutoEnabled),
            4, "Attenuation Auto Enabled");

    if (attenuationAutoEnabled == VI_FALSE)
    {
        viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_SWEEP_LIST_RANGE_ATTENUATION_MANUAL, attenuationManual),
                5, "Attenuation Manual");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns the preamplifier for a SEM range on and off.
/// HIFN     In case of high speed measurements, the state of the preamplifier has
/// HIFN  to be identical for all ranges.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR subBlock/Defines the number of sub block.
/// HIPAR range/Defines the number of range.
/// HIPAR preamplifierEnabled/Turns the preamplifier for a SEM range on and off.
ViStatus _VI_FUNC rsfpl_ConfigureSweepListRangePreamplifierEnabled(
                            ViSession instrSession,
                            ViInt32   subBlock,
                            ViInt32   range,
                            ViBoolean preamplifierEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_range[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subBlock, 1, 3),
            2, "Sub Block");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, range, 1, 30),
            3, "Range");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SWEEP_LIST_RANGE_PREAMPLIFIER_ENABLED,
            "Subblock", subBlock - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sub Block");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SWEEP_LIST_RANGE_PREAMPLIFIER_ENABLED,
            "Range", range - 1, RS_REPCAP_BUF_SIZE, repCap_range),
            3, "Range");

    (void)strncat(repCap, ",", RS_REPCAP_BUF_SIZE - strlen(repCap) - 1);
    (void)strncat(repCap, repCap_range, RS_REPCAP_BUF_SIZE - strlen(repCap) - 1);

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_SWEEP_LIST_RANGE_PREAMPLIFIER_ENABLED, preamplifierEnabled),
            4, "Preamplifier Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the gain for a SEM range. In case of high speed measurements,
/// HIFN  the level of the preamplifier has to be identical for all ranges.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR subBlock/Defines the number of sub block.
/// HIPAR range/Defines the number of range.
/// HIPAR level/Selects the gain for a SEM range. In case of high speed measurements,
/// HIPAR level/the level of the preamplifier has to be identical for all ranges.
ViStatus _VI_FUNC rsfpl_ConfigureSweepListRangePreamplifierLevel(
                            ViSession instrSession,
                            ViInt32   subBlock,
                            ViInt32   range,
                            ViReal64  level)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_range[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subBlock, 1, 3),
            2, "Sub Block");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, range, 1, 30),
            3, "Range");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SWEEP_LIST_RANGE_PREAMPLIFIER_ENABLED,
            "Subblock", subBlock - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sub Block");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SWEEP_LIST_RANGE_PREAMPLIFIER_ENABLED,
            "Range", range - 1, RS_REPCAP_BUF_SIZE, repCap_range),
            3, "Range");

    (void)strncat(repCap, ",", RS_REPCAP_BUF_SIZE - strlen(repCap) - 1);
    (void)strncat(repCap, repCap_range, RS_REPCAP_BUF_SIZE - strlen(repCap) - 1);

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_SWEEP_LIST_RANGE_PREAMPLIFIER_LEVEL, level),
            4, "Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects a transducer factor for a SEM range.
/// HIFN     Note that
/// HIFN     - the transducer must cover at least the span of the range
/// HIFN     - the x-axis has to be linear
/// HIFN     - the unit has to be dB
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR subBlock/Defines the number of sub block.
/// HIPAR range/Defines the number of range.
/// HIPAR transducerFileName/Selects a transducer factor for a SEM range.
ViStatus _VI_FUNC rsfpl_ConfigureSweepListRangeTransducerFileName(
                            ViSession instrSession,
                            ViInt32   subBlock,
                            ViInt32   range,
                            ViString  transducerFileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_range[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subBlock, 1, 3),
            2, "Sub Block");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, range, 1, 30),
            3, "Range");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SWEEP_LIST_RANGE_TRANSDUCER_FILE_NAME,
            "Subblock", subBlock - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sub Block");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SWEEP_LIST_RANGE_TRANSDUCER_FILE_NAME,
            "Range", range - 1, RS_REPCAP_BUF_SIZE, repCap_range),
            3, "Range");

    (void)strncat(repCap, ",", RS_REPCAP_BUF_SIZE - strlen(repCap) - 1);
    (void)strncat(repCap, repCap_range, RS_REPCAP_BUF_SIZE - strlen(repCap) - 1);

    viCheckParm(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_SWEEP_LIST_RANGE_TRANSDUCER_FILE_NAME, transducerFileName),
            4, "Transducer File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the function used to calculate the limit line for the n-th
/// HIFN  power class for overlapping ranges in Multi-SEM measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR subBlock/Defines the number of sub block.
/// HIPAR range/Defines the number of range.
/// HIPAR multiLimitCalculation/Defines the function used to calculate the limit line for the n-th
/// HIPAR multiLimitCalculation/power class for overlapping ranges in Multi-SEM measurements.
ViStatus _VI_FUNC rsfpl_ConfigureSweepListRangeMultiLimitCalculation(
                            ViSession instrSession,
                            ViInt32   subBlock,
                            ViInt32   range,
                            ViInt32   multiLimitCalculation)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_range[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subBlock, 1, 3),
            2, "Sub Block");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, range, 1, 30),
            3, "Range");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SWEEP_LIST_RANGE_MULTI_LIMIT_CALCULATION,
            "Subblock", subBlock - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sub Block");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SWEEP_LIST_RANGE_MULTI_LIMIT_CALCULATION,
            "Range", range - 1, RS_REPCAP_BUF_SIZE, repCap_range),
            3, "Range");

    (void)strncat(repCap, ",", RS_REPCAP_BUF_SIZE - strlen(repCap) - 1);
    (void)strncat(repCap, repCap_range, RS_REPCAP_BUF_SIZE - strlen(repCap) - 1);

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_SWEEP_LIST_RANGE_MULTI_LIMIT_CALCULATION, multiLimitCalculation),
            4, "Multi Limit Calculation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the minimum number of sweep points for the range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR subBlock/Defines the number of sub block.
/// HIPAR range/Defines the number of range.
/// HIPAR minSweepPoints/Defines the minimum number of sweep points for the range.
ViStatus _VI_FUNC rsfpl_ConfigureSweepListRangeMinSweepPoints(
                            ViSession instrSession,
                            ViInt32   subBlock,
                            ViInt32   range,
                            ViInt32   minSweepPoints)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_range[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subBlock, 1, 3),
            2, "Sub Block");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, range, 1, 30),
            3, "Range");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SWEEP_LIST_RANGE_MIN_SWEEP_POINTS,
            "Subblock", subBlock - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sub Block");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SWEEP_LIST_RANGE_MIN_SWEEP_POINTS,
            "Range", range - 1, RS_REPCAP_BUF_SIZE, repCap_range),
            3, "Range");

    (void)strncat(repCap, ",", RS_REPCAP_BUF_SIZE - strlen(repCap) - 1);
    (void)strncat(repCap, repCap_range, RS_REPCAP_BUF_SIZE - strlen(repCap) - 1);

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_SWEEP_LIST_RANGE_MIN_SWEEP_POINTS, minSweepPoints),
            4, "Min Sweep Points");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the power class used by the measurement if the mode is set to
/// HIFN  manual.
/// HIFN     Note that:
/// HIFN     You can only use power classes for which limits are defined.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR subBlock/Defines the number of sub block.
/// HIPAR powerClass/Defines the number of power class.
/// HIPAR powerClassEnabled/Selects the power class used by the measurement if the mode is set to
/// HIPAR powerClassEnabled/manual.
ViStatus _VI_FUNC rsfpl_ConfigureSweepListPowerClassEnabled(ViSession instrSession,
                                                            ViInt32   subBlock,
                                                            ViInt32   powerClass,
                                                            ViBoolean powerClassEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_pclass[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subBlock, 1, 3),
            2, "Sub Block");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, powerClass, 1, 4),
            3, "Power Class");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SWEEP_LIST_POWER_CLASS_ENABLED,
            "Subblock", subBlock - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sub Block");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SWEEP_LIST_POWER_CLASS_ENABLED,
            "PowerClass", powerClass - 1, RS_REPCAP_BUF_SIZE, repCap_pclass),
            3, "Power Class");

    (void)strncat(repCap, ",", RS_REPCAP_BUF_SIZE - strlen(repCap) - 1);
    (void)strncat(repCap, repCap_pclass, RS_REPCAP_BUF_SIZE - strlen(repCap) - 1);

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_SWEEP_LIST_POWER_CLASS_ENABLED, powerClassEnabled),
            4, "Power Class Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the limit check mode for each power class.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR subBlock/Defines the number of sub block.
/// HIPAR powerClass/Defines the number of power class.
/// HIPAR limitCheckMode/Selects the limit check mode for each power class.
ViStatus _VI_FUNC rsfpl_ConfigureSweepListPowerClassLimitCheckMode(
                            ViSession instrSession,
                            ViInt32   subBlock,
                            ViInt32   powerClass,
                            ViInt32   limitCheckMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_pclass[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subBlock, 1, 3),
            2, "Sub Block");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, powerClass, 1, 4),
            3, "Power Class");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SWEEP_LIST_POWER_CLASS_LIMIT_CHECK_MODE,
            "Subblock", subBlock - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sub Block");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SWEEP_LIST_POWER_CLASS_LIMIT_CHECK_MODE,
            "PowerClass", powerClass - 1, RS_REPCAP_BUF_SIZE, repCap_pclass),
            3, "Power Class");

    (void)strncat(repCap, ",", RS_REPCAP_BUF_SIZE - strlen(repCap) - 1);
    (void)strncat(repCap, repCap_pclass, RS_REPCAP_BUF_SIZE - strlen(repCap) - 1);

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_SWEEP_LIST_POWER_CLASS_LIMIT_CHECK_MODE, limitCheckMode),
            4, "Limit Check Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the limit check mode for all SEM ranges.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR subBlock/Defines the number of sub block.
/// HIPAR powerClass/Defines the number of power class.
/// HIPAR limitCheckModeAllRanges/Selects the limit check mode for all SEM ranges.
ViStatus _VI_FUNC rsfpl_ConfigureSweepListPowerClassLimitCheckModeAllRanges(
                            ViSession instrSession,
                            ViInt32   subBlock,
                            ViInt32   powerClass,
                            ViInt32   limitCheckModeAllRanges)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_pclass[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subBlock, 1, 3),
            2, "Sub Block");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, powerClass, 1, 4),
            3, "Power Class");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SWEEP_LIST_POWER_CLASS_LIMIT_CHECK_MODE_ALL_RANGES,
            "Subblock", subBlock - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sub Block");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SWEEP_LIST_POWER_CLASS_LIMIT_CHECK_MODE_ALL_RANGES,
            "PowerClass", powerClass - 1, RS_REPCAP_BUF_SIZE, repCap_pclass),
            3, "Power Class");

    (void)strncat(repCap, ",", RS_REPCAP_BUF_SIZE - strlen(repCap) - 1);
    (void)strncat(repCap, repCap_pclass, RS_REPCAP_BUF_SIZE - strlen(repCap) - 1);

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_SWEEP_LIST_POWER_CLASS_LIMIT_CHECK_MODE_ALL_RANGES, limitCheckModeAllRanges),
            4, "Limit Check Mode All Ranges");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the lower and upper limit of a particular power class.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR subBlock/Defines the number of sub block.
/// HIPAR powerClass/Defines the number of power class.
/// HIPAR min/Defines the lower limit of a particular power class.
/// HIPAR max/Defines the upper limit of a particular power class.
ViStatus _VI_FUNC rsfpl_ConfigureSweepListPowerClassLimit(ViSession instrSession,
                                                          ViInt32   subBlock,
                                                          ViInt32   powerClass,
                                                          ViReal64  min,
                                                          ViReal64  max)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_pclass[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subBlock, 1, 3),
            2, "Sub Block");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, powerClass, 1, 4),
            3, "Power Class");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SWEEP_LIST_POWER_CLASS_LIMIT_MIN,
            "Subblock", subBlock - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sub Block");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SWEEP_LIST_POWER_CLASS_LIMIT_MIN,
            "PowerClass", powerClass - 1, RS_REPCAP_BUF_SIZE, repCap_pclass),
            3, "Power Class");

    (void)strncat(repCap, ",", RS_REPCAP_BUF_SIZE - strlen(repCap) - 1);
    (void)strncat(repCap, repCap_pclass, RS_REPCAP_BUF_SIZE - strlen(repCap) - 1);

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_SWEEP_LIST_POWER_CLASS_LIMIT_MIN, min),
            4, "Min");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_SWEEP_LIST_POWER_CLASS_LIMIT_MAX, max),
            5, "Max");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines an absolute limit for a SEM range.
/// HIFN     Unlike manual operation, you can define an absolute limit anytime and
/// HIFN  regardless of the limit check mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR subBlock/Defines the number of sub block.
/// HIPAR range/Defines the number of range.
/// HIPAR powerClass/Defines the number of power class.
/// HIPAR absoluteStart/Defines an absolute limit for a SEM range.
/// HIPAR absoluteStop/Defines an absolute limit for a SEM range.
ViStatus _VI_FUNC rsfpl_ConfigureSweepListPowerClassLimitAbsolute(
                            ViSession instrSession,
                            ViInt32   subBlock,
                            ViInt32   range,
                            ViInt32   powerClass,
                            ViReal64  absoluteStart,
                            ViReal64  absoluteStop)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_range[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_pclass[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subBlock, 1, 3),
            2, "Sub Block");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, range, 1, 30),
            3, "Range");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, powerClass, 1, 4),
            3, "Power Class");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SWEEP_LIST_POWER_CLASS_LIMIT_ABSOLUTE_START,
            "Subblock", subBlock - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sub Block");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SWEEP_LIST_POWER_CLASS_LIMIT_ABSOLUTE_START,
            "Range", range - 1, RS_REPCAP_BUF_SIZE, repCap_range),
            3, "Range");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SWEEP_LIST_POWER_CLASS_LIMIT_ABSOLUTE_START,
            "PowerClass", powerClass - 1, RS_REPCAP_BUF_SIZE, repCap_pclass),
            4, "Power Class");

    (void)strncat(repCap, ",", RS_REPCAP_BUF_SIZE - strlen(repCap) - 1);
    (void)strncat(repCap, repCap_range, RS_REPCAP_BUF_SIZE - strlen(repCap) - 1);
    (void)strncat(repCap, ",", RS_REPCAP_BUF_SIZE - strlen(repCap) - 1);
    (void)strncat(repCap, repCap_pclass, RS_REPCAP_BUF_SIZE - strlen(repCap) - 1);

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_SWEEP_LIST_POWER_CLASS_LIMIT_ABSOLUTE_START, absoluteStart),
            5, "Absolute Start");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_SWEEP_LIST_POWER_CLASS_LIMIT_ABSOLUTE_STOP, absoluteStop),
            6, "Absolute Stop");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines a relative limit for a SEM range.
/// HIFN     Unlike manual operation, you can define a relative limit regardless of
/// HIFN  the limit check mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR subBlock/Defines the number of sub block.
/// HIPAR range/Defines the number of range.
/// HIPAR powerClass/Defines the number of power class.
/// HIPAR relativeStart/Defines a relative limit for a SEM range.
/// HIPAR relativeStop/Defines a relative limit for a SEM range.
ViStatus _VI_FUNC rsfpl_ConfigureSweepListPowerClassLimitRelative(
                            ViSession instrSession,
                            ViInt32   subBlock,
                            ViInt32   range,
                            ViInt32   powerClass,
                            ViReal64  relativeStart,
                            ViReal64  relativeStop)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_range[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_pclass[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subBlock, 1, 3),
            2, "Sub Block");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, range, 1, 30),
            3, "Range");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, powerClass, 1, 4),
            3, "Power Class");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SWEEP_LIST_POWER_CLASS_LIMIT_RELATIVE_START,
            "Subblock", subBlock - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sub Block");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SWEEP_LIST_POWER_CLASS_LIMIT_RELATIVE_START,
            "Range", range - 1, RS_REPCAP_BUF_SIZE, repCap_range),
            3, "Range");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SWEEP_LIST_POWER_CLASS_LIMIT_RELATIVE_START,
            "PowerClass", powerClass - 1, RS_REPCAP_BUF_SIZE, repCap_pclass),
            4, "Power Class");

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_range, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_pclass, NULL);

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_SWEEP_LIST_POWER_CLASS_LIMIT_RELATIVE_START, relativeStart),
            5, "Relative Start");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_SWEEP_LIST_POWER_CLASS_LIMIT_RELATIVE_STOP, relativeStop),
            6, "Relative Stop");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables the use of a function when defining the relative limit for a
/// HIFN  SEM range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR subBlock/Defines the number of sub block.
/// HIPAR range/Defines the number of range.
/// HIPAR powerClass/Defines the number of power class.
/// HIPAR relativeStartFunction/Enables the use of a function when defining the relative limit for a
/// HIPAR relativeStartFunction/SEM range.
/// HIPAR maxStartFunctionLimit/Defines an absolute limit for the MAX function of the relative limit
/// HIPAR maxStartFunctionLimit/for a SEM range. This control has effect only if Relative Start
/// HIPAR maxStartFunctionLimit/Function == Max.
ViStatus _VI_FUNC rsfpl_ConfigureSweepListPowerClassLimitRelativeStartFunction(
                            ViSession instrSession,
                            ViInt32   subBlock,
                            ViInt32   range,
                            ViInt32   powerClass,
                            ViInt32   relativeStartFunction,
                            ViReal64  maxStartFunctionLimit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_range[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_pclass[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subBlock, 1, 3),
            2, "Sub Block");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, range, 1, 30),
            3, "Range");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, powerClass, 1, 4),
            3, "Power Class");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SWEEP_LIST_POWER_CLASS_LIMIT_RELATIVE_START_FUNCTION,
            "Subblock", subBlock - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sub Block");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SWEEP_LIST_POWER_CLASS_LIMIT_RELATIVE_START_FUNCTION,
            "Range", range - 1, RS_REPCAP_BUF_SIZE, repCap_range),
            3, "Range");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SWEEP_LIST_POWER_CLASS_LIMIT_RELATIVE_START_FUNCTION,
            "PowerClass", powerClass - 1, RS_REPCAP_BUF_SIZE, repCap_pclass),
            4, "Power Class");

    (void)strncat(repCap, ",", RS_REPCAP_BUF_SIZE - strlen(repCap) - 1);
    (void)strncat(repCap, repCap_range, RS_REPCAP_BUF_SIZE - strlen(repCap) - 1);
    (void)strncat(repCap, ",", RS_REPCAP_BUF_SIZE - strlen(repCap) - 1);
    (void)strncat(repCap, repCap_pclass, RS_REPCAP_BUF_SIZE - strlen(repCap) - 1);

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_SWEEP_LIST_POWER_CLASS_LIMIT_RELATIVE_START_FUNCTION, relativeStartFunction),
            5, "Relative Start Function");

    if (relativeStartFunction == RSFPL_VAL_STOP_FUNCTION_MAX)
    {
        viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_SWEEP_LIST_POWER_CLASS_LIMIT_RELATIVE_MAX_START_FUNCTION_LIMIT, maxStartFunctionLimit),
                6, "Max Start Function Limit");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the function to be used to determine the relative limit line
/// HIFN  stop value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR subBlock/Defines the number of sub block.
/// HIPAR range/Defines the number of range.
/// HIPAR powerClass/Defines the number of power class.
/// HIPAR relativeStopFunction/Defines the function to be used to determine the relative limit line
/// HIPAR relativeStopFunction/stop value.
/// HIPAR maxStopFunctionLimit/Defines an absolute limit for the MAX function of the relative limit
/// HIPAR maxStopFunctionLimit/for a SEM range. This control has effect only if Relative Stop Function
/// HIPAR maxStopFunctionLimit/== Max.
ViStatus _VI_FUNC rsfpl_ConfigureSweepListPowerClassLimitRelativeStopFunction(
                            ViSession instrSession,
                            ViInt32   subBlock,
                            ViInt32   range,
                            ViInt32   powerClass,
                            ViInt32   relativeStopFunction,
                            ViReal64  maxStopFunctionLimit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_range[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_pclass[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subBlock, 1, 3),
            2, "Sub Block");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, range, 1, 30),
            3, "Range");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, powerClass, 1, 4),
            3, "Power Class");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SWEEP_LIST_POWER_CLASS_LIMIT_RELATIVE_STOP_FUNCTION,
            "Subblock", subBlock - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sub Block");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SWEEP_LIST_POWER_CLASS_LIMIT_RELATIVE_STOP_FUNCTION,
            "Range", range - 1, RS_REPCAP_BUF_SIZE, repCap_range),
            3, "Range");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SWEEP_LIST_POWER_CLASS_LIMIT_RELATIVE_STOP_FUNCTION,
            "PowerClass", powerClass - 1, RS_REPCAP_BUF_SIZE, repCap_pclass),
            4, "Power Class");

    (void)strncat(repCap, ",", RS_REPCAP_BUF_SIZE - strlen(repCap) - 1);
    (void)strncat(repCap, repCap_range, RS_REPCAP_BUF_SIZE - strlen(repCap) - 1);
    (void)strncat(repCap, ",", RS_REPCAP_BUF_SIZE - strlen(repCap) - 1);
    (void)strncat(repCap, repCap_pclass, RS_REPCAP_BUF_SIZE - strlen(repCap) - 1);

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_SWEEP_LIST_POWER_CLASS_LIMIT_RELATIVE_STOP_FUNCTION, relativeStopFunction),
            5, "Relative Stop Function");

    if (relativeStopFunction == RSFPL_VAL_STOP_FUNCTION_MAX)
    {
        viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_SWEEP_LIST_POWER_CLASS_LIMIT_RELATIVE_MAX_STOP_FUNCTION_LIMIT, maxStopFunctionLimit),
                6, "Max Stop Function Limit");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the type of the power reference.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR subBlock/Defines the number of sub block.
/// HIPAR referenceType/Defines the type of the power reference.
ViStatus _VI_FUNC rsfpl_ConfigureReferenceRangePowerReferenceType(
                            ViSession instrSession,
                            ViInt32   subBlock,
                            ViInt32   referenceType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_REFERENCE_RANGE_POWER_REFERENCE_TYPE,
            "Subblock", subBlock - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sub Block");

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_REFERENCE_RANGE_POWER_REFERENCE_TYPE, referenceType),
            3, "Reference Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the channel bandwidth of the reference range.
/// HIFN     The bandwidth is available if the power reference is the channel
/// HIFN  power.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR subBlock/Defines the number of sub block.
/// HIPAR txBandwidth/Defines the channel bandwidth of the reference range.
ViStatus _VI_FUNC rsfpl_ConfigureReferenceRangeTxBandwidth(ViSession instrSession,
                                                           ViInt32   subBlock,
                                                           ViReal64  txBandwidth)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_REFERENCE_RANGE_TX_BANDWIDTH,
            "Subblock", subBlock - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sub Block");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_REFERENCE_RANGE_TX_BANDWIDTH, txBandwidth),
            3, "Tx Bandwidth");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the RRC filter in the reference range.
/// HIFN     The RRC filter is available if the power reference is the channel
/// HIFN  power.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR subBlock/Defines the number of sub block.
/// HIPAR rrcFilterEnabled/Turns the RRC filter in the reference range on and off.
/// HIPAR alpha/Defines the roll-off factor for the RRC filter.
ViStatus _VI_FUNC rsfpl_ConfigureReferenceRangeRRCFilter(ViSession instrSession,
                                                         ViInt32   subBlock,
                                                         ViBoolean rrcFilterEnabled,
                                                         ViReal64  alpha)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_REFERENCE_RANGE_RRC_FILTER_ENABLED,
            "Subblock", subBlock - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sub Block");

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_REFERENCE_RANGE_RRC_FILTER_ENABLED, rrcFilterEnabled),
            3, "RRC Filter Enabled");

    if (rrcFilterEnabled)
    {
        viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_REFERENCE_RANGE_ALPHA, alpha),
                3, "Alpha");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the reference range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR subBlock/Defines the number of sub block.
/// HIPAR referenceRangeNumber/Queries the reference range.
ViStatus _VI_FUNC rsfpl_QueryReferenceRangeNumber(ViSession instrSession,
                                                  ViInt32   subBlock,
                                                  ViInt32*  referenceRangeNumber)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_REFERENCE_RANGE_NUMBER,
            "Subblock", subBlock - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sub Block");

    viCheckParm(rsfpl_GetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_REFERENCE_RANGE_NUMBER, referenceRangeNumber),
            3, "Reference Range Number");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns the list evaluation on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR evaluationEnabled/Turns the list evaluation on and off.
ViStatus _VI_FUNC rsfpl_ConfigureSEMListEvaluationEnabled(ViSession instrSession,
                                                          ViBoolean evaluationEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_SEM_LIST_EVALUATION_ENABLED, evaluationEnabled),
            2, "Evaluation Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures how detailed the list in the Result Summary is.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR allDetectedPeaksIncluded/Configures how detailed the list in the Result Summary is.
ViStatus _VI_FUNC rsfpl_ConfigureSEMListEvaluationAllDetectedPeaksIncluded(
                            ViSession instrSession,
                            ViBoolean allDetectedPeaksIncluded)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_SEM_LIST_EVALUATION_ALL_DETECTED_PEAKS_INCLUDED, allDetectedPeaksIncluded),
            2, "All Detected Peaks Included");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the threshold of the list evaluation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR threshold/Defines the threshold of the list evaluation.
ViStatus _VI_FUNC rsfpl_ConfigureSEMListEvaluationThreshold(ViSession instrSession,
                                                            ViReal64  threshold)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_SEM_LIST_EVALUATION_THRESHOLD, threshold),
            2, "Threshold");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns the peak labels in the diagram on and off.
/// HIFN     Peak labels are blue squares.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR evaluationPeakLabelsEnabled/Turns the peak labels in the diagram on and off.
ViStatus _VI_FUNC rsfpl_ConfigureSEMListEvaluationPeakLabelsEnabled(
                            ViSession instrSession,
                            ViBoolean evaluationPeakLabelsEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_SEM_LIST_EVALUATION_PEAK_LABELS_ENABLED, evaluationPeakLabelsEnabled),
            2, "Evaluation Peak Labels Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Initiates a list evaluation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_SEMListEvaluationInitiate(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_SEM_LIST_EVALUATION_INITIATE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Initiates a Spurious Emission measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_SpuriousEmissionsInitiate(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_SPURIOUS_EMISSIONS_INITIATE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns the limit check for all spurious emission measurement ranges on
/// HIFN  and off (<range> is irrelevant).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR rangesEnabled/Turns the limit check for all spurious emission measurement ranges on
/// HIPAR rangesEnabled/and off (<range> is irrelevant).
ViStatus _VI_FUNC rsfpl_ConfigureSpuriousEmissionsRangesEnabled(
                            ViSession instrSession,
                            ViBoolean rangesEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_SPURIOUS_EMISSIONS_RANGES_ENABLED, rangesEnabled),
            2, "Ranges Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the number of ranges in the sweep list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR numberOfRanges/Queries the number of ranges in the sweep list.
ViStatus _VI_FUNC rsfpl_QuerySpuriousEmissionsNumberOfRanges(ViSession instrSession,
                                                             ViInt32*  numberOfRanges)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_GetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_SPURIOUS_EMISSIONS_NUMBER_OF_RANGES, numberOfRanges),
            2, "Number Of Ranges");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Controls the sweep for all ranges.
/// HIFN     Note(s):
/// HIFN     ON - The R&S FPL1000 stops after measuring one range. To continue with
/// HIFN  the next range, you have to use
/// HIFN  RSFPL_ATTR_SWEEP_MODE_CONTINUOUS_ENABLED.
/// HIFN     OFF - The R&S FPL1000 sweeps all ranges in one go.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Controls the sweep (for all ranges, <range> is irrelevant).
ViStatus _VI_FUNC rsfpl_ConfigureSpuriousEmissionsBreakMeasurementEnabled(
                            ViSession instrSession,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_SPURIOUS_EMISSIONS_BREAK_MEASUREMENT_ENABLED, enabled),
            2, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the x-axis range for the spurious emission measurement from the
/// HIFN  start frequency of the first sweep range to the stop frequency of the
/// HIFN  last sweep range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_AdjustSpuriousEmissionsXAxis(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_SPURIOUS_EMISSIONS_ADJUST_X_AXIS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Removes a range from the sweep list.
/// HIFN     Note that
/// HIFN     - you cannot delete the reference range
/// HIFN     - a minimum of three ranges is mandatory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR range/Defines the number of range.
ViStatus _VI_FUNC rsfpl_SpuriousEmissionsRangeDelete(ViSession instrSession,
                                                     ViInt32   range)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, range, 1, 30),
            2, "Range");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SPURIOUS_EMISSIONS_RANGE_DELETE,
            "Range", range - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Range");

    checkErr(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_SPURIOUS_EMISSIONS_RANGE_DELETE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the start and stop frequency of a spurious emission
/// HIFN  measurement range.
/// HIFN     Make sure to set an appropriate span. If you set a span that is
/// HIFN     - smaller than the span the sweep list covers, the FPL will not
/// HIFN  measure the ranges that are outside the span - results may be invalid.
/// HIFN     - greater than the span the sweep list covers, the FPL will adjust the
/// HIFN  start frequency of the first range and the stop frequency of the last
/// HIFN  range to the span
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR range/Defines the number of range.
/// HIPAR start/Defines the start frequency of a spurious emission measurement range.
/// HIPAR stop/Defines the stop frequency of a spurious emission measurement range.
ViStatus _VI_FUNC rsfpl_ConfigureSpuriousEmissionsRangeStartStop(
                            ViSession instrSession,
                            ViInt32   range,
                            ViReal64  start,
                            ViReal64  stop)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, range, 1, 30),
            2, "Range");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SPURIOUS_EMISSIONS_RANGE_START,
            "Range", range - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Range");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_SPURIOUS_EMISSIONS_RANGE_START, start),
            3, "Start");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_SPURIOUS_EMISSIONS_RANGE_STOP, stop),
            4, "Stop");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the filter type for a spurious emission measurement range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR range/Defines the number of range.
/// HIPAR filterType/Selects the filter type for a spurious emission measurement range.
ViStatus _VI_FUNC rsfpl_ConfigureSpuriousEmissionsRangeFilterType(
                            ViSession instrSession,
                            ViInt32   range,
                            ViInt32   filterType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, range, 1, 30),
            2, "Range");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SPURIOUS_EMISSIONS_RANGE_FILTER_TYPE,
            "Range", range - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Range");

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_SPURIOUS_EMISSIONS_RANGE_FILTER_TYPE, filterType),
            3, "Filter Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the resolution bandwidth for a spurious emission measurement
/// HIFN  range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR range/Defines the number of range.
/// HIPAR rbw/Defines the resolution bandwidth for a spurious emission measurement
/// HIPAR rbw/range.
ViStatus _VI_FUNC rsfpl_ConfigureSpuriousEmissionsRangeRBW(ViSession instrSession,
                                                           ViInt32   range,
                                                           ViReal64  rbw)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, range, 1, 30),
            2, "Range");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SPURIOUS_EMISSIONS_RANGE_RBW,
            "Range", range - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Range");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_SPURIOUS_EMISSIONS_RANGE_RBW, rbw),
            3, "RBW");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the video bandwidth for a spurious emission measurement range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR range/Defines the number of range.
/// HIPAR vbw/Defines the video bandwidth for a spurious emission measurement range.
ViStatus _VI_FUNC rsfpl_ConfigureSpuriousEmissionsRangeVBW(ViSession instrSession,
                                                           ViInt32   range,
                                                           ViReal64  vbw)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, range, 1, 30),
            2, "Range");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SPURIOUS_EMISSIONS_RANGE_VBW,
            "Range", range - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Range");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_SPURIOUS_EMISSIONS_RANGE_VBW, vbw),
            3, "VBW");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the sweep time for a spurious emission measurement range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR range/Defines the number of range.
/// HIPAR sweepTimeAutoEnabled/Turns automatic selection of the sweep time for a spurious emission
/// HIPAR sweepTimeAutoEnabled/measurement range on and off.
/// HIPAR sweepTimeManual/Defines the sweep time for a spurious emission measurement range. This
/// HIPAR sweepTimeManual/control has effect only if Sweep Time Auto Enabled == Off.
ViStatus _VI_FUNC rsfpl_ConfigureSpuriousEmissionsRangeSweepTime(
                            ViSession instrSession,
                            ViInt32   range,
                            ViBoolean sweepTimeAutoEnabled,
                            ViReal64  sweepTimeManual)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, range, 1, 30),
            2, "Range");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SPURIOUS_EMISSIONS_RANGE_SWEEP_TIME_AUTO_ENABLED,
            "Range", range - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Range");

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_SPURIOUS_EMISSIONS_RANGE_SWEEP_TIME_AUTO_ENABLED, sweepTimeAutoEnabled),
            3, "Sweep Time Auto Enabled");

    if (sweepTimeAutoEnabled == VI_FALSE)
    {
        viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_SPURIOUS_EMISSIONS_RANGE_SWEEP_TIME_MANUAL, sweepTimeManual),
                4, "Sweep Time Manual");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the detector for a spurious emission measurement range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR range/Defines the number of range.
/// HIPAR detector/Selects the detector for a spurious emission measurement range.
ViStatus _VI_FUNC rsfpl_ConfigureSpuriousEmissionsRangeDetector(
                            ViSession instrSession,
                            ViInt32   range,
                            ViInt32   detector)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, range, 1, 30),
            2, "Range");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SPURIOUS_EMISSIONS_RANGE_DETECTOR,
            "Range", range - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Range");

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_SPURIOUS_EMISSIONS_RANGE_DETECTOR, detector),
            3, "Detector");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the reference level for a spurious emission measurement range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR range/Defines the number of range.
/// HIPAR referenceLevel/Defines the reference level for a spurious emission measurement range.
ViStatus _VI_FUNC rsfpl_ConfigureSpuriousEmissionsRangeReferenceLevel(
                            ViSession instrSession,
                            ViInt32   range,
                            ViReal64  referenceLevel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, range, 1, 30),
            2, "Range");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SPURIOUS_EMISSIONS_RANGE_REFERENCE_LEVEL,
            "Range", range - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Range");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_SPURIOUS_EMISSIONS_RANGE_REFERENCE_LEVEL, referenceLevel),
            3, "Reference Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the input attenuation for a spurious emission measurement
/// HIFN  range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR range/Defines the number of range.
/// HIPAR attenuationAutoEnabled/Turns automatic selection of the input attenuation for a spurious
/// HIPAR attenuationAutoEnabled/emission measurement range on and off.
/// HIPAR attenuationManual/Defines the input attenuation for a spurious emission measurement
/// HIPAR attenuationManual/range. This control has effect only if Attenuation Auto Enabled == On.
ViStatus _VI_FUNC rsfpl_ConfigureSpuriousEmissionsRangeAttenuation(
                            ViSession instrSession,
                            ViInt32   range,
                            ViBoolean attenuationAutoEnabled,
                            ViReal64  attenuationManual)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, range, 1, 30),
            2, "Range");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SPURIOUS_EMISSIONS_RANGE_ATTENUATION_AUTO_ENABLED,
            "Range", range - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Range");

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_SPURIOUS_EMISSIONS_RANGE_ATTENUATION_AUTO_ENABLED, attenuationAutoEnabled),
            3, "Attenuation Auto Enabled");

    if (attenuationAutoEnabled)
    {
        viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_SPURIOUS_EMISSIONS_RANGE_ATTENUATION_MANUAL, attenuationManual),
                4, "Attenuation Manual");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the preamplifier for a spurious emission measurement range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR range/Defines the number of range.
/// HIPAR preamplifierEnabled/Turns the preamplifier for a spurious emission measurement range on
/// HIPAR preamplifierEnabled/and off.
/// HIPAR preamplificationLevel/Selects the preamplification level for the range. This control has
/// HIPAR preamplificationLevel/effect only if Preamplifier Enabled == On.
ViStatus _VI_FUNC rsfpl_ConfigureSpuriousEmissionsRangePreamplifier(
                            ViSession instrSession,
                            ViInt32   range,
                            ViBoolean preamplifierEnabled,
                            ViReal64  preamplificationLevel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, range, 1, 30),
            2, "Range");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SPURIOUS_EMISSIONS_RANGE_PREAMPLIFIER_ENABLED,
            "Range", range - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Range");

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_SPURIOUS_EMISSIONS_RANGE_PREAMPLIFIER_ENABLED, preamplifierEnabled),
            3, "Preamplifier Enabled");

    if (preamplifierEnabled)
    {
        viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_SPURIOUS_EMISSIONS_RANGE_PREAMPLIFICATION_LEVEL, preamplificationLevel),
                4, "Preamplification Level");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the number of sweep points in a spurious emission measurement
/// HIFN  range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR range/Defines the number of range.
/// HIPAR sweepPoints/Defines the number of sweep points in a spurious emission measurement
/// HIPAR sweepPoints/range.
ViStatus _VI_FUNC rsfpl_ConfigureSpuriousEmissionsRangeSweepPoints(
                            ViSession instrSession,
                            ViInt32   range,
                            ViInt32   sweepPoints)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, range, 1, 30),
            2, "Range");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SPURIOUS_EMISSIONS_RANGE_SWEEP_POINTS,
            "Range", range - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Range");

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_SPURIOUS_EMISSIONS_RANGE_SWEEP_POINTS, sweepPoints),
            3, "Sweep Points");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects a transducer factor for a spurious emission measurement range.
/// HIFN     Note that
/// HIFN     - the transducer must cover at least the span of the range
/// HIFN     - the x-axis has to be linear
/// HIFN     - the unit has to be dB
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR range/Defines the number of range.
/// HIPAR transducerFileName/Selects a transducer factor for a spurious emission measurement range.
ViStatus _VI_FUNC rsfpl_ConfigureSpuriousEmissionsRangeTransducerFileName(
                            ViSession instrSession,
                            ViInt32   range,
                            ViString  transducerFileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, range, 1, 30),
            2, "Range");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SPURIOUS_EMISSIONS_RANGE_TRANSDUCER_FILE_NAME,
            "Range", range - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Range");

    viCheckParm(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_SPURIOUS_EMISSIONS_RANGE_TRANSDUCER_FILE_NAME, transducerFileName),
            3, "Transducer File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines an absolute limit for a spurious emission measurement range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR range/Defines the number of range.
/// HIPAR start/Defines an absolute limit for a spurious emission measurement range.
/// HIPAR stop/Defines an absolute limit for a spurious emission measurement range.
ViStatus _VI_FUNC rsfpl_ConfigureSpuriousEmissionsRangeAbsLimit(
                            ViSession instrSession,
                            ViInt32   range,
                            ViReal64  start,
                            ViReal64  stop)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, range, 1, 30),
            2, "Range");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SPURIOUS_EMISSIONS_RANGE_ABS_LIMIT_START,
            "Range", range - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Range");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_SPURIOUS_EMISSIONS_RANGE_ABS_LIMIT_START, start),
            3, "Start");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_SPURIOUS_EMISSIONS_RANGE_ABS_LIMIT_STOP, stop),
            4, "Stop");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns the list evaluation on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR listEvaluationEnabled/Turns the list evaluation on and off.
ViStatus _VI_FUNC rsfpl_ConfigureSpuriousEmissionsListEvaluationEnabled(
                            ViSession instrSession,
                            ViBoolean listEvaluationEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_SPURIOUS_EMISSIONS_LIST_EVALUATION_ENABLED, listEvaluationEnabled),
            2, "List Evaluation Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the number of peaks included in the peak list.
/// HIFN     After this number of peaks has been found, the FPL stops the peak
/// HIFN  search and continues the search in the next measurement range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR numberOfPeaks/Defines the number of peaks included in the peak list.
ViStatus _VI_FUNC rsfpl_ConfigureSpuriousEmissionsListEvaluationNumberOfPeaks(
                            ViSession instrSession,
                            ViInt32   numberOfPeaks)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_SPURIOUS_EMISSIONS_LIST_EVALUATION_NUMBER_OF_PEAKS, numberOfPeaks),
            2, "Number Of Peaks");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures how detailed the list in the Result Summary is.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR allDetectedPeaksIncluded/Configures how detailed the list in the Result Summary is.
ViStatus _VI_FUNC rsfpl_ConfigureSpuriousEmissionsListEvaluationAllDetectedPeaksIncluded(
                            ViSession instrSession,
                            ViBoolean allDetectedPeaksIncluded)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_SPURIOUS_EMISSIONS_LIST_EVALUATION_ALL_DETECTED_PEAKS_INCLUDED, allDetectedPeaksIncluded),
            2, "All Detected Peaks Included");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the threshold of the list evaluation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR threshold/Defines the threshold of the list evaluation.
ViStatus _VI_FUNC rsfpl_ConfigureSpuriousEmissionsListEvaluationThreshold(
                            ViSession instrSession,
                            ViReal64  threshold)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_SPURIOUS_EMISSIONS_LIST_EVALUATION_THRESHOLD, threshold),
            2, "Threshold");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns the peak labels in the diagram on and off.
/// HIFN     Peak labels are blue squares.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR labelsEnabled/Turns the peak labels in the diagram on and off.
ViStatus _VI_FUNC rsfpl_ConfigureSpuriousEmissionsListEvaluationPeakLabelsEnabled(
                            ViSession instrSession,
                            ViBoolean labelsEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_SPURIOUS_EMISSIONS_LIST_EVALUATION_PEAK_LABELS_ENABLED, labelsEnabled),
            2, "Labels Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures statistical measurement options.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR statisticsEnabled/Turns statistical measurement on and off. Only APD or CCDF
/// HIPAR statisticsEnabled/measurements can be enabled at one time.
/// HIPAR measurement/Selects APD or CCDF measurements. Only one of these can be enabled at
/// HIPAR measurement/one time.
/// HIPAR numberOfSamples/Defines the number of samples included in the analysis of statistical
/// HIPAR numberOfSamples/measurement functions.
ViStatus _VI_FUNC rsfpl_ConfigureStatisticalMeasurement(ViSession instrSession,
                                                        ViBoolean statisticsEnabled,
                                                        ViInt32   measurement,
                                                        ViInt32   numberOfSamples)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   arrStatsAttr[2] = {
             RSFPL_ATTR_SIGNAL_STATISTIC_APD_ENABLED,
             RSFPL_ATTR_SIGNAL_STATISTIC_CCDF_ENABLED
             };

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, RSFPL_VAL_STATS_APD, RSFPL_VAL_STATS_CCDF),
            3, "Measurement");


    checkErr(RsCore_LockSession(instrSession));

    if (statisticsEnabled)
    {
        viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, arrStatsAttr[measurement], VI_TRUE),
                2, "Statistics Enabled");

        viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_SIGNAL_STATISTIC_NUMBER_OF_SAMPLES, numberOfSamples),
                4, "Number Of Samples");
    }
    else
    {
        for (ViInt32 i = 0; i < 2; i++)
        {
            viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, arrStatsAttr[i], VI_FALSE),
                    2, "Statistics Enabled");
        }
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures settings for the X and Y axes for statistical measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR xRefLevel/Sets the reference level for statistical measurements. The effects are
/// HIPAR xRefLevel/identical to RSFPL_ATTR_REFERENCE_LEVEL. Note that in case of
/// HIPAR xRefLevel/statistical measurements the reference level applies to the x-axis.
/// HIPAR xRange/Defines the display range of the x-axis for statistical measurements.
/// HIPAR xRange/The effects are identical to RSFPL_ATTR_Y_AXIS_RANGE.
/// HIPAR yUnit/Selects the unit of the y-axis.
/// HIPAR yMinimum/Defines the lower vertical limit of the diagram.
/// HIPAR yMaximum/Defines the upper vertical limit of the diagram.
ViStatus _VI_FUNC rsfpl_ConfigureSignalStatisticScaling(ViSession instrSession,
                                                        ViReal64  xRefLevel,
                                                        ViReal64  xRange,
                                                        ViInt32   yUnit,
                                                        ViReal64  yMinimum,
                                                        ViReal64  yMaximum)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_SIGNAL_STATISTIC_SCALING_X_REF_LEVEL, xRefLevel),
            2, "X Ref Level");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_SIGNAL_STATISTIC_SCALING_X_RANGE, xRange),
            3, "X Range");

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_SIGNAL_STATISTIC_SCALING_Y_UNIT, yUnit),
            4, "Y Unit");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_SIGNAL_STATISTIC_SCALING_Y_MINIMUM, yMinimum),
            5, "Y Minimum");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_SIGNAL_STATISTIC_SCALING_Y_MAXIMUM, yMaximum),
            6, "Y Maximum");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Applies the default or automatically configured scaling settings to
/// HIFN  the X and Y axes for statistical measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR configuration/Applies the default or automatically configured scaling settings to
/// HIPAR configuration/the X and Y axes for statistical measurement.
ViStatus _VI_FUNC rsfpl_SetSignalStatisticScalingSettings(ViSession instrSession,
                                                          ViInt32   configuration)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   arrScalingAttr[] = {
             RSFPL_ATTR_SIGNAL_STATISTIC_DEFAULT_SETTING,
             RSFPL_ATTR_SIGNAL_STATISTIC_ADJUST_SETTINGS
             };

    checkErr(RsCore_LockSession(instrSession));

    switch (configuration)
    {
    case RSFPL_VAL_STATS_SETTINGS_DEFAULT:
    case RSFPL_VAL_STATS_SETTINGS_AUTO:
        viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, arrScalingAttr[configuration], NULL),
                2, "Configuration");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, configuration), 2, "Configuration");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the results of a CCDF or ADP measurement for a specific trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trace/Selects the trace.
/// HIPAR resultType/Selects the result type.
/// HIPAR arraySize/This control defines the size of statistical measurement results
/// HIPAR arraySize/array.
/// HIPAR results/Returns the set of statistical measurements.
/// HIPAR returnedResults/Returns the number of values in the statistical measurement returned
/// HIPAR returnedResults/from the instrument.
ViStatus _VI_FUNC rsfpl_QueryStatisticalMeasurementResults(ViSession instrSession,
                                                           ViInt32   trace,
                                                           ViInt32   resultType,
                                                           ViInt32   arraySize,
                                                           ViReal64  results[],
                                                           ViInt32*  returnedResults)
{
    ViStatus error = VI_SUCCESS;
    ViInt32  traceMax = 6;
    ViChar   cmd[RS_MAX_MESSAGE_LEN] = "";
    ViString arrTypeCmd[] = { "MEAN", "PEAK", "CFAC", "ALL" };

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Trace", &traceMax));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trace, 1, traceMax),
            2, "Trace");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, resultType, RSFPL_VAL_STAT_RESULT_MEAN, RSFPL_VAL_STAT_RESULT_ALL),
            3, "Result Type");
    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, arraySize, 0), 3, "Array Size");

    snprintf(cmd, RS_MAX_MESSAGE_LEN, "CALC:STAT:RES%ld? %s", trace, arrTypeCmd[resultType]);


    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, results, returnedResults));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the results of the CCDF.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trace/Selects the trace.
/// HIPAR probability/Selects the probability.
/// HIPAR result/
ViStatus _VI_FUNC rsfpl_QueryStatisticalMeasurementCCDFLevel(ViSession instrSession,
                                                             ViInt32   trace,
                                                             ViInt32   probability,
                                                             ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViInt32  traceMax = 6;
    ViChar   cmd[RS_MAX_MESSAGE_LEN] = "";
    ViString arrProbabilityCmd[] = { "P0_01", "P0_1", "P1", "P10" };

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Trace", &traceMax));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trace, 1, traceMax),
            2, "Trace");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, probability, RSFPL_VAL_PROBABILITY_0_01, RSFPL_VAL_PROBABILITY_10),
            3, "Probability");

    snprintf(cmd, RS_MAX_MESSAGE_LEN, "CALC:STAT:CCDF:X%ld? %s", trace,
        arrProbabilityCmd[probability]);

    checkErr(RsCore_QueryViReal64(instrSession, cmd, result));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the length of the gate for all traces. The gate length applies
/// HIFN  to all traces.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR gatePeriod/Defines the length of the gate for all traces. The gate length applies
/// HIPAR gatePeriod/to all traces.
ViStatus _VI_FUNC rsfpl_ConfigureSignalStatisticGatePeriod(ViSession instrSession,
                                                           ViInt32   trace,
                                                           ViReal64  gatePeriod)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SIGNAL_STATISTIC_GATE_COMMENT,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trace");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_SIGNAL_STATISTIC_GATE_PERIOD, gatePeriod),
            2, "Gate Period");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines a comment for the gate of a particular trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trace/Selects the trace.
/// HIPAR gateComment/Defines a comment for the gate of a particular trace.
ViStatus _VI_FUNC rsfpl_ConfigureSignalStatisticGateComment(ViSession instrSession,
                                                            ViInt32   trace,
                                                            ViString  gateComment)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SIGNAL_STATISTIC_GATE_COMMENT,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trace");

    viCheckParm(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_SIGNAL_STATISTIC_GATE_COMMENT, gateComment),
            3, "Gate Comment");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures a gate range for a particular trace for statistical measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trace/Selects the trace.
/// HIPAR gate/Selects the trigger gate.
/// HIPAR gateRangeEnabled/Includes or excludes a gate range for a particular trace.
/// HIPAR gateRangeStart/Defines the start time for a gate range.
/// HIPAR gateRangeStop/Defines the stop time for a gate range.
ViStatus _VI_FUNC rsfpl_ConfigureSignalStatisticGateRange(ViSession instrSession,
                                                          ViInt32   trace,
                                                          ViInt32   gate,
                                                          ViBoolean gateRangeEnabled,
                                                          ViReal64  gateRangeStart,
                                                          ViReal64  gateRangeStop)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_trace[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_gate[RS_REPCAP_BUF_SIZE] = "";

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SIGNAL_STATISTIC_GATE_RANGE_ENABLED,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCap_trace),
            2, "Trace");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_SIGNAL_STATISTIC_GATE_RANGE_ENABLED,
            "EGateRange", gate - 1, RS_REPCAP_BUF_SIZE, repCap_gate),
            3, "Gate");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap_trace,
        repCap_gate);


    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_SIGNAL_STATISTIC_GATE_RANGE_ENABLED, gateRangeEnabled),
            4, "Gate Range Enabled");

    if (gateRangeEnabled)
    {
        viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_SIGNAL_STATISTIC_GATE_RANGE_START, gateRangeStart),
                5, "Gate Range Start");

        viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_SIGNAL_STATISTIC_GATE_RANGE_STOP, gateRangeStop),
                6, "Gate Range Stop");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns time domain power measurements on and off. This measurement in
/// HIFN  only available in zero span.
/// HIFN     When you turn the measurement on, the FPL activates a marker and
/// HIFN  positions it on the peak power level in the marker search range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR timeDomainPowerEnabled/Turns time domain power measurements on and off. This measurement in
/// HIPAR timeDomainPowerEnabled/only available in zero span.
ViStatus _VI_FUNC rsfpl_ConfigureTimeDomainPowerEnabled(ViSession instrSession,
                                                        ViBoolean timeDomainPowerEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_TIME_DOMAIN_POWER_ENABLED, timeDomainPowerEnabled),
            2, "Time Domain Power Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Switches on or off averaging for the active power measurement in zero
/// HIFN  span. If activated, a time domain value is calculated from the trace
/// HIFN  after each sweep; in the end, all values are averaged to calculate the
/// HIFN  final result.
/// HIFN     The number of results required for the calculation of average is
/// HIFN  defined with RSFPL_ATTR_TRACE_AVERAGE_COUNT/Trace.AverageCount
/// HIFN     Averaging is reset by switching it off and on again.
/// HIFN     Synchronization to the end of averaging is only possible in single
/// HIFN  sweep mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR averagingEnabled/Switches on or off averaging for the active power measurement in zero
/// HIPAR averagingEnabled/span. If activated, a time domain value is calculated from the trace
/// HIPAR averagingEnabled/after each sweep; in the end, all values are averaged to calculate the
/// HIPAR averagingEnabled/final result.
ViStatus _VI_FUNC rsfpl_ConfigureTimeDomainPowerAveragingEnabled(
                            ViSession instrSession,
                            ViBoolean averagingEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_TIME_DOMAIN_POWER_AVERAGING_ENABLED, averagingEnabled),
            2, "Averaging Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns all time domain power evaluation modes off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_TimeDomainPowerAllOff(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_TIME_DOMAIN_POWER_ALL_OFF, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Switches on or off the peak-hold function for the active power
/// HIFN  measurement in zero span. If activated, the peak for each sweep is
/// HIFN  compared to the previously stored peak; the maximum of the two is
/// HIFN  stored as the current peak.
/// HIFN     The peak-hold function is reset by switching it off and on again.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR peakHoldEnabled/Switches on or off the peak-hold function for the active power
/// HIPAR peakHoldEnabled/measurement in zero span. If activated, the peak for each sweep is
/// HIPAR peakHoldEnabled/compared to the previously stored peak; the maximum of the two is
/// HIPAR peakHoldEnabled/stored as the current peak.
ViStatus _VI_FUNC rsfpl_ConfigureTimeDomainPowerPeakHoldEnabled(
                            ViSession instrSession,
                            ViBoolean peakHoldEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_TIME_DOMAIN_POWER_PEAK_HOLD_ENABLED, peakHoldEnabled),
            2, "Peak Hold Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns the evaluation to determine the selected time domain power on
/// HIFN  and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR timeDomainPower/Selects time domain power.
/// HIPAR measurementEnabled/Turns the evaluation to determine the selected time domain power on
/// HIPAR measurementEnabled/and off.
ViStatus _VI_FUNC rsfpl_ConfigureTimeDomainPowerMeasurementEnabled(
                            ViSession instrSession,
                            ViInt32   timeDomainPower,
                            ViBoolean measurementEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr[] = {
             RSFPL_ATTR_TIME_DOMAIN_POWER_RMS_ENABLED,
             RSFPL_ATTR_TIME_DOMAIN_POWER_MEAN_ENABLED,
             RSFPL_ATTR_TIME_DOMAIN_POWER_POSITIVE_PEAK_ENABLED,
             RSFPL_ATTR_TIME_DOMAIN_POWER_STANDARD_DEVIATION_ENABLED
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, timeDomainPower, RSFPL_VAL_TDOM_POWER_RMS, RSFPL_VAL_TDOM_POWER_SDEV),
            2, "Time Domain Power");

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, attr[timeDomainPower], measurementEnabled),
            3, "Measurement Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the selected type of the time domain power.
/// HIFN     To get a valid result, you have to perform a complete measurement with
/// HIFN  synchronization to the end of the measurement before reading out the
/// HIFN  result. This is only possible for single sweep mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR timeDomainPower/Selects time domain power.
/// HIPAR statisticalType/Selects the statistical type.
/// HIPAR result/Queries the selected result of the time domain power.
ViStatus _VI_FUNC rsfpl_QueryTimeDomainPowerResult(ViSession instrSession,
                                                   ViInt32   timeDomainPower,
                                                   ViInt32   statisticalType,
                                                   ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr[3][4] = {
             {
             RSFPL_ATTR_TIME_DOMAIN_POWER_RMS_RESULT,
             RSFPL_ATTR_TIME_DOMAIN_POWER_MEAN_RESULT,
             RSFPL_ATTR_TIME_DOMAIN_POWER_POSITIVE_PEAK_RESULT,
             RSFPL_ATTR_TIME_DOMAIN_POWER_STANDARD_DEVIATION_RESULT
             },
             {
             RSFPL_ATTR_TIME_DOMAIN_POWER_AVERAGE_RMS_RESULT,
             RSFPL_ATTR_TIME_DOMAIN_POWER_AVERAGE_MEAN_RESULT,
             RSFPL_ATTR_TIME_DOMAIN_POWER_AVERAGE_POSITIVE_PEAK_RESULT,
             RSFPL_ATTR_TIME_DOMAIN_POWER_AVERAGE_STANDARD_DEVIATION_RESULT
             },
             {
             RSFPL_ATTR_TIME_DOMAIN_POWER_MAXIMUM_RMS_RESULT,
             RSFPL_ATTR_TIME_DOMAIN_POWER_MAXIMUM_MEAN_RESULT,
             RSFPL_ATTR_TIME_DOMAIN_POWER_MAXIMUM_POSITIVE_PEAK_RESULT,
             RSFPL_ATTR_TIME_DOMAIN_POWER_MAXIMUM_STANDARD_DEVIATION_RESULT
             }
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, timeDomainPower, RSFPL_VAL_TDOM_POWER_RMS, RSFPL_VAL_TDOM_POWER_SDEV),
            2, "Time Domain Power");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, statisticalType, RSFPL_VAL_STAT_CURRENT, RSFPL_VAL_STAT_MAX),
            3, "Statistical Type");

    viCheckParm(rsfpl_GetAttributeViReal64(instrSession, NULL, attr[statisticalType][timeDomainPower], result),
            4, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns the harmonic distortion measurement on and off.
/// HIFN     Note the following:
/// HIFN     If you perform the measurement in the frequency domain, the search
/// HIFN  range for the frequency of the first harmonic, whose power is
/// HIFN  determined, is defined by the last span.If you perform the measurement
/// HIFN  in the time domain, the current center frequency is used as the
/// HIFN  frequency of the first harmonic. Thus, the frequency search is
/// HIFN  bypassed. The first harmonic frequency is set by a specific center
/// HIFN  frequency in zero span before the harmonic measurement is started.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR harmonicDistortionEnabled/Turns the harmonic distortion measurement on and off.
ViStatus _VI_FUNC rsfpl_ConfigureHarmonicDistortionEnabled(ViSession instrSession,
                                                           ViBoolean harmonicDistortionEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_HARMONIC_DISTORTION_ENABLED, harmonicDistortionEnabled),
            2, "Harmonic Distortion Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the number of harmonics that the FPL looks for.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR numberOfHarmonics/Selects the number of harmonics that the FPL looks for.
ViStatus _VI_FUNC rsfpl_ConfigureHarmonicDistortionNumberOfHarmonics(
                            ViSession instrSession,
                            ViInt32   numberOfHarmonics)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_HARMONIC_DISTORTION_NUMBER_OF_HARMONICS, numberOfHarmonics),
            2, "Number Of Harmonics");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the resolution bandwidth of the harmonic in respect to the
/// HIFN  bandwidth of the first harmonic.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR rbwAutoEnabled/Selects the resolution bandwidth of the harmonic in respect to the
/// HIPAR rbwAutoEnabled/bandwidth of the first harmonic.
ViStatus _VI_FUNC rsfpl_ConfigureHarmonicDistortionRBWAutoEnabled(
                            ViSession instrSession,
                            ViBoolean rbwAutoEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_HARMONIC_DISTORTION_RBW_AUTO_ENABLED, rbwAutoEnabled),
            2, "RBW Auto Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Initiates a measurement to determine the ideal configuration for the
/// HIFN  harmonic distortion measurement.
/// HIFN
/// HIFN     The method depends on the span.
/// HIFN
/// HIFN     Frequency domain (span > 0)
/// HIFN     Frequency and level of the first harmonic are determined and used for
/// HIFN  the measurement list.
/// HIFN
/// HIFN     Time domain (span = 0)
/// HIFN     The level of the first harmonic is determined. The frequency remains
/// HIFN  unchanged.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_HarmonicDistortionInitiate(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_HARMONIC_DISTORTION_INITIATE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the total harmonic distortion of the signal.
/// HIFN     To get a valid result, you have to perform a complete measurement with
/// HIFN  synchronization to the end of the measurement before reading out the
/// HIFN  result. This is only possible for single sweep mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR thdinPct/THD in %.
/// HIPAR thdindB/THD in dB.
ViStatus _VI_FUNC rsfpl_QueryHarmonicDistortionTHDResult(ViSession instrSession,
                                                         ViReal64* thdinPct,
                                                         ViReal64* thdindB)
{
    ViStatus  error = VI_SUCCESS;
    ViReal64* buffer = NULL;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, thdinPct),
        2, "THD in Pct");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, thdindB),
        3, "THD in dB");

    checkErr(RsCore_QueryAsciiViReal64Array(instrSession, "CALC:MARK:FUNC:HARM:DIST? TOT", &buffer, NULL));

    *thdinPct = buffer[0];
    *thdindB = buffer[1];

    checkErr(rsfpl_CheckStatus(instrSession));

Error:
    if (buffer) free(buffer);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the position of the harmonics.
/// HIFN     To get a valid result, you have to perform a complete measurement with
/// HIFN  synchronization to the end of the measurement before reading out the
/// HIFN  result. This is only possible for single sweep mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR arraySize/This control defines the size of harmonic list array.
/// HIPAR harmonicsList/Returns the list of harmonics. The first value is the absolute power
/// HIPAR harmonicsList/of the first harmonic. The
/// HIPAR returnedValues/Returns the number of values in harmonic list returned from the
/// HIPAR returnedValues/instrument.
ViStatus _VI_FUNC rsfpl_QueryHarmonicDistortionHarmonicsPositions(
                            ViSession instrSession,
                            ViUInt32  arraySize,
                            ViReal64  harmonicsList[],
                            ViUInt32* returnedValues)
{
    ViStatus  error = VI_SUCCESS;
    ViReal64* buffer = NULL;
    ViInt32   count = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, arraySize, 1),
        2, "Array Size");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, harmonicsList),
        3, "Harmonics List");

    checkErr(RsCore_QueryAsciiViReal64Array(instrSession, "CALC:MARK:FUNC:HARM:LIST?", &buffer, &count));
    checkErr(rsfpl_CheckStatus(instrSession));
    checkErr(RsCore_CopyToUserBufferViReal64Array(harmonicsList, arraySize, buffer, count));

    if (returnedValues)
        *returnedValues = (ViUInt32)count;

Error:
    if (buffer) free(buffer);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Initiates a measurement to determine the third intercept point.
/// HIFN     A two-tone signal with equal carrier levels is expected at the RF
/// HIFN  input of the instrument. Marker 1 and marker 2 (both normal markers)
/// HIFN  are set to the maximum of the two signals. Delta marker 3 and delta
/// HIFN  marker 4 are positioned to the intermodulation products. The delta
/// HIFN  markers can be modified separately afterwards.
/// HIFN     The third-order intercept is calculated from the level spacing between
/// HIFN  the normal markers and the delta markers.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR thirdOrderInterceptEnabled/Initiates a measurement to determine the third intercept point.
ViStatus _VI_FUNC rsfpl_ConfigureThirdOrderInterceptEnabled(ViSession instrSession,
                                                            ViBoolean thirdOrderInterceptEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_THIRD_ORDER_INTERCEPT_ENABLED, thirdOrderInterceptEnabled),
            2, "Third Order Intercept Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Initiates a search for signals in the current trace to determine the
/// HIFN  third intercept point.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_ThirdOrderInterceptSearchSignal(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_THIRD_ORDER_INTERCEPT_SEARCH_SIGNAL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the results for the third order intercept point measurement.
/// HIFN     To get a valid result, you have to perform a complete measurement with
/// HIFN  synchronization to the end of the measurement before reading out the
/// HIFN  result. This is only possible for single sweep mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR thirdOrderInterceptResult/Queries the results for the third order intercept point measurement.
ViStatus _VI_FUNC rsfpl_QueryThirdOrderInterceptResult(ViSession instrSession,
                                                       ViReal64* thirdOrderInterceptResult)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_GetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_THIRD_ORDER_INTERCEPT_RESULT, thirdOrderInterceptResult),
            2, "Third Order Intercept Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the results for the maximum third order intercept point
/// HIFN  measurement.
/// HIFN     To get a valid result, you have to perform a complete measurement with
/// HIFN  synchronization to the end of the measurement before reading out the
/// HIFN  result. This is only possible for single sweep mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR maximum/Queries the results for the maximum third order intercept point
/// HIPAR maximum/measurement.
ViStatus _VI_FUNC rsfpl_QueryThirdOrderInterceptResultMaximum(
                            ViSession instrSession,
                            ViReal64* maximum)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViReal64(instrSession, NULL,
        RSFPL_ATTR_THIRD_ORDER_INTERCEPT_RESULT_MAXIMUM,
        RS_VAL_DIRECT_USER_CALL, maximum),
        2, "Maximum");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the results for the minimum third order intercept point
/// HIFN  measurement.
/// HIFN     To get a valid result, you have to perform a complete measurement with
/// HIFN  synchronization to the end of the measurement before reading out the
/// HIFN  result. This is only possible for single sweep mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR minimum/Queries the results for the minimum third order intercept point
/// HIPAR minimum/measurement.
ViStatus _VI_FUNC rsfpl_QueryThirdOrderInterceptResultMinimum(
                            ViSession instrSession,
                            ViReal64* minimum)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViReal64(instrSession, NULL,
        RSFPL_ATTR_THIRD_ORDER_INTERCEPT_RESULT_MINIMUM,
        RS_VAL_DIRECT_USER_CALL, minimum),
        2, "Minimum");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns the AM Modulation Depth measurement on and off.
/// HIFN     To work correctly, the measurement requires an AM modulated signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR amModulationDepthEnabled/Turns the AM Modulation Depth measurement on and off.
ViStatus _VI_FUNC rsfpl_ConfigureAMModulationDepthEnabled(ViSession instrSession,
                                                          ViBoolean amModulationDepthEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_AM_MODULATION_DEPTH_ENABLED, amModulationDepthEnabled),
            2, "AM Modulation Depth Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Initiates a search for the signals required for the AM depth
/// HIFN  measurement.
/// HIFN     Note that the method/attribute does not perform a new measurement, but
/// HIFN  looks for the signals on the current trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_AMModulationDepthSearchSignal(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_AM_MODULATION_DEPTH_SEARCH_SIGNAL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the results of the AM modulation depth measurement.
/// HIFN     To get a valid result, you have to perform a complete measurement with
/// HIFN  synchronization to the end of the measurement before reading out the
/// HIFN  result. This is only possible for single sweep mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR amModulationDepthResult/Queries the results of the AM modulation depth measurement.
ViStatus _VI_FUNC rsfpl_QueryAMModulationDepthResult(ViSession instrSession,
                                                     ViReal64* amModulationDepthResult)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_GetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_AM_MODULATION_DEPTH_RESULT, amModulationDepthResult),
            2, "AM Modulation Depth Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines the constant settings for the list during
/// HIFN  multiple power measurement. Parameters Peak Meas, RMS Meas and AVG Meas
/// HIFN  define, which measurements are to be performed at the same time at the
/// HIFN  frequency point. Correspondingly, one, two or three results per
/// HIFN  frequency point are returned for the function Get List Power Sequence
/// HIFN  (actListPwrSeq). If all three parameters are set to OFF, the command
/// HIFN  generates an execution error.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR peakPower/Turns peak power evaluation on and off.
/// HIPAR rmsPower/Turns RMS power evaluation on and off.
/// HIPAR avgPower/Turns average power evaluation on and off.
/// HIPAR triggerSource/Selects a trigger source.
/// HIPAR triggerSlope/Selects the trigger slope.
/// HIPAR triggerOffset/Defines the trigger delay.
/// HIPAR gateLength/Defines the gate length for gated measurements. Setting 0 seconds
/// HIPAR gateLength/turns gated measurements off.
ViStatus _VI_FUNC rsfpl_DefineListEvaluationParameters(ViSession instrSession,
                                                       ViBoolean peakPower,
                                                       ViBoolean rmsPower,
                                                       ViBoolean avgPower,
                                                       ViString  triggerSource,
                                                       ViInt32   triggerSlope,
                                                       ViReal64  triggerOffset,
                                                       ViReal64  gateLength)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  p2triggSlope;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsFpl_rngPosNeg, triggerSlope, &p2triggSlope),
        7, "Trigger Slope");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, gateLength, 0.0, 30.0), 8, "Gate Length");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:LIST:POW:SET %s,%s,%s,%s,%s,%.12lG,%.12lG", peakPower ? "ON" : "OFF", rmsPower ? "ON" : "OFF", avgPower ? "ON" : "OFF",
        triggerSource, p2triggSlope,
        triggerOffset, gateLength);
    checkErr(rsfpl_WriteCommandWithOPCSync(instrSession, cmd, 0));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures and initiates the List Evaluation measurement.
/// HIFN     The list can contain up to 200 entries (frequencies). You can define a
/// HIFN  different instrument setup for each frequency that is in the list.
/// HIFN     This method returns immedatiately without waiting for measurement to
/// HIFN  be finished.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR noofListItems/Set number of list items. This value must be equal to array size of
/// HIPAR noofListItems/each list parameter.
/// HIPAR frequency/Defines the frequency. Each frequency corresponds to one list entry.
/// HIPAR referenceLevel/Defines the reference level for a list entry.
/// HIPAR rfAttenuation/Defines the RF attenuation for a list entry.
/// HIPAR filterType/Selects the filter type for a list entry.
/// HIPAR rbw/Defines the resolution bandwidth for a list entry.
/// HIPAR vbw/Defines the video bandwidth for a list entry.
/// HIPAR measTime/Defines the measurement time for a list entry.
/// HIPAR triggerLevel/Array of trigger level values for each list item.
ViStatus _VI_FUNC rsfpl_ConfigureAndInitiateListEvaluationMeasurement(
                            ViSession instrSession,
                            ViInt32   noofListItems,
                            ViReal64  frequency[],
                            ViReal64  referenceLevel[],
                            ViReal64  rfAttenuation[],
                            ViInt32   filterType[],
                            ViReal64  rbw[],
                            ViReal64  vbw[],
                            ViReal64  measTime[],
                            ViReal64  triggerLevel[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2buf = cmd;
    ViChar*  p2rng;
    ViInt32  i = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, noofListItems, 1, 200),
            2, "No Of List Items");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, frequency), 3, "Frequency");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, referenceLevel), 4, "Reference Level");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, rfAttenuation), 5, "RF Attenuation");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, filterType), 6, "Filter Type");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, rbw), 7, "RBW");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, vbw), 8, "VBW");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, measTime), 9, "Meas Time");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, triggerLevel), 10, "Trigger Level");

    p2buf += snprintf(p2buf, RS_MAX_MESSAGE_BUF_SIZE, ":SENS:LIST:POW:SEQ ");

    for (i = 0; i < noofListItems; i++)
    {
        checkErr(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsFpl_rngFilterType, filterType[i], &p2rng));

        p2buf += snprintf(p2buf, RS_MAX_MESSAGE_BUF_SIZE, "%.12lG,%.12lG,%.12lG,OFF,%s,%.12lG,%.12lG,%.12lG,%.12lG,",
            frequency[i],
            referenceLevel[i],
            rfAttenuation[i],
            p2rng,
            rbw[i],
            vbw[i],
            measTime[i],
            triggerLevel[i]);
    }

    *--p2buf = '\0';
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsfpl_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures and initiates the List Evaluation measurement with
/// HIFN  electronic attenuation.
/// HIFN     The list can contain up to 200 entries (frequencies). You can define a
/// HIFN  different instrument setup for each frequency that is in the list.
/// HIFN     This method returns immedatiately without waiting for measurement to
/// HIFN  be finished.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR noofListItems/Set number of list items. This value must be equal to array size of
/// HIPAR noofListItems/each list parameter.
/// HIPAR frequency/Defines the frequency. Each frequency corresponds to one list entry.
/// HIPAR referenceLevel/Defines the reference level for a list entry.
/// HIPAR rfAttenuation/Defines the RF attenuation for a list entry.
/// HIPAR electronicAttenuation/Defines the electronic attenuation for a list entry. A setting other
/// HIPAR electronicAttenuation/than 0 (OFF) requires option R&S FPL1-B25.
/// HIPAR filterType/Selects the resolution filter type.
/// HIPAR rbw/Defines the resolution bandwidth for a list entry.
/// HIPAR vbw/Defines the video bandwidth for a list entry.
/// HIPAR measTime/Defines the measurement time for a list entry.
/// HIPAR triggerLevel/Array of trigger level values for each list item.
ViStatus _VI_FUNC rsfpl_ConfigureAndInitiateListEvaluationMeasurementElectronic(
                            ViSession instrSession,
                            ViInt32   noofListItems,
                            ViReal64  frequency[],
                            ViReal64  referenceLevel[],
                            ViReal64  rfAttenuation[],
                            ViReal64  electronicAttenuation[],
                            ViInt32   filterType[],
                            ViReal64  rbw[],
                            ViReal64  vbw[],
                            ViReal64  measTime[],
                            ViReal64  triggerLevel[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2buf = cmd;
    ViChar*  p2rng;
    ViInt32  i = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, noofListItems, 1, 200),
            2, "No Of List Items");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, frequency), 3, "Frequency");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, referenceLevel), 4, "Reference Level");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, rfAttenuation), 5, "RF Attenuation");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, electronicAttenuation), 6, "Electronic Attenuation");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, filterType), 7, "Filter Type");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, rbw), 8, "RBW");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, vbw), 9, "VBW");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, measTime), 10, "Meas Time");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, triggerLevel), 11, "Trigger Level");

    p2buf += snprintf(p2buf, RS_MAX_MESSAGE_BUF_SIZE, ":SENS:LIST:POW:SEQ ");

    for (i = 0; i < noofListItems; i++)
    {
        checkErr(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsFpl_rngFilterType, filterType[i], &p2rng));

        p2buf += snprintf(p2buf, RS_MAX_MESSAGE_BUF_SIZE, "%.12lG,%.12lG,%.12lG,%.12lG,%s,%.12lG,%.12lG,%.12lG,%.12lG,",
            frequency[i],
            referenceLevel[i],
            rfAttenuation[i],
            electronicAttenuation[i],
            p2rng,
            rbw[i],
            vbw[i],
            measTime[i],
            triggerLevel[i]);
    }

    *--p2buf = '\0';
    checkErr(rsfpl_WriteCommand(instrSession, cmd));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures and initiates the List Evaluation measurement.
/// HIFN     The list can contain up to 200 entries (frequencies). You can define a
/// HIFN  different instrument setup for each frequency that is in the list.
/// HIFN     This method returns immedatiately without waiting for measurement to
/// HIFN  be finished.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR noofListItems/Set number of list items. This value must be equal to array size of
/// HIPAR noofListItems/each list parameter.
/// HIPAR frequency/Defines the frequency. Each frequency corresponds to one list entry.
/// HIPAR referenceLevel/Defines the reference level for a list entry.
/// HIPAR rfAttenuation/Defines the RF attenuation for a list entry.
/// HIPAR filterType/Selects the filter type for a list entry.
/// HIPAR rbw/Defines the resolution bandwidth for a list entry.
/// HIPAR vbw/Defines the video bandwidth for a list entry.
/// HIPAR measTime/Defines the measurement time for a list entry.
/// HIPAR triggerLevel/Array of trigger level values for each list item.
/// HIPAR timeout/This control sets the timeout value in ms.
ViStatus _VI_FUNC rsfpl_ConfigureAndInitiateListEvaluationMeasurementSynchronized(
                            ViSession instrSession,
                            ViInt32   noofListItems,
                            ViReal64  frequency[],
                            ViReal64  referenceLevel[],
                            ViReal64  rfAttenuation[],
                            ViInt32   filterType[],
                            ViReal64  rbw[],
                            ViReal64  vbw[],
                            ViReal64  measTime[],
                            ViReal64  triggerLevel[],
                            ViUInt32  timeout)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2buf = cmd;
    ViChar*  p2filter;
    ViInt32  i = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, noofListItems, 1, 200),
            2, "No Of List Items");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, frequency), 3, "Frequency");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, referenceLevel), 4, "Reference Level");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, rfAttenuation), 5, "RF Attenuation");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, filterType), 6, "Filter Type");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, rbw), 7, "RBW");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, vbw), 8, "VBW");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, measTime), 9, "Meas Time");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, triggerLevel), 10, "Trigger Level");

    p2buf += snprintf(p2buf, RS_MAX_MESSAGE_BUF_SIZE, ":SENS:LIST:POW:SEQ ");

    for (i = 0; i < noofListItems; i++)
    {
        viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsFpl_rngFilterType, filterType[i], &p2filter),
            7, "Filter Type");

        p2buf += sprintf(p2buf, "%.12lG,%.12lG,%.12lG,OFF,%s,%.12lG,%.12lG,%.12lG,%.12lG,",
            frequency[i],
            referenceLevel[i],
            rfAttenuation[i],
            p2filter,
            rbw[i],
            vbw[i],
            measTime[i],
            triggerLevel[i]);
    }

    *--p2buf = '\0';
    checkErr(rsfpl_WriteCommandWithOPCSync(instrSession, cmd, timeout));
    checkErr(rsfpl_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures and initiates the List Evaluation measurement with
/// HIFN  electronic attenuation.
/// HIFN     The list can contain up to 200 entries (frequencies). You can define a
/// HIFN  different instrument setup for each frequency that is in the list.
/// HIFN     This method waits for the measurement to finish.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR noofListItems/Set number of list items. This value must be equal to array size of
/// HIPAR noofListItems/each list parameter.
/// HIPAR frequency/Defines the frequency. Each frequency corresponds to one list entry.
/// HIPAR referenceLevel/Defines the reference level for a list entry.
/// HIPAR rfAttenuation/Defines the RF attenuation for a list entry.
/// HIPAR electronicAttenuation/Defines the electronic attenuation for a list entry. A setting other
/// HIPAR electronicAttenuation/than 0 (OFF) requires option R&S FPL1-B25.
/// HIPAR filterType/Selects the filter type for a list entry.
/// HIPAR rbw/Defines the resolution bandwidth for a list entry.
/// HIPAR vbw/Defines the video bandwidth for a list entry.
/// HIPAR measTime/Defines the measurement time for a list entry.
/// HIPAR triggerLevel/Array of trigger level values for each list item.
/// HIPAR timeout/This control sets the timeout value in ms.
ViStatus _VI_FUNC rsfpl_ConfigureAndInitiateListEvaluationMeasurementElectronicSynchronized(
                            ViSession instrSession,
                            ViInt32   noofListItems,
                            ViReal64  frequency[],
                            ViReal64  referenceLevel[],
                            ViReal64  rfAttenuation[],
                            ViReal64  electronicAttenuation[],
                            ViInt32   filterType[],
                            ViReal64  rbw[],
                            ViReal64  vbw[],
                            ViReal64  measTime[],
                            ViReal64  triggerLevel[],
                            ViUInt32  timeout)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2buf = cmd;
    ViChar*  p2rng;
    ViInt32  i = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, noofListItems, 1, 200),
            2, "No Of List Items");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, frequency), 3, "Frequency");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, referenceLevel), 4, "Reference Level");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, rfAttenuation), 5, "RF Attenuation");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, electronicAttenuation), 6, "Electronic Attenuation");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, filterType), 7, "Filter Type");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, rbw), 8, "RBW");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, vbw), 9, "VBW");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, measTime), 10, "Meas Time");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, triggerLevel), 11, "Trigger Level");

    p2buf += snprintf(p2buf, RS_MAX_MESSAGE_BUF_SIZE, ":SENS:LIST:POW:SEQ ");

    for (i = 0; i < noofListItems; i++)
    {
        checkErr(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsFpl_rngFilterType, filterType[i], &p2rng));

        p2buf += snprintf(p2buf, RS_MAX_MESSAGE_BUF_SIZE, "%.12lG,%.12lG,%.12lG,%.12lG,%s,%.12lG,%.12lG,%.12lG,%.12lG,",
            frequency[i],
            referenceLevel[i],
            rfAttenuation[i],
            electronicAttenuation[i],
            p2rng,
            rbw[i],
            vbw[i],
            measTime[i],
            triggerLevel[i]);
    }

    *--p2buf = '\0';
    checkErr(rsfpl_WriteCommandWithOPCSync(instrSession, cmd, timeout));
    checkErr(rsfpl_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the results of the list evaluation.
/// HIFN     May be used to obtain measurement results in an asynchronous way,
/// HIFN  using the service request mechanism for synchronization to the end of
/// HIFN  the measurement.
/// HIFN     If there are no results, the instrument returns an error.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR noofResults/Expected number of values to be returned from the instrument. This
/// HIPAR noofResults/value must be equal to the List Power results array number of items.
/// HIPAR listPowerResults/This control returns array of elements as the result of a previous
/// HIPAR listPowerResults/list measurement as configured and initiated.
/// HIPAR returnedValues/Returns the number of results.
ViStatus _VI_FUNC rsfpl_QueryListEvaluationResults(ViSession instrSession,
                                                   ViUInt32  noofResults,
                                                   ViReal64  listPowerResults[],
                                                   ViUInt32* returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, listPowerResults), 3, "List Power Results");

    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, "SENS:LIST:POW:RES?", noofResults, listPowerResults, (ViInt32*)returnedValues));
    checkErr(rsfpl_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures and initiates the List Evaluation measurement.
/// HIFN     The list can contain up to 200 entries (frequencies). You can define a
/// HIFN  different instrument setup for each frequency that is in the list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR noofListItems/Set number of list items. This value must be equal to array size of
/// HIPAR noofListItems/each list parameter.
/// HIPAR frequency/Defines the frequency. Each frequency corresponds to one list entry.
/// HIPAR referenceLevel/Defines the reference level for a list entry.
/// HIPAR rfAttenuation/Defines the RF attenuation for a list entry.
/// HIPAR filterType/Selects the filter type for a list entry.
/// HIPAR rbw/Defines the resolution bandwidth for a list entry.
/// HIPAR vbw/Defines the video bandwidth for a list entry.
/// HIPAR measTime/Defines the measurement time for a list entry.
/// HIPAR triggerLevel/Array of trigger level values for each list item.
/// HIPAR arraySize/Defines the memory size allocated for Results.
/// HIPAR results/Returns the measured values:
/// HIPAR returnedValues/Returns number of returned values.
ViStatus _VI_FUNC rsfpl_ReadListEvaluationMeasurement(ViSession instrSession,
                                                      ViUInt32  noofListItems,
                                                      ViReal64  frequency[],
                                                      ViReal64  referenceLevel[],
                                                      ViReal64  rfAttenuation[],
                                                      ViInt32   filterType[],
                                                      ViReal64  rbw[],
                                                      ViReal64  vbw[],
                                                      ViReal64  measTime[],
                                                      ViReal64  triggerLevel[],
                                                      ViUInt32  arraySize,
                                                      ViReal64  results[],
                                                      ViUInt32* returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2buf = cmd;
    ViChar*  p2filter;
    ViUInt32 i = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, noofListItems, 1, 200),
            2, "No Of List Items");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, frequency), 3, "Frequency");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, referenceLevel), 4, "Reference Level");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, rfAttenuation), 5, "RF Attenuation");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, filterType), 6, "Filter Type");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, rbw), 7, "RBW");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, vbw), 8, "VBW");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, measTime), 9, "Meas Time");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, triggerLevel), 10, "Trigger Level");
    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, arraySize, 0), 11, "Array Size");

    if (arraySize > 0)
    {
        viCheckParm(RsCore_InvalidNullPointer(instrSession, results), 12, "Results");
    }

    p2buf += snprintf(p2buf, RS_MAX_MESSAGE_BUF_SIZE, ":SENS:LIST:POW:SEQ? ");

    for (i = 0; i < noofListItems; i++)
    {
        viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsFpl_rngFilterType, filterType[i], &p2filter),
            7, "Filter Type");

        p2buf += snprintf(p2buf, RS_MAX_MESSAGE_BUF_SIZE, "%.12lG,%.12lG,%.12lG,OFF,%s,%.12lG,%.12lG,%.12lG,%.12lG,",
            frequency[i],
            referenceLevel[i],
            rfAttenuation[i],
            p2filter,
            rbw[i],
            vbw[i],
            measTime[i],
            triggerLevel[i]);
    }

    *--p2buf = '\0';
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, results, (ViInt32*)returnedValues));
    checkErr(rsfpl_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns the List Evaluation off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_DisableListEvaluation(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_DISABLE_LIST_EVALUATION, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the filter type for pulse power measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR powerMeasurementFilterType/Selects the filter type for pulse power measurements.
ViStatus _VI_FUNC rsfpl_ConfigurePulsePowerMeasurementFilterType(
                            ViSession instrSession,
                            ViInt32   powerMeasurementFilterType)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_PULSE_POWER_MEASUREMENT_FILTER_TYPE, powerMeasurementFilterType),
            2, "Power Measurement Filter Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the lowest pulse power that has been measured during a pulse
/// HIFN  power measurement.
/// HIFN     If there are no results, the instrument returns an error.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR powerMeasurementMinimum/Queries the lowest pulse power that has been measured during a pulse
/// HIPAR powerMeasurementMinimum/power measurement.
ViStatus _VI_FUNC rsfpl_QueryPulsePowerMeasurementMinimum(ViSession instrSession,
                                                          ViReal64* powerMeasurementMinimum)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_GetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_PULSE_POWER_MEASUREMENT_MINIMUM, powerMeasurementMinimum),
            2, "Power Measurement Minimum");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the results of the pulse power measurement.
/// HIFN     If there are no results, the instrument returns an error.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR noofResults/Expected number of values to be returned from the instrument.
/// HIPAR listPowerResults/This control returns array of elements as the result of a previous
/// HIPAR listPowerResults/list measurement as configured and initiated.
/// HIPAR returnedValues/Returns the number of results.
ViStatus _VI_FUNC rsfpl_QueryPulsePowerMeasurementListOfPulsePowers(
                            ViSession instrSession,
                            ViUInt32  noofResults,
                            ViReal64  listPowerResults[],
                            ViUInt32* returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, noofResults, 1),
        2, "No Of Results");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, listPowerResults),
        3, "List Power Results");

    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, "SENS:MPOW:RES:LIST?", noofResults, listPowerResults, (ViInt32*)returnedValues));
    checkErr(rsfpl_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures power measurements on pulses in the time domain.
/// HIFN     To evaluate the pulse power, the instrument uses the data captured
/// HIFN  during a previous measurement. The data recorded during the set
/// HIFN  measurement time is combined to a measured value for each pulse
/// HIFN  according to the detector specified and the indicated number of results
/// HIFN  is output as a list.
/// HIFN
/// HIFN     To get a valid result, you have to perform a complete measurement with
/// HIFN  synchronization to the end of the measurement before reading out the
/// HIFN  result. This is only possible for single sweep mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR timeOffset/Defines a time offset to start the measurement at the first pulse of a
/// HIPAR timeOffset/trace.
/// HIPAR measTime/Defines the measurement time.
/// HIPAR pulsePeriod/Defines the pulse period
/// HIPAR numberOfPulses/Defines the number of pulses to measure.
ViStatus _VI_FUNC rsfpl_ConfigurePulsePowerMeasurement(ViSession instrSession,
                                                       ViReal64  timeOffset,
                                                       ViReal64  measTime,
                                                       ViReal64  pulsePeriod,
                                                       ViInt32   numberOfPulses)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:MARK:FUNC:MSUM %.12lG,%.12lG,%.12lG,%ld", timeOffset, measTime, pulsePeriod, numberOfPulses);
    checkErr(rsfpl_WriteCommandWithOPCSync(instrSession, cmd, 0));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures and initiates the pulse power measurement.
/// HIFN     The FPL caches all measurement parameters that you can set with this
/// HIFN  method. If you use the method repeatedly, the FPL only changes those
/// HIFN  settings that you have actually changed before initiating the
/// HIFN  measurement. Thus, measurement times are kept as low as possible.
/// HIFN     This method returns immediately without waiting for all frequencies
/// HIFN  have been measured and the number of individual measurements has been
/// HIFN  performed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frequency/Defines the pulse frequency.
/// HIPAR rbw/Defines the resolution bandwidth.
/// HIPAR measTime/Defines the measurement time.
/// HIPAR triggerSource/Selects the trigger source.
/// HIPAR triggerLevel/Defines a trigger level.
/// HIPAR triggerOffset/Defines the trigger delay.
/// HIPAR detector/Selects the detector and therefore the way the measurement is
/// HIPAR detector/evaluated.
/// HIPAR numberOfPulses/Defines the number of pulses included in the measurement.
ViStatus _VI_FUNC rsfpl_ConfigureAndInitiatePulsePowerMeasurement(
                            ViSession instrSession,
                            ViReal64  frequency,
                            ViReal64  rbw,
                            ViReal64  measTime,
                            ViInt32   triggerSource,
                            ViReal64  triggerLevel,
                            ViReal64  triggerOffset,
                            ViInt32   detector,
                            ViInt32   numberOfPulses)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2stringTr;
    ViChar*  p2stringDet;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViReal64Range(instrSession, measTime, 1.0E-6, 30.0), 4, "Meas Time");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, triggerLevel, 0.0, 100.0), 6, "Trigger Level");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, triggerOffset, 0.0, 30.0), 7, "Trigger Offset");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, detector, RSFPL_VAL_PULSE_POWER_DETECTOR_MEAN, RSFPL_VAL_PULSE_POWER_DETECTOR_PEAK),
            8, "Detector");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, numberOfPulses, 1, 32001),
            9, "Number Of Pulses");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsFpl_rngTriggerSource, triggerSource, &p2stringTr),
        7, "Filter Type");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsFpl_rngPulsePowerDetector, detector, &p2stringDet),
        7, "Filter Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":SENS:MPOW:SEQ %.12lG,%.12lG,%.12lG,%s,%.12lG,%.12lG,%s,%ld",
        frequency,
        rbw,
        measTime,
        p2stringTr,
        triggerLevel,
        triggerOffset,
        p2stringDet,
        numberOfPulses);
    checkErr(rsfpl_WriteCommand(instrSession, cmd));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures and initiates the pulse power measurement.
/// HIFN     The FPL caches all measurement parameters that you can set with this
/// HIFN  method. If you use the method repeatedly, the FPL only changes those
/// HIFN  settings that you have actually changed before initiating the
/// HIFN  measurement. Thus, measurement times are kept as low as possible.
/// HIFN     This method returns immediately without waiting for all frequencies
/// HIFN  have been measured and the number of individual measurements has been
/// HIFN  performed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frequency/Defines the pulse frequency.
/// HIPAR rbw/Defines the resolution bandwidth.
/// HIPAR measTime/Defines the measurement time.
/// HIPAR triggerSource/Selects the trigger source.
/// HIPAR triggerLevel/Defines a trigger level.
/// HIPAR triggerOffset/Defines the trigger delay.
/// HIPAR detector/Selects the detector and therefore the way the measurement is
/// HIPAR detector/evaluated.
/// HIPAR numberOfPulses/Defines the number of pulses included in the measurement.
ViStatus _VI_FUNC rsfpl_ConfigureAndInitiatePulsePowerMeasurementSynchronized(
                            ViSession instrSession,
                            ViReal64  frequency,
                            ViReal64  rbw,
                            ViReal64  measTime,
                            ViInt32   triggerSource,
                            ViReal64  triggerLevel,
                            ViReal64  triggerOffset,
                            ViInt32   detector,
                            ViInt32   numberOfPulses)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2stringTr;
    ViChar*  p2stringDet;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViReal64Range(instrSession, measTime, 1.0E-6, 30.0), 4, "Meas Time");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, triggerLevel, 0.0, 100.0), 6, "Trigger Level");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, triggerOffset, 0.0, 30.0), 7, "Trigger Offset");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, detector, RSFPL_VAL_PULSE_POWER_DETECTOR_MEAN, RSFPL_VAL_PULSE_POWER_DETECTOR_PEAK),
            8, "Detector");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, numberOfPulses, 1, 32001),
            9, "Number Of Pulses");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsFpl_rngTriggerSource, triggerSource, &p2stringTr),
        7, "Filter Type");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsFpl_rngPulsePowerDetector, detector, &p2stringDet),
        7, "Filter Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":SENS:MPOW:SEQ %.12lG,%.12lG,%.12lG,%s,%.12lG,%.12lG,%s,%ld", frequency,
        rbw,
        measTime,
        p2stringTr,
        triggerLevel,
        triggerOffset,
        p2stringDet,
        numberOfPulses);
    checkErr(rsfpl_WriteCommandWithOPCSync(instrSession, cmd, 0));

    checkErr(rsfpl_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures and initiates the pulse power measurement.
/// HIFN     The FPL caches all measurement parameters that you can set with this
/// HIFN  method. If you use the method repeatedly, the FPL only changes those
/// HIFN  settings that you have actually changed before initiating the
/// HIFN  measurement. Thus, measurement times are kept as low as possible.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frequency/Defines the pulse frequency.
/// HIPAR rbw/Defines the resolution bandwidth.
/// HIPAR measTime/Defines the measurement time.
/// HIPAR triggerSource/Selects the trigger source.
/// HIPAR triggerLevel/Defines a trigger level.
/// HIPAR triggerOffset/Defines the trigger delay.
/// HIPAR detector/Selects the detector and therefore the way the measurement is
/// HIPAR detector/evaluated.
/// HIPAR numberOfPulses/Defines the number of pulses included in the measurement.
/// HIPAR noofResults/Expected number of values to be returned from the instrument.
/// HIPAR listPowerResults/This control returns array of elements as the result of a previous
/// HIPAR listPowerResults/list measurement as configured and initiated.
/// HIPAR returnedValues/Returns the number of results.
ViStatus _VI_FUNC rsfpl_ConfigureAndReadPulsePowerMeasurement(
                            ViSession instrSession,
                            ViReal64  frequency,
                            ViReal64  rbw,
                            ViReal64  measTime,
                            ViInt32   triggerSource,
                            ViReal64  triggerLevel,
                            ViReal64  triggerOffset,
                            ViInt32   detector,
                            ViInt32   numberOfPulses,
                            ViUInt32  noofResults,
                            ViReal64  listPowerResults[],
                            ViUInt32* returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2stringTr;
    ViChar*  p2stringDet;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViReal64Range(instrSession, measTime, 1.0E-6, 30.0), 4, "Meas Time");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, triggerLevel, 0.0, 100.0), 6, "Trigger Level");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, triggerOffset, 0.0, 30.0), 7, "Trigger Offset");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, detector, RSFPL_VAL_PULSE_POWER_DETECTOR_MEAN, RSFPL_VAL_PULSE_POWER_DETECTOR_PEAK),
            8, "Detector");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, numberOfPulses, 1, 32001),
            9, "Number Of Pulses");
    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, noofResults, 1),
        10, "No Of Results");
    if (noofResults > 0)
    {
        viCheckParm(RsCore_InvalidNullPointer(instrSession, listPowerResults),
            11, "List Power Results");
    }

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsFpl_rngTriggerSource, triggerSource, &p2stringTr),
        5, "Trigger Source");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsFpl_rngPulsePowerDetector, detector, &p2stringDet),
        8, "Detector");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":SENS:MPOW:SEQ? %.12lG,%.12lG,%.12lG,%s,%.12lG,%.12lG,%s,%ld",
        frequency,
        rbw,
        measTime,
        p2stringTr,
        triggerLevel,
        triggerOffset,
        p2stringDet,
        numberOfPulses);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, noofResults, listPowerResults, (ViInt32*)returnedValues));
    checkErr(rsfpl_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures gated measurements with the I/Q analyzer on and off.
/// HIFN     Before you can use the function you have to turn on the I/Q analyzer
/// HIFN  and select an external or IF power trigger source.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR gatingEnabled/Turns gated measurements with the I/Q analyzer on and off.
/// HIPAR mode/Selects the gate mode for gated measurements with the I/Q analyzer.
/// HIPAR mode/This control has effect only if Gating Enabled == On.
/// HIPAR gap/Defines the interval between several gate periods for gated
/// HIPAR gap/measurements with the I/Q analyzer. This control has effect only if
/// HIPAR gap/Gating Enabled == On.
/// HIPAR length/Defines the gate length for gated measurements with the I/Q analyzer.
/// HIPAR numberOfGatePeriods/This command defines the number of gate periods after the trigger
/// HIPAR numberOfGatePeriods/signal for gated measurements with the I/Q analyzer.
ViStatus _VI_FUNC rsfpl_ConfigureIQAnalyzerGating(ViSession instrSession,
                                                  ViBoolean gatingEnabled,
                                                  ViInt32   mode,
                                                  ViInt32   gap,
                                                  ViInt32   length,
                                                  ViInt32   numberOfGatePeriods)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_IQ_ANALYZER_GATING_ENABLED, gatingEnabled),
            2, "Gating Enabled");
    if (gatingEnabled)
    {
        viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_IQ_ANALYZER_GATING_MODE, mode),
                3, "Mode");

        viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_IQ_ANALYZER_GATING_GAP, gap),
                4, "Gap");

        viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_IQ_ANALYZER_GATING_LENGTH, length),
                5, "Length");

        viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_IQ_ANALYZER_GATING_NUMBER_OF_GATE_PERIODS, numberOfGatePeriods),
                6, "Number Of Gate Periods");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Executing this attribute also has the following effects:
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR iqAnalyzerEnabled/Executing this control also has the following effects:
ViStatus _VI_FUNC rsfpl_ConfigureIQAnalyzerEnabled(ViSession instrSession,
                                                   ViBoolean iqAnalyzerEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_IQ_ANALYZER_ENABLED, iqAnalyzerEnabled),
            2, "IQ Analyzer Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns I/Q data analysis on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR evaluationEnabled/Turns I/Q data analysis on and off.
ViStatus _VI_FUNC rsfpl_ConfigureIQAnalyzerEvaluationEnabled(ViSession instrSession,
                                                             ViBoolean evaluationEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_IQ_ANALYZER_EVALUATION_ENABLED, evaluationEnabled),
            2, "Evaluation Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets up the instrument for I/Q measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR sampleRate/Sample rate for the data acquisition.
/// HIPAR triggerMode/Selection of the trigger source used for the measurement.
/// HIPAR triggerSlope/Used trigger slope.
/// HIPAR pretriggerSamples/Defines the trigger offset in terms of pretrigger samples. Negative
/// HIPAR pretriggerSamples/values correspond to a trigger delay.
/// HIPAR recordLength/Number of measurement values to record (including the pretrigger
/// HIPAR recordLength/samples).
ViStatus _VI_FUNC rsfpl_ConfigureIQAnalyzerSettings(ViSession instrSession,
                                                    ViReal64  sampleRate,
                                                    ViInt32   triggerMode,
                                                    ViInt32   triggerSlope,
                                                    ViInt32   pretriggerSamples,
                                                    ViInt32   recordLength)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2src;
    ViChar*  p2slope;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViReal64Range(instrSession, sampleRate, 100.0, 10.0e+9), 2, "Sampling Rate");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, pretriggerSamples, -1399999999, 1399999999), 5, "Pretrigger Samples");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsFpl_rngTriggerSource, triggerMode, &p2src),
        3, "Trigger Mode");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsFpl_rngPosNeg, triggerSlope, &p2slope),
        4, "Trigger Slope");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRAC:IQ:SET NORM,0,%.12lG,%s,%s,%ld,%ld", sampleRate,
        p2src,
        p2slope,
        pretriggerSamples,
        recordLength);
    checkErr(rsfpl_WriteCommandWithOPCSync(instrSession, cmd, 0));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the final user sample rate for the acquired I/Q data. Thus, the
/// HIFN  user sample rate can be modified without affecting the actual data
/// HIFN  capturing settings on the FPL.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR sampleRate/Sets the final user sample rate for the acquired I/Q data. Thus, the
/// HIPAR sampleRate/user sample rate can be modified without affecting the actual data
/// HIPAR sampleRate/capturing settings on the FPL.
ViStatus _VI_FUNC rsfpl_ConfigureIQAnalyzerSampleRate(ViSession instrSession,
                                                      ViReal64  sampleRate)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_IQ_ANALYZER_SAMPLE_RATE, sampleRate),
            2, "Sample Rate");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This command defines or queries the bandwidth of the resampling
/// HIFN  filter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR analysisBandwidth/This command defines or queries the bandwidth of the resampling
/// HIPAR analysisBandwidth/filter.
ViStatus _VI_FUNC rsfpl_ConfigureIQAnalyzerAnalysisBandwidth(ViSession instrSession,
                                                             ViReal64  analysisBandwidth)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_IQ_ANALYZER_ANALYSIS_BANDWIDTH, analysisBandwidth),
            2, "Analysis Bandwidth");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines whether or not the recorded I/Q pairs should be swapped
/// HIFN  (I<->Q) before being processed. Swapping I and Q inverts the sideband.
/// HIFN     This is useful if the DUT interchanged the I and Q parts of the
/// HIFN  signal; then the instrument can do the same to compensate for it.
/// HIFN     For GSM measurements: Try this function if the TSC can not be found.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR swapIQEnabled/Defines whether or not the recorded I/Q pairs should be swapped
/// HIPAR swapIQEnabled/(I<->Q) before being processed. Swapping I and Q inverts the sideband.
ViStatus _VI_FUNC rsfpl_ConfigureIQAnalyzerSwapIQEnabled(ViSession instrSession,
                                                         ViBoolean swapIQEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_IQ_ANALYZER_SWAP_IQ_ENABLED, swapIQEnabled),
            2, "Swap IQ Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the record length for the acquired I/Q data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR recordLength/Sets the record length for the acquired I/Q data.
ViStatus _VI_FUNC rsfpl_ConfigureIQAnalyzerRecordLength(ViSession instrSession,
                                                        ViInt32   recordLength)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_IQ_ANALYZER_RECORD_LENGTH, recordLength),
            2, "Record Length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines how the resolution bandwidth is determined.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR rbwMode/Defines how the resolution bandwidth is determined.
/// HIPAR manualRBW/Defines the resolution bandwidth manually if 'RBW Mode' is set to
/// HIPAR manualRBW/Manual.
ViStatus _VI_FUNC rsfpl_ConfigureIQAnalyzerRBW(ViSession instrSession,
                                               ViInt32   rbwMode,
                                               ViReal64  manualRBW)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_IQ_ANALYZER_RBW_MODE, rbwMode),
            2, "RBW Mode");

    if (rbwMode == RSFPL_VAL_IQ_RBW_MODE_MANUAL)
    {
        viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_IQ_ANALYZER_MANUAL_RBW, manualRBW),
                3, "Manual RBW");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the advanced FFT parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR transformationAlgorithm/Defines the FFT calculation method.
/// HIPAR fftLength/Defines the number of frequency points determined by each FFT
/// HIPAR fftLength/calculation. The more points are used, the higher the resolution in the
/// HIPAR fftLength/spectrum becomes, but the longer the calculation takes.
/// HIPAR windowFunction/In the I/Q Analyzer you can select one of several FFT window types.
/// HIPAR windowOverlap/Defines the part of a single FFT window that is re-calculated by the
/// HIPAR windowOverlap/next FFT calculation.
/// HIPAR windowLength/Defines the number of samples to be included in a single FFT window
/// HIPAR windowLength/when multiple FFT windows are used.
ViStatus _VI_FUNC rsfpl_ConfigureIQAnalyzerAdvancedFFTMode(ViSession instrSession,
                                                           ViInt32   transformationAlgorithm,
                                                           ViInt32   fftLength,
                                                           ViInt32   windowFunction,
                                                           ViReal64  windowOverlap,
                                                           ViInt32   windowLength)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_IQ_ANALYZER_RBW_MODE, RSFPL_VAL_IQ_RBW_MODE_FFT),
            2, "IQ Analyzer RBW Mode");

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_IQ_ANALYZER_TRANSFORMATION_ALGORITHM, transformationAlgorithm),
            2, "Transformation Algorithm");

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_IQ_ANALYZER_WINDOW_FUNCTION, windowFunction),
            4, "Window Function");

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_IQ_ANALYZER_FFT_LENGTH, fftLength),
            3, "FFT Length");
    if (transformationAlgorithm == RSFPL_VAL_FFT_AVERAGE)
    {
        viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_IQ_ANALYZER_WINDOW_OVERLAP, windowOverlap),
                5, "Window Overlap");

        viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_IQ_ANALYZER_WINDOW_LENGTH, windowLength),
                6, "Window Length");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the time offset between the sample start and the trigger event
/// HIFN  (trigger point in sample = TPIS). Since the instrument usually samples
/// HIFN  with a much higher sample rate than the specific application actually
/// HIFN  requires, the trigger point determined internally is much more precise
/// HIFN  than the one determined from the (downsampled) data in the application.
/// HIFN  Thus, the TPIS indicates the offset between the sample start and the
/// HIFN  actual trigger event.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR triggerOffset/Queries the time offset between the sample start and the trigger event
/// HIPAR triggerOffset/(trigger point in sample = TPIS). Since the instrument usually samples
/// HIPAR triggerOffset/with a much higher sample rate than the specific application actually
/// HIPAR triggerOffset/requires, the trigger point determined internally is much more precise
/// HIPAR triggerOffset/than the one determined from the (downsampled) data in the application.
/// HIPAR triggerOffset/Thus, the TPIS indicates the offset between the sample start and the
/// HIPAR triggerOffset/actual trigger event.
ViStatus _VI_FUNC rsfpl_QueryIQAnalyzerTriggerOffset(ViSession instrSession,
                                                     ViReal64* triggerOffset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_GetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_IQ_ANALYZER_TRIGGER_OFFSET, triggerOffset),
            2, "Trigger Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the trace type a marker search is performed on.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR branchForPeaksearch/Selects the trace type a marker search is performed on.
ViStatus _VI_FUNC rsfpl_ConfigureIQAnalyzerBranchForPeaksearch(
                            ViSession instrSession,
                            ViInt32   branchForPeaksearch)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_IQ_ANALYZER_BRANCH_FOR_PEAKSEARCH, branchForPeaksearch),
            2, "Branch For Peaksearch");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Adds a window to the display in the active channel setup.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR windowName/String containing the name of the existing window the new window is
/// HIPAR windowName/inserted next to.
/// HIPAR direction/Direction the new window is added relative to the existing window.
/// HIPAR windowType/Type of result display (evaluation method) you want to add.
/// HIPAR returnedWindowName/String containing the name of the existing window the new window is
/// HIPAR returnedWindowName/inserted next to.
/// HIPAR arraySize/Defines the memory size allocated for Returned Window Name.
ViStatus _VI_FUNC rsfpl_IQAnalyzerLayoutAddWindow(ViSession instrSession,
                                                  ViString  windowName,
                                                  ViInt32   direction,
                                                  ViInt32   windowType,
                                                  ViUInt32  arraySize,
                                                  ViChar    returnedWindowName[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   response[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2direction;
    ViChar*  p2type;

    checkErr(RsCore_LockSession(instrSession));

    if (direction == RSFPL_VAL_DIRECTION_REPLACE)
    {
        checkErr(rsfpl_IQAnalyzerLayoutReplaceWindow(instrSession, windowName, windowType));
        (void)strcpy(returnedWindowName, windowName);
    }
    else
    {
        viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsFpl_rngLayoutDirection, direction, &p2direction),
            3, "Direction");
        viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsFpl_rngIQAnalyzerLayoutWindowType, windowType, &p2type),
            4, "Window Type");

        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:ADD? '%s',%s,%s", windowName, p2direction, p2type);
        checkErr(RsCore_QueryViString(instrSession, cmd, response));
        checkErr(rsfpl_CheckStatus(instrSession));

        if (returnedWindowName != NULL)
        {
            RsCore_TrimString(response, RS_VAL_TRIM_WHITESPACES_AND_ALL_QUOTES);
            strncpy(returnedWindowName, response, arraySize);
        }
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Replaces the window type (for example from 'Diagram' to 'Result
/// HIFN  Summary') of an already existing window in the active channel setup
/// HIFN  while keeping its position, index and window name.
/// HIFN     To add a new window, use the Layout.AddWindow method.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR windowName/String containing the name of the existing window the new window is
/// HIPAR windowName/inserted next to.
/// HIPAR windowType/Type of result display (evaluation method) you want to replace.
ViStatus _VI_FUNC rsfpl_IQAnalyzerLayoutReplaceWindow(ViSession instrSession,
                                                      ViString  windowName,
                                                      ViInt32   windowType)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  p2type;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsFpl_rngIQAnalyzerLayoutWindowType, windowType, &p2type),
        3, "Window Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:REPL '%s',%s", windowName, p2type);
    checkErr(rsfpl_WriteCommandWithOPCSync(instrSession, cmd, 0));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the I/Q data currently stored in the capture buffer of the
/// HIFN  instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR offsetSamples/This parameter sets offset of the values to be read related to the
/// HIPAR offsetSamples/start of the acquired data.
/// HIPAR numberOfSamples/This parameter sets the number of measurement values to be read. The
/// HIPAR numberOfSamples/value also defines size of arrays you specify for the real parts and
/// HIPAR numberOfSamples/imaginary parameters.
/// HIPAR returnedValues/Returns the number of I/Q trace data points.
/// HIPAR realParts_I/Returns an array of trace data (I-value x, real part) point. The array
/// HIPAR realParts_I/must contain at least as many elements as the value you specify with
/// HIPAR realParts_I/the Number Of Samples parameter.
/// HIPAR imaginaryParts_Q/Returns an array of trace data (Q-value y, imaginary part) point. The
/// HIPAR imaginaryParts_Q/array must contain at least as many elements as the value you specify
/// HIPAR imaginaryParts_Q/with the Number Of Samples parameter.
ViStatus _VI_FUNC rsfpl_FetchIQData(ViSession instrSession,
                                    ViInt32   offsetSamples,
                                    ViUInt32  numberOfSamples,
                                    ViReal64  realParts_I[],
                                    ViReal64  imaginaryParts_Q[],
                                    ViUInt32* returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    if (offsetSamples == 0 || numberOfSamples == 0)
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRAC:IQ:DATA:FORM IQP;:FORM REAL,32;:TRAC:IQ:DATA:MEM?");
    else
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRAC:IQ:DATA:FORM IQP;:FORM REAL,32;:TRAC:IQ:DATA:MEM? %ld,%ld", offsetSamples, numberOfSamples);

    checkErr(rsfpl_dataReadInterleaved(instrSession, cmd, -1, (ViInt32)numberOfSamples, realParts_I, imaginaryParts_Q, (ViInt32*)returnedValues));
    checkErr(rsfpl_CheckStatus(instrSession));

Error:
    (void)RsCore_Write(instrSession, "FORM ASC");
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Initiates a measurement with the current settings and returns the
/// HIFN  captured data from I/Q measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR timeout/This control sets the timeout value in ms.
/// HIPAR arraySize/Pass the number of doubles in the array you specify for the real parts
/// HIPAR arraySize/and imaginary parameters.
/// HIPAR realParts_I/Returns an array of trace data (I-value x, real part) point. The array
/// HIPAR realParts_I/must contain at least as many elements as the value you specify with
/// HIPAR realParts_I/the Number Of Samples parameter.
/// HIPAR imaginaryParts_Q/Returns an array of trace data (Q-value y, imaginary part) point. The
/// HIPAR imaginaryParts_Q/array must contain at least as many elements as the value you specify
/// HIPAR imaginaryParts_Q/with the Number Of Samples parameter.
/// HIPAR returnedValues/Returns the number of I/Q trace data points.
ViStatus _VI_FUNC rsfpl_ReadIQData(ViSession instrSession,
                                   ViUInt32  timeout,
                                   ViUInt32  arraySize,
                                   ViReal64  realParts_I[],
                                   ViReal64  imaginaryParts_Q[],
                                   ViUInt32* returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetTimeout(instrSession, timeout), 2, "Timeout");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRAC:IQ:DATA:FORM IQP;:FORM REAL,32;:TRAC:IQ:DATA?");
    checkErr(rsfpl_dataReadInterleaved(instrSession, cmd, timeout, (ViInt32)arraySize, realParts_I, imaginaryParts_Q, (ViInt32*)returnedValues));
    checkErr(rsfpl_CheckStatus(instrSession));

Error:
    (void)RsCore_Write(instrSession, "FORM ASC");
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Initiates a measurement with the current settings and returns the
/// HIFN  captured data from I/Q measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR timeout/This control sets the timeout value in ms.
/// HIPAR fileName/PC file to store the IQ data to.
ViStatus _VI_FUNC rsfpl_ReadIQDataToPCFile(ViSession instrSession,
                                           ViUInt32  timeout,
                                           ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViUInt32 oldTimeout = 0;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsfpl_GetTimeout(instrSession, &oldTimeout));
    viCheckParm(rsfpl_SetTimeout(instrSession, timeout), 2, "Timeout");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRAC:IQ:DATA:FORM IQP;:FORM REAL,32;:TRAC:IQ:DATA?");
    checkErr(RsCore_QueryBinaryDataBlockToFile(instrSession, cmd, fileName, RS_VAL_TRUNCATE));
    strncpy(gsCommand, "FORM ASC", RS_MAX_MESSAGE_BUF_SIZE);
    checkErr(RsCore_Write(instrSession, gsCommand));
    checkErr(rsfpl_CheckStatus(instrSession));

Error:
    if (oldTimeout > 0)
        (void)rsfpl_SetTimeout(instrSession, oldTimeout);

    (void)RsCore_Write(instrSession, "FORM ASC");
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Adds a comment to a file that contains I/Q data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR comment/Adds a comment to a file that contains I/Q data.
ViStatus _VI_FUNC rsfpl_ConfigureIQAnalyzerIQDataComment(ViSession instrSession,
                                                         ViString  comment)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_IQ_ANALYZER_IQ_DATA_COMMENT, comment),
            2, "Comment");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Writes the captured I/Q data to a file. The file extension is
/// HIFN  *.iq.tar.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR fileName/Writes the captured I/Q data to a file. The file extension is
/// HIPAR fileName/*.iq.tar.
ViStatus _VI_FUNC rsfpl_SaveIQData(ViSession instrSession,
                                   ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:STOR:IQ:STAT 1,'%s'", fileName);
    checkErr(rsfpl_WriteCommandWithOPCSync(instrSession, cmd, 0));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This method loads the I/Q data from the specified  file.
/// HIFN     The file extension is *.iq.tar.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR fileName/This parameter selects  file with IQ Data.
ViStatus _VI_FUNC rsfpl_LoadIQData(ViSession instrSession,
                                   ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:LOAD:IQ:STAT 1,'%s'", fileName);
    checkErr(rsfpl_WriteCommandWithOPCSync(instrSession, cmd, 0));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: rsfpl_SetAttribute<type> Functions
 * Purpose:  These functions enable the instrument driver user to set
 *           attribute values directly.  There are typesafe versions for
 *           ViInt32, ViReal64, ViString, ViBoolean, and ViSession datatypes.
 *****************************************************************************/
 /* ViInt32 */
ViStatus _VI_FUNC rsfpl_SetAttributeViInt32(ViSession instrSession,
                                            ViString  repeatedCapabilityName,
                                            ViAttr    attributeId,
                                            ViInt32   value)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_SetAttributeViInt32(instrSession, repeatedCapabilityName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* ViInt64 */
ViStatus _VI_FUNC rsfpl_SetAttributeViInt64(ViSession instrSession,
                                            ViString  repeatedCapabilityName,
                                            ViAttr    attributeId,
                                            ViInt64   value)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_SetAttributeViInt64(instrSession, repeatedCapabilityName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* ViReal64 */
ViStatus _VI_FUNC rsfpl_SetAttributeViReal64(ViSession instrSession,
                                             ViString  repeatedCapabilityName,
                                             ViAttr    attributeId,
                                             ViReal64  value)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_SetAttributeViReal64(instrSession, repeatedCapabilityName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* ViString */
ViStatus _VI_FUNC rsfpl_SetAttributeViString(ViSession instrSession,
                                             ViString  repeatedCapabilityName,
                                             ViAttr    attributeId,
                                             ViString  value)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_SetAttributeViStringOrRawString(instrSession, repeatedCapabilityName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* ViBoolean */
ViStatus _VI_FUNC rsfpl_SetAttributeViBoolean(ViSession instrSession,
                                              ViString  repeatedCapabilityName,
                                              ViAttr    attributeId,
                                              ViBoolean value)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_SetAttributeViBoolean(instrSession, repeatedCapabilityName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: rsfpl_GetAttribute<type> Functions
 * Purpose:  These functions enable the instrument driver user to get
 *           attribute values directly.  There are typesafe versions for
 *           ViInt32, ViReal64, ViString, ViBoolean, and ViSession attributes.
 *****************************************************************************/

 /* ViInt32 */
ViStatus _VI_FUNC rsfpl_GetAttributeViInt32(ViSession instrSession,
                                            ViString  repeatedCapabilityName,
                                            ViAttr    attributeId,
                                            ViInt32*  value)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);
    error = RsCore_GetAttributeViInt32(instrSession, repeatedCapabilityName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* ViInt64 */
ViStatus _VI_FUNC rsfpl_GetAttributeViInt64(ViSession instrSession,
                                            ViString  repeatedCapabilityName,
                                            ViAttr    attributeId,
                                            ViInt64*  value)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);
    error = RsCore_GetAttributeViInt64(instrSession, repeatedCapabilityName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* ViReal64 */
ViStatus _VI_FUNC rsfpl_GetAttributeViReal64(ViSession instrSession,
                                             ViString  repeatedCapabilityName,
                                             ViAttr    attributeId,
                                             ViReal64* value)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_GetAttributeViReal64(instrSession, repeatedCapabilityName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* ViString */
ViStatus _VI_FUNC rsfpl_GetAttributeViString(ViSession instrSession,
                                             ViString  repeatedCapabilityName,
                                             ViAttr    attributeId,
                                             ViInt32   bufferSize,
                                             ViChar    attributeValue[])
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_GetAttributeViStringOrRawString(instrSession, repeatedCapabilityName, attributeId, RS_VAL_DIRECT_USER_CALL, bufferSize,
        attributeValue);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* ViBoolean */
ViStatus _VI_FUNC rsfpl_GetAttributeViBoolean(ViSession  instrSession,
                                              ViString   repeatedCapabilityName,
                                              ViAttr     attributeId,
                                              ViBoolean* value)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_GetAttributeViBoolean(instrSession, repeatedCapabilityName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Function used by Hi-level functions to obtain a repCap name from:
/// HIFN - attributeID
/// HIFN - repCapNameID
/// HIFN - 0-based index in the repCapNames
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwgs_init or
/// HIPAR instrSession/rscmwgs_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR repCapNameId/RepCap Name ID.
/// HIPAR index/Zero based index in the repCapNames.
/// HIPAR bufferSize/Pass the number of bytes in the string you allocate for the repCapName.
/// HIPAR repCapName/String returning RepCap name
ViStatus _VI_FUNC rsfpl_GetAttributeRepCapName(ViSession instrSession,
                                               ViAttr    attributeId,
                                               ViString  repCapNameId,
                                               ViInt32   index,
                                               ViInt32   bufferSize,
                                               ViChar    repCapName[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, attributeId, repCapNameId, index, bufferSize, repCapName));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Rs_GetAttributeRepCapNameId
 * Purpose:  This function returns the repeated capability id(s) that
 *           belongs to the attribute you specify. More then one id is listed
 *           in comma separated string.
 *****************************************************************************/
ViStatus _VI_FUNC rsfpl_GetAttributeRepeatedCapabilityIds(ViSession instrSession,
                                                          ViAttr    attributeID,
                                                          ViInt32   bufferSize,
                                                          ViChar    repeatedCapabilityIds[])
{
    ViStatus error = VI_SUCCESS;

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, bufferSize, 1, -1),
        3, "Buffer Size");

    checkErr(RsCore_GetAttributeRepCapNameIds(instrSession, attributeID, bufferSize, repeatedCapabilityIds));

Error:
    return error;
}

/*****************************************************************************
 * Function: Get Attribute Repeated Capability Id Name(s)
 * Purpose:  This function returns the repeated capability name(s) that
 *           belongs to the attribute and single repeated capability id you
 *           may specify. More then one name is listed in comma separated
 *           string.
 *****************************************************************************/
ViStatus _VI_FUNC rsfpl_GetAttributeRepeatedCapabilityIdNames(
                            ViSession instrSession,
                            ViAttr    attributeID,
                            ViString  repeatedCapabilityId,
                            ViInt32   bufferSize,
                            ViChar    repeatedCapabilityIdNames[])
{
    ViStatus error = VI_SUCCESS;

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, bufferSize, 1, -1),
        4, "Buffer Size");

    checkErr(RsCore_GetAttributeRepCapNamesAll(instrSession,
        attributeID,
        repeatedCapabilityId,
        bufferSize,
        repeatedCapabilityIdNames));

Error:
    return error;
}

/*****************************************************************************
 * Function: Reset
 * Purpose:  This function resets the instrument.
 *****************************************************************************/
ViStatus _VI_FUNC rsfpl_reset(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    if (RsCore_Simulating(instrSession) == VI_FALSE)
    {
        checkErr(rsfpl_ClearStatus(instrSession));
        checkErr(RsCore_Write(instrSession, "*RST")); // IGNORE: Check double write
        checkErr(RsCore_QueryViStringShort(instrSession, "*OPC?", NULL));
    }

    checkErr(rsfpl_DefaultInstrSetup(instrSession));
    checkErr(rsfpl_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);

    return error;
}


/*****************************************************************************
 * Function: Self Test
 * Purpose:  This function executes the instrument self-test and returns the
 *           result.
 *****************************************************************************/
ViStatus _VI_FUNC rsfpl_self_test(ViSession instrSession,
                                  ViInt16*  testResult,
                                  ViChar    testMessage[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, testResult), 2, "Null address for Test Result");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, testMessage), 3, "Null address for Test Message");

    checkErr(RsCore_SelfTest(instrSession, testResult, 48, testMessage));

    checkErr(rsfpl_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/*****************************************************************************
 * Function: Revision Query
 * Purpose:  This function returns the driver and instrument revisions.
 *****************************************************************************/
ViStatus _VI_FUNC rsfpl_revision_query(ViSession instrSession,
                                       ViChar    instrumentDriverRevision[],
                                       ViChar    firmwareRevision[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, instrumentDriverRevision), 2, "Null address for Instrument Driver Revision");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, firmwareRevision), 3, "Null address for Instrument Revision");

    checkErr(RsCore_GetAttributeViString(instrSession, NULL, RS_ATTR_SPECIFIC_DRIVER_REVISION, 0, 256, instrumentDriverRevision));
    checkErr(RsCore_GetAttributeViString(instrSession, NULL, RS_ATTR_INSTRUMENT_FIRMWARE_REVISION, 0, 256, firmwareRevision));
    checkErr(rsfpl_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/*****************************************************************************
 * Function: Error Query
 * Purpose:  This function queries the instrument error queue and returns
 *           the result.
 *****************************************************************************/
ViStatus _VI_FUNC rsfpl_error_query(ViSession instrSession,
                                    ViInt32*  errorCode,
                                    ViChar    errorMessage[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, errorCode), 2, "Null address for Error Code");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, errorMessage), 3, "Null address for Error Message");

    checkErr(RsCore_ErrorQueryAll(instrSession, errorCode, 1024, errorMessage));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function reads all system messages and returns a list of comma
/// HIFN  separated strings. Each string corresponds to an entry in the table
/// HIFN  SYSTEM MESSAGES.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Sets the size of Errors string.
/// HIPAR errors/Returns the error message string read from the instrument's error
/// HIPAR errors/message queue.
ViStatus _VI_FUNC rsfpl_ErrorList(ViSession instrSession,
                                  ViInt32   bufferSize,
                                  ViChar    errors[],
                                  ViInt32*  returnCount)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, bufferSize, 0),
        2, "Buffer Size");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, errors),
        3, "Null address for Errors buffer");

    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession,
        "SYST:ERR:LIST?", bufferSize, errors, returnCount));

    checkErr(rsfpl_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This command queries all system messages, or all messages of a defined
/// HIFN  type, displayed in the status bar for a specific measurement channel
/// HIFN  (application).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR messageType/This control selects the message type.
/// HIPAR channelName/String containing the name of the channel. This parameter is optional.
/// HIPAR channelName/Pass NULL if you don't want to use it.
/// HIPAR bufferSize/Sets the size of Messages string.
/// HIPAR messages/String containing all messages of the selected type for the specified
/// HIPAR messages/measurement channel. Each message is separated by a comma and inserted
/// HIPAR messages/in parentheses. If no messages are available, empty parentheses are
/// HIPAR messages/returned.
ViStatus _VI_FUNC rsfpl_ErrorListSpecificType(ViSession instrSession,
                                              ViInt32   messageType,
                                              ViString  channelName,
                                              ViInt32   bufferSize,
                                              ViChar    messages[],
                                              ViInt32*  returnCount)
{
    ViStatus error = VI_SUCCESS;
    ViString messageTypeArr[] = { "ALL", "INFO", "WARN", "FAT", "ERR", "MESS" };
    ViChar   cmd[RS_MAX_MESSAGE_LEN] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, messageType, RSFPL_VAL_MESSAGE_TYPE_ALL, RSFPL_VAL_MESSAGE_TYPE_MESSAGE),
            2, "Message Type");
    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, bufferSize, 0),
        4, "Buffer Size");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, messages),
        5, "Null address for Messages buffer");

    if (channelName == NULL)
        snprintf(cmd, RS_MAX_MESSAGE_LEN, "SYST:ERR:EXT? %s", messageTypeArr[messageType]);
    else
    {
        viCheckParm(RsCore_InvalidStringLength(instrSession, channelName, 0,
            (RS_MAX_MESSAGE_LEN - 24)),
            3, "Channel Name");
        snprintf(cmd, RS_MAX_MESSAGE_LEN, "SYST:ERR:EXT? %s,\'%s\'", messageTypeArr[messageType], channelName);
    }
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd,
        (ViInt32)bufferSize, messages, returnCount));

    checkErr(rsfpl_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Error Message
 * Purpose:  This function translates the error codes returned by this
 *           instrument driver into user-readable strings.
 *
 *           Note:  The caller can pass NULL for the vi parameter.  This
 *           is useful if one of the init functions fail.
 *****************************************************************************/
ViStatus _VI_FUNC rsfpl_error_message(ViSession instrSession,
                                      ViStatus  errorCode,
                                      ViChar    errorMessage[])
{
    return rsfpl_utility_error_message( instrSession, errorCode, errorMessage);
}

/// HIFN Returns information about the system.
/// HIRET Document return value here.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.
/// HIPAR numberOfBytesToRead/Pass the maximum number of bytes to read from the instruments.
/// HIPAR systemInformation/Returns information about the system.
ViStatus _VI_FUNC rsfpl_QuerySystemInformation(ViSession instrSession,
                                               ViUInt32  numberOfBytesToRead,
                                               ViChar    systemInformation[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViUInt32 byte_count = 0;
    ViChar*  p2buf = NULL;
    size_t   count = 0;

    checkErr(RsCore_LockSession(instrSession));

    strcpy(cmd, "SYST:INFO?");
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, "SYST:INFO?", &p2buf));
    count = ((ViUInt32)numberOfBytesToRead - 1 < byte_count) ? (size_t)numberOfBytesToRead - 1 : (size_t)byte_count;
    memset(systemInformation, 0, numberOfBytesToRead);
    strncpy(systemInformation, p2buf, count);

    checkErr(rsfpl_CheckStatus(instrSession));

Error:
    if (p2buf) free(p2buf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function sets the OPC timeout.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.
/// HIPAR OPCtimeout/This control sets the timeout value.
ViStatus _VI_FUNC rsfpl_SetTimeout(ViSession instrSession,
                                   ViUInt32  timeout)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_SetAttributeViInt32(instrSession, NULL, RS_ATTR_OPC_TIMEOUT, 0, (ViInt32)timeout));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function returns the OPC timeout.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.
/// HIPAR timeout/This control returns the timeout value.
ViStatus _VI_FUNC rsfpl_GetTimeout(ViSession instrSession,
                                   ViUInt32* timeout)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeViInt32(instrSession, NULL, RS_ATTR_OPC_TIMEOUT, 0, (ViInt32*)timeout));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function specifies the minimum timeout value to use (in
/// HIFN milliseconds) when accessing the device associated with the
/// HIFN given session.
/// HIFN Note that the actual timeout used may be higher than the one
/// HIFN requested.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function. The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR VISATimeout/Specifies the minimum timeout value to use (in milliseconds)
/// HIPAR VISATimeout/when accessing the device associated with the given session.
/// HIPAR VISATimeout/Note that the actual timeout used may be higher than the one
/// HIPAR VISATimeout/requested.
ViStatus _VI_FUNC rsfpl_SetIOTimeout(ViSession instrSession,
                                     ViUInt32  VISATimeout)
{
    return RsCore_SetVisaTimeout(instrSession, VISATimeout);
}

/// HIFN This function queries the minimum timeout value to use (in
/// HIFN milliseconds) when accessing the device associated with the
/// HIFN given session.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function. The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR VISATimeout/Queries the minimum timeout value to use (in milliseconds)
/// HIPAR VISATimeout/when accessing the device associated with the given session.
ViStatus _VI_FUNC rsfpl_GetIOTimeout(ViSession instrSession,
                                     ViUInt32* VISATimeout)
{
    return RsCore_GetVisaTimeout(instrSession, VISATimeout);
}

/// HIFN This function switches ON/OFF various error checkings performed inside of the driver.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR optionChecking/This control switches option checking On or Off.
/// HIPAR rangeChecking/This control enables or disables range checking.
/// HIPAR statusChecking/This control enables or disables instrument state checking
ViStatus _VI_FUNC rsfpl_ConfigureErrorChecking(ViSession instrSession,
                                               ViBoolean optionChecking,
                                               ViBoolean rangeChecking,
                                               ViBoolean statusChecking)
{
    ViStatus         error = VI_SUCCESS;
    RsCoreSessionPtr rsSession = NULL;

    checkErr(RsCore_GetRsSession(instrSession, &rsSession));
    rsSession->optionChecking = optionChecking;
    checkErr(RsCore_SetAttributeViBoolean(instrSession, "", RS_ATTR_QUERY_INSTRUMENT_STATUS, 0, statusChecking));
    checkErr(RsCore_SetAttributeViBoolean(instrSession, "", RS_ATTR_RANGE_CHECK, 0, rangeChecking));

Error:
    return error;
}

/// HIFN  This function clears status.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_ClearStatus(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_ClearStatus(instrSession));
    checkErr(rsfpl_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the ID Query response string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of IDQueryResponse buffer
/// HIPAR idQueryResponse/Returns the ID Query response string. The array should consist of at
/// HIPAR idQueryResponse/least 256 elements.
ViStatus _VI_FUNC rsfpl_IDQueryResponse(ViSession instrSession,
                                        ViUInt32  bufferSize,
                                        ViChar    IDQueryResponse[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsfpl_QueryViString(instrSession, "*IDN?", bufferSize, IDQueryResponse, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Stops further command processing until all commands sent before *WAI
/// HIFN  have been executed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_ProcessAllPreviousCommands(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_Write(instrSession, "*WAI"));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the OPC.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR opc/Queries the OPC.
ViStatus _VI_FUNC rsfpl_QueryOPC(ViSession instrSession,
                                 ViInt32*  opc)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_QueryViInt32(instrSession, "*OPC?", opc));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN If TRUE (default after init is FALSE), the driver calls SYST:ERR? automatically in CheckStatus() when Instrument Status Error is detected.
/// HIFN Use the function rsfph_GetError afterwards to get the detailed information about the instrument error(s).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfph_init or
/// HIPAR instrSession/rsfph_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR autoSystErrQuery/If TRUE (default after init is FALSE), the driver calls SYST:ERR? automatically in CheckStatus() when Instrument Status Error is detected.
ViStatus _VI_FUNC rsfpl_ConfigureAutoSystemErrQuery(ViSession instrSession,
                                                    ViBoolean autoSystErrQuery)
{
    ViStatus         error = VI_SUCCESS;
    RsCoreSessionPtr rsSession = NULL;

    checkErr(RsCore_GetRsSession(instrSession, &rsSession));
    rsSession->autoSystErrQuery = autoSystErrQuery;

Error:
    return error;
}
/// HIFN Session-related mutex switch.
/// HIFN If TRUE (default after init is FALSE), the driver sets a mutual-exclusion mechanism for every driver's hi-level function and attribute access.
/// HIFN Set this value to TRUE if you plan to use one session in more than one thread.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfph_init or
/// HIPAR instrSession/rsfph_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR multiThreadLocking/Session-related mutex switch
ViStatus _VI_FUNC rsfpl_ConfigureMultiThreadLocking(ViSession instrSession,
                                                    ViBoolean multiThreadLocking)
{
    ViStatus         error = VI_SUCCESS;
    RsCoreSessionPtr rsSession = NULL;

    checkErr(RsCore_GetRsSession(instrSession, &rsSession));
    rsSession->multiThreadLocking = multiThreadLocking;

Error:
    return error;
}


/// HIFN  Reads the footprint file of the sensor.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Measurement channel if more than one channel is available.
/// HIPAR bufferSize/Defines the size of buffer in argument 'Device Footprint'.
/// HIPAR deviceFootprint/
/// HIPAR returnedValues/Returns count of characters read from instrument and stored in buffer
/// HIPAR returnedValues/passed to 'Measurement'.
ViStatus _VI_FUNC rsfpl_GetDeviceFootprint(ViSession instrSession,
                                           ViInt32   channel,
                                           ViUInt32  bufferSize,
                                           ViChar    deviceFootprint[],
                                           ViUInt32* returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, 0, 4),
            2, "Channel");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SYST:DFPR%ld?", channel);
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, bufferSize, deviceFootprint, (ViInt32*)returnedValues));

    checkErr(rsfpl_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Get Error and Clear Error Functions
 * Purpose:  These functions enable the instrument driver user to
 *           get or clear the error information the driver associates with the
 *           RS session.
 *****************************************************************************/
ViStatus _VI_FUNC rsfpl_GetError(ViSession instrSession,
                                 ViStatus* errorCode,
                                 ViUInt32  bufferSize,
                                 ViChar    description[])
{
    ViStatus error = VI_SUCCESS;

    // Lock Session - do not jump to the end, even with invalid instrSession the function continues further
    (void)(RsCore_LockSession(instrSession));

    // Test for nulls and acquire error data
    viCheckParm(RsCore_InvalidNullPointer(instrSession, errorCode), 2, "Null address for Error");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, description), 4, "Null address for Description");

    checkErr(RsCore_GetErrorCompleteDescription(instrSession, &rsfpl_utility_error_message, errorCode, bufferSize, description)
    );

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


ViStatus _VI_FUNC rsfpl_ClearError(ViSession instrSession)
{
    return RsCore_ClearErrorInfo(instrSession);
}

/*===========================================================================*
 *------------------- Instrument I/O ----------------------------------------*
 *===========================================================================*/
 /// HIFN This function is used to read data from the instrument and write
 /// HIFN it to a user specified file on the host computer.
 /// HIRET Returns the status code of this operation.
 /// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
 /// HIPAR instrSession/function to select the desired instrument driver session.
 /// HIPAR source/This control selects file for the data transfer from instrument
 /// HIPAR source/to control computer.
 /// HIPAR destination/This control defines destination file to which the data transfer
 /// HIPAR destination/from instrument to control computer applies.
ViStatus _VI_FUNC rsfpl_ReadToFileFromInstrument(ViSession instrSession,
                                                 ViString  source,
                                                 ViString  destination)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:DATA? '%s'", source);
    checkErr(RsCore_QueryBinaryDataBlockToFile(instrSession, cmd, destination, RS_VAL_TRUNCATE));

    checkErr(rsfpl_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function is used to read data from the host computer and
/// HIFN write it to a user specified file in the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR source/This control selects file for the data transfer from control
/// HIPAR source/computer to instrument.
/// HIPAR destination/This control defines destination file to which the data transfer
/// HIPAR destination/from control computer to instrument applies.
ViStatus _VI_FUNC rsfpl_WriteFromFileToInstrument(ViSession instrSession,
                                                  ViString  source,
                                                  ViString  destination)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:DATA '%s',", destination);
    checkErr(RsCore_WriteBinaryDataFromFile(instrSession, cmd, source));
    checkErr(rsfpl_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/*****************************************************************************
 * Function: Query ViBoolean
 * Purpose:  This function queries the ViBoolean value.
 *****************************************************************************/
ViStatus _VI_FUNC rsfpl_QueryViBoolean(ViSession  instrSession,
                                       ViString   command,
                                       ViBoolean* value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidStringLength(instrSession, command, 1, -1), 2, "Command (string length)");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, value), 3, "Value");
    checkErr(RsCore_QueryViBoolean(instrSession, command, value));
    checkErr(rsfpl_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Query ViInt32
 * Purpose:  This function queries the ViInt32 value.
 *****************************************************************************/
ViStatus _VI_FUNC rsfpl_QueryViInt32(ViSession instrSession,
                                     ViString  command,
                                     ViInt32*  value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidStringLength(instrSession, command, 1, -1), 2, "Command (string length)");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, value), 3, "Value");
    checkErr(RsCore_QueryViInt32(instrSession, command, value));
    checkErr(rsfpl_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Query ViReal64
 * Purpose:  This function queries the ViReal64 value.
 *****************************************************************************/
ViStatus _VI_FUNC rsfpl_QueryViReal64(ViSession instrSession,
                                      ViString  command,
                                      ViReal64* value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidStringLength(instrSession, command, 1, -1), 2, "Command (string length)");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, value), 3, "Value");
    checkErr(RsCore_QueryViReal64(instrSession, command, value));
    checkErr(rsfpl_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/*****************************************************************************
 * Function: Query ViString
 * Purpose:  This function queries the ViString value.
 *****************************************************************************/
ViStatus _VI_FUNC rsfpl_QueryViString(ViSession instrSession,
                                      ViString  command,
                                      ViUInt32  bufferSize,
                                      ViChar    value[],
                                      ViPUInt32 returnCount)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidStringLength(instrSession, command, 1, -1), 2, "Command (string length)");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, value), 3, "Value");
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, command, bufferSize, value, (ViInt32*)returnCount));
    checkErr(rsfpl_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Sends a command to the instrument synchronised with OPC-polling.
/// HIFN Use this function to wait for an instrument operation that can
/// HIFN take a long time to complete, e.g. self-alignment.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR writeBuffer/Pass the string to be written to the instrument.
ViStatus _VI_FUNC rsfpl_WriteCommand(ViSession instrSession,
                                     ViString  writeBuffer)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_Write(instrSession, writeBuffer));
    checkErr(rsfpl_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Sends a command to the instrument synchronised with OPC-polling.
/// HIFN Use this function to wait for an instrument operation that can
/// HIFN take a long time to complete, e.g. self-alignment.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR writeBuffer/Pass the string to be written to the instrument.
/// HIPAR timeout/This control sets the timeout value.
ViStatus _VI_FUNC rsfpl_WriteCommandWithOPCSync(ViSession instrSession,
                                                ViString  writeBuffer,
                                                ViUInt32  timeout)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_WriteWithOpc(instrSession, writeBuffer, timeout));
    checkErr(rsfpl_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Sends a query to the instrument synchronised with OPC-polling.
/// HIFN Use this function to wait for a response that can take a long
/// HIFN time to complete e.g. self-alignment, calibration or self-test.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR writeBuffer/Pass the string to be written to the instrument.
/// HIPAR timeout/This control sets the timeout value.
/// HIPAR bufferSize/Pass the maximum number of bytes to read from the instruments.
/// HIPAR readBuffer/After this function executes, this parameter contains the data
/// HIPAR readBuffer/that was read from the instrument.
/// HIPAR numBytesRead/Returns the number of bytes actually read from the instrument
/// HIPAR numBytesRead/and stored in the Read Buffer.
ViStatus _VI_FUNC rsfpl_QueryWithOPCSync(ViSession instrSession,
                                         ViString  writeBuffer,
                                         ViUInt32  timeout,
                                         ViUInt32  bufferSize,
                                         ViChar    readBuffer[],
                                         ViUInt32* numBytesRead)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_QueryViStringUnknownLengthToUserBufferWithOpc(instrSession, writeBuffer, timeout, bufferSize, readBuffer, (ViInt32*)numBytesRead));
    checkErr(rsfpl_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function contains the commands for the querying status reporting
/// HIFN  system.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
ViStatus _VI_FUNC rsfpl_PresetStatusRegister(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_STATUS_PRESET, NULL),
            2, "Status Preset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the Enable, NTransition, and PTransition bits
/// HIFN  of the Main, Power, Limit, LMargin, ACP Limit, Frequency, status
/// HIFN  questionable and operating registers.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR registerOperation/Specifies the operation to perform.
/// HIPAR questionableRegister/This control selects the registers that will be configured.
/// HIPAR enable/This control sets the enable bits of the status register.
/// HIPAR ptransition/This control sets the PTransition bits (edge detectors) of the status
/// HIPAR ptransition/register.
/// HIPAR ntransition/This control sets the NTransition bits (edge detectors) of the status
/// HIPAR ntransition/register.
ViStatus _VI_FUNC rsfpl_SetStatusRegister(ViSession instrSession,
                                          ViInt32   registerOperation,
                                          ViInt32   questionableRegister,
                                          ViUInt16  enable,
                                          ViUInt16  PTransition,
                                          ViUInt16  NTransition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2reg;

    checkErr(RsCore_LockSession(instrSession));

    switch (registerOperation)
    {
    case RSFPL_VAL_STAT_REG_QUESTIONABLE:
        viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsFpl_rngStatusRegister, questionableRegister, &p2reg),
            3, "Questionable Register");
        switch (questionableRegister)
        {
        case RSFPL_VAL_STAT_REG_MAIN:
            snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":STAT:QUES:ENAB %hu;PTR %hu;NTR %hu", enable, PTransition, NTransition);
            checkErr(rsfpl_WriteCommandWithOPCSync(instrSession, cmd, 0));
            break;
        default:
            snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":STAT:QUES:%s:ENAB %hu;PTR %hu;NTR %hu", p2reg, enable, PTransition, NTransition);
            checkErr(rsfpl_WriteCommandWithOPCSync(instrSession, cmd, 0));
        }
        break;
    case RSFPL_VAL_STAT_REG_OPERATION:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":STAT:OPER:ENAB %hu;PTR %hu;NTR %hu", enable, PTransition, NTransition);
        checkErr(rsfpl_WriteCommandWithOPCSync(instrSession, cmd, 0));
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, registerOperation), 2, "Register Operation");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function contains the commands for the querying status reporting
/// HIFN  system.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR registerOperation/Specifies the operation to perform.
/// HIPAR questionableRegister/This control selects the registers that will be configured.
/// HIPAR registerPart/Sets the register part.
/// HIPAR registerValue/Returns content of selected register.
ViStatus _VI_FUNC rsfpl_GetStatusRegister(ViSession instrSession,
                                          ViInt32   registerOperation,
                                          ViInt32   questionableRegister,
                                          ViInt32   registerPart,
                                          ViUInt16* registerValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2reg;
    ViInt32  result = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, registerPart, 0, 1),
            4, "Register Part");

    switch (registerOperation)
    {
    case RSFPL_VAL_STAT_REG_QUESTIONABLE:
        viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsFpl_rngStatusRegister, questionableRegister, &p2reg),
            3, "Questionable Register");
        switch (questionableRegister)
        {
        case RSFPL_VAL_STAT_REG_MAIN:
            snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "STAT:QUES:%s?", RsFpl_rngStatusRegisterQuery.entries[registerPart].cmdString);
            checkErr(rsfpl_QueryViInt32(instrSession, cmd, &result));
            break;
        default:
            snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "STAT:QUES:%s:%s?", p2reg, RsFpl_rngStatusRegisterQuery.entries[registerPart].cmdString);
            checkErr(rsfpl_QueryViInt32(instrSession, cmd, &result));
        }

        break;
    case RSFPL_VAL_STAT_REG_OPERATION:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "STAT:OPER:%s?", RsFpl_rngStatusRegisterQuery.entries[registerPart].cmdString);
        checkErr(rsfpl_QueryViInt32(instrSession, cmd, &result));
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, registerOperation), 2, "Register Operation");
    }

    *registerValue = (ViUInt16)result;

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: rsfpl_close
 * Purpose:  This function closes the instrument.
 *
 *           Note:  This function must unlock the session before calling
 *           Rs_Dispose.
 *****************************************************************************/
ViStatus _VI_FUNC rsfpl_close(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_ViClose(instrSession));

Error:
    (void)RsCore_Dispose(instrSession);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}
