/*****************************************************************************
 *  rsfpl_vsa.c
 *
 *  Rohde & Schwarz FPL spectrum analyzer Instrument Driver
 *  Original Release: August 2019
 *  By: Petr Kominek
 *
 *  Should you have any technical questions please contact the hotline of
 *  Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
 *
 *  e-mail: CustomerSupport@rohde-schwarz.com
 *
 *  Modification History:
 *    see ChangeLog
 *
 *  Copyright (c) 2019, Rohde & Schwarz GmbH & Co. KG
 *  All rights reserved.
 *****************************************************************************/

#include "rsfpl.h"

#if (defined(WIN32) || defined(_WIN32) || defined(__WIN32__) || defined(__NT__))
#define ANSI
#endif
#ifndef ANSI                /* UNIX compatible */
#include <sys/timeb.h>
#include <sys/types.h>
#include <unistd.h>
#define Sleep sleep
#else                       /* ANSI compatible */
#include <windows.h>
#endif
#include <string.h>
#if defined (_CVI_)
#include <utility.h>
#define _strnicmp strnicmp
#endif

static ViString pattArr[] = { "CURR", "ALL", NULL };
static ViString resultAdrArr[] = { "", "AVG", "RPE", "SDEV", "PCTL", NULL };
static ViString resultEVMArr[] = { "", "AVG", "PAVG", "PCTL", "PEAK", "PPCT", "PSD", "RPE", "SDEV", "TPE", NULL };
static ViString IQFormatArr[] = { "FLOAT32", "INT32",NULL };
static ViString IQDataFormatArr[] = { "COMP", "REAL",NULL };

/// HIFN  This function selects the VSA mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_SetVSAMode(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsfpl_SetAttributeViString(instrSession, "", RSFPL_ATTR_VSA_MODE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Restores the factory settings of standards or patterns for the VSA
/// HIFN  application.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR defaults/Restores the factory settings of standards or patterns for the VSA
/// HIPAR defaults/application.
ViStatus _VI_FUNC rsfpl_SetVSAFactoryDefaults(ViSession instrSession,
                                              ViInt32   defaults)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, "", RSFPL_ATTR_VSA_RESTORES_FACTORY_SETTINGS, defaults),
            2, "Defaults");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function selects a predefined 'signal overview' consisting of
/// HIFN  four windows. The top left window (1) shows magnitude data from capture
/// HIFN  buffer, the top right window (2) spectrum data from capture buffer, the
/// HIFN  bottom left window (3) the result summary and the bottom right window
/// HIFN  (4) constellation I/Q data. Using this setup, scripts written for R&S
/// HIFN  FSV instruments will continue to work.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_PresetVSAScreens(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsfpl_SetAttributeViString(instrSession, "", RSFPL_ATTR_VSA_WINDOW_PRESET, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function initiates a measurement that evaluates and sets the
/// HIFN  ideal reference level for the current measurement. This ensures that
/// HIFN  the settings of the RF attenuation and the reference level are
/// HIFN  optimally adjusted to the signal level without overloading the R&S
/// HIFN  FPL1000 or limiting the dynamic range by an S/N ratio that is too
/// HIFN  small.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR timeout/This control sets the timeout value.
ViStatus _VI_FUNC rsfpl_VSAReferenceLevelAdjust(ViSession instrSession,
                                                ViUInt32  timeout)
{
    ViStatus error = VI_SUCCESS;
    ViUInt32 old_timeout = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsfpl_GetTimeout(instrSession, &old_timeout));
    checkErr(rsfpl_SetTimeout(instrSession, timeout));
    checkErr(rsfpl_SetAttributeViString(instrSession, "", RSFPL_ATTR_VSA_OUTPUT_PRESET_REFERENCE_LEVEL, NULL));

Error:
    if (old_timeout > 0)
        rsfpl_SetTimeout(instrSession, old_timeout);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the RF Input.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR coupling/This control selects the coupling type of the RF input. The control is
/// HIPAR coupling/not available for measurements with the optional Digital Baseband
/// HIPAR coupling/Interface.
/// HIPAR yigFilter/Sets the state of the YIG Filter.
/// HIPAR padType/Defines the type of matching pad used for impedance conversion for RF
/// HIPAR padType/input.
ViStatus _VI_FUNC rsfpl_ConfigureVSARFInput(ViSession instrSession,
                                            ViInt32   coupling,
                                            ViBoolean yigFilter,
                                            ViInt32   padType)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, "", RSFPL_ATTR_VSA_RF_INPUT_COUPLING, coupling),
            2, "Coupling");

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, "", RSFPL_ATTR_VSA_RF_INPUT_YIG_FILTER, yigFilter),
            3, "YIG Filter");

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, "", RSFPL_ATTR_VSA_RF_INPUT_PAD_TYPE, padType),
            4, "PAD Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the priority for signal processing after the RF attenuation
/// HIFN  has been applied.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR optimizationMode/Selects the priority for signal processing after the RF attenuation
/// HIPAR optimizationMode/has been applied.
ViStatus _VI_FUNC rsfpl_ConfigureVSAOutputOpimizationMode(ViSession instrSession,
                                                          ViInt32   optimizationMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, "", RSFPL_ATTR_VSA_OUTPUT_OPIMIZATION_MODE, optimizationMode),
            2, "Optimization Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the IQ State.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fileName/Sets the file name parameter.
/// HIPAR dataFormat/Sets the data format.
ViStatus _VI_FUNC rsfpl_ConfigureVSAImportExportIQData(ViSession instrSession,
                                                       ViString  fileName,
                                                       ViInt32   IQFormat,
                                                       ViInt32   dataFormat)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar   cmd2[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:STOR:IQ:STAT 1,'%s'", fileName);
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:STOR:IQ:FORM %s,%s", IQFormatArr[IQFormat], IQDataFormatArr[dataFormat]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsfpl_CheckStatus(instrSession));

    checkErr(RsCore_Write(instrSession, cmd2));
    checkErr(rsfpl_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Restores the default settings of the currently selected standard.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_ConfigureVSAStandardPreset(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsfpl_SetAttributeViString(instrSession, "", RSFPL_ATTR_VSA_STANDARD_PRESET, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enters the comment for a new standard. The comment is stored with the
/// HIFN  standard and is only displayed in the selection menu (manual
/// HIFN  operation). When remote control is used, the string is deleted after
/// HIFN  the standard has been stored, allowing a new comment to be entered for
/// HIFN  the next standard. In this case a blank string is returned when a query
/// HIFN  is made.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR comment/Enters the comment for a new standard. The comment is stored with the
/// HIPAR comment/standard and is only displayed in the selection menu (manual
/// HIPAR comment/operation). When remote control is used, the string is deleted after
/// HIPAR comment/the standard has been stored, allowing a new comment to be entered for
/// HIPAR comment/the next standard. In this case a blank string is returned when a query
/// HIPAR comment/is made.
ViStatus _VI_FUNC rsfpl_ConfigureVSAStandardComment(ViSession instrSession,
                                                    ViString  comment)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, "", RSFPL_ATTR_VSA_CONFIGURATION_COMMENT, comment),
            2, "Comment");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects an automatic setting of all modulation parameters according to
/// HIFN  a standardized transmission method or a user-defined transmission
/// HIFN  method. The standardized transmission methods are available in the
/// HIFN  instrument as predefined standards.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR operation/This control selects the type of operation to be performed.
/// HIPAR standard/Selects an automatic setting of all modulation parameters according to
/// HIPAR standard/a standardized transmission method or a user-defined transmission
/// HIPAR standard/method. The standardized transmission methods are available in the
/// HIPAR standard/instrument as predefined standards.
ViStatus _VI_FUNC rsfpl_ConfigureVSAStandardOperations(ViSession instrSession,
                                                       ViInt32   operation,
                                                       ViString  standard)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    switch (operation)
    {
    case RSFPL_VAL_VSA_STANDARD_LOAD:
        viCheckParm(rsfpl_SetAttributeViString(instrSession, "", RSFPL_ATTR_VSA_LOAD_STANDARD, standard),
                3, "Standard");
        break;
    case RSFPL_VAL_VSA_STANDARD_SAVE:
        viCheckParm(rsfpl_SetAttributeViString(instrSession, "", RSFPL_ATTR_VSA_SAVE_STANDARD, standard),
                3, "Standard");
        break;
    case RSFPL_VAL_VSA_STANDARD_DELETE:
        checkErr(rsfpl_SetAttributeViString(instrSession, "", RSFPL_ATTR_VSA_DELETE_STANDARD, NULL));
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, operation), 2, "Operation");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the VSA Modulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR type/Selects the digital demodulation mode.
/// HIPAR symbolRate/Defines the symbol rate.
ViStatus _VI_FUNC rsfpl_ConfigureVSAModulationSettings(ViSession instrSession,
                                                       ViInt32   type,
                                                       ViReal64  symbolRate)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, "", RSFPL_ATTR_VSA_MODULATION_TYPE, type),
            2, "Type");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, "", RSFPL_ATTR_VSA_MODULATION_SYMBOL_RATE, symbolRate),
            3, "Symbol Rate");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the file that contains the user-defined modulation to be
/// HIFN  loaded.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fileName/File name parameter.
ViStatus _VI_FUNC rsfpl_LoadVSAModulationUserDefined(ViSession instrSession,
                                                     ViString  fileName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, "", RSFPL_ATTR_VSA_MODULATION_LOAD_USER_DEFINED, fileName),
            2, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the names of all mappings that are available for the current
/// HIFN  modulation type and order. A mapping describes the assignment of
/// HIFN  constellation points to symbols.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR arraySize/Sets the Array Size parameter.
/// HIPAR catalog/Queries the names of all mappings that are available for the current
/// HIPAR catalog/modulation type and order. A mapping describes the assignment of
/// HIPAR catalog/constellation points to symbols.
ViStatus _VI_FUNC rsfpl_QueryVSAModulationMappingCatalog(ViSession instrSession,
                                                         ViInt32   arraySize,
                                                         ViChar    catalog[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_GetAttributeViString(instrSession, "", RSFPL_ATTR_VSA_MODULATION_MAPPING_CATALOG,
            arraySize, catalog),
            3, "Catalog");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the mapping for digital demodulation. The mapping describes
/// HIFN  the assignment of constellation points to symbols.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mapping/Selects the mapping for digital demodulation. The mapping describes
/// HIPAR mapping/the assignment of constellation points to symbols.
ViStatus _VI_FUNC rsfpl_ConfigureVSAModulationMapping(ViSession instrSession,
                                                      ViString  mapping)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, "", RSFPL_ATTR_VSA_MODULATION_MAPPING, mapping),
            2, "Mapping");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the PSK Modulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR order/Together with DDEMod:PSK:NST, this command defines the demodulation
/// HIPAR order/order for PSK (see also [SENSe:]DDEMod:PSK:NSTate). Depending on the
/// HIPAR order/demodulation format and state, the following orders are available:
/// HIPAR nState/Together with DDEMod:PSK:FORMat, this command defines the demodulation
/// HIPAR nState/order for PSK
ViStatus _VI_FUNC rsfpl_ConfigureVSAModulationPSK(ViSession instrSession,
                                                  ViInt32   order,
                                                  ViInt32   nState)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, "", RSFPL_ATTR_VSA_MODULATION_PSK_FORMAT, order),
            2, "Order");

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, "", RSFPL_ATTR_VSA_MODULATION_PSK_N_STATE, nState),
            3, "N State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the QAM Modulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR order/Defines the specific demodulation order for QAM.
/// HIPAR nState/Defines the demodulation order for QAM.
ViStatus _VI_FUNC rsfpl_ConfigureVSAModulationQAM(ViSession instrSession,
                                                  ViInt32   order,
                                                  ViInt32   nState)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, "", RSFPL_ATTR_VSA_MODULATION_QAM_FORMAT, order),
            2, "Order");

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, "", RSFPL_ATTR_VSA_MODULATION_QAM_N_STATE, nState),
            3, "N State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the FSK Modulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR deviation/Selects the deviation to set.
/// HIPAR fskRefDeviation/Defines the deviation to the reference frequency for FSK modulation as
/// HIPAR fskRefDeviation/a multiple of the symbol rate.
/// HIPAR nState/Defines the demodulation of the FSK modulation scheme.
ViStatus _VI_FUNC rsfpl_ConfigureVSAModulationFSK(ViSession instrSession,
                                                  ViInt32   deviation,
                                                  ViReal64  fskRefDeviation,
                                                  ViInt32   nState)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (deviation)
    {
    case RSFPL_VAL_ABS:
        attr = RSFPL_ATTR_VSA_MODULATION_FSK_DEVIATION_ABSOLUTE;
        break;
    case RSFPL_VAL_REL:
        attr = RSFPL_ATTR_VSA_MODULATION_FSK_DEVIATION;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, deviation),
            2, "Deviation");
        break;
    }

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, attr, fskRefDeviation),
            3, "FSK Reference Deviation");

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, "", RSFPL_ATTR_VSA_MODULATION_FSK_N_STATE, nState),
            4, "N State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the specific demodulation mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR askOrder/Defines the specific demodulation mode for ASK.
/// HIPAR apskOrder/Defines the specific demodulation mode for APSK.
/// HIPAR mskOrder/Defines the specific demodulation order for MSK.
/// HIPAR qpskOrder/Defines the demodulation order for QPSK.
ViStatus _VI_FUNC rsfpl_ConfigureVSAModulationOrders(ViSession instrSession,
                                                     ViInt32   askOrder,
                                                     ViInt32   apskOrder,
                                                     ViInt32   mskOrder,
                                                     ViInt32   qpskOrder)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, "", RSFPL_ATTR_VSA_MODULATION_ASK_N_STATE, askOrder),
            2, "ASK Order");

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, "", RSFPL_ATTR_VSA_MODULATION_APSK_N_STATE, apskOrder),
            3, "APSK Order");

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, "", RSFPL_ATTR_VSA_MODULATION_MSK_FORMAT, mskOrder),
            4, "MSK Order");

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, "", RSFPL_ATTR_VSA_MODULATION_QPSK_FORMAT, qpskOrder),
            2, "QPSK Order");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the VSA Modulation Transmit Filter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR transmitFilterSelect/Selects a transmit filter and automatically switches it on.
/// HIPAR alphaBT/Determines the filter characteristic (ALPHA/BT). The resolution is
/// HIPAR alphaBT/0.01.
ViStatus _VI_FUNC rsfpl_ConfigureVSAModulationTransmitFilter(ViSession instrSession,
                                                             ViString  transmitFilterSelect,
                                                             ViReal64  alphaBT)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, "", RSFPL_ATTR_VSA_MODULATION_TRANSMIT_FILTER_SELECT, transmitFilterSelect),
            2, "Transmit Filter Select");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, "", RSFPL_ATTR_VSA_MODULATION_TRANSMIT_FILTER_ALPHA_BT, alphaBT),
            3, "Alpha BT");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects a user-defined transmit filter file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR userDefined/Selects a user-defined transmit filter file.
ViStatus _VI_FUNC rsfpl_ConfigureVSAModulationTransmitFilterUser(
                            ViSession instrSession,
                            ViString  userDefined)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, "", RSFPL_ATTR_VSA_MODULATION_TRANSMIT_FILTER_SELECT_USER_DEFINED, userDefined),
            2, "User Defined");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Use this function to switch the transmit filter off. To switch a
/// HIFN  transmit filter on, use the [SENSe:]DDEMod:TFILter:NAME command.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_ConfigureVSAModulationTransmitFilterOff(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsfpl_SetAttributeViString(instrSession, "", RSFPL_ATTR_VSA_MODULATION_TRANSMIT_FILTER_OFF, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the VSA modulation filter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Defines whether the input signal that is evaluated is filtered by the
/// HIPAR enabled/measurement filter. This command has no effect on the transmit filter.
/// HIPAR alphaBT/Determines the filter characteristic (ALPHA/BT).
ViStatus _VI_FUNC rsfpl_ConfigureVSAModulationFilter(ViSession instrSession,
                                                     ViBoolean enabled,
                                                     ViReal64  alphaBT)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, "", RSFPL_ATTR_VSA_MODULATION_FILTER_ENABLED, enabled),
            2, "Enabled");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, "", RSFPL_ATTR_VSA_MODULATION_FILTER_ALPHA_BT, alphaBT),
            3, "Alpha BT");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Specifies whether the signal is bursted or continuous.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR type/Specifies whether the signal is bursted or continuous.
ViStatus _VI_FUNC rsfpl_ConfigureVSASignalStructureType(ViSession instrSession,
                                                        ViInt32   type)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, "", RSFPL_ATTR_VSA_SIGNAL_STRUCTURE_TYPE, type),
            2, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the burst.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR minLength/Defines the minimum useful length of a burst. Only those bursts will
/// HIPAR minLength/be recognized that exceed this length. The default unit is symbols.
/// HIPAR maxLength/Defines the maximum length of a burst. Only those bursts will be
/// HIPAR maxLength/recognized that fall below this length. The default unit is symbols.
/// HIPAR runIn/Defines the length of the rising burst edge which is not considered
/// HIPAR runIn/when evaluating the result. The default unit is symbols.
/// HIPAR runOut/Defines the length of the falling burst edge which is not considered
/// HIPAR runOut/when evaluating the result. The default unit is symbols.
ViStatus _VI_FUNC rsfpl_ConfigureVSASignalStructureBurst(ViSession instrSession,
                                                         ViInt32   minLength,
                                                         ViInt32   maxLength,
                                                         ViInt32   runIn,
                                                         ViInt32   runOut)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, "", RSFPL_ATTR_VSA_SIGNAL_STRUCTURE_BURST_MIN_LENGTH, minLength),
            2, "Min Length");

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, "", RSFPL_ATTR_VSA_SIGNAL_STRUCTURE_BURST_MAX_LENGTH, maxLength),
            3, "Max Length");

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, "", RSFPL_ATTR_VSA_SIGNAL_STRUCTURE_RUN_IN, runIn),
            4, "Run In");

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, "", RSFPL_ATTR_VSA_SIGNAL_STRUCTURE_RUN_OUT, runOut),
            5, "Run Out");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Specifies whether the signal contains a pattern or not.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Specifies whether the signal contains a pattern or not.
ViStatus _VI_FUNC rsfpl_ConfigureVSASignalStructurePatternEnabled(
                            ViSession instrSession,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, "", RSFPL_ATTR_VSA_SIGNAL_STRUCTURE_PATTERN_ENABLED, enabled),
            2, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Reads the names of all patterns stored on the hard disk. The file
/// HIFN  names are returned as a comma-separated list of strings, one for each
/// HIFN  file name (without the file extension).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR patterns/Selects the type of patterns to be read.
ViStatus _VI_FUNC rsfpl_ConfigureVSASignalStructurePatternsCatalog(
                            ViSession instrSession,
                            ViInt32   patterns,
                            ViChar    catalog[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, patterns, 1, 2),
            2, "Window");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:DDEM:SEAR:SYNC:CAT? %s", pattArr[patterns]);
    checkErr(RsCore_QueryViString(instrSession, cmd, catalog));

    checkErr(rsfpl_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the pattern offset.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/This control (de)activates the pattern offset.
/// HIPAR offset/Defines a number of symbols which are ignored before the comparison
/// HIPAR offset/with the pattern starts.
ViStatus _VI_FUNC rsfpl_ConfigureVSASignalStructurePatternOffset(
                            ViSession instrSession,
                            ViBoolean enabled,
                            ViInt32   offset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, "", RSFPL_ATTR_VSA_SIGNAL_STRUCTURE_PATTERN_OFFSET_ENABLED, enabled),
            2, "Enabled");

    if (enabled == VI_TRUE)
    {
        viCheckParm(rsfpl_SetAttributeViInt32(instrSession, "", RSFPL_ATTR_VSA_SIGNAL_STRUCTURE_PATTERN_OFFSET, offset),
                3, "Offset");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the Modulation Settings for Pattern.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR sameAsDataSymbols/Determines whether the pattern uses a different modulation type than
/// HIPAR sameAsDataSymbols/the data symbols.
/// HIPAR type/Configures the frame format.
ViStatus _VI_FUNC rsfpl_ConfigureVSAFramePattern(ViSession instrSession,
                                                 ViBoolean sameAsDataSymbols,
                                                 ViInt32   type)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, "", RSFPL_ATTR_VSA_FRAME_PATTERN_ENABLED, sameAsDataSymbols),
            2, "Same As Data Symbols");

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, "", RSFPL_ATTR_VSA_FRAME_FORMAT, type),
            3, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the names of all mappings that are available for the pattern
/// HIFN  for the current modulation type and order. A mapping describes the
/// HIFN  assignment of constellation points to symbols.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Pass the number of bytes in the string you specify for the catalog
/// HIPAR bufferSize/list parameter.
/// HIPAR mappingList/Queries the names of all mappings that are available for the pattern
/// HIPAR mappingList/for the current modulation type and order. A mapping describes the
/// HIPAR mappingList/assignment of constellation points to symbols.
ViStatus _VI_FUNC rsfpl_QueryVSAFrameMappingCatalog(ViSession instrSession,
                                                    ViInt32   bufferSize,
                                                    ViChar    mappingList[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_GetAttributeViString(instrSession, "", RSFPL_ATTR_VSA_FRAME_MAPPING_CATALOG,
            bufferSize, mappingList),
            2, "Mapping List");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Specifies an xml file for a user-defined frame structure
/// HIFN  configuration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR config/Specifies an xml file for a user-defined frame structure
/// HIPAR config/configuration.
ViStatus _VI_FUNC rsfpl_VSALoadFrameConfig(ViSession instrSession,
                                           ViString  config)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, "", RSFPL_ATTR_VSA_FRAME_EDIT, config),
            2, "Config");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the mapping for pattern demodulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frameMapping/Selects the mapping for pattern demodulation.
ViStatus _VI_FUNC rsfpl_ConfigureVSAFrameMappingSelect(ViSession instrSession,
                                                       ViString  frameMapping)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, "", RSFPL_ATTR_VSA_FRAME_MAPPING, frameMapping),
            2, "Frame Mapping");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Loads a user-defined frame structure configuration to be used by the
/// HIFN  measurement from an xml file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fileName/File name parameter.
ViStatus _VI_FUNC rsfpl_VSALoadFrameStructure(ViSession instrSession,
                                              ViString  fileName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, "", RSFPL_ATTR_VSA_FRAME_LOAD, fileName),
            2, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Stores the current frame structure configuration to the specified
/// HIFN  file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fileName/File name parameter.
ViStatus _VI_FUNC rsfpl_VSASaveFrameStructure(ViSession instrSession,
                                              ViString  fileName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, "", RSFPL_ATTR_VSA_FRAME_SAVE, fileName),
            2, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the frame structure for a previously loaded file. For each
/// HIFN  subframe, all parameters must be defined.
/// HIFN     This command is only available if the additional Multi-Modulation
/// HIFN  Analysis option (R&S&nbsp;FPL1000-K70M) is installed.
/// HIFN     Note that the file must be loaded for editing before the structure can
/// HIFN  be defined using this command (see [SENSe:]DDEMod:PATTern:FRAMe:EDIT).
/// HIFN  Loading the file for use in the current measurement is not sufficient
/// HIFN  (see [SENSe:]DDEMod:PATTern:FRAMe:LOAD). Therefore, you can edit a
/// HIFN  frame structure while simultaneously performing a measurement with
/// HIFN  another frame structure configuration.
/// HIFN     The configuration is only stored by a subsequent
/// HIFN  [SENSe:]DDEMod:PATTern:FRAMe:EDIT:SAVE command.
/// HIFN     The modulation for the 'previous frame' and 'next frame' are defined
/// HIFN  by separate commands (see [SENS:]DDEM:PATT:FRAM:PREV:... and
/// HIFN  [SENS:]DDEM:PATT:FRAM:NEXT:...).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR structure/Sets the VSA Frame structure.
ViStatus _VI_FUNC rsfpl_VSAFrameStructureEdit(ViSession instrSession,
                                              ViString  structure)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:DDEM:PATT:FRAM:EDIT:STR %s", structure);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsfpl_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Determines whether the frame structure of the signal is configured in
/// HIFN  reference to the result range or user-defined.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mode/Determines whether the frame structure of the signal is configured in
/// HIPAR mode/reference to the result range or user-defined.
ViStatus _VI_FUNC rsfpl_ConfigureVSAFrameMode(ViSession instrSession,
                                              ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, "", RSFPL_ATTR_VSA_FRAME_MODE, mode),
            2, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the file that contains a user-defined modulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fileName/Selects the file that contains a user-defined modulation.
ViStatus _VI_FUNC rsfpl_ConfigureVSAFrameUserFile(ViSession instrSession,
                                                  ViString  fileName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, "", RSFPL_ATTR_VSA_FRAME_USER_DEFINED_MODULATION, fileName),
            2, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This command defines the demodulation order for APSK for the pattern
/// HIFN  (see also RSFPL_ATTR_VSA_FRAME_FORMAT).
/// HIFN     Depending on the demodulation state, the following orders are
/// HIFN  available:
/// HIFN     16 - 16APSK
/// HIFN     32 - 32APSK
/// HIFN     This command is only available if the additional Multi-Modulation
/// HIFN  Analysis option (R&S FPL1000-K70M) is installed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR nState/Defines the demodulation order for APSK for the pattern (see also
/// HIPAR nState/RSFPL_ATTR_VSA_FRAME_FORMAT).
ViStatus _VI_FUNC rsfpl_ConfigureVSAFrameAPSKNState(ViSession instrSession,
                                                    ViInt32   nState)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_VSA_FRAME_APSK_N_STATE, nState),
            2, "N State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This command defines the demodulation order for ASK for the pattern
/// HIFN  (see also RSFPL_ATTR_VSA_FRAME_FORMAT).
/// HIFN     Depending on the demodulation state, the following orders are
/// HIFN  available:
/// HIFN     2 - 2ASK
/// HIFN     4 - 4ASK
/// HIFN     This command is only available if the additional Multi-Modulation
/// HIFN  Analysis option (R&S FPL1000-K70M) is installed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR nState/Defines the demodulation order for ASK for the pattern (see also
/// HIPAR nState/RSFPL_ATTR_VSA_FRAME_FORMAT).
ViStatus _VI_FUNC rsfpl_ConfigureVSAFrameASKNState(ViSession instrSession,
                                                   ViInt32   nState)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_VSA_FRAME_ASK_N_STATE, nState),
            2, "N State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the Frame PSK Modulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR format/Together with DDEMod:PATT:PSK:NST, this command defines the
/// HIPAR format/demodulation order for PSK for the pattern (see also
/// HIPAR format/[SENSe:]DDEMod:PATTern:PSK:NSTate).
/// HIPAR nState/Together with DDEMod:PATT:PSK:FORMat, this control defines the
/// HIPAR nState/demodulation order for PSK for the pattern (see also
/// HIPAR nState/[SENSe:]DDEMod:PATTern:PSK:FORMat).
ViStatus _VI_FUNC rsfpl_ConfigureVSAFramePSK(ViSession instrSession,
                                             ViInt32   format,
                                             ViInt32   nState)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, "", RSFPL_ATTR_VSA_FRAME_PSK_FORMAT, format),
            2, "Format");

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, "", RSFPL_ATTR_VSA_FRAME_PSK_N_STATE, nState),
            3, "N State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the Frame QAM Modulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR format/This control defines the specific demodulation order for QAM for the
/// HIPAR format/pattern.
/// HIPAR nState/This control defines the demodulation order for QAM for the pattern.
ViStatus _VI_FUNC rsfpl_ConfigureVSAFrameQAM(ViSession instrSession,
                                             ViInt32   format,
                                             ViInt32   nState)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, "", RSFPL_ATTR_VSA_FRAME_QAM_FORMAT, format),
            2, "Format");

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, "", RSFPL_ATTR_VSA_FRAME_QAM_N_STATE, nState),
            3, "N State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function defines the demodulation order for QPSK for the pattern.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR format/This control defines the demodulation order for QPSK for the pattern.
ViStatus _VI_FUNC rsfpl_ConfigureVSAFrameQPSK(ViSession instrSession,
                                              ViInt32   format)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, "", RSFPL_ATTR_VSA_FRAME_QPSK_FORMAT, format),
            2, "Format");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Determines which boosting is used to demodulate the frame next or
/// HIFN  previous to the last configured subframe.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR type/Defines the type of the operation.
/// HIPAR boosting/Determines which boosting is used to demodulate the frame next or
/// HIPAR boosting/previous to the last configured subframe.
ViStatus _VI_FUNC rsfpl_ConfigureVSAFrameBoosting(ViSession instrSession,
                                                  ViInt32   type,
                                                  ViReal64  boosting)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    switch (type)
    {
    case RSFPL_VAL_VSA_FRAME_TYPE_NEXT:
        viCheckParm(rsfpl_SetAttributeViReal64(instrSession, "", RSFPL_ATTR_VSA_FRAME_NEXT_BOOSTING, boosting),
                3, "Boosting");
        break;
    case RSFPL_VAL_VSA_FRAME_TYPE_PREV:
        viCheckParm(rsfpl_SetAttributeViReal64(instrSession, "", RSFPL_ATTR_VSA_FRAME_PREVIOUS_BOOSTING, boosting),
                3, "Boosting");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, type), 2, "Type");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Determines which modulation is used to demodulate the frame next or
/// HIFN  previous to the last configured subframe.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR type/Defines the type of the operation.
/// HIPAR modulation/Determines which modulation is used to demodulate the frame next or
/// HIPAR modulation/previous to the last configured subframe.
ViStatus _VI_FUNC rsfpl_ConfigureVSAFrameModulation(ViSession instrSession,
                                                    ViInt32   type,
                                                    ViInt32   modulation)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    switch (type)
    {
    case RSFPL_VAL_VSA_FRAME_TYPE_NEXT:
        viCheckParm(rsfpl_SetAttributeViInt32(instrSession, "", RSFPL_ATTR_VSA_FRAME_NEXT_MODULATION, modulation),
                3, "Modulation");
        break;
    case RSFPL_VAL_VSA_FRAME_TYPE_PREV:
        viCheckParm(rsfpl_SetAttributeViInt32(instrSession, "", RSFPL_ATTR_VSA_FRAME_PREVIOUS_MODULATION, modulation),
                3, "Modulation");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, type), 2, "Type");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the description for the frame structure in a previously loaded
/// HIFN  file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR description/Defines the description for the frame structure in a previously loaded
/// HIPAR description/file.
ViStatus _VI_FUNC rsfpl_ConfigureVSAFrameDescription(ViSession instrSession,
                                                     ViString  description)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, "", RSFPL_ATTR_VSA_FRAME_DESCRIPTION, description),
            2, "Description");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the start sample of the current result range
/// HIFN  within the capture buffer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR startSamples/This control queries the start sample of the current result range
/// HIPAR startSamples/within the capture buffer.
ViStatus _VI_FUNC rsfpl_QueryVSAFrameStartSample(ViSession instrSession,
                                                 ViReal64* startSamples)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_GetAttributeViReal64(instrSession, "", RSFPL_ATTR_VSA_MEASUREMENT_START_SAMPLES, startSamples),
            2, "Start Samples");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the VSA Known Data
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/This control selects the Known Data state. The use of known data is a
/// HIPAR enabled/prerequisite for the BER measurement and can also be used for the fine
/// HIPAR enabled/sync.
/// HIPAR source/This control selects the Known Data source.
ViStatus _VI_FUNC rsfpl_ConfigureVSAKnownData(ViSession instrSession,
                                              ViBoolean enabled,
                                              ViInt32   source)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, "", RSFPL_ATTR_VSA_KNOWN_DATA_ENABLED, enabled),
            2, "Enabled");

    if (enabled == VI_TRUE)
    {
        viCheckParm(rsfpl_SetAttributeViInt32(instrSession, "", RSFPL_ATTR_VSA_KNOWN_DATA_SOURCE, source),
                3, "Source");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function selects the Known Data file. Note that known data must
/// HIFN  be activated ([SENSe:]DDEMod:KDATa:STATe) before you can select a file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fileName/This control selects the Known Data file. Note that known data must be
/// HIPAR fileName/activated ([SENSe:]DDEMod:KDATa:STATe) before you can select a file.
ViStatus _VI_FUNC rsfpl_VSAKnownDataLoadDataFile(ViSession instrSession,
                                                 ViString  fileName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, "", RSFPL_ATTR_VSA_KNOWN_DATA_LOAD_DATA_FILE, fileName),
            2, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the type of the used PRBS model. The type of the model defines
/// HIFN  the degree, complexity and number of terms in the polynomial model. If
/// HIFN  the PRBS data is generated in accordance with the ITU-T standard, no
/// HIFN  further settings are required.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR type/Defines the type of the used PRBS model. The type of the model defines
/// HIPAR type/the degree, complexity and number of terms in the polynomial model. If
/// HIPAR type/the PRBS data is generated in accordance with the ITU-T standard, no
/// HIPAR type/further settings are required.
ViStatus _VI_FUNC rsfpl_ConfigureVSAKnownDataPRBSType(ViSession instrSession,
                                                      ViInt32   type)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, "", RSFPL_ATTR_VSA_KNOWN_DATA_PRBS_TYPE, type),
            2, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the PRBS generator polynomial.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR polynomialAuto/Determines the coefficients of the polynomial and thus the feedback
/// HIPAR polynomialAuto/positions for the PRBS algorithm.
/// HIPAR polynomialOrder/Determines the coefficients of the polynomial and thus the feedback
/// HIPAR polynomialOrder/positions for the PRBS algorithm.
ViStatus _VI_FUNC rsfpl_ConfigureVSAKnownDataPRBSPolynomial(ViSession instrSession,
                                                            ViBoolean polynomialAuto,
                                                            ViString  polynomialOrder)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, "", RSFPL_ATTR_VSA_KNOWN_DATA_PRBS_POLYNOMIAL_AUTO, polynomialAuto),
            2, "Polynomial Auto");

    if (polynomialAuto == VI_FALSE)
    {
        viCheckParm(rsfpl_SetAttributeViString(instrSession, "", RSFPL_ATTR_VSA_KNOWN_DATA_PRBS_POLYNOMIAL_ORDER, polynomialOrder),
                3, "Polynomial Order");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Determines how the feedback value is calculated.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR feedbackAuto/Determines how the feedback value is calculated.
/// HIPAR feedback/Determines how the feedback bit is calculated.
ViStatus _VI_FUNC rsfpl_ConfigureVSAKnownDataPRBSNegateFeedbackPath(
                            ViSession instrSession,
                            ViBoolean feedbackAuto,
                            ViBoolean feedback)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, "", RSFPL_ATTR_VSA_KNOWN_DATA_FEEDBACK_AUTO, feedbackAuto),
            2, "Feedback Auto");

    if (feedbackAuto == VI_FALSE)
    {
        viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, "", RSFPL_ATTR_VSA_KNOWN_DATA_FEEDBACK_PRBS, feedback),
                3, "Feedback");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the PRBS Pattern.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR patternAuto/If enabled, the R&S FPL1000 VSA application assumes the pattern is
/// HIPAR patternAuto/part of the PRBS sequence.
/// HIPAR patternEnabled/Determines whether the pattern symbols are part of the PRBS sequence
/// HIPAR patternEnabled/and are thus treated as data symbols.
ViStatus _VI_FUNC rsfpl_ConfigureVSAKnownDataPRBSPattern(ViSession instrSession,
                                                         ViBoolean patternAuto,
                                                         ViBoolean patternEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, "", RSFPL_ATTR_VSA_KNOWN_DATA_PRBS_PATTERN_AUTO, patternAuto),
            2, "Pattern Auto");

    if (patternAuto == VI_FALSE)
    {
        viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, "", RSFPL_ATTR_VSA_KNOWN_DATA_PRBS_PATTERN_ENABLED, patternEnabled),
                3, "Pattern Enabled");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the Signal Capture.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR signalCaptureAuto/If enabled, the capture length is automatically adapted as required
/// HIPAR signalCaptureAuto/according to the current result length, burst and pattern search
/// HIPAR signalCaptureAuto/settings, and network-specific characteristics (e.g. burst and frame
/// HIPAR signalCaptureAuto/structures).
/// HIPAR sampleRate/Defines the number of samples that are captured per symbol, i.e. the
/// HIPAR sampleRate/factor by which the symbol rate is multiplied to obtain the sample
/// HIPAR sampleRate/rate. This parameter also affects the demodulation bandwidth and thus
/// HIPAR sampleRate/the usable I/Q bandwidth.
ViStatus _VI_FUNC rsfpl_ConfigureVSASignalCapture(ViSession instrSession,
                                                  ViBoolean signalCaptureAuto,
                                                  ViInt32   sampleRate)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, "", RSFPL_ATTR_VSA_SIGNAL_CAPTURE_RECORD_LENGTH_AUTO, signalCaptureAuto),
            2, "Signal Capture Auto");

    if (signalCaptureAuto == VI_FALSE)
    {
        viCheckParm(rsfpl_SetAttributeViInt32(instrSession, "", RSFPL_ATTR_VSA_SIGNAL_CAPTURE_SAMPLE_RATE, sampleRate),
                3, "Sample Rate");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function defines or queries the capture length for further
/// HIFN  processing, e.g. for burst search.
/// HIFN     Note that the maximum capture length depends on the sample rate for
/// HIFN  signal capture (see [SENSe:]DDEMod:PRATe).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR length/This control defines or queries the capture length for further
/// HIPAR length/processing, e.g. for burst search.
ViStatus _VI_FUNC rsfpl_ConfigureVSASignalCaptureRecordLength(
                            ViSession instrSession,
                            ViReal64  length)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, "", RSFPL_ATTR_VSA_SIGNAL_CAPTURE_RECORD_LENGTH, length),
            2, "Length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function defines the capture length for further processing, e.g.
/// HIFN  for burst search, in symbols.
/// HIFN     Note that the maximum record length depends on the sample rate for
/// HIFN  signal capture (see [SENSe:]DDEMod:PRATe). The maximum record length
/// HIFN  (in symbols) can be calculated as:
/// HIFN     RecordlengthMAX = 256000/ <points per symbol>
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR lengthSymbols/This control defines the capture length for further processing, e.g.
/// HIPAR lengthSymbols/for burst search, in symbols.
ViStatus _VI_FUNC rsfpl_ConfigureVSASignalCaptureRecordLengthSymbols(
                            ViSession instrSession,
                            ViInt32   lengthSymbols)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, "", RSFPL_ATTR_VSA_SIGNAL_CAPTURE_RECORD_LENGTH_SYMBOLS, lengthSymbols),
            2, "Length Symbols");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the bandwidth of the resampling filter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR usableIQBandwidth/This control queries the bandwidth of the resampling filter.
ViStatus _VI_FUNC rsfpl_QuerySignalCaptureIQBandwidth(ViSession instrSession,
                                                      ViReal64* usableIQBandwidth)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_GetAttributeViReal64(instrSession, "", RSFPL_ATTR_QUERY_SIGNAL_CAPTURE_IQ_BANDWIDTH, usableIQBandwidth),
            2, "Usable IQ Bandwidth");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function links the burst search to the type of signal. When a
/// HIFN  signal is marked as bursted, burst search is switched on automatically.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR burstSearch/This control links the burst search to the type of signal. When a
/// HIPAR burstSearch/signal is marked as bursted, burst search is switched on automatically.
ViStatus _VI_FUNC rsfpl_ConfigureVSABurstSearchAuto(ViSession instrSession,
                                                    ViInt32   burstSearch)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, "", RSFPL_ATTR_VSA_BURST_SEARCH_AUTO, burstSearch),
            2, "Burst Search");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the search for a signal burst.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/This control switches the search for a signal burst on or off.
/// HIPAR mode/This control sets the vector analyzer so that a measurement is
/// HIPAR mode/performed only if a burst is found.
ViStatus _VI_FUNC rsfpl_ConfigureVSABurstSearch(ViSession instrSession,
                                                ViBoolean enabled,
                                                ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, "", RSFPL_ATTR_VSA_BURST_SEARCH_ENABLED, enabled),
            2, "Enabled");

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, "", RSFPL_ATTR_VSA_BURST_SEARCH_MEASURING, mode),
            3, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configure the search tolerance and the min gap length
/// HIFN  and their default values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR autoConfiguration/This control sets the search tolerance and the min gap length to their
/// HIPAR autoConfiguration/default values.
/// HIPAR tolerance/This control controls burst search tolerance.
/// HIPAR minGapLength/This control defines the minimum time between two bursts. A minimum
/// HIPAR minGapLength/time with decreased level must occur between two bursts. The default
/// HIPAR minGapLength/unit is symbol. The value can also be given in seconds.
ViStatus _VI_FUNC rsfpl_ConfigureVSABurstSearchAutoConfiguration(
                            ViSession instrSession,
                            ViBoolean autoConfiguration,
                            ViInt32   tolerance,
                            ViInt32   minGapLength)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, "", RSFPL_ATTR_VSA_BURST_SEARCH_CONFIGURE_AUTO, autoConfiguration),
            2, "Auto Configuration");

    if (autoConfiguration == VI_FALSE)
    {
        viCheckParm(rsfpl_SetAttributeViInt32(instrSession, "", RSFPL_ATTR_VSA_BURST_SEARCH_TOLERANCE, tolerance),
                3, "Tolerance");

        viCheckParm(rsfpl_SetAttributeViInt32(instrSession, "", RSFPL_ATTR_VSA_BURST_SEARCH_MIN_GAP_LENGTH, minGapLength),
                4, "Min Gap Length");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function links the pattern search to the type of signal. When a
/// HIFN  signal is marked as patterned, pattern search is switched on
/// HIFN  automatically.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR patternSearchAuto/This control links the pattern search to the type of signal. When a
/// HIPAR patternSearchAuto/signal is marked as patterned, pattern search is switched on
/// HIPAR patternSearchAuto/automatically.
ViStatus _VI_FUNC rsfpl_ConfigureVSAPatternSearchAuto(ViSession instrSession,
                                                      ViInt32   patternSearchAuto)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, "", RSFPL_ATTR_VSA_PATTERN_SEARCH_AUTO, patternSearchAuto),
            2, "Pattern Search Auto");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the search for a signal burst.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/This control switches the search for a sync sequence on or off.
/// HIPAR searchMode/This control sets the vector analyzer so that the measurement is
/// HIPAR searchMode/performed only if the measurement was synchronous to the selected sync
/// HIPAR searchMode/pattern.
/// HIPAR pattern/This control selects a predefined sync pattern file.
ViStatus _VI_FUNC rsfpl_ConfigureVSAPatternSearch(ViSession instrSession,
                                                  ViBoolean enabled,
                                                  ViInt32   searchMode,
                                                  ViString  pattern)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, "", RSFPL_ATTR_VSA_PATTERN_SEARCH_ENABLED, enabled),
            2, "Enabled");

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, "", RSFPL_ATTR_VSA_PATTERN_SEARCH_SYNC_MODE, searchMode),
            3, "Search Mode");

    viCheckParm(rsfpl_SetAttributeViString(instrSession, "", RSFPL_ATTR_VSA_PATTERN_SEARCH_SELECT_FILE, pattern),
            4, "Pattern");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures IQ Correlation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR autoThreshold/This control sets the I/Q correlation threshold to its default value.
/// HIPAR level/This control sets the I/Q correlation threshold for pattern matching
/// HIPAR level/in percent. A high level means stricter matching.
ViStatus _VI_FUNC rsfpl_ConfigureVSAPatternSearchIQCorrelation(
                            ViSession instrSession,
                            ViBoolean autoThreshold,
                            ViReal64  level)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, "", RSFPL_ATTR_VSA_PATTERN_SEARCH_IQ_CORRELATION_THRESHOLD, autoThreshold),
            2, "Auto Threshold");

    if (autoThreshold == VI_FALSE)
    {
        viCheckParm(rsfpl_SetAttributeViReal64(instrSession, "", RSFPL_ATTR_VSA_PATTERN_SEARCH_IQ_CORRELATION_LEVEL, level),
                3, "Level");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function performs the selected operation with channel table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR operation/This control selects the type of memory operation to be performed.
/// HIPAR patternName/This control selects a predefined channel table file.
ViStatus _VI_FUNC rsfpl_ConfigureVSAPatternOperations(ViSession instrSession,
                                                      ViInt32   operation,
                                                      ViString  patternName)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (operation)
    {
    case RSFPL_VAL_SYNC_PATT_ADD:
        attr = RSFPL_ATTR_VSA_PATTERN_ADD;
        break;
    case RSFPL_VAL_SYNC_PATT_SELECT:
        attr = RSFPL_ATTR_VSA_PATTERN_NAME;
        break;
    case RSFPL_VAL_SYNC_PATT_DELETE:
        checkErr(rsfpl_SetAttributeViString(instrSession, "", RSFPL_ATTR_VSA_PATTERN_DELETE, NULL));
        break;
    case RSFPL_VAL_SYNC_PATT_REMOVE:
        checkErr(rsfpl_SetAttributeViString(instrSession, "", RSFPL_ATTR_VSA_PATTERN_REMOVE, NULL));
        break;
    case RSFPL_VAL_SYNC_PATT_COPY:
        attr = RSFPL_ATTR_VSA_PATTERN_COPY;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, operation), 2, "Operation");
        break;
    }

    viCheckParm(rsfpl_SetAttributeViString(instrSession, "", attr, patternName),
            3, "Pattern Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the selected pattern.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR patternName/This control selects a predefined channel table file.
/// HIPAR description/Sets a text to explain the pattern. The text is displayed only in the
/// HIPAR description/selection menu (manual control). This text is supposed to be short and
/// HIPAR description/concise. Detailed information about the pattern is given in the
/// HIPAR description/comment.
/// HIPAR comment/Defines a comment to a sync pattern.
/// HIPAR degreeofModulation/Sets the degree of modulation (number of permitted states).
/// HIPAR patternData/Sets a sync sequence for the sync pattern. Four values represent a
/// HIPAR patternData/symbol. The value range of a symbol depends on the degree of
/// HIPAR patternData/modulation. FFFF entries are interpreted as "Don't Care Bits".
ViStatus _VI_FUNC rsfpl_ConfigureVSAPatternSettings(ViSession instrSession,
                                                    ViString  patternName,
                                                    ViString  description,
                                                    ViString  comment,
                                                    ViInt32   degreeofModulation,
                                                    ViString  patternData)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, "", RSFPL_ATTR_VSA_PATTERN_NAME, patternName),
            2, "Pattern Name");

    viCheckParm(rsfpl_SetAttributeViString(instrSession, "", RSFPL_ATTR_VSA_PATTERN_DESCRIPTION, description),
            3, "Description");

    viCheckParm(rsfpl_SetAttributeViString(instrSession, "", RSFPL_ATTR_VSA_PATTERN_COMMENT, comment),
            4, "Comment");

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, "", RSFPL_ATTR_VSA_PATTERN_N_STATE, degreeofModulation),
            5, "Degree Of Modulation");

    viCheckParm(rsfpl_SetAttributeViString(instrSession, "", RSFPL_ATTR_VSA_PATTERN_DATA, patternData),
            6, "Pattern Data");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Determines how the result length is defined for multi-modulation
/// HIFN  analysis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR toFrameStructureFile/Determines how the result length is defined for multi-modulation
/// HIPAR toFrameStructureFile/analysis.
ViStatus _VI_FUNC rsfpl_ConfigureVSAResultRangeAccordingToFrameStructureFile(
                            ViSession instrSession,
                            ViBoolean toFrameStructureFile)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, "", RSFPL_ATTR_VSA_RESULT_RANGE_ACCORDING_TO_FRAME_STRUCTURE_FILE, toFrameStructureFile),
            2, "To Frame Structure File");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  The function determines the number of displayed symbols (result
/// HIFN  length).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR resultLength/The control determines the number of displayed symbols (result
/// HIPAR resultLength/length).
ViStatus _VI_FUNC rsfpl_ConfigureVSAResultLength(ViSession instrSession,
                                                 ViInt32   resultLength)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, "", RSFPL_ATTR_VSA_RESULT_RANGE_RESULT_LENGTH, resultLength),
            2, "Result Length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function defines the reference point for the display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR trace/Sets the trace.
/// HIPAR reference/This control defines the reference point for the display.
ViStatus _VI_FUNC rsfpl_ConfigureVSAResultRangeReference(ViSession instrSession,
                                                         ViInt32   window,
                                                         ViInt32   trace,
                                                         ViInt32   reference)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_trace[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_VSA_RESULT_RANGE_REFERENCE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap));
    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_VSA_RESULT_RANGE_REFERENCE,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCap_trace));
    strncat(repCap, ",", strlen(repCap) - RS_MAX_MESSAGE_BUF_SIZE - 1);
    strncat(repCap, repCap_trace, strlen(repCap) - RS_MAX_MESSAGE_BUF_SIZE - 1);

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_VSA_RESULT_RANGE_REFERENCE, reference),
            4, "Reference");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the display alignemnt.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR trace/Sets the trace.
/// HIPAR alignment/This control defines where the reference point is to appear in the
/// HIPAR alignment/result range.
/// HIPAR offset/This control shifts the display range (relative to the reference time)
/// HIPAR offset/by the number of given symbols. The resolution is 1 symbol. A value >0
/// HIPAR offset/results in a shift towards the right, and a value <0 results in a shift
/// HIPAR offset/towards the left.
ViStatus _VI_FUNC rsfpl_ConfigureVSADisplayAlignment(ViSession instrSession,
                                                     ViInt32   window,
                                                     ViInt32   trace,
                                                     ViInt32   alignment,
                                                     ViInt32   offset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_trace[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_VSA_RESULT_RANGE_ALIGNMENT,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap));
    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_VSA_RESULT_RANGE_ALIGNMENT,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCap_trace));
    strncat(repCap, ",", strlen(repCap) - RS_MAX_MESSAGE_BUF_SIZE - 1);
    strncat(repCap, repCap_trace, strlen(repCap) - RS_MAX_MESSAGE_BUF_SIZE - 1);

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_VSA_RESULT_RANGE_ALIGNMENT, alignment),
            4, "Alignment");

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_VSA_RESULT_RANGE_ALIGNMENT_OFFSET, offset),
            5, "Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function defines an offset to numbering of the symbols (Except
/// HIFN  capture buffer).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR trace/Sets the trace.
/// HIPAR symbolNumber/This control defines an offset to numbering of the symbols (Except
/// HIPAR symbolNumber/capture buffer).
ViStatus _VI_FUNC rsfpl_ConfigureVSASymbolNumberAtReference(ViSession instrSession,
                                                            ViInt32   window,
                                                            ViInt32   trace,
                                                            ViInt32   symbolNumber)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_trace[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_VSA_RESULT_RANGE_SYMBOL_NUMBER_AT_REFERENCE_START,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap));
    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_VSA_RESULT_RANGE_SYMBOL_NUMBER_AT_REFERENCE_START,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCap_trace));
    strncat(repCap, ",", strlen(repCap) - RS_MAX_MESSAGE_BUF_SIZE - 1);
    strncat(repCap, repCap_trace, strlen(repCap) - RS_MAX_MESSAGE_BUF_SIZE - 1);

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_VSA_RESULT_RANGE_SYMBOL_NUMBER_AT_REFERENCE_START, symbolNumber),
            4, "Symbol Number");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function restricts the evaluation range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/This control restricts the evaluation range.
/// HIPAR start/Defines the start value for the evaluation range.
/// HIPAR stop/Defines the stop value for the evaluation range.
ViStatus _VI_FUNC rsfpl_ConfigureVSAEvaluationRange(ViSession instrSession,
                                                    ViBoolean enabled,
                                                    ViInt32   start,
                                                    ViInt32   stop)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, "SS1", RSFPL_ATTR_VSA_EVALUATION_RANGE_ENABLED, enabled),
            2, "Enabled");

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, "SS2", RSFPL_ATTR_VSA_EVALUATION_RANGE_ENABLED, enabled),
            2, "Enabled");

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, "SS1", RSFPL_ATTR_VSA_EVALUATION_RANGE, start),
            3, "Start");

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, "SS2", RSFPL_ATTR_VSA_EVALUATION_RANGE, stop),
            4, "Stop");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the result range to be displayed after a single sweep (e.g. a
/// HIFN  burst number).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR resultRange/Sets the result range to be displayed after a single sweep (e.g. a
/// HIPAR resultRange/burst number).
ViStatus _VI_FUNC rsfpl_ConfigureVSAResultRangeNumber(ViSession instrSession,
                                                      ViInt32   resultRange)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, "", RSFPL_ATTR_VSA_SELECT_RESULT_RANGE, resultRange),
            2, "Result Range");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the Result Range PPS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR mode/Defines the number of display points that are displayed per symbol
/// HIPAR mode/automatically, i.e. according to [SENSe:]DDEMod:PRATe.
/// HIPAR pps/This control determines the number of points to be displayed per
/// HIPAR pps/symbol if manual mode is selected.
ViStatus _VI_FUNC rsfpl_ConfigureVSAResultRangePPS(ViSession instrSession,
                                                   ViInt32   window,
                                                   ViInt32   mode,
                                                   ViInt32   pps)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_VSA_RESULT_RANGE_PPS_MODE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_VSA_RESULT_RANGE_PPS_MODE, mode),
            3, "Mode");

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_VSA_RESULT_RANGE_PPS, pps),
            4, "PPS");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This attribute enables the display of the decision instants (time when
/// HIFN  the signals occurred) as dots on the trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR trace/Sets the trace.
/// HIPAR highlightSymbols/This control enables the display of the decision instants (time when
/// HIPAR highlightSymbols/the signals occurred) as dots on the trace.
ViStatus _VI_FUNC rsfpl_ConfigureVSAResultRangeHighlightSymbols(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   trace,
                            ViBoolean highlightSymbols)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_trace[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_VSA_RESULT_RANGE_HIGHLIGHT_SYMBOLS,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap));
    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_VSA_RESULT_RANGE_HIGHLIGHT_SYMBOLS,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCap_trace));
    strncat(repCap, ",", strlen(repCap) - RS_MAX_MESSAGE_BUF_SIZE - 1);
    strncat(repCap, repCap_trace, strlen(repCap) - RS_MAX_MESSAGE_BUF_SIZE - 1);

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_VSA_RESULT_RANGE_HIGHLIGHT_SYMBOLS, highlightSymbols),
            4, "Highlight Symbol");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the VSA Result Range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR resultType/This control defines the result type of the traces.
/// HIPAR transformation/This control switches the measurement of the statistical distribution
/// HIPAR transformation/of magnitude, phase or frequency values on or off.
/// HIPAR oversampling/This control defines whether only the symbol points or all points are
/// HIPAR oversampling/considered for the statistical calculations.
/// HIPAR individualResults/This control switches between the whole Result Summary and the diagram
/// HIPAR individualResults/showing only a single value, e.g. the EVM RMS value as a bargraph.
ViStatus _VI_FUNC rsfpl_ConfigureVSAResultRange(ViSession instrSession,
                                                ViInt32   window,
                                                ViInt32   resultType,
                                                ViBoolean transformation,
                                                ViInt32   oversampling,
                                                ViInt32   individualResults)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_VSA_RESULT_RANGE_RESULT_TYPE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_VSA_RESULT_RANGE_RESULT_TYPE, resultType),
            3, "Result Type");

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_VSA_RESULT_RANGE_TYPE_TRANSFORMATION, transformation),
            4, "Transformation");

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_VSA_RESULT_RANGE_OVERSAMPLING, oversampling),
            5, "Oversampling");

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_VSA_RESULT_RANGE_INDIVIDUAL_RESULTS, individualResults),
            6, "Individual Results");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the demodulation compensate.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR iqOffset/This control switches the compensation of the I/Q offset on or off.
/// HIPAR iqImbalance/This control switches the compensation of the I/Q imbalance on or off.
/// HIPAR amplitudeDroop/This control switches the compensation of the amplitude droop on or
/// HIPAR amplitudeDroop/off.
/// HIPAR symbolRateError/This control switches the compensation for symbol rate error on or
/// HIPAR symbolRateError/off.
/// HIPAR channel/This control switches the channel compensation on or off. (With
/// HIPAR channel/equalizer only)
/// HIPAR iqSkew/This control switches the compensation of the I/Q skew on or off.
/// HIPAR fsk/This control defines whether the carrier frequency drift is
/// HIPAR fsk/compensated for FSK modulation.
/// HIPAR deviationError/This control defines whether the deviation error is compensated for
/// HIPAR deviationError/when calculating the frequency error for FSK modulation.
ViStatus _VI_FUNC rsfpl_ConfigureVSACompensate(ViSession instrSession,
                                               ViBoolean iqOffset,
                                               ViBoolean iqImbalance,
                                               ViBoolean amplitudeDroop,
                                               ViBoolean symbolRateError,
                                               ViBoolean channel,
                                               ViBoolean iqSkew,
                                               ViBoolean fsk,
                                               ViBoolean deviationError)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, "", RSFPL_ATTR_VSA_DEMODULATION_COMPENSATE_IQ_OFFSET, iqOffset),
            2, "IQ Offset");

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, "", RSFPL_ATTR_VSA_DEMODULATION_COMPENSATE_IQ_IMBALANCE, iqImbalance),
            3, "IQ Imbalance");

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, "", RSFPL_ATTR_VSA_DEMODULATION_COMPENSATE_AMPLITUDE_DROOP_ENABLED, amplitudeDroop),
            4, "Amplitude Droop");

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, "", RSFPL_ATTR_VSA_DEMODULATION_COMPENSATE_SYMBOL_CLOCK_ERROR, symbolRateError),
            5, "Symbol Rate Error");

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, "", RSFPL_ATTR_VSA_DEMODULATION_COMPENSATE_TRANSMIT_CHANNEL, channel),
            6, "Channel");

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, "", RSFPL_ATTR_VSA_DEMODULATION_COMPENSATE_IQ_SKEW, iqSkew),
            7, "IQ Skew");

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, "", RSFPL_ATTR_VSA_DEMODULATION_COMPENSATE_FSK, fsk),
            8, "FSK");

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, "", RSFPL_ATTR_VSA_DEMODULATION_COMPENSATE_DEVIATION_ERROR, deviationError),
            9, "Deviation Error");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the equalizer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/This control activates or deactivates the equalizer.
/// HIPAR mode/Switches between the equalizer modes.
/// HIPAR filterLength/This control defines the length of the equalizer in terms of symbols.
ViStatus _VI_FUNC rsfpl_ConfigureVSAEqualizer(ViSession instrSession,
                                              ViBoolean enabled,
                                              ViInt32   mode,
                                              ViInt32   filterLength)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, "", RSFPL_ATTR_VSA_DEMODULATION_EQUALIZER_ENABLED, enabled),
            2, "Enabled");

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, "", RSFPL_ATTR_VSA_DEMODULATION_EQUALIZER_MODE, mode),
            3, "Mode");

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, "", RSFPL_ATTR_VSA_DEMODULATION_EQUALIZER_FILTER_LENGTH, filterLength),
            4, "Filter Length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Determines the file format for stored equalizer results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fileFormat/Determines the file format for stored equalizer results.
ViStatus _VI_FUNC rsfpl_ConfigureVSAEqualizerFileFormat(ViSession instrSession,
                                                        ViInt32   fileFormat)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, "", RSFPL_ATTR_VSA_DEMODULATION_EQUALIZER_FILE_FORMAT, fileFormat),
            2, "File Format");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function performs the selected operation with equalizer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR operation/This control selects the type of operation to be performed.
/// HIPAR equalizerName/This control selects a equalizer name
ViStatus _VI_FUNC rsfpl_ConfigureVSAEqualizerOperations(ViSession instrSession,
                                                        ViInt32   operation,
                                                        ViString  equalizerName)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (operation)
    {
    case RSFPL_VAL_VSA_EQUALIZER_LOAD:
        attr = RSFPL_ATTR_VSA_DEMODULATION_EQUALIZER_LOAD;
        break;
    case RSFPL_VAL_VSA_EQUALIZER_SAVE:
        attr = RSFPL_ATTR_VSA_DEMODULATION_EQUALIZER_SAVE;
        break;
    case RSFPL_VAL_VSA_EQUALIZER_RESET:
        checkErr(rsfpl_SetAttributeViString(instrSession, "", RSFPL_ATTR_VSA_DEMODULATION_EQUALIZER_RESET, NULL));
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, operation), 2, "Operation");
        break;
    }

    viCheckParm(rsfpl_SetAttributeViString(instrSession, "", attr, equalizerName),
            3, "Equalizer Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the way the VSA application calculates the error vector
/// HIFN  results for offset QPSK.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Configures the way the VSA application calculates the error vector
/// HIPAR enabled/results for offset QPSK.
ViStatus _VI_FUNC rsfpl_ConfigureVSAEVMModulation(ViSession instrSession,
                                                  ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, "", RSFPL_ATTR_VSA_DEMODULATION_OFFSET_EVM_ENABLED, enabled),
            2, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the Advanced Demodulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR normalizeEVMTo/This control defines the calculation formula for EVM.
/// HIPAR optimization/This control determines the optimization criteria for the
/// HIPAR optimization/demodulation. RMSMin Optimization goal is the minimization of RMS of
/// HIPAR optimization/the error vector. EVMMin Optimization goal is the minimization of the
/// HIPAR optimization/EVM.
ViStatus _VI_FUNC rsfpl_ConfigureVSADemodulationAdvanced(ViSession instrSession,
                                                         ViInt32   normalizeEVMTo,
                                                         ViInt32   optimization,
                                                         ViBoolean points)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, "", RSFPL_ATTR_VSA_DEMODULATION_NORMALIZE_EVM_TO, normalizeEVMTo),
            2, "Normalize EVM To");

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, "", RSFPL_ATTR_VSA_DEMODULATION_OPTIMIZATION, optimization),
            3, "Optimization");

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, "", RSFPL_ATTR_VSA_DEMODULATION_ESTIMATION_POINTS_ENABLED, points),
            4, "Points");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures Estimation Points.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Defines how many sample points are used at each symbol to calculate
/// HIPAR enabled/modulation accuracy results automatically.
/// HIPAR points/Defines how many sample points are used at each symbol to calculate
/// HIPAR points/modulation accuracy results.
ViStatus _VI_FUNC rsfpl_ConfigureVSADemodulationAdvancedEstimationPoints(
                            ViSession instrSession,
                            ViBoolean enabled,
                            ViInt32   points)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, "", RSFPL_ATTR_VSA_DEMODULATION_ESTIMATION_POINTS_ENABLED, enabled),
            2, "Enabled");

    if (enabled == VI_TRUE)
    {
        viCheckParm(rsfpl_SetAttributeViInt32(instrSession, "", RSFPL_ATTR_VSA_DEMODULATION_ESTIMATION_POINTS, points),
                3, "Points");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the Advanced Demodulation Synchronization.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR coarseAuto/This control switches fast synchronization on and off, if you manually
/// HIPAR coarseAuto/synchronize with a waveform pattern.
/// HIPAR coarseMode/This control selects manual or automatic synchronization with a
/// HIPAR coarseMode/pattern waveform to speed up measurements.
/// HIPAR fineSyncAuto/This control selects manual or automatic Fine Sync.
/// HIPAR fineSyncMode/This control defines the fine synchronization mode used to calculate
/// HIPAR fineSyncMode/results, e.g. the bit error rate.
/// HIPAR fineSyncSERLevel/This control sets the Fine Sync Level if fine sync works on Known
/// HIPAR fineSyncSERLevel/Data.
ViStatus _VI_FUNC rsfpl_ConfigureVSADemodulationAdvancedSynchronization(
                            ViSession instrSession,
                            ViBoolean coarseAuto,
                            ViInt32   coarseMode,
                            ViBoolean fineSyncAuto,
                            ViInt32   fineSyncMode,
                            ViReal64  fineSyncSERLevel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, "", RSFPL_ATTR_VSA_DEMODULATION_COARSE_SYNCHRONIZATION_ENABLED, coarseAuto),
            2, "Coarse Auto");
    if (coarseAuto == VI_FALSE)
    {
        viCheckParm(rsfpl_SetAttributeViInt32(instrSession, "", RSFPL_ATTR_VSA_DEMODULATION_COARSE_SYNCHRONIZATION, coarseMode),
                3, "Coarse Mode");
    }

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, "", RSFPL_ATTR_VSA_DEMODULATION_FINE_SYNC_AUTO, fineSyncAuto),
            4, "Fine Sync Auto");

    if (fineSyncAuto == VI_FALSE)
    {
        viCheckParm(rsfpl_SetAttributeViInt32(instrSession, "", RSFPL_ATTR_VSA_DEMODULATION_FINE_SYNC_MODE, fineSyncMode),
                5, "Fine Sync Mode");

        viCheckParm(rsfpl_SetAttributeViReal64(instrSession, "", RSFPL_ATTR_VSA_DEMODULATION_FINE_SYNC_SER_LEVEL, fineSyncSERLevel),
                6, "Fine Sync SER Level");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  The result of this query is 0 if the fine sync with known data failed,
/// HIFN  otherwise 1.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fineSyncResult/The result of this query is 0 if the fine sync with known data failed,
/// HIPAR fineSyncResult/otherwise 1.
ViStatus _VI_FUNC rsfpl_QueryVSADemodulationAdvancedFineSyncResult(
                            ViSession instrSession,
                            ViInt32*  fineSyncResult)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_GetAttributeViInt32(instrSession, "", RSFPL_ATTR_VSA_DEMODULATION_FINE_SYNC_RESULT, fineSyncResult),
            2, "Fine Sync Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the Measurement Filter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementFilterAuto/If this control is set to 'ON', the measurement filter is defined
/// HIPAR measurementFilterAuto/automatically depending on the transmit filter (see
/// HIPAR measurementFilterAuto/[SENSe:]DDEMod:TFILter:NAME).
/// HIPAR filterName/This control selects a measurement filter and automatically sets its
/// HIPAR filterName/state to 'ON'.
/// HIPAR alphaBT/This control sets the alpha value of the measurement filter.
ViStatus _VI_FUNC rsfpl_ConfigureVSAMeasurementFilter(ViSession instrSession,
                                                      ViBoolean measurementFilterAuto,
                                                      ViString  filterName,
                                                      ViReal64  alphaBT)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, "", RSFPL_ATTR_VSA_MEASUREMENT_FILTER_AUTO, measurementFilterAuto),
            2, "Measurement Filter Auto");

    viCheckParm(rsfpl_SetAttributeViString(instrSession, "", RSFPL_ATTR_VSA_MEASUREMENT_FILTER_NAME, filterName),
            3, "Filter Name");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, "", RSFPL_ATTR_VSA_MEASUREMENT_FILTER_ALPHA_BT, alphaBT),
            4, "Alpha BT");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Use this attribute to switch the measurement filter off. To switch a
/// HIFN  measurement filter on, use the [SENSe:]DEMod:MFILter:NAME command.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_ConfigureVSAMeasurementFilterOff(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsfpl_SetAttributeViString(instrSession, "", RSFPL_ATTR_VSA_MEASUREMENT_FILTER_OFF, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function selects the user-defined measurement filter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fileName/This control selects the user-defined measurement filter.
ViStatus _VI_FUNC rsfpl_ConfigureVSAMeasurementFilterUser(ViSession instrSession,
                                                          ViString  fileName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, "", RSFPL_ATTR_VSA_MEASUREMENT_FILTER_USER, fileName),
            2, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Before this command can be used, the external preamplifier must be
/// HIFN  connected to the R&S&nbsp;FPL1000. See the preamplifier's documentation
/// HIFN  for details.
/// HIFN     When activated, the R&S FPL1000 automatically compensates the
/// HIFN  magnitude and phase characteristics of the external preamplifier in the
/// HIFN  measurement results.
/// HIFN     Note that when an optional external preamplifier is activated, the
/// HIFN  internal preamplifier is automatically disabled, and vice versa.
/// HIFN     When deactivated, no compensation is performed even if an external
/// HIFN  preamplifier remains connected.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Before this command can be used, the external preamplifier must be
/// HIPAR enabled/connected to the R&S&nbsp;FPL1000. See the preamplifier's documentation
/// HIPAR enabled/for details.
ViStatus _VI_FUNC rsfpl_ConfigureVSAPreamplifierCompensationEnabled(
                            ViSession instrSession,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, "", RSFPL_ATTR_VSA_PREAMPLIFIER_COMPENSATION_ENABLED, enabled),
            2, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the Electronic Attenuation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/This control turns the electronic attenuator on and off. This control
/// HIPAR enabled/requires the electronic attenuation hardware option.
/// HIPAR attenuation/This command defines an electronic attenuation manually.
ViStatus _VI_FUNC rsfpl_ConfigureVSAAttenuationElectronic(ViSession instrSession,
                                                          ViBoolean enabled,
                                                          ViReal64  attenuation)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, "", RSFPL_ATTR_VSA_ATTENUATION_ELECTRONIC_ENABLED, enabled),
            2, "Enabled");

    if (enabled == VI_TRUE)
    {
        viCheckParm(rsfpl_SetAttributeViReal64(instrSession, "", RSFPL_ATTR_VSA_ATTENUATION_ELECTRONIC, attenuation),
                3, "Attenuation");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function defines the number of columns for the statistical
/// HIFN  distribution.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR quantize/This control defines the number of columns for the statistical
/// HIPAR quantize/distribution.
ViStatus _VI_FUNC rsfpl_ConfigureVSAScallingQuantize(ViSession instrSession,
                                                     ViInt32   window,
                                                     ViInt32   quantize)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_VSA_SCALLING_QUANTIZE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_VSA_SCALLING_QUANTIZE, quantize),
            3, "Quantize");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function selects the global unit for phase results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR angleUnit/This control selects the global unit for phase results.
ViStatus _VI_FUNC rsfpl_ConfigureVSAScallingAngleUnit(ViSession instrSession,
                                                      ViInt32   window,
                                                      ViInt32   angleUnit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_VSA_SCALLING_ANGLE_UNIT,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_VSA_SCALLING_ANGLE_UNIT, angleUnit),
            3, "Angle Unit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the time unit.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR timeUnitX/This control selects the unit (symbols or seconds) for the x axis.
/// HIPAR timeUnitY/This control selects the unit (symbols or seconds) for the y axis of
/// HIPAR timeUnitY/equalizer group delay measurements.
ViStatus _VI_FUNC rsfpl_ConfigureVSAScallingTimeUnit(ViSession instrSession,
                                                     ViInt32   window,
                                                     ViInt32   timeUnitX,
                                                     ViInt32   timeUnitY)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_VSA_SCALLING_TIME_UNIT_X,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_VSA_SCALLING_TIME_UNIT_X, timeUnitX),
            3, "Time Unit X");

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_VSA_SCALLING_TIME_UNIT_Y, timeUnitY),
            4, "Time Unit Y");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function defines the scaling of the x-axis for statistical result
/// HIFN  displays.
/// HIFN     For all other result displays, this command is only available as a
/// HIFN  query.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR trace/Sets the trace.
/// HIPAR rangePerDivision/This control defines the scaling of the x-axis for statistical result
/// HIPAR rangePerDivision/displays.
ViStatus _VI_FUNC rsfpl_ConfigureVSAScallingRangePerDivision(ViSession instrSession,
                                                             ViInt32   window,
                                                             ViInt32   trace,
                                                             ViInt32   rangePerDivision)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_trace[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_VSA_SCALLING_RANGE_PER_DIVISION,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap));
    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_VSA_SCALLING_RANGE_PER_DIVISION,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCap_trace));
    strncat(repCap, ",", strlen(repCap) - RS_MAX_MESSAGE_BUF_SIZE - 1);
    strncat(repCap, repCap_trace, strlen(repCap) - RS_MAX_MESSAGE_BUF_SIZE - 1);

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_VSA_SCALLING_RANGE_PER_DIVISION, rangePerDivision),
            4, "Range Per Division");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This attribute defines the reference value for the x-axis for
/// HIFN  statistical result displays.
/// HIFN     For all other result displays, this command is only available as a
/// HIFN  query.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR trace/Sets the trace.
/// HIPAR referenceValue/This control defines the reference value for the x-axis for
/// HIPAR referenceValue/statistical result displays.
ViStatus _VI_FUNC rsfpl_ConfigureVSAScallingXAxisReferenceValue(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   trace,
                            ViInt32   referenceValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_trace[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_VSA_SCALLING_X_AXIS_REFERENCE_VALUE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap));
    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_VSA_SCALLING_X_AXIS_REFERENCE_VALUE,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCap_trace));
    strncat(repCap, ",", strlen(repCap) - RS_MAX_MESSAGE_BUF_SIZE - 1);
    strncat(repCap, repCap_trace, strlen(repCap) - RS_MAX_MESSAGE_BUF_SIZE - 1);

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_VSA_SCALLING_X_AXIS_REFERENCE_VALUE, referenceValue),
            4, "Reference Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function switches the result type transformation to spectrum
/// HIFN  mode. Spectral evaluation is available for the following result types:
/// HIFN  MAGNitudePHASe/UPHaseFREQuencyReal/Imag (RIMAG)
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR addEvaluation/This control switches the result type transformation to spectrum mode.
/// HIPAR addEvaluation/Spectral evaluation is available for the following result types:
/// HIPAR addEvaluation/MAGNitudePHASe/UPHaseFREQuencyReal/Imag (RIMAG)
ViStatus _VI_FUNC rsfpl_ConfigureVSAWindowAddEvaluation(ViSession instrSession,
                                                        ViInt32   window,
                                                        ViBoolean addEvaluation)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_VSA_WINDOW_ADD_EVALUATION,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_VSA_WINDOW_ADD_EVALUATION, addEvaluation),
            3, "Add Evaluation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the signal source (and for the equalizer also the result type)
/// HIFN  for evaluation.
/// HIFN     Note that this command is maintained for compatibility reasons only.
/// HIFN  Use the LAYout commands for new remote control programs (see 'Working
/// HIFN  with Windows in the Display').
/// HIFN     Only for the 'Equalizer Impulse Response' and 'Equalizer Frequency
/// HIFN  Response', as well as the multi-source diagrams, this command is
/// HIFN  required.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR feed/Selects the signal source (and for the equalizer also the result type)
/// HIPAR feed/for evaluation.
ViStatus _VI_FUNC rsfpl_ConfigureVSAWindowFeed(ViSession instrSession,
                                               ViInt32   window,
                                               ViString  feed)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_VSA_WINDOW_FEED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_VSA_WINDOW_FEED, feed),
            3, "Feed");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Repeats the evaluation of the data currently in the capture buffer
/// HIFN  without capturing new data. This is useful after changing settings, for
/// HIFN  example filters, patterns or evaluation ranges.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_ConfigureVSAMeasurementRefresh(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsfpl_SetAttributeViString(instrSession, "", RSFPL_ATTR_VSA_MEASUREMENT_REFRESH, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the start symbol of the current result range
/// HIFN  within the capture buffer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR startSymbol/This control queries the start symbol of the current result range
/// HIPAR startSymbol/within the capture buffer.
ViStatus _VI_FUNC rsfpl_QueryVSAMeasurementStartSymbol(ViSession instrSession,
                                                       ViReal64* startSymbol)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_GetAttributeViReal64(instrSession, "", RSFPL_ATTR_VSA_MEASUREMENT_START_SYMBOL, startSymbol),
            2, "Start Symbol");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the validity of the measurement data, as indicated in the
/// HIFN  channel bar in manual operation. A result of false indicates the
/// HIFN  measurement results are valid; a result of true indicates that invalid
/// HIFN  or inconsistent data is displayed, that is: the trace no longer matches
/// HIFN  the discplayed instrument settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementDirty/Queries the validity of the measurement data, as indicated in the
/// HIPAR measurementDirty/channel bar in manual operation. A result of false indicates the
/// HIPAR measurementDirty/measurement results are valid; a result of true indicates that invalid
/// HIPAR measurementDirty/or inconsistent data is displayed, that is: the trace no longer matches
/// HIPAR measurementDirty/the discplayed instrument settings.
/// HIPAR window/Selects the window.
ViStatus _VI_FUNC rsfpl_QueryVSAMeasurementDirty(ViSession  instrSession,
                                                 ViInt32    window,
                                                 ViBoolean* measurementDirty)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_VSA_WINDOW_MEASUREMENT_DIRTY,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsfpl_GetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_VSA_WINDOW_MEASUREMENT_DIRTY, measurementDirty),
            3, "Measurement Dirty");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the selected Bit Error Rate results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR resultType/Specifies a particular BER result to be queried.
/// HIPAR result/Returns the selected bit error rate result.
ViStatus _VI_FUNC rsfpl_QueryVSABitErrorRateResults(ViSession instrSession,
                                                    ViInt32   window,
                                                    ViInt32   resultType,
                                                    ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViPChar  p2cmdString;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_VSA_RESULTS_CURRENT_BER,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsFpl_rngVSABitErrorRateResult, resultType, &p2cmdString),
        3, "Result Type");
    viCheckParm(RsCore_GetAttributeViReal64WithQparam(instrSession, repCap,
        RSFPL_ATTR_VSA_RESULTS_CURRENT_BER,
        p2cmdString,
        result),
        4, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the length of a detected burst.
/// HIFN     Note that since the R&S FPL1000 VSA application has no knowledge on
/// HIFN  the ramp length, there is an uncertainty in the burst search algorithm.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR burstLength/This control queries the length of a detected burst.
ViStatus _VI_FUNC rsfpl_QueryVSAResultsBurstLength(ViSession instrSession,
                                                   ViInt32   window,
                                                   ViInt32*  burstLength)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_VSA_RESULTS_BURST_LENGTH,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsfpl_GetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_VSA_RESULTS_BURST_LENGTH, burstLength),
            3, "Burst Length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  The command queries all results of the result summary.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR marker/Selects the marker.
/// HIPAR allValues/Displays all results values.
ViStatus _VI_FUNC rsfpl_GetVSAResultsAll(ViSession instrSession,
                                         ViInt32   window,
                                         ViInt32   arraySize,
                                         ViInt32   marker,
                                         ViReal64  results[],
                                         ViInt32*  actualPoints)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K70"));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, results),
        3, "Results");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:MARK%ld:FUNC:DDEM:STAT:ALL?", window, marker);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, results, actualPoints));

    checkErr(rsfpl_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the results of the amplitude droop error
/// HIFN  measurement performed for digital demodulation. The output values are
/// HIFN  the same as those provided in the Modulation Accuracy table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR marker/Selects the marker.
/// HIPAR type/Sets the result type for the selected measurement.
/// HIPAR result/Displays the result.
ViStatus _VI_FUNC rsfpl_GetVSAResultsAmplitudeDroop(ViSession instrSession,
                                                    ViInt32   window,
                                                    ViInt32   marker,
                                                    ViInt32   type,
                                                    ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, 16),
            3, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, type, RSFPL_VAL_VSA_MEAS_RESULT_TYPE_NONE, RSFPL_VAL_VSA_MEAS_RESULT_TYPE_PCTL),
            4, "Type");

    if (type == RSFPL_VAL_VSA_MEAS_RESULT_TYPE_NONE)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:MARK%ld:FUNC:DDEM:STAT:ADR?", window, marker);
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:MARK%ld:FUNC:DDEM:STAT:ADR? %s", window, marker, resultAdrArr[type]);
    }

    checkErr(RsCore_QueryViReal64(instrSession, cmd, result));

    checkErr(rsfpl_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the results of the carrier frequency error
/// HIFN  measurement performed for digital demodulation. The output values are
/// HIFN  the same as those provided in the Modulation Accuracy table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR marker/Selects the marker.
/// HIPAR type/Sets the result type for the selected measurement.
/// HIPAR result/Displays the result.
ViStatus _VI_FUNC rsfpl_GetVSAResultsCarrierFrequencyError(ViSession instrSession,
                                                           ViInt32   window,
                                                           ViInt32   marker,
                                                           ViInt32   type,
                                                           ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, 16),
            3, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, type, RSFPL_VAL_VSA_MEAS_RESULT_TYPE_NONE, RSFPL_VAL_VSA_MEAS_RESULT_TYPE_PCTL),
            4, "Type");

    if (type == RSFPL_VAL_VSA_MEAS_RESULT_TYPE_NONE)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:MARK%ld:FUNC:DDEM:STAT:CFER?", window, marker);
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:MARK%ld:FUNC:DDEM:STAT:CFER? %s", window, marker, resultAdrArr[type]);
    }

    checkErr(RsCore_QueryViReal64(instrSession, cmd, result));

    checkErr(rsfpl_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This attribute queries the results of the error vector magnitude
/// HIFN  measurement of digital demodulation. The output values are the same as
/// HIFN  those provided in the Modulation Accuracy table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR marker/Selects the marker.
/// HIPAR type/Sets the result type for the selected measurement.
/// HIPAR result/Displays the result.
ViStatus _VI_FUNC rsfpl_GetVSAResultsEVM(ViSession instrSession,
                                         ViInt32   window,
                                         ViInt32   marker,
                                         ViInt32   type,
                                         ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, 16),
            3, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, type, RSFPL_VAL_VSA_RESULTS_EVM_NONE, RSFPL_VAL_VSA_RESULTS_EVM_TPE),
            4, "Type");

    if (type == RSFPL_VAL_VSA_RESULTS_EVM_NONE)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:MARK%ld:FUNC:DDEM:STAT:EVM?", window, marker);
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:MARK%ld:FUNC:DDEM:STAT:EVM? %s", window, marker, resultEVMArr[type]);
    }

    checkErr(RsCore_QueryViReal64(instrSession, cmd, result));

    checkErr(rsfpl_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This attribute queries the results of the FSK deviation error of FSK
/// HIFN  modulated signals.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR marker/Selects the marker.
/// HIPAR type/Sets the result type for the selected measurement.
/// HIPAR result/Displays the result.
ViStatus _VI_FUNC rsfpl_GetVSAResultsFSKDeviation(ViSession instrSession,
                                                  ViInt32   window,
                                                  ViInt32   marker,
                                                  ViInt32   type,
                                                  ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, 16),
            3, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, type, RSFPL_VAL_VSA_MEAS_RESULT_TYPE_NONE, RSFPL_VAL_VSA_MEAS_RESULT_TYPE_PCTL),
            4, "Type");

    if (type == RSFPL_VAL_VSA_MEAS_RESULT_TYPE_NONE)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:MARK%ld:FUNC:DDEM:STAT:FDER?", window, marker);
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:MARK%ld:FUNC:DDEM:STAT:FDER? %s", window, marker, resultAdrArr[type]);
    }

    checkErr(RsCore_QueryViReal64(instrSession, cmd, result));

    checkErr(rsfpl_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the results of the carrier frequency drift for
/// HIFN  FSK modulated signals.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR marker/Selects the marker.
/// HIPAR type/Sets the result type for the selected measurement.
/// HIPAR result/Displays the result.
ViStatus _VI_FUNC rsfpl_GetVSAResultsFSKCarrierFrequency(ViSession instrSession,
                                                         ViInt32   window,
                                                         ViInt32   marker,
                                                         ViInt32   type,
                                                         ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, 16),
            3, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, type, RSFPL_VAL_VSA_MEAS_RESULT_TYPE_NONE, RSFPL_VAL_VSA_MEAS_RESULT_TYPE_PCTL),
            4, "Type");

    if (type == RSFPL_VAL_VSA_MEAS_RESULT_TYPE_NONE)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:MARK%ld:FUNC:DDEM:STAT:FSK:CFDR?", window, marker);
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:MARK%ld:FUNC:DDEM:STAT:FSK:CFDR? %s", window, marker, resultAdrArr[type]);
    }

    checkErr(RsCore_QueryViReal64(instrSession, cmd, result));

    checkErr(rsfpl_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the results of the frequency error of FSK
/// HIFN  modulated signals.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR marker/Selects the marker.
/// HIPAR type/Sets the result type for the selected measurement.
/// HIPAR result/Displays the result.
ViStatus _VI_FUNC rsfpl_GetVSAResultsFSKFrequencyError(ViSession instrSession,
                                                       ViInt32   window,
                                                       ViInt32   marker,
                                                       ViInt32   type,
                                                       ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, 16),
            3, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, type, RSFPL_VAL_VSA_RESULTS_EVM_NONE, RSFPL_VAL_VSA_RESULTS_EVM_TPE),
            4, "Type");

    if (type == RSFPL_VAL_VSA_RESULTS_EVM_NONE)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:MARK%ld:FUNC:DDEM:STAT:FSK:DERR?", window, marker);
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:MARK%ld:FUNC:DDEM:STAT:FSK:DERR? %s", window, marker, resultEVMArr[type]);
    }

    checkErr(RsCore_QueryViReal64(instrSession, cmd, result));

    checkErr(rsfpl_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the results of the measurement deviation of FSK
/// HIFN  modulated signals.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR marker/Selects the marker.
/// HIPAR type/Sets the result type for the selected measurement.
/// HIPAR result/Displays the result.
ViStatus _VI_FUNC rsfpl_GetVSAResultsFSKMeasurementDeviation(ViSession instrSession,
                                                             ViInt32   window,
                                                             ViInt32   marker,
                                                             ViInt32   type,
                                                             ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, 16),
            3, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, type, RSFPL_VAL_VSA_MEAS_RESULT_TYPE_NONE, RSFPL_VAL_VSA_MEAS_RESULT_TYPE_PCTL),
            4, "Type");

    if (type == RSFPL_VAL_VSA_MEAS_RESULT_TYPE_NONE)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:MARK%ld:FUNC:DDEM:STAT:FSK:MDEV?", window, marker);
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:MARK%ld:FUNC:DDEM:STAT:FSK:MDEV? %s", window, marker, resultAdrArr[type]);
    }

    checkErr(RsCore_QueryViReal64(instrSession, cmd, result));

    checkErr(rsfpl_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the results of the reference deviation of FSK
/// HIFN  modulated signals.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR marker/Selects the marker.
/// HIPAR type/Sets the result type for the selected measurement.
/// HIPAR result/Displays the result.
ViStatus _VI_FUNC rsfpl_GetVSAResultsFSKReferenceDeviation(ViSession instrSession,
                                                           ViInt32   window,
                                                           ViInt32   marker,
                                                           ViInt32   type,
                                                           ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, 16),
            3, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, type, RSFPL_VAL_VSA_MEAS_RESULT_TYPE_NONE, RSFPL_VAL_VSA_MEAS_RESULT_TYPE_PCTL),
            4, "Type");

    if (type == RSFPL_VAL_VSA_MEAS_RESULT_TYPE_NONE)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:MARK%ld:FUNC:DDEM:STAT:FSK:RDEV?", window, marker);
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:MARK%ld:FUNC:DDEM:STAT:FSK:RDEV? %s", window, marker, resultAdrArr[type]);
    }

    checkErr(RsCore_QueryViReal64(instrSession, cmd, result));

    checkErr(rsfpl_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the results of the Gain Imbalance error
/// HIFN  measurement of digital demodulation. The output values are the same as
/// HIFN  those provided in the Modulation Accuracy table .
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR marker/Selects the marker.
/// HIPAR type/Sets the result type for the selected measurement.
/// HIPAR result/Displays the result.
ViStatus _VI_FUNC rsfpl_GetVSAResultsGainImbalance(ViSession instrSession,
                                                   ViInt32   window,
                                                   ViInt32   marker,
                                                   ViInt32   type,
                                                   ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, 16),
            3, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, type, RSFPL_VAL_VSA_MEAS_RESULT_TYPE_NONE, RSFPL_VAL_VSA_MEAS_RESULT_TYPE_PCTL),
            4, "Type");

    if (type == RSFPL_VAL_VSA_MEAS_RESULT_TYPE_NONE)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:MARK%ld:FUNC:DDEM:STAT:GIMB?", window, marker);
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:MARK%ld:FUNC:DDEM:STAT:GIMB? %s", window, marker, resultAdrArr[type]);
    }

    checkErr(RsCore_QueryViReal64(instrSession, cmd, result));

    checkErr(rsfpl_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the results of the I/Q imbalance error
/// HIFN  measurement of digital demodulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR marker/Selects the marker.
/// HIPAR type/Sets the result type for the selected measurement.
/// HIPAR result/Displays the result.
ViStatus _VI_FUNC rsfpl_GetVSAResultsIQImbalance(ViSession instrSession,
                                                 ViInt32   window,
                                                 ViInt32   marker,
                                                 ViInt32   type,
                                                 ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, 16),
            3, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, type, RSFPL_VAL_VSA_MEAS_RESULT_TYPE_NONE, RSFPL_VAL_VSA_MEAS_RESULT_TYPE_PCTL),
            4, "Type");

    if (type == RSFPL_VAL_VSA_MEAS_RESULT_TYPE_NONE)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:MARK%ld:FUNC:DDEM:STAT:IQIM?", window, marker);
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:MARK%ld:FUNC:DDEM:STAT:IQIM? %s", window, marker, resultAdrArr[type]);
    }

    checkErr(RsCore_QueryViReal64(instrSession, cmd, result));

    checkErr(rsfpl_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the results of the I/Q skew error measurement of
/// HIFN  digital demodulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR marker/Selects the marker.
/// HIPAR type/Sets the result type for the selected measurement.
/// HIPAR result/Displays the result.
ViStatus _VI_FUNC rsfpl_GetVSAResultsIQSkewError(ViSession instrSession,
                                                 ViInt32   window,
                                                 ViInt32   marker,
                                                 ViInt32   type,
                                                 ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, 16),
            3, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, type, RSFPL_VAL_VSA_MEAS_RESULT_TYPE_NONE, RSFPL_VAL_VSA_MEAS_RESULT_TYPE_PCTL),
            4, "Type");

    if (type == RSFPL_VAL_VSA_MEAS_RESULT_TYPE_NONE)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:MARK%ld:FUNC:DDEM:STAT:IQSK?", window, marker);
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:MARK%ld:FUNC:DDEM:STAT:IQSK? %s", window, marker, resultAdrArr[type]);
    }

    checkErr(RsCore_QueryViReal64(instrSession, cmd, result));

    checkErr(rsfpl_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the results of the I/Q offset measurement
/// HIFN  performed for digital demodulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR marker/Selects the marker.
/// HIPAR type/Sets the result type for the selected measurement.
/// HIPAR result/Displays the result.
ViStatus _VI_FUNC rsfpl_GetVSAResultsIQOffset(ViSession instrSession,
                                              ViInt32   window,
                                              ViInt32   marker,
                                              ViInt32   type,
                                              ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, 16),
            3, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, type, RSFPL_VAL_VSA_MEAS_RESULT_TYPE_NONE, RSFPL_VAL_VSA_MEAS_RESULT_TYPE_PCTL),
            4, "Type");

    if (type == RSFPL_VAL_VSA_MEAS_RESULT_TYPE_NONE)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:MARK%ld:FUNC:DDEM:STAT:OOFF?", window, marker);
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:MARK%ld:FUNC:DDEM:STAT:OOFF? %s", window, marker, resultAdrArr[type]);
    }

    checkErr(RsCore_QueryViReal64(instrSession, cmd, result));

    checkErr(rsfpl_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the results of the magnitude error measurement
/// HIFN  of digital demodulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR marker/Selects the marker.
/// HIPAR type/Sets the result type for the selected measurement.
/// HIPAR result/Displays the result.
ViStatus _VI_FUNC rsfpl_GetVSAResultsMagnitudeError(ViSession instrSession,
                                                    ViInt32   window,
                                                    ViInt32   marker,
                                                    ViInt32   type,
                                                    ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, 16),
            3, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, type, RSFPL_VAL_VSA_RESULTS_EVM_NONE, RSFPL_VAL_VSA_RESULTS_EVM_TPE),
            4, "Type");

    if (type == RSFPL_VAL_VSA_RESULTS_EVM_NONE)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:MARK%ld:FUNC:DDEM:STAT:MERR?", window, marker);
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:MARK%ld:FUNC:DDEM:STAT:MERR? %s", window, marker, resultEVMArr[type]);
    }

    checkErr(RsCore_QueryViReal64(instrSession, cmd, result));

    checkErr(rsfpl_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the results of the power measurement of digital
/// HIFN  demodulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR marker/Selects the marker.
/// HIPAR type/Sets the result type for the selected measurement.
/// HIPAR result/Displays the result.
ViStatus _VI_FUNC rsfpl_GetVSAResultsPowerMeasurement(ViSession instrSession,
                                                      ViInt32   window,
                                                      ViInt32   marker,
                                                      ViInt32   type,
                                                      ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, 16),
            3, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, type, RSFPL_VAL_VSA_MEAS_RESULT_TYPE_NONE, RSFPL_VAL_VSA_MEAS_RESULT_TYPE_PCTL),
            4, "Type");

    if (type == RSFPL_VAL_VSA_MEAS_RESULT_TYPE_NONE)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:MARK%ld:FUNC:DDEM:STAT:MPOW?", window, marker);
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:MARK%ld:FUNC:DDEM:STAT:MPOW? %s", window, marker, resultAdrArr[type]);
    }

    checkErr(RsCore_QueryViReal64(instrSession, cmd, result));

    checkErr(rsfpl_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the results of the phase error measurement
/// HIFN  performed for digital demodulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR marker/Selects the marker.
/// HIPAR type/Sets the result type for the selected measurement.
/// HIPAR result/Displays the result.
ViStatus _VI_FUNC rsfpl_GetVSAResultsPhaseError(ViSession instrSession,
                                                ViInt32   window,
                                                ViInt32   marker,
                                                ViInt32   type,
                                                ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, 16),
            3, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, type, RSFPL_VAL_VSA_MEAS_RESULT_TYPE_NONE, RSFPL_VAL_VSA_MEAS_RESULT_TYPE_PCTL),
            4, "Type");

    if (type == RSFPL_VAL_VSA_MEAS_RESULT_TYPE_NONE)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:MARK%ld:FUNC:DDEM:STAT:PERR?", window, marker);
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:MARK%ld:FUNC:DDEM:STAT:PERR? %s", window, marker, resultAdrArr[type]);
    }

    checkErr(RsCore_QueryViReal64(instrSession, cmd, result));

    checkErr(rsfpl_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the results of the Quadrature error measurement
/// HIFN  performed for digital demodulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR marker/Selects the marker.
/// HIPAR type/Sets the result type for the selected measurement.
/// HIPAR result/Displays the result.
ViStatus _VI_FUNC rsfpl_GetVSAResultsQuadratureError(ViSession instrSession,
                                                     ViInt32   window,
                                                     ViInt32   marker,
                                                     ViInt32   type,
                                                     ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, 16),
            3, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, type, RSFPL_VAL_VSA_MEAS_RESULT_TYPE_NONE, RSFPL_VAL_VSA_MEAS_RESULT_TYPE_PCTL),
            4, "Type");

    if (type == RSFPL_VAL_VSA_MEAS_RESULT_TYPE_NONE)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:MARK%ld:FUNC:DDEM:STAT:QERR?", window, marker);
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:MARK%ld:FUNC:DDEM:STAT:QERR? %s", window, marker, resultAdrArr[type]);
    }

    checkErr(RsCore_QueryViReal64(instrSession, cmd, result));

    checkErr(rsfpl_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the results of the Rho factor measurement
/// HIFN  performed for digital demodulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR marker/Selects the marker.
/// HIPAR type/Sets the result type for the selected measurement.
/// HIPAR result/Displays the result.
ViStatus _VI_FUNC rsfpl_GetVSAResultsRHO(ViSession instrSession,
                                         ViInt32   window,
                                         ViInt32   marker,
                                         ViInt32   type,
                                         ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, 16),
            3, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, type, RSFPL_VAL_VSA_MEAS_RESULT_TYPE_NONE, RSFPL_VAL_VSA_MEAS_RESULT_TYPE_PCTL),
            4, "Type");

    if (type == RSFPL_VAL_VSA_MEAS_RESULT_TYPE_NONE)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:MARK%ld:FUNC:DDEM:STAT:RHO?", window, marker);
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:MARK%ld:FUNC:DDEM:STAT:RHO? %s", window, marker, resultAdrArr[type]);
    }

    checkErr(RsCore_QueryViReal64(instrSession, cmd, result));

    checkErr(rsfpl_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the results of the SNR error measurement
/// HIFN  performed for digital demodulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR marker/Selects the marker.
/// HIPAR type/Sets the result type for the selected measurement.
/// HIPAR result/Displays the result.
ViStatus _VI_FUNC rsfpl_GetVSAResultsSNRError(ViSession instrSession,
                                              ViInt32   window,
                                              ViInt32   marker,
                                              ViInt32   type,
                                              ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, 16),
            3, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, type, RSFPL_VAL_VSA_RESULTS_EVM_NONE, RSFPL_VAL_VSA_RESULTS_EVM_TPE),
            4, "Type");

    if (type == RSFPL_VAL_VSA_RESULTS_EVM_NONE)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:MARK%ld:FUNC:DDEM:STAT:SNR?", window, marker);
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:MARK%ld:FUNC:DDEM:STAT:SNR? %s", window, marker, resultEVMArr[type]);
    }

    checkErr(RsCore_QueryViReal64(instrSession, cmd, result));

    checkErr(rsfpl_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the symbol rate error.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR marker/Selects the marker.
/// HIPAR type/Sets the result type for the selected measurement.
/// HIPAR result/Displays the result.
ViStatus _VI_FUNC rsfpl_GetVSAResultsSymbolRateError(ViSession instrSession,
                                                     ViInt32   window,
                                                     ViInt32   marker,
                                                     ViInt32   type,
                                                     ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, 16),
            3, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, type, RSFPL_VAL_VSA_MEAS_RESULT_TYPE_NONE, RSFPL_VAL_VSA_MEAS_RESULT_TYPE_PCTL),
            4, "Type");

    if (type == RSFPL_VAL_VSA_MEAS_RESULT_TYPE_NONE)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:MARK%ld:FUNC:DDEM:STAT:SRER?", window, marker);
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:MARK%ld:FUNC:DDEM:STAT:SRER? %s", window, marker, resultAdrArr[type]);
    }

    checkErr(RsCore_QueryViReal64(instrSession, cmd, result));

    checkErr(rsfpl_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the first and last value of the x-axis in the
/// HIFN  specified window in symbols or time, depending on the unit setting for
/// HIFN  the x-axis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR trace/Sets the trace number.
/// HIPAR start/This control queries the first value of the x-axis in the specified
/// HIPAR start/window in symbols or time, depending on the unit setting for the
/// HIPAR start/x-axis.
/// HIPAR stop/This control queries the last value of the x-axis in the specified
/// HIPAR stop/window in symbols or time, depending on the unit setting for the
/// HIPAR stop/x-axis.
ViStatus _VI_FUNC rsfpl_QueryVSATraceStartStop(ViSession instrSession,
                                               ViInt32   window,
                                               ViInt32   trace,
                                               ViReal64* start,
                                               ViReal64* stop)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_trace[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_VSA_TRACE_START,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap));
    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_VSA_TRACE_START,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCap_trace));
    strncat(repCap, ",", strlen(repCap) - RS_MAX_MESSAGE_BUF_SIZE - 1);
    strncat(repCap, repCap_trace, strlen(repCap) - RS_MAX_MESSAGE_BUF_SIZE - 1);

    viCheckParm(rsfpl_GetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_VSA_TRACE_START, start),
            4, "Start");

    viCheckParm(rsfpl_GetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_VSA_TRACE_STOP, stop),
            5, "Stop");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Stores the I/Q data for the displayed trace in the selected window to
/// HIFN  a file in iq.tar format.
/// HIFN     This attribute  is only available for result types that provide I/Q
/// HIFN  data based on the error vector, such as the Vector I/Q or Real/Imag
/// HIFN  displays.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fileName/Sets the filename parameter.
ViStatus _VI_FUNC rsfpl_SaveVSATrace(ViSession instrSession,
                                     ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:STOR:IQ:TRAC '%s'", fileName);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsfpl_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the marker to the start symbol of the currently selected result
/// HIFN  range. This is useful to determine the start position of the selected
/// HIFN  result range within the capture buffer. Only available in result
/// HIFN  diagrams based on the capture buffer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR marker/Selects the marker.
ViStatus _VI_FUNC rsfpl_VSADSPMarkerToStartOfSelectedResultRange(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   marker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_VSA_DSP_MARKER_TO_START_OF_SELECTED_RESULT_RANGE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_VSA_DSP_MARKER_TO_START_OF_SELECTED_RESULT_RANGE,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap_marker),
            3, "Marker");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap_window, repCap_marker);

    checkErr(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_VSA_DSP_MARKER_TO_START_OF_SELECTED_RESULT_RANGE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the delta marker to the start symbol of the currently selected
/// HIFN  result range. This is useful to determine the start position of the
/// HIFN  selected result range within the capture buffer. Only available in
/// HIFN  result diagrams based on the capture buffer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR marker/Selects the marker.
ViStatus _VI_FUNC rsfpl_VSADSPDeltaMarkerToStartOfSelectedResultRange(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   marker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_VSA_DSP_DELTA_MARKER_TO_START_OF_SELECTED_RESULT_RANGE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_VSA_DSP_DELTA_MARKER_TO_START_OF_SELECTED_RESULT_RANGE,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap_marker),
            3, "Marker");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap_window, repCap_marker);

    checkErr(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_VSA_DSP_DELTA_MARKER_TO_START_OF_SELECTED_RESULT_RANGE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the number of bursts found across the internal capture buffer.
/// HIFN  Note that the internal capture buffer is slightly larger than the
/// HIFN  displayed capture buffer in order to allow for sufficient filter
/// HIFN  settling times for further processing.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR numberOfBursts/Returns the number of bursts found across the internal capture buffer.
ViStatus _VI_FUNC rsfpl_QueryVSADSPCaptureNumberOfBursts(ViSession instrSession,
                                                         ViInt32*  numberOfBursts)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_GetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_VSA_DSP_CAPTURE_NUMBER_OF_BURSTS, numberOfBursts),
            2, "Number Of Bursts");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the number of patterns found across the internal capture
/// HIFN  buffer. Note that the internal capture buffer is slightly larger than
/// HIFN  the displayed capture buffer in order to allow for sufficient filter
/// HIFN  settling times for further processing.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR numberOfPatterns/Returns the number of patterns found across the internal capture
/// HIPAR numberOfPatterns/buffer.
ViStatus _VI_FUNC rsfpl_QueryVSADSPCaptureNumberOfPatterns(ViSession instrSession,
                                                           ViInt32*  numberOfPatterns)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_GetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_VSA_DSP_CAPTURE_NUMBER_OF_PATTERNS, numberOfPatterns),
            2, "Number Of Patterns");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the length in samples of the burst in the current result
/// HIFN  range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR length/Returns the length in samples of the burst in the current result
/// HIPAR length/range.
ViStatus _VI_FUNC rsfpl_QueryVSADSPCurrentRangeBurstLength(ViSession instrSession,
                                                           ViInt32*  length)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_GetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_VSA_DSP_CURRENT_RANGE_BURST_LENGTH, length),
            2, "Length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries whether a burst is present or not in the current result range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR burstPresent/Returns whether a burst is present or not in the current result range.
ViStatus _VI_FUNC rsfpl_QueryVSADSPCurrentRangeBurstPresent(ViSession  instrSession,
                                                            ViBoolean* burstPresent)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_GetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_VSA_DSP_CURRENT_RANGE_BURST_PRESENT, burstPresent),
            2, "Burst Present");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the burst start in the current result range as an offset in
/// HIFN  symbols to the capture buffer start. To determine the capture buffer
/// HIFN  start, use rsfpl_QueryVSATraceStartStop for a window with a capture
/// HIFN  buffer display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR burstStart/Queries the burst start in the current result range as an offset in
/// HIPAR burstStart/symbols to the capture buffer start. To determine the capture buffer
/// HIPAR burstStart/start, use the RSFPL_ATTR_VSA_TRACE_START control for a window with a
/// HIPAR burstStart/capture buffer display.
ViStatus _VI_FUNC rsfpl_QueryVSADSPCurrentRangeBurstStart(ViSession instrSession,
                                                          ViInt32*  burstStart)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_GetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_VSA_DSP_CURRENT_RANGE_BURST_START, burstStart),
            2, "Burst Start");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the percentage confidence with which the pattern was detected
/// HIFN  in the current result range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR confidence/Returns the percentage confidence with which the pattern was detected
/// HIPAR confidence/in the current result range.
ViStatus _VI_FUNC rsfpl_QueryVSADSPCurrentRangePatternConfidence(
                            ViSession instrSession,
                            ViReal64* confidence)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_GetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_VSA_DSP_CURRENT_RANGE_PATTERN_CONFIDENCE, confidence),
            2, "Confidence");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries whether the pattern is correct or not in the current result
/// HIFN  range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR patternCorrect/Returns whether the pattern is correct or not in the current result
/// HIPAR patternCorrect/range.
ViStatus _VI_FUNC rsfpl_QueryVSADSPCurrentRangePatternCorrect(
                            ViSession  instrSession,
                            ViBoolean* patternCorrect)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_GetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_VSA_DSP_CURRENT_RANGE_PATTERN_CORRECT, patternCorrect),
            2, "Pattern Correct");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries whether a pattern is present or not in the current result
/// HIFN  range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR patternPresent/Returns whether a pattern is present or not in the current result
/// HIPAR patternPresent/range.
ViStatus _VI_FUNC rsfpl_QueryVSADSPCurrentRangePatternPresent(
                            ViSession  instrSession,
                            ViBoolean* patternPresent)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_GetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_VSA_DSP_CURRENT_RANGE_PATTERN_PRESENT, patternPresent),
            2, "Pattern Present");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the pattern start in the current result range as an offset in
/// HIFN  symbols to the capture buffer start. To determine the capture buffer
/// HIFN  start, use rsfpl_QueryVSATraceStartStop for a window with a capture
/// HIFN  buffer display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR patternStart/Returns the pattern start in the current result range as an offset in
/// HIPAR patternStart/symbols to the capture buffer start.
ViStatus _VI_FUNC rsfpl_QueryVSADSPCurrentRangePatternStart(ViSession instrSession,
                                                            ViInt32*  patternStart)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_GetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_VSA_DSP_CURRENT_RANGE_PATTERN_START, patternStart),
            2, "Pattern Start");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the Analysis Trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trace/Sets the trace number.
/// HIPAR traceEvaluation/Selects the signal to be used as the data source for a trace.
/// HIPAR symbolSelection/This control selects which symbols are displayed by a trace (in a
/// HIPAR symbolSelection/constellation graph with 2 modulations).
ViStatus _VI_FUNC rsfpl_ConfigureVSAAnalysisTrace(ViSession instrSession,
                                                  ViInt32   trace,
                                                  ViInt32   traceEvaluation,
                                                  ViInt32   symbolSelection)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_VSA_TRACE_EVALUATION,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_VSA_TRACE_EVALUATION, traceEvaluation),
            4, "Trace Evaluation");

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_VSA_TRACE_SYMBOL_SELECTION, symbolSelection),
            5, "Symbol Selection");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the marker to the largest absolute peak value (maximum or
/// HIFN  minimum) of the selected trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR marker/Selects the marker.
ViStatus _VI_FUNC rsfpl_ConfigureVSAMarkerAbsolutePeak(ViSession instrSession,
                                                       ViInt32   window,
                                                       ViInt32   marker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_VSA_MARKER_ABSOLUTE_PEAK,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window));
    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_VSA_MARKER_ABSOLUTE_PEAK,
            "Marker", marker , RS_REPCAP_BUF_SIZE, repCap_marker));
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap_window, repCap_marker);

    checkErr(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_VSA_MARKER_ABSOLUTE_PEAK, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function positions the active marker or delta marker on the
/// HIFN  largest absolute peak value (maximum or minimum) of the selected trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR deltamarker/Selects the marker.
ViStatus _VI_FUNC rsfpl_ConfigureVSADeltamarkerAbsolutePeak(ViSession instrSession,
                                                            ViInt32   window,
                                                            ViInt32   deltamarker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap_window[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_VSA_MARKER_ABSOLUTE_PEAK,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_window));
    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_VSA_MARKER_ABSOLUTE_PEAK,
            "Marker", deltamarker , RS_REPCAP_BUF_SIZE, repCap_marker));
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap_window, repCap_marker);

    checkErr(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_VSA_DELTAMARKER_ABSOLUTE_PEAK, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Displays an absolute horizontal line in the specified window. This
/// HIFN  command is only available for eye diagrams.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR enabled/Displays an absolute horizontal line in the specified window. This
/// HIPAR enabled/command is only available for eye diagrams.
ViStatus _VI_FUNC rsfpl_ConfigureVSADisplayLinesEnabled(ViSession instrSession,
                                                        ViInt32   window,
                                                        ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_VSA_DISPLAY_LINES_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_VSA_DISPLAY_LINES_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines value of horizontal absolute line.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR horizontalLine/Defines value of horizontal absolute line.
ViStatus _VI_FUNC rsfpl_ConfigureVSADisplayLinesHorizontalLine(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64  horizontalLine)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_VSA_DISPLAY_LINES_HORIZONTAL_LINE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_VSA_DISPLAY_LINES_HORIZONTAL_LINE, horizontalLine),
            3, "Horizontal Line");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures relative horizontal display line.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR enabled/Displays a relative horizontal line in the specified window. This
/// HIPAR enabled/control is only available for eye diagrams, and only if an absolute
/// HIPAR enabled/horizontal line is already available in the same diagram (see
/// HIPAR enabled/CALCulate<n>:DLABs:STATe).
/// HIPAR horizontalLine/Defines or queries the y-value of the relative horizontal line in the
/// HIPAR horizontalLine/specified window. This command is only available for eye diagrams, and
/// HIPAR horizontalLine/only if an absolute horizontal line and a relative horizontal line are
/// HIPAR horizontalLine/already available in the same diagram.
ViStatus _VI_FUNC rsfpl_ConfigureVSADisplayLineRelativeHorizontal(
                            ViSession instrSession,
                            ViInt32   window,
                            ViBoolean enabled,
                            ViReal64  horizontalLine)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_VSA_DISPLAY_LINE_RELATIVE_HORIZONTAL_LINE_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_VSA_DISPLAY_LINE_RELATIVE_HORIZONTAL_LINE_ENABLED, enabled),
            3, "Enabled");

    if (enabled == VI_TRUE)
    {
        viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_VSA_DISPLAY_LINE_RELATIVE_HORIZONTAL_LINE, horizontalLine),
                4, "Horizontal Line");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures absolute vertical display line.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR enabled/Displays an absolute vertical line in the specified window. This
/// HIPAR enabled/command is only available for eye diagrams.
/// HIPAR verticalLine/Defines or queries the x-value of the absolute vertical line in the
/// HIPAR verticalLine/specified window. This command is only available for eye diagrams, and
/// HIPAR verticalLine/only if an absolute vertical line is already available in the diagram.
ViStatus _VI_FUNC rsfpl_ConfigureVSADisplayLineAbsoluteVertical(
                            ViSession instrSession,
                            ViInt32   window,
                            ViBoolean enabled,
                            ViReal64  verticalLine)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_VSA_DISPLAY_LINE_ABSOLUTE_VERTICAL_LINE_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_VSA_DISPLAY_LINE_ABSOLUTE_VERTICAL_LINE_ENABLED, enabled),
            3, "Enabled");

    if (enabled == VI_TRUE)
    {
        viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_VSA_DISPLAY_LINE_ABSOLUTE_VERTICAL_LINE, verticalLine),
                4, "Vertical Line");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures relative vertical display line.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR enabled/Displays a relative vertical line in the specified window. This
/// HIPAR enabled/control is only available for eye diagrams, and only if an absolute
/// HIPAR enabled/vertical line is already available in the same diagram.
/// HIPAR verticalLine/Defines or queries the x-value of the relative vertical line in the
/// HIPAR verticalLine/specified window. This control is only available for eye diagrams, and
/// HIPAR verticalLine/only if an absolute vertical line and a relative vertical line are
/// HIPAR verticalLine/already available in the same diagram.
ViStatus _VI_FUNC rsfpl_ConfigureVSADisplayLineRelativeVertical(
                            ViSession instrSession,
                            ViInt32   window,
                            ViBoolean enabled,
                            ViReal64  verticalLine)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_VSA_DISPLAY_LINE_RELATIVE_VERTICAL_LINE_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_VSA_DISPLAY_LINE_RELATIVE_VERTICAL_LINE_ENABLED, enabled),
            3, "Enabled");

    if (enabled == VI_TRUE)
    {
        viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_VSA_DISPLAY_LINE_RELATIVE_VERTICAL_LINE, verticalLine),
                4, "Vertical Line");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Restores the default limits and deactivates all checks in all windows.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
ViStatus _VI_FUNC rsfpl_ConfigureVSAMACCLimitDefault(ViSession instrSession,
                                                     ViInt32   window)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_VSA_MACC_LIMIT_DEFAULT,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap));

    checkErr(rsfpl_SetAttributeViString(instrSession, repCap, RSFPL_ATTR_VSA_MACC_LIMIT_DEFAULT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Limits checks for all evaluations based on modulation accuracy (e.g.
/// HIFN  Result Summary) are enabled or disabled.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the window.
/// HIPAR enabled/Limits checks for all evaluations based on modulation accuracy (e.g.
/// HIPAR enabled/Result Summary) are enabled or disabled.
ViStatus _VI_FUNC rsfpl_ConfigureVSAMACCLimitEnabled(ViSession instrSession,
                                                     ViInt32   window,
                                                     ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_VSA_MACC_LIMIT_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_VSA_MACC_LIMIT_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries whether the limit for the specified result type
/// HIFN  and limit type was violated.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/Selects the window.
/// HIPAR measurementType/Selects the measurement limit result.
/// HIPAR limitType/Selects the limit type.
/// HIPAR enabled/This control switches the limit check for the selected result type and
/// HIPAR enabled/limit type on or off.
/// HIPAR value/This control defines the limit for the current, peak or mean center
/// HIPAR value/frequency error limit. Note that the limits for the current and the
/// HIPAR value/peak value are always kept identical.
ViStatus _VI_FUNC rsfpl_ConfigureVSAMACCLimitCheck(ViSession instrSession,
                                                   ViInt32   window,
                                                   ViInt32   measurementType,
                                                   ViInt32   limitType,
                                                   ViBoolean enabled,
                                                   ViReal64  value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attrEnabled = 0;
    ViAttr   attrValue = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_VSA_MACC_LIMIT_CARRIER_FREQUENCY_ERROR_CURRENT_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap));

    switch (measurementType)
    {
    case RSFPL_VAL_VSA_RESULT_EVM:
        switch (limitType)
        {
        case RSFPL_VAL_VSA_LIMIT_CURRENT_PEAK:
            attrEnabled = RSFPL_ATTR_VSA_MACC_LIMIT_EVM_PEAK_CURRENT_ENABLED;
            attrValue = RSFPL_ATTR_VSA_MACC_LIMIT_EVM_PEAK_CURRENT_VALUE;
            break;
        case RSFPL_VAL_VSA_LIMIT_MEAN_PEAK:
            attrEnabled = RSFPL_ATTR_VSA_MACC_LIMIT_EVM_PEAK_MEAN_ENABLED;
            attrValue = RSFPL_ATTR_VSA_MACC_LIMIT_EVM_PEAK_MEAN_VALUE;
            break;
        case RSFPL_VAL_VSA_LIMIT_PEAK_PEAK:
            attrEnabled = RSFPL_ATTR_VSA_MACC_LIMIT_EVM_PEAK_PEAK_ENABLED;
            attrValue = RSFPL_ATTR_VSA_MACC_LIMIT_EVM_PEAK_PEAK_VALUE;
            break;
        case RSFPL_VAL_VSA_LIMIT_CURRENT_RMS:
            attrEnabled = RSFPL_ATTR_VSA_MACC_LIMIT_EVM_RMS_CURRENT_ENABLED;
            attrValue = RSFPL_ATTR_VSA_MACC_LIMIT_EVM_RMS_CURRENT_VALUE;
            break;
        case RSFPL_VAL_VSA_LIMIT_MEAN_RMS:
            attrEnabled = RSFPL_ATTR_VSA_MACC_LIMIT_EVM_RMS_MEAN_ENABLED;
            attrValue = RSFPL_ATTR_VSA_MACC_LIMIT_EVM_RMS_MEAN_VALUE;
            break;
        case RSFPL_VAL_VSA_LIMIT_PEAK_RMS:
            attrEnabled = RSFPL_ATTR_VSA_MACC_LIMIT_EVM_RMS_PEAK_ENABLED;
            attrValue = RSFPL_ATTR_VSA_MACC_LIMIT_EVM_RMS_PEAK_VALUE;
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 4, "Limit Type");
            break;
        }

        break;
    case RSFPL_VAL_VSA_RESULT_FREQ_ERROR:
        switch (limitType)
        {
        case RSFPL_VAL_VSA_LIMIT_CURRENT_PEAK:
            attrEnabled = RSFPL_ATTR_VSA_MACC_LIMIT_FREQUENCY_ERROR_PEAK_CURRENT_ENABLED;
            attrValue = RSFPL_ATTR_VSA_MACC_LIMIT_FREQUENCY_ERROR_PEAK_CURRENT_VALUE;
            break;
        case RSFPL_VAL_VSA_LIMIT_MEAN_PEAK:
            attrEnabled = RSFPL_ATTR_VSA_MACC_LIMIT_FREQUENCY_ERROR_PEAK_MEAN_ENABLED;
            attrValue = RSFPL_ATTR_VSA_MACC_LIMIT_FREQUENCY_ERROR_PEAK_MEAN_VALUE;
            break;
        case RSFPL_VAL_VSA_LIMIT_PEAK_PEAK:
            attrEnabled = RSFPL_ATTR_VSA_MACC_LIMIT_FREQUENCY_ERROR_PEAK_PEAK_ENABLED;
            attrValue = RSFPL_ATTR_VSA_MACC_LIMIT_FREQUENCY_ERROR_PEAK_PEAK_VALUE;
            break;
        case RSFPL_VAL_VSA_LIMIT_CURRENT_RMS:
            attrEnabled = RSFPL_ATTR_VSA_MACC_LIMIT_FREQUENCY_ERROR_RMS_CURRENT_ENABLED;
            attrValue = RSFPL_ATTR_VSA_MACC_LIMIT_FREQUENCY_ERROR_RMS_CURRENT_VALUE;
            break;
        case RSFPL_VAL_VSA_LIMIT_MEAN_RMS:
            attrEnabled = RSFPL_ATTR_VSA_MACC_LIMIT_FREQUENCY_ERROR_RMS_MEAN_ENABLED;
            attrValue = RSFPL_ATTR_VSA_MACC_LIMIT_FREQUENCY_ERROR_RMS_MEAN_VALUE;
            break;
        case RSFPL_VAL_VSA_LIMIT_PEAK_RMS:
            attrEnabled = RSFPL_ATTR_VSA_MACC_LIMIT_FREQUENCY_ERROR_RMS_PEAK_ENABLED;
            attrValue = RSFPL_ATTR_VSA_MACC_LIMIT_FREQUENCY_ERROR_RMS_PEAK_VALUE;
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 4, "Limit Type");
            break;
        }
        break;
    case RSFPL_VAL_VSA_RESULT_MAGNITUDE_ERROR:
        switch (limitType)
        {
        case RSFPL_VAL_VSA_LIMIT_CURRENT_PEAK:
            attrEnabled = RSFPL_ATTR_VSA_MACC_LIMIT_MAGNITUDE_ERROR_PEAK_CURRENT_ENABLED;
            attrValue = RSFPL_ATTR_VSA_MACC_LIMIT_MAGNITUDE_ERROR_PEAK_CURRENT_VALUE;
            break;
        case RSFPL_VAL_VSA_LIMIT_MEAN_PEAK:
            attrEnabled = RSFPL_ATTR_VSA_MACC_LIMIT_MAGNITUDE_ERROR_PEAK_MEAN_ENABLED;
            attrValue = RSFPL_ATTR_VSA_MACC_LIMIT_MAGNITUDE_ERROR_PEAK_MEAN_VALUE;
            break;
        case RSFPL_VAL_VSA_LIMIT_PEAK_PEAK:
            attrEnabled = RSFPL_ATTR_VSA_MACC_LIMIT_MAGNITUDE_ERROR_PEAK_PEAK_ENABLED;
            attrValue = RSFPL_ATTR_VSA_MACC_LIMIT_MAGNITUDE_ERROR_PEAK_PEAK_VALUE;
            break;
        case RSFPL_VAL_VSA_LIMIT_CURRENT_RMS:
            attrEnabled = RSFPL_ATTR_VSA_MACC_LIMIT_MAGNITUDE_ERROR_RMS_CURRENT_ENABLED;
            attrValue = RSFPL_ATTR_VSA_MACC_LIMIT_MAGNITUDE_ERROR_RMS_CURRENT_VALUE;
            break;
        case RSFPL_VAL_VSA_LIMIT_MEAN_RMS:
            attrEnabled = RSFPL_ATTR_VSA_MACC_LIMIT_MAGNITUDE_ERROR_RMS_MEAN_ENABLED;
            attrValue = RSFPL_ATTR_VSA_MACC_LIMIT_MAGNITUDE_ERROR_RMS_MEAN_VALUE;
            break;
        case RSFPL_VAL_VSA_LIMIT_PEAK_RMS:
            attrEnabled = RSFPL_ATTR_VSA_MACC_LIMIT_MAGNITUDE_ERROR_RMS_PEAK_ENABLED;
            attrValue = RSFPL_ATTR_VSA_MACC_LIMIT_MAGNITUDE_ERROR_RMS_PEAK_VALUE;
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 4, "Limit Type");
            break;
        }
        break;
    case RSFPL_VAL_VSA_RESULT_PHASE_ERROR:
        switch (limitType)
        {
        case RSFPL_VAL_VSA_LIMIT_CURRENT_PEAK:
            attrEnabled = RSFPL_ATTR_VSA_MACC_LIMIT_PHASE_ERROR_PEAK_CURRENT_ENABLED;
            attrValue = RSFPL_ATTR_VSA_MACC_LIMIT_PHASE_ERROR_PEAK_CURRENT_VALUE;
            break;
        case RSFPL_VAL_VSA_LIMIT_MEAN_PEAK:
            attrEnabled = RSFPL_ATTR_VSA_MACC_LIMIT_PHASE_ERROR_PEAK_MEAN_ENABLED;
            attrValue = RSFPL_ATTR_VSA_MACC_LIMIT_PHASE_ERROR_PEAK_MEAN_VALUE;
            break;
        case RSFPL_VAL_VSA_LIMIT_PEAK_PEAK:
            attrEnabled = RSFPL_ATTR_VSA_MACC_LIMIT_PHASE_ERROR_PEAK_PEAK_ENABLED;
            attrValue = RSFPL_ATTR_VSA_MACC_LIMIT_PHASE_ERROR_PEAK_PEAK_VALUE;
            break;
        case RSFPL_VAL_VSA_LIMIT_CURRENT_RMS:
            attrEnabled = RSFPL_ATTR_VSA_MACC_LIMIT_PHASE_ERROR_RMS_CURRENT_ENABLED;
            attrValue = RSFPL_ATTR_VSA_MACC_LIMIT_PHASE_ERROR_RMS_CURRENT_VALUE;
            break;
        case RSFPL_VAL_VSA_LIMIT_MEAN_RMS:
            attrEnabled = RSFPL_ATTR_VSA_MACC_LIMIT_PHASE_ERROR_RMS_MEAN_ENABLED;
            attrValue = RSFPL_ATTR_VSA_MACC_LIMIT_PHASE_ERROR_RMS_MEAN_VALUE;
            break;
        case RSFPL_VAL_VSA_LIMIT_PEAK_RMS:
            attrEnabled = RSFPL_ATTR_VSA_MACC_LIMIT_PHASE_ERROR_RMS_PEAK_ENABLED;
            attrValue = RSFPL_ATTR_VSA_MACC_LIMIT_PHASE_ERROR_RMS_PEAK_VALUE;
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 4, "Limit Type");
            break;
        }
        break;
    case RSFPL_VAL_VSA_RESULT_CF_ERROR:
        switch (limitType)
        {
        case RSFPL_VAL_VSA_LIMIT_CURRENT:
            attrEnabled = RSFPL_ATTR_VSA_MACC_LIMIT_CARRIER_FREQUENCY_ERROR_CURRENT_ENABLED;
            attrValue = RSFPL_ATTR_VSA_MACC_LIMIT_CARRIER_FREQUENCY_ERROR_CURRENT_VALUE;
            break;
        case RSFPL_VAL_VSA_LIMIT_MEAN:
            attrEnabled = RSFPL_ATTR_VSA_MACC_LIMIT_CARRIER_FREQUENCY_ERROR_MEAN_ENABLED;
            attrValue = RSFPL_ATTR_VSA_MACC_LIMIT_CARRIER_FREQUENCY_ERROR_MEAN_VALUE;
            break;
        case RSFPL_VAL_VSA_LIMIT_PEAK:
            attrEnabled = RSFPL_ATTR_VSA_MACC_LIMIT_CARRIER_FREQUENCY_ERROR_PEAK_ENABLED;
            attrValue = RSFPL_ATTR_VSA_MACC_LIMIT_CARRIER_FREQUENCY_ERROR_PEAK_VALUE;
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 4, "Limit Type");
            break;
        }
        break;
    case RSFPL_VAL_VSA_RESULT_IQ_OFFSET:
        switch (limitType)
        {
        case RSFPL_VAL_VSA_LIMIT_CURRENT:
            attrEnabled = RSFPL_ATTR_VSA_MACC_LIMIT_IQ_OFFSET_CURRENT_ENABLED;
            attrValue = RSFPL_ATTR_VSA_MACC_LIMIT_IQ_OFFSET_CURRENT_VALUE;
            break;
        case RSFPL_VAL_VSA_LIMIT_MEAN:
            attrEnabled = RSFPL_ATTR_VSA_MACC_LIMIT_IQ_OFFSET_MEAN_ENABLED;
            attrValue = RSFPL_ATTR_VSA_MACC_LIMIT_IQ_OFFSET_MEAN_VALUE;
            break;
        case RSFPL_VAL_VSA_LIMIT_PEAK:
            attrEnabled = RSFPL_ATTR_VSA_MACC_LIMIT_IQ_OFFSET_PEAK_ENABLED;
            attrValue = RSFPL_ATTR_VSA_MACC_LIMIT_IQ_OFFSET_PEAK_VALUE;
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 4, "Limit Type");
            break;
        }
        break;
    case RSFPL_VAL_VSA_RESULT_RHO:
        switch (limitType)
        {
        case RSFPL_VAL_VSA_LIMIT_CURRENT:
            attrEnabled = RSFPL_ATTR_VSA_MACC_LIMIT_RHO_CURRENT_ENABLED;
            attrValue = RSFPL_ATTR_VSA_MACC_LIMIT_RHO_CURRENT_VALUE;
            break;
        case RSFPL_VAL_VSA_LIMIT_MEAN:
            attrEnabled = RSFPL_ATTR_VSA_MACC_LIMIT_RHO_MEAN_ENABLED;
            attrValue = RSFPL_ATTR_VSA_MACC_LIMIT_RHO_MEAN_VALUE;
            break;
        case RSFPL_VAL_VSA_LIMIT_PEAK:
            attrEnabled = RSFPL_ATTR_VSA_MACC_LIMIT_RHO_PEAK_ENABLED;
            attrValue = RSFPL_ATTR_VSA_MACC_LIMIT_RHO_PEAK_VALUE;
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 4, "Limit Type");
            break;
        }
        break;
    case RSFPL_VAL_VSA_RESULT_FSK_DEV_ERROR:
        switch (limitType)
        {
        case RSFPL_VAL_VSA_LIMIT_CURRENT:
            attrEnabled = RSFPL_ATTR_VSA_MACC_LIMIT_FREQUENCY_DEVIATION_CURRENT_ENABLED;
            attrValue = RSFPL_ATTR_VSA_MACC_LIMIT_FREQUENCY_DEVIATION_CURRENT_VALUE;
            break;
        case RSFPL_VAL_VSA_LIMIT_MEAN:
            attrEnabled = RSFPL_ATTR_VSA_MACC_LIMIT_FREQUENCY_DEVIATION_MEAN_ENABLED;
            attrValue = RSFPL_ATTR_VSA_MACC_LIMIT_FREQUENCY_DEVIATION_MEAN_VALUE;
            break;
        case RSFPL_VAL_VSA_LIMIT_PEAK:
            attrEnabled = RSFPL_ATTR_VSA_MACC_LIMIT_FREQUENCY_DEVIATION_PEAK_ENABLED;
            attrValue = RSFPL_ATTR_VSA_MACC_LIMIT_FREQUENCY_DEVIATION_PEAK_VALUE;
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 4, "Limit Type");
            break;
        }
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 3, "Measurement Type");
        break;
    }

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, attrEnabled, enabled),
            5, "Enabled");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, attrValue, value),
            6, "Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries whether the limit for the specified result type
/// HIFN  and limit type was violated.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/Selects the window.
/// HIPAR measurementType/Selects the measurement limit result.
/// HIPAR limitType/Selects the limit type.
/// HIPAR checkResults/Returns limit state code.
ViStatus _VI_FUNC rsfpl_QueryVSAMACCLimitCheckResults(ViSession instrSession,
                                                      ViInt32   window,
                                                      ViInt32   measurementType,
                                                      ViInt32   limitType,
                                                      ViInt32*  checkResults)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_VSA_MACC_LIMIT_CARRIER_FREQUENCY_ERROR_CURRENT_RESULT,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap));

    switch (measurementType)
    {
    case RSFPL_VAL_VSA_RESULT_EVM:
        switch (limitType)
        {
        case RSFPL_VAL_VSA_LIMIT_CURRENT_PEAK:
            attr = RSFPL_ATTR_VSA_MACC_LIMIT_EVM_PEAK_CURRENT_RESULT;
            break;
        case RSFPL_VAL_VSA_LIMIT_MEAN_PEAK:
            attr = RSFPL_ATTR_VSA_MACC_LIMIT_EVM_PEAK_MEAN_RESULT;
            break;
        case RSFPL_VAL_VSA_LIMIT_PEAK_PEAK:
            attr = RSFPL_ATTR_VSA_MACC_LIMIT_EVM_PEAK_PEAK_RESULT;
            break;
        case RSFPL_VAL_VSA_LIMIT_CURRENT_RMS:
            attr = RSFPL_ATTR_VSA_MACC_LIMIT_EVM_RMS_CURRENT_RESULT;
            break;
        case RSFPL_VAL_VSA_LIMIT_MEAN_RMS:
            attr = RSFPL_ATTR_VSA_MACC_LIMIT_EVM_RMS_MEAN_RESULT;
            break;
        case RSFPL_VAL_VSA_LIMIT_PEAK_RMS:
            attr = RSFPL_ATTR_VSA_MACC_LIMIT_EVM_RMS_PEAK_RESULT;
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 4, "Limit Type");
            break;
        }

        break;
    case RSFPL_VAL_VSA_RESULT_FREQ_ERROR:
        switch (limitType)
        {
        case RSFPL_VAL_VSA_LIMIT_CURRENT_PEAK:
            attr = RSFPL_ATTR_VSA_MACC_LIMIT_CARRIER_FREQUENCY_ERROR_PEAK_CURRENT_RESULT;
            break;
        case RSFPL_VAL_VSA_LIMIT_MEAN_PEAK:
            attr = RSFPL_ATTR_VSA_MACC_LIMIT_CARRIER_FREQUENCY_ERROR_PEAK_MEAN_RESULT;
            break;
        case RSFPL_VAL_VSA_LIMIT_PEAK_PEAK:
            attr = RSFPL_ATTR_VSA_MACC_LIMIT_CARRIER_FREQUENCY_ERROR_PEAK_PEAK_RESULT;
            break;
        case RSFPL_VAL_VSA_LIMIT_CURRENT_RMS:
            attr = RSFPL_ATTR_VSA_MACC_LIMIT_CARRIER_FREQUENCY_ERROR_RMS_CURRENT_RESULT;
            break;
        case RSFPL_VAL_VSA_LIMIT_MEAN_RMS:
            attr = RSFPL_ATTR_VSA_MACC_LIMIT_CARRIER_FREQUENCY_ERROR_RMS_MEAN_RESULT;
            break;
        case RSFPL_VAL_VSA_LIMIT_PEAK_RMS:
            attr = RSFPL_ATTR_VSA_MACC_LIMIT_CARRIER_FREQUENCY_ERROR_RMS_PEAK_RESULT;
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 4, "Limit Type");
            break;
        }
        break;
    case RSFPL_VAL_VSA_RESULT_MAGNITUDE_ERROR:
        switch (limitType)
        {
        case RSFPL_VAL_VSA_LIMIT_CURRENT_PEAK:
            attr = RSFPL_ATTR_VSA_MACC_LIMIT_MAGNITUDE_ERROR_PEAK_CURRENT_RESULT;
            break;
        case RSFPL_VAL_VSA_LIMIT_MEAN_PEAK:
            attr = RSFPL_ATTR_VSA_MACC_LIMIT_MAGNITUDE_ERROR_PEAK_MEAN_RESULT;
            break;
        case RSFPL_VAL_VSA_LIMIT_PEAK_PEAK:
            attr = RSFPL_ATTR_VSA_MACC_LIMIT_MAGNITUDE_ERROR_PEAK_PEAK_RESULT;
            break;
        case RSFPL_VAL_VSA_LIMIT_CURRENT_RMS:
            attr = RSFPL_ATTR_VSA_MACC_LIMIT_MAGNITUDE_ERROR_RMS_CURRENT_RESULT;
            break;
        case RSFPL_VAL_VSA_LIMIT_MEAN_RMS:
            attr = RSFPL_ATTR_VSA_MACC_LIMIT_MAGNITUDE_ERROR_RMS_MEAN_RESULT;
            break;
        case RSFPL_VAL_VSA_LIMIT_PEAK_RMS:
            attr = RSFPL_ATTR_VSA_MACC_LIMIT_MAGNITUDE_ERROR_RMS_PEAK_RESULT;
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 4, "Limit Type");
            break;
        }
        break;
    case RSFPL_VAL_VSA_RESULT_PHASE_ERROR:
        switch (limitType)
        {
        case RSFPL_VAL_VSA_LIMIT_CURRENT_PEAK:
            attr = RSFPL_ATTR_VSA_MACC_LIMIT_PHASE_ERROR_PEAK_CURRENT_RESULT;
            break;
        case RSFPL_VAL_VSA_LIMIT_MEAN_PEAK:
            attr = RSFPL_ATTR_VSA_MACC_LIMIT_PHASE_ERROR_PEAK_MEAN_RESULT;
            break;
        case RSFPL_VAL_VSA_LIMIT_PEAK_PEAK:
            attr = RSFPL_ATTR_VSA_MACC_LIMIT_PHASE_ERROR_PEAK_PEAK_RESULT;
            break;
        case RSFPL_VAL_VSA_LIMIT_CURRENT_RMS:
            attr = RSFPL_ATTR_VSA_MACC_LIMIT_PHASE_ERROR_RMS_CURRENT_RESULT;
            break;
        case RSFPL_VAL_VSA_LIMIT_MEAN_RMS:
            attr = RSFPL_ATTR_VSA_MACC_LIMIT_PHASE_ERROR_RMS_MEAN_RESULT;
            break;
        case RSFPL_VAL_VSA_LIMIT_PEAK_RMS:
            attr = RSFPL_ATTR_VSA_MACC_LIMIT_PHASE_ERROR_RMS_PEAK_RESULT;
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 4, "Limit Type");
            break;
        }
        break;
    case RSFPL_VAL_VSA_RESULT_CF_ERROR:
        switch (limitType)
        {
        case RSFPL_VAL_VSA_LIMIT_CURRENT:
            attr = RSFPL_ATTR_VSA_MACC_LIMIT_CARRIER_FREQUENCY_ERROR_CURRENT_RESULT;
            break;
        case RSFPL_VAL_VSA_LIMIT_MEAN:
            attr = RSFPL_ATTR_VSA_MACC_LIMIT_CARRIER_FREQUENCY_ERROR_MEAN_RESULT;
            break;
        case RSFPL_VAL_VSA_LIMIT_PEAK:
            attr = RSFPL_ATTR_VSA_MACC_LIMIT_CARRIER_FREQUENCY_ERROR_PEAK_RESULT;
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 4, "Limit Type");
            break;
        }
        break;
    case RSFPL_VAL_VSA_RESULT_IQ_OFFSET:
        switch (limitType)
        {
        case RSFPL_VAL_VSA_LIMIT_CURRENT:
            attr = RSFPL_ATTR_VSA_MACC_LIMIT_IQ_OFFSET_CURRENT_RESULT;
            break;
        case RSFPL_VAL_VSA_LIMIT_MEAN:
            attr = RSFPL_ATTR_VSA_MACC_LIMIT_IQ_OFFSET_MEAN_RESULT;
            break;
        case RSFPL_VAL_VSA_LIMIT_PEAK:
            attr = RSFPL_ATTR_VSA_MACC_LIMIT_IQ_OFFSET_PEAK_RESULT;
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 4, "Limit Type");
            break;
        }
        break;
    case RSFPL_VAL_VSA_RESULT_RHO:
        switch (limitType)
        {
        case RSFPL_VAL_VSA_LIMIT_CURRENT:
            attr = RSFPL_ATTR_VSA_MACC_LIMIT_RHO_CURRENT_RESULT;
            break;
        case RSFPL_VAL_VSA_LIMIT_MEAN:
            attr = RSFPL_ATTR_VSA_MACC_LIMIT_RHO_MEAN_RESULT;
            break;
        case RSFPL_VAL_VSA_LIMIT_PEAK:
            attr = RSFPL_ATTR_VSA_MACC_LIMIT_RHO_PEAK_RESULT;
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 4, "Limit Type");
            break;
        }
        break;
    case RSFPL_VAL_VSA_RESULT_FSK_DEV_ERROR:
        switch (limitType)
        {
        case RSFPL_VAL_VSA_LIMIT_CURRENT:
            attr = RSFPL_ATTR_VSA_MACC_LIMIT_FREQUENCY_DEVIATION_CURRENT_RESULT;
            break;
        case RSFPL_VAL_VSA_LIMIT_MEAN:
            attr = RSFPL_ATTR_VSA_MACC_LIMIT_FREQUENCY_DEVIATION_MEAN_RESULT;
            break;
        case RSFPL_VAL_VSA_LIMIT_PEAK:
            attr = RSFPL_ATTR_VSA_MACC_LIMIT_FREQUENCY_DEVIATION_PEAK_RESULT;
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 4, "Limit Type");
            break;
        }
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 3, "Measurement Type");
        break;
    }

    viCheckParm(rsfpl_GetAttributeViInt32(instrSession, repCap, attr, checkResults),
            5, "Check Results");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This method sets the Enable, NTransition, and PTransition bits of the
/// HIFN  Main, Power, Limit, LMargin, ACP Limit, Frequency, status questionable
/// HIFN  and operating registers.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR questionableregister/This control selects the registers that will be configured.
/// HIPAR enable/This control sets the enable bits of the status register.
/// HIPAR ptransition/This control sets the PTransition bits (edge detectors) of the status
/// HIPAR ptransition/register.
/// HIPAR ntransition/This control sets the NTransition bits (edge detectors) of the status
/// HIPAR ntransition/register.
/// HIPAR channelname/String containing the name of the channel.
ViStatus _VI_FUNC rsfpl_SetVSAStatusRegister(ViSession instrSession,
                                             ViInt32   questionableRegister,
                                             ViInt32   enable,
                                             ViInt32   PTransition,
                                             ViInt32   NTransition,
                                             ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2stat;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsFpl_rngVSAStatusRegister, questionableRegister, &p2stat),
        3, "Questionable Register");

    if (questionableRegister == 0)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":STAT:QUES:MOD:ENAB %ld;PTR %ld;NTR %ld", enable, PTransition, NTransition);
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":STAT:QUES:MOD%ld:%s:ENAB %ld;PTR %ld;NTR %ld", channel, p2stat, enable, PTransition, NTransition);
    }

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsfpl_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This method contains the commands for the querying status reporting
/// HIFN  system.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR register/This parameter selects the registers.
/// HIPAR statusregisterquery/This parameter selects the registers to be queried.
/// HIPAR channelname/String containing the name of the channel.
/// HIPAR registerValue/Returns content of selected register.
ViStatus _VI_FUNC rsfpl_GetVSAStatusRegister(ViSession instrSession,
                                             ViInt32   registerPart,
                                             ViInt32   statusRegisterQuery,
                                             ViInt32   channel,
                                             ViInt32*  registerValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2stat;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsFpl_rngVSAStatusRegister, statusRegisterQuery, &p2stat),
        3, "Questionable Register");
    if (statusRegisterQuery == 0)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":STAT:QUES:MOD:%s?", RsFpl_rngStatusRegisterQuery.entries[registerPart].cmdString);
        checkErr(RsCore_QueryViInt32(instrSession, cmd, registerValue));
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":STAT:QUES:MOD%ld:%s:%s?", channel, p2stat, RsFpl_rngStatusRegisterQuery.entries[registerPart].cmdString);
        checkErr(RsCore_QueryViInt32(instrSession, cmd, registerValue));
    }

    checkErr(rsfpl_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}
